@@ -1,7 +1,7 @@
 #ifndef IVL_PClass_H
 #define IVL_PClass_H
 /*
- * Copyright (c) 2012-2014 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2012-2019 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -20,7 +20,7 @@
  */
 
 # include  "PScope.h"
-# include  "LineInfo.h"
+# include  "PNamedItem.h"
 # include  "StringHeap.h"
 # include  <iostream>
 
@@ -32,7 +32,7 @@ class PChainConstructor;
  * collected.
  */
 
-class PClass : public PScopeExtra, public LineInfo {
+class PClass : public PScopeExtra, public PNamedItem {
 
     public:
       explicit PClass (perm_string name, LexicalScope*parent);
@@ -40,6 +40,8 @@ class PClass : public PScopeExtra, public LineInfo {
 
       void dump(std::ostream&out, unsigned indent) const;
 
+      SymbolType symbol_type() const;
+
     public:
       class_type_t*type;
 };
