@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2004-2019 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -35,3 +35,7 @@ perm_string  PEvent::name() const
       return name_;
 }
 
+PNamedItem::SymbolType PEvent::symbol_type() const
+{
+      return EVENT;
+}
