@@ -1,7 +1,7 @@
 #ifndef IVL_PExpr_H
 #define IVL_PExpr_H
 /*
- * Copyright (c) 1998-2019 Stephen Williams <steve@icarus.com>
+ * Copyright (c) 1998-2021 Stephen Williams <steve@icarus.com>
  * Copyright CERN 2013 / Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
@@ -23,6 +23,7 @@
 # include  <string>
 # include  <vector>
 # include  <valarray>
+# include  <memory>
 # include  "netlist.h"
 # include  "verinum.h"
 # include  "LineInfo.h"
@@ -35,6 +36,7 @@ class NetNet;
 class NetExpr;
 class NetScope;
 class PPackage;
+struct symbol_search_results;
 
 /*
  * The PExpr class hierarchy supports the description of
@@ -60,7 +62,7 @@ class PExpr : public LineInfo {
       PExpr();
       virtual ~PExpr();
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
 
         // This method tests whether the expression contains any identifiers
         // that have not been previously declared in the specified scope or
@@ -172,21 +174,12 @@ class PExpr : public LineInfo {
 					 bool is_cassign,
 					 bool is_force) const;
 
-	// This attempts to evaluate a constant expression, and return
-	// a verinum as a result. If the expression cannot be
-	// evaluated, return 0.
-      virtual verinum* eval_const(Design*des, NetScope*sc) const;
-
 	// This method returns true if the expression represents a
         // structural net that can have multiple drivers. This is
         // used to test whether an input port connection can be
         // collapsed to a single wire.
-      virtual bool is_collapsible_net(Design*des, NetScope*scope) const;
-
-	// This method returns true if that expression is the same as
-	// this expression. This method is used for comparing
-	// expressions that must be structurally "identical".
-      virtual bool is_the_same(const PExpr*that) const;
+      virtual bool is_collapsible_net(Design*des, NetScope*scope,
+                                      NetNet::PortType port_type) const;
 
     protected:
       unsigned fix_width_(width_mode_t mode);
@@ -202,7 +195,7 @@ class PExpr : public LineInfo {
       PExpr& operator= (const PExpr&);
 };
 
-ostream& operator << (ostream&, const PExpr&);
+std::ostream& operator << (std::ostream&, const PExpr&);
 
 class PEAssignPattern : public PExpr {
     public:
@@ -230,11 +223,10 @@ class PEAssignPattern : public PExpr {
 class PEConcat : public PExpr {
 
     public:
-      PEConcat(const list<PExpr*>&p, PExpr*r =0);
+      explicit PEConcat(const std::list<PExpr*>&p, PExpr*r =0);
       ~PEConcat();
 
-      virtual verinum* eval_const(Design*des, NetScope*sc) const;
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
 
       virtual void declare_implicit_nets(LexicalScope*scope, NetNet::Type type);
 
@@ -256,12 +248,13 @@ class PEConcat : public PExpr {
 					 NetScope*scope,
 					 bool is_cassign,
 					 bool is_force) const;
-      virtual bool is_collapsible_net(Design*des, NetScope*scope) const;
+      virtual bool is_collapsible_net(Design*des, NetScope*scope,
+                                      NetNet::PortType port_type) const;
     private:
       NetNet* elaborate_lnet_common_(Design*des, NetScope*scope,
 				     bool bidirectional_flag) const;
     private:
-      vector<PExpr*>parms_;
+      std::vector<PExpr*>parms_;
       std::valarray<width_mode_t>width_modes_;
 
       PExpr*repeat_;
@@ -279,7 +272,7 @@ class PEConcat : public PExpr {
 class PEEvent : public PExpr {
 
     public:
-      enum edge_t {ANYEDGE, POSEDGE, NEGEDGE, POSITIVE};
+      enum edge_t {ANYEDGE, POSEDGE, NEGEDGE, EDGE, POSITIVE};
 
 	// Use this constructor to create events based on edges or levels.
       PEEvent(edge_t t, PExpr*e);
@@ -289,7 +282,7 @@ class PEEvent : public PExpr {
       edge_t type() const;
       PExpr* expr() const;
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
 
       virtual bool has_aa_term(Design*des, NetScope*scope) const;
 
@@ -309,11 +302,6 @@ class PEFNumber : public PExpr {
 
       const verireal& value() const;
 
-	/* The eval_const method as applied to a floating point number
-	   gets the *integer* value of the number. This accounts for
-	   any rounding that is needed to get the value. */
-      virtual verinum* eval_const(Design*des, NetScope*sc) const;
-
       virtual unsigned test_width(Design*des, NetScope*scope,
 				  width_mode_t&mode);
       virtual NetExpr*elaborate_expr(Design*des, NetScope*,
@@ -322,7 +310,7 @@ class PEFNumber : public PExpr {
 				     unsigned expr_wid,
                                      unsigned flags) const;
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
 
     private:
       verireal*value_;
@@ -340,7 +328,7 @@ class PEIdent : public PExpr {
 	// current identifier.
       void append_name(perm_string);
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
 
       virtual void declare_implicit_nets(LexicalScope*scope, NetNet::Type type);
 
@@ -375,9 +363,8 @@ class PEIdent : public PExpr {
 	// expressions can can be unpacked arrays.
       NetNet* elaborate_unpacked_net(Design*des, NetScope*sc) const;
 
-      verinum* eval_const(Design*des, NetScope*sc) const;
-
-      virtual bool is_collapsible_net(Design*des, NetScope*scope) const;
+      virtual bool is_collapsible_net(Design*des, NetScope*scope,
+                                      NetNet::PortType port_type) const;
 
       const PPackage* package() const { return package_; }
 
@@ -404,10 +391,6 @@ class PEIdent : public PExpr {
 	// flag is set to *false*.
       bool calculate_parts_(Design*, NetScope*, long&msb, long&lsb, bool&defined) const;
       NetExpr* calculate_up_do_base_(Design*, NetScope*, bool need_const) const;
-      bool calculate_param_range_(Design*, NetScope*,
-				  const NetExpr*msb_ex, long&msb,
-				  const NetExpr*lsb_ex, long&lsb,
-				  long length) const;
 
       bool calculate_up_do_width_(Design*, NetScope*, unsigned long&wid) const;
 
@@ -434,6 +417,7 @@ class PEIdent : public PExpr {
                                    index_component_t::ctype_t,
 				   bool need_const_idx) const;
       NetAssign_*elaborate_lval_net_class_member_(Design*, NetScope*,
+						   const netclass_t *class_type,
 						   NetNet*,
 						   pform_name_t) const;
       bool elaborate_lval_net_packed_member_(Design*, NetScope*,
@@ -443,41 +427,46 @@ class PEIdent : public PExpr {
 				       NetAssign_*) const;
 
     private:
+      NetExpr* elaborate_expr_(Design *des, NetScope *scope,
+			      unsigned expr_wid, unsigned flags) const;
+
+      NetExpr*elaborate_expr_param_or_specparam_(Design*des,
+						 NetScope*scope,
+						 const NetExpr*par,
+						 NetScope*found_in,
+						 ivl_type_t par_type,
+						 unsigned expr_wid,
+						 unsigned flags) const;
       NetExpr*elaborate_expr_param_(Design*des,
 				    NetScope*scope,
 				    const NetExpr*par,
-				    NetScope*found_in,
-				    const NetExpr*par_msb,
-				    const NetExpr*par_lsb,
+				    const NetScope*found_in,
+				    ivl_type_t par_type,
 				    unsigned expr_wid,
                                     unsigned flags) const;
       NetExpr*elaborate_expr_param_bit_(Design*des,
 					NetScope*scope,
 					const NetExpr*par,
-					NetScope*found_in,
-					const NetExpr*par_msb,
-					const NetExpr*par_lsb,
+					const NetScope*found_in,
+					ivl_type_t par_type,
                                         bool need_const) const;
       NetExpr*elaborate_expr_param_part_(Design*des,
 					 NetScope*scope,
 					 const NetExpr*par,
-					 NetScope*found_in,
-					 const NetExpr*par_msb,
-					 const NetExpr*par_lsb,
+					 const NetScope*found_in,
+					 ivl_type_t par_type,
 				         unsigned expr_wid) const;
       NetExpr*elaborate_expr_param_idx_up_(Design*des,
 					   NetScope*scope,
 					   const NetExpr*par,
-					   NetScope*found_in,
-					   const NetExpr*par_msb,
-					   const NetExpr*par_lsb,
+					   const NetScope*found_in,
+					   ivl_type_t par_type,
                                            bool need_const) const;
       NetExpr*elaborate_expr_param_idx_do_(Design*des,
 					   NetScope*scope,
 					   const NetExpr*par,
-					   NetScope*found_in,
-					   const NetExpr*par_msb,
-					   const NetExpr*par_lsb,
+					   const NetScope*found_in,
+					   ivl_type_t par_type,
                                            bool need_const) const;
       NetExpr*elaborate_expr_net(Design*des,
 				 NetScope*scope,
@@ -522,8 +511,14 @@ class PEIdent : public PExpr {
 					   unsigned expr_wid,
 					   unsigned flags) const;
 
+      NetExpr *elaborate_expr_class_field_(Design*des, NetScope*scope,
+					   const symbol_search_results &sr,
+					   unsigned expr_wid,
+					   unsigned flags) const;
+
       unsigned test_width_method_(Design*des, NetScope*scope, width_mode_t&mode);
 
+      unsigned test_width_parameter_(const NetExpr *par, width_mode_t&mode);
 
     private:
       NetNet* elaborate_lnet_common_(Design*des, NetScope*scope,
@@ -540,7 +535,7 @@ class PENewArray : public PExpr {
       explicit PENewArray (PExpr*s, PExpr*i);
       ~PENewArray();
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
       virtual unsigned test_width(Design*des, NetScope*scope,
 				  width_mode_t&mode);
       virtual NetExpr*elaborate_expr(Design*des, NetScope*scope,
@@ -560,11 +555,12 @@ class PENewClass : public PExpr {
 	// New without (or with default) constructor
       explicit PENewClass ();
 	// New with constructor arguments
-      explicit PENewClass (const std::list<PExpr*>&p);
+      explicit PENewClass (const std::list<PExpr*>&p,
+			   data_type_t *class_type = nullptr);
 
       ~PENewClass();
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
 	// Class objects don't have a useful width, but the expression
 	// is IVL_VT_CLASS.
       virtual unsigned test_width(Design*des, NetScope*scope,
@@ -582,6 +578,7 @@ class PENewClass : public PExpr {
 
     private:
       std::vector<PExpr*>parms_;
+      data_type_t *class_type_;
 };
 
 class PENewCopy : public PExpr {
@@ -589,7 +586,7 @@ class PENewCopy : public PExpr {
       explicit PENewCopy(PExpr*src);
       ~PENewCopy();
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
 	// Class objects don't have a useful width, but the expression
 	// is IVL_VT_CLASS.
       virtual unsigned test_width(Design*des, NetScope*scope,
@@ -609,7 +606,7 @@ class PENull : public PExpr {
       explicit PENull();
       ~PENull();
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
       virtual unsigned test_width(Design*des, NetScope*scope,
 				  width_mode_t&mode);
       virtual NetExpr*elaborate_expr(Design*des, NetScope*scope,
@@ -627,7 +624,7 @@ class PENumber : public PExpr {
 
       const verinum& value() const;
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
       virtual unsigned test_width(Design*des, NetScope*scope,
 				  width_mode_t&mode);
 
@@ -640,10 +637,6 @@ class PENumber : public PExpr {
 					 bool is_cassign,
 					 bool is_force) const;
 
-      virtual verinum* eval_const(Design*des, NetScope*sc) const;
-
-      virtual bool is_the_same(const PExpr*that) const;
-
     private:
       verinum*const value_;
 };
@@ -661,8 +654,8 @@ class PEString : public PExpr {
       explicit PEString(char*s);
       ~PEString();
 
-      string value() const;
-      virtual void dump(ostream&) const;
+      std::string value() const;
+      virtual void dump(std::ostream&) const;
 
       virtual unsigned test_width(Design*des, NetScope*scope,
 				  width_mode_t&mode);
@@ -672,7 +665,6 @@ class PEString : public PExpr {
 
       virtual NetEConst*elaborate_expr(Design*des, NetScope*,
 				       unsigned expr_wid, unsigned) const;
-      verinum* eval_const(Design*, NetScope*) const;
 
     private:
       char*text_;
@@ -683,7 +675,7 @@ class PETypename : public PExpr {
       explicit PETypename(data_type_t*data_type);
       ~PETypename();
 
-      virtual void dump(ostream&) const;
+      virtual void dump(std::ostream&) const;
       virtual unsigned test_width(Design*des, NetScope*scope,
 				  width_mode_t&mode);
       virtual NetExpr*elaborate_expr(Design*des, NetScope*scope,
@@ -701,7 +693,7 @@ class PEUnary : public PExpr {
       explicit PEUnary(char op, PExpr*ex);
       ~PEUnary();
 
-      virtual void dump(ostream&out) const;
+      virtual void dump(std::ostream&out) const;
 
       virtual void declare_implicit_nets(LexicalScope*scope, NetNet::Type type);
 
@@ -713,7 +705,6 @@ class PEUnary : public PExpr {
       virtual NetExpr*elaborate_expr(Design*des, NetScope*,
 				     unsigned expr_wid,
                                      unsigned flags) const;
-      virtual verinum* eval_const(Design*des, NetScope*sc) const;
 
     public:
       inline char get_op() const { return op_; }
@@ -733,7 +724,7 @@ class PEBinary : public PExpr {
       explicit PEBinary(char op, PExpr*l, PExpr*r);
       ~PEBinary();
 
-      virtual void dump(ostream&out) const;
+      virtual void dump(std::ostream&out) const;
 
       virtual void declare_implicit_nets(LexicalScope*scope, NetNet::Type type);
 
@@ -745,7 +736,6 @@ class PEBinary : public PExpr {
       virtual NetExpr*elaborate_expr(Design*des, NetScope*,
 				     unsigned expr_wid,
                                      unsigned flags) const;
-      virtual verinum* eval_const(Design*des, NetScope*sc) const;
 
     protected:
       char op_;
@@ -858,7 +848,7 @@ class PETernary : public PExpr {
       explicit PETernary(PExpr*e, PExpr*t, PExpr*f);
       ~PETernary();
 
-      virtual void dump(ostream&out) const;
+      virtual void dump(std::ostream&out) const;
 
       virtual void declare_implicit_nets(LexicalScope*scope, NetNet::Type type);
 
@@ -870,7 +860,6 @@ class PETernary : public PExpr {
       virtual NetExpr*elaborate_expr(Design*des, NetScope*,
 		                     unsigned expr_wid,
                                      unsigned flags) const;
-      virtual verinum* eval_const(Design*des, NetScope*sc) const;
 
     private:
       NetExpr* elab_and_eval_alternative_(Design*des, NetScope*scope,
@@ -890,7 +879,7 @@ class PETernary : public PExpr {
  */
 class PECallFunction : public PExpr {
     public:
-      explicit PECallFunction(const pform_name_t&n, const vector<PExpr *> &parms);
+      explicit PECallFunction(const pform_name_t&n, const std::vector<PExpr *> &parms);
 	// Call function defined in package.
       explicit PECallFunction(PPackage*pkg, perm_string n, const std::vector<PExpr *> &parms);
       explicit PECallFunction(PPackage*pkg, perm_string n, const std::list<PExpr *> &parms);
@@ -899,16 +888,16 @@ class PECallFunction : public PExpr {
       explicit PECallFunction(PPackage*pkg, const pform_name_t&n, const std::vector<PExpr *> &parms);
 
 	// Call of system function (name is not hierarchical)
-      explicit PECallFunction(perm_string n, const vector<PExpr *> &parms);
+      explicit PECallFunction(perm_string n, const std::vector<PExpr *> &parms);
       explicit PECallFunction(perm_string n);
 
 	// std::list versions. Should be removed!
-      explicit PECallFunction(const pform_name_t&n, const list<PExpr *> &parms);
-      explicit PECallFunction(perm_string n, const list<PExpr *> &parms);
+      explicit PECallFunction(const pform_name_t&n, const std::list<PExpr *> &parms);
+      explicit PECallFunction(perm_string n, const std::list<PExpr *> &parms);
 
       ~PECallFunction();
 
-      virtual void dump(ostream &) const;
+      virtual void dump(std::ostream &) const;
 
       virtual void declare_implicit_nets(LexicalScope*scope, NetNet::Type type);
 
@@ -936,29 +925,33 @@ class PECallFunction : public PExpr {
 
       NetExpr* cast_to_width_(NetExpr*expr, unsigned wid) const;
 
+      NetExpr* elaborate_expr_(Design *des, NetScope *scope,
+			       unsigned flags) const;
+
       NetExpr*elaborate_expr_pkg_(Design*des, NetScope*scope,
-				  unsigned expr_wid, unsigned flags)const;
-      NetExpr*elaborate_expr_method_(Design*des, NetScope*scope,
-				     unsigned expr_wid,
-				     bool add_this_flag = false) const;
-#if 0
-      NetExpr*elaborate_expr_string_method_(Design*des, NetScope*scope) const;
-      NetExpr*elaborate_expr_enum_method_(Design*des, NetScope*scope,
-					  unsigned expr_wid) const;
-#endif
+				  unsigned flags)const;
+
+      NetExpr* elaborate_expr_method_(Design*des, NetScope*scope,
+				      symbol_search_results&search_results)
+				      const;
+      NetExpr* elaborate_expr_method_par_(Design*des, NetScope*scope,
+					  symbol_search_results&search_results)
+					  const;
+
 
       NetExpr* elaborate_sfunc_(Design*des, NetScope*scope,
                                 unsigned expr_wid,
                                 unsigned flags) const;
-      NetExpr* elaborate_access_func_(Design*des, NetScope*scope, ivl_nature_t,
-                                      unsigned expr_wid) const;
+      NetExpr* elaborate_access_func_(Design*des, NetScope*scope, ivl_nature_t)
+                                      const;
       unsigned test_width_sfunc_(Design*des, NetScope*scope,
 			         width_mode_t&mode);
       unsigned test_width_method_(Design*des, NetScope*scope,
+				  symbol_search_results&search_results,
 				  width_mode_t&mode);
 
       NetExpr*elaborate_base_(Design*des, NetScope*scope, NetScope*dscope,
-			      unsigned expr_wid, unsigned flags) const;
+			      unsigned flags) const;
 
       unsigned elaborate_arguments_(Design*des, NetScope*scope,
 				    NetFuncDef*def, bool need_const,
@@ -975,12 +968,14 @@ class PECastSize  : public PExpr {
       explicit PECastSize(PExpr*size, PExpr*base);
       ~PECastSize();
 
-      void dump(ostream &out) const;
+      void dump(std::ostream &out) const;
 
       virtual NetExpr*elaborate_expr(Design*des, NetScope*scope,
 				     unsigned expr_wid,
                                      unsigned flags) const;
 
+      virtual bool has_aa_term(Design *des, NetScope *scope) const;
+
       virtual unsigned test_width(Design*des, NetScope*scope,
 				  width_mode_t&mode);
 
@@ -998,7 +993,7 @@ class PECastType  : public PExpr {
       explicit PECastType(data_type_t*target, PExpr*base);
       ~PECastType();
 
-      void dump(ostream &out) const;
+      void dump(std::ostream &out) const;
 
       virtual NetExpr*elaborate_expr(Design*des, NetScope*scope,
 				     ivl_type_t type, unsigned flags) const;
@@ -1006,14 +1001,39 @@ class PECastType  : public PExpr {
       virtual NetExpr*elaborate_expr(Design*des, NetScope*scope,
 				     unsigned expr_wid, unsigned flags) const;
 
+      virtual bool has_aa_term(Design *des, NetScope *scope) const;
+
       virtual unsigned test_width(Design*des, NetScope*scope,
 				  width_mode_t&mode);
 
     private:
       data_type_t* target_;
+      ivl_type_t target_type_;
       PExpr* base_;
 };
 
+/*
+ * Support the SystemVerilog sign cast.
+ */
+class PECastSign : public PExpr {
+
+    public:
+      explicit PECastSign(bool signed_flag, PExpr *base);
+      ~PECastSign() = default;
+
+      void dump(std::ostream &out) const;
+
+      NetExpr* elaborate_expr(Design *des, NetScope *scope,
+			      unsigned expr_wid, unsigned flags) const;
+
+      virtual bool has_aa_term(Design *des, NetScope *scope) const;
+
+      unsigned test_width(Design *des, NetScope *scope, width_mode_t &mode);
+
+    private:
+      std::unique_ptr<PExpr> base_;
+};
+
 /*
  * This class is used for error recovery. All methods do nothing and return
  * null or default values.
