@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -23,6 +23,8 @@
 # include <cassert>
 # include "ivl_assert.h"
 
+using namespace std;
+
 PFunction::PFunction(perm_string name, LexicalScope*parent, bool is_auto__)
 : PTaskFunc(name, parent), statement_(0)
 {
@@ -43,7 +45,7 @@ void PFunction::set_statement(Statement*s)
 
 void PFunction::push_statement_front(Statement*stmt)
 {
-	// This can only happen after the statement is initially set.
+        // This should not be possible.
       ivl_assert(*this, statement_);
 
 	// Get the PBlock of the statement. If it is not a PBlock,
@@ -75,7 +77,8 @@ PChainConstructor* PFunction::extract_chain_constructor()
       PChainConstructor*res = 0;
 
       if ((res = dynamic_cast<PChainConstructor*> (statement_))) {
-	    statement_ = 0;
+	    statement_ = new PBlock(PBlock::BL_SEQ);
+	    statement_->set_line(*this);
 
       } else if (PBlock*blk = dynamic_cast<PBlock*>(statement_)) {
 	    res = blk->extract_chain_constructor();
@@ -88,3 +91,14 @@ PNamedItem::SymbolType PFunction::symbol_type() const
 {
       return FUNCTION;
 }
+
+
+PLet::PLet(perm_string name, LexicalScope*parent, list<let_port_t*>*ports,
+           PExpr*expr)
+: PTaskFunc(name, parent), ports_(ports), expr_(expr)
+{
+}
+
+PLet::~PLet()
+{
+}
