@@ -1,7 +1,7 @@
 #ifndef IVL_PGenerate_H
 #define IVL_PGenerate_H
 /*
- * Copyright (c) 2006-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2006-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -69,6 +69,8 @@ class PGenerate : public PNamedItem, public LexicalScope {
 		     GS_CASE, GS_CASE_ITEM, GS_NBLOCK};
       scheme_t scheme_type;
 
+      bool directly_nested;
+
 	// generate loops have an index variable and three
 	// expressions: for (index = <init>; <test>; index=<step>)
 	// the index is local if it was declared in the init expression,
@@ -84,18 +86,18 @@ class PGenerate : public PNamedItem, public LexicalScope {
       std::valarray<PExpr*> item_test;
 
 	// defparam assignments found in this scope.
-      typedef pair<pform_name_t,PExpr*> named_expr_t;
-      list<named_expr_t>defparms;
+      typedef std::pair<pform_name_t,PExpr*> named_expr_t;
+      std::list<named_expr_t>defparms;
 
-      list<PGate*> gates;
+      std::list<PGate*> gates;
       void add_gate(PGate*);
 
 	// Tasks instantiated within this scheme.
-      map<perm_string,PTask*> tasks;
-      map<perm_string,PFunction*>funcs;
+      std::map<perm_string,PTask*> tasks;
+      std::map<perm_string,PFunction*>funcs;
 
 	// Generate schemes can contain further generate schemes.
-      list<PGenerate*> generate_schemes;
+      std::list<PGenerate*> generate_schemes;
 //      PGenerate*parent;
 
 	// This method is called by the elaboration of a module to
@@ -108,29 +110,23 @@ class PGenerate : public PNamedItem, public LexicalScope {
       bool elaborate_sig(Design*des, NetScope*container) const;
       bool elaborate(Design*des, NetScope*container) const;
 
-      void dump(ostream&out, unsigned indent) const;
+      void dump(std::ostream&out, unsigned indent) const;
 
       SymbolType symbol_type() const;
 
     private:
+      void check_for_valid_genvar_value_(long value);
       bool generate_scope_loop_(Design*des, NetScope*container);
       bool generate_scope_condit_(Design*des, NetScope*container, bool else_flag);
       bool generate_scope_case_(Design*des, NetScope*container);
       bool generate_scope_nblock_(Design*des, NetScope*container);
 
-	// Call probe during elaborate_scope to calculate the
-	// direct_nested_ flag. It is OK to store the direct_nested_
-	// information here because "direct nested" is a property of
-	// the lexical generate code.
-      void probe_for_direct_nesting_(void);
-      bool direct_nested_;
-
 	// Elaborate_scope within a generated scope.
       void elaborate_subscope_(Design*des, NetScope*scope);
       void elaborate_subscope_direct_(Design*des, NetScope*scope);
 
 	// These are the scopes created by generate_scope.
-      list<NetScope*>scope_list_;
+      std::list<NetScope*>scope_list_;
 	// internal function called on each scope generated by this scheme.
       bool elaborate_sig_(Design*des, NetScope*scope) const;
       bool elaborate_sig_direct_(Design*des, NetScope*scope) const;
