@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -18,24 +18,32 @@
  */
 
 # include "config.h"
+# include "ivl_assert.h"
 # include  "PWire.h"
 # include  "PExpr.h"
 # include  <cassert>
 
+using namespace std;
+
 PWire::PWire(perm_string n,
 	     NetNet::Type t,
 	     NetNet::PortType pt,
-	     ivl_variable_type_t dt)
-: name_(n), type_(t), port_type_(pt), data_type_(dt),
-  signed_(false), isint_(false),
+	     PWSRType rt)
+: name_(n), type_(t), port_type_(pt), signed_(false),
   port_set_(false), net_set_(false), is_scalar_(false),
-  error_cnt_(0), uarray_type_(0), set_data_type_(0),
-  discipline_(0)
+  error_cnt_(0), discipline_(0)
 {
-      if (t == NetNet::INTEGER) {
-	    type_ = NetNet::REG;
-	    signed_ = true;
-	    isint_ = true;
+      switch (rt) {
+	  case SR_PORT:
+	    port_set_ = true;
+	    break;
+	  case SR_NET:
+	    net_set_ = true;
+	    break;
+	  case SR_BOTH:
+	    port_set_ = true;
+	    net_set_ = true;
+	    break;
       }
 }
 
@@ -62,18 +70,9 @@ bool PWire::set_wire_type(NetNet::Type t)
 		  type_ = t;
 		  return true;
 	    }
-	    if (t == NetNet::INTEGER) {
-		  type_ = NetNet::REG;
-		  isint_ = true;
-		  return true;
-	    }
 	    if (t == NetNet::IMPLICIT_REG) return true;
 	    return false;
 	  case NetNet::REG:
-	    if (t == NetNet::INTEGER) {
-		  isint_ = true;
-		  return true;
-	    }
 	    if (t == NetNet::REG) return true;
 	    return false;
 	  default:
@@ -96,12 +95,10 @@ bool PWire::set_port_type(NetNet::PortType pt)
 
       switch (port_type_) {
 	  case NetNet::PIMPLICIT:
+	  case NetNet::NOT_A_PORT:
 	    port_type_ = pt;
 	    return true;
 
-	  case NetNet::NOT_A_PORT:
-	    return false;
-
 	  default:
 	    if (port_type_ != pt)
 		  return false;
@@ -110,29 +107,15 @@ bool PWire::set_port_type(NetNet::PortType pt)
       }
 }
 
-bool PWire::set_data_type(ivl_variable_type_t dt)
-{
-      if (data_type_ != IVL_VT_NO_TYPE) {
-	    if (data_type_ != dt)
-		  return false;
-	    else
-		  return true;
-      }
-
-      assert(data_type_ == IVL_VT_NO_TYPE);
-      data_type_ = dt;
-
-      return true;
-}
-
-ivl_variable_type_t PWire::get_data_type() const
-{
-      return data_type_;
-}
-
 void PWire::set_signed(bool flag)
 {
-      signed_ = flag;
+	// For a non-ANSI style port declaration where the data type is
+	// specified in a corresponding variable declaration, the signed
+	// attribute may be attached to either the port declaration or to
+	// the variable declaration (IEEE 1364-2005 section 12.3.3). The
+	// signal is signed if either the port or the variable is signed.
+	// Handle that here.
+      signed_ = signed_ || flag;
 }
 
 bool PWire::get_signed() const
@@ -140,64 +123,23 @@ bool PWire::get_signed() const
       return signed_;
 }
 
-bool PWire::get_isint() const
+void PWire::set_port(NetNet::PortType pt)
 {
-      if (isint_)
-	    return true;
+      ivl_assert(*this, !port_set_);
+      port_set_ = true;
 
-      if (vector_type_t*tmp = dynamic_cast<vector_type_t*>(set_data_type_)) {
-	    return tmp->integer_flag;
-      }
-
-      return false;
+      bool rc = set_port_type(pt);
+      ivl_assert(*this, rc);
 }
 
-bool PWire::get_scalar() const
+void PWire::set_net(NetNet::Type t)
 {
-      return is_scalar_;
-}
+      ivl_assert(*this, !net_set_);
+      net_set_ = true;
 
-void PWire::set_range_scalar(PWSRType type)
-{
-      is_scalar_ = true;
-      switch (type) {
-	  case SR_PORT:
-	    if (port_set_) {
-		  cerr << get_fileline() << ": error: Port ``" << name_
-		       << "'' has already been declared a port." << endl;
-		  error_cnt_ += 1;
-	    } else {
-		  port_set_ = true;
-	    }
-	    return;
-
-	  case SR_NET:
-	    if (net_set_) {
-		  cerr << get_fileline() << ": error: Net ``" << name_
-		       << "'' has already been declared." << endl;
-		  error_cnt_ += 1;
-	    } else {
-		  net_set_ = true;
-	    }
-	    return;
-
-	  case SR_BOTH:
-	    if (port_set_ || net_set_) {
-		  if (port_set_) {
-		        cerr << get_fileline() << ": error: Port ``" << name_
-		             << "'' has already been declared a port." << endl;
-		        error_cnt_ += 1;
-		  }
-		  if (net_set_) {
-		        cerr << get_fileline() << ": error: Net ``" << name_
-		             << "'' has already been declared." << endl;
-		        error_cnt_ += 1;
-		  }
-	    } else {
-		  port_set_ = true;
-		  net_set_ = true;
-	    }
-	    return;
+      if (t != NetNet::IMPLICIT) {
+	    bool rc = set_wire_type(t);
+	    ivl_assert(*this, rc);
       }
 }
 
@@ -205,49 +147,21 @@ void PWire::set_range(const list<pform_range_t>&rlist, PWSRType type)
 {
       switch (type) {
 	  case SR_PORT:
-	    if (port_set_) {
-		  cerr << get_fileline() << ": error: Port ``" << name_
-		       << "'' has already been declared a port." << endl;
-		  error_cnt_ += 1;
-	    } else {
-		  port_ = rlist;
-		  port_set_ = true;
-		  is_scalar_ = false;
-	    }
-	    return;
-
+	    if (!port_.empty())
+		  return;
+	    port_ = rlist;
+	    break;
 	  case SR_NET:
-	    if (net_set_) {
-		  cerr << get_fileline() << ": error: Net ``" << name_
-		       << "'' has already been declared." << endl;
-		  error_cnt_ += 1;
-	    } else {
-		  net_ = rlist;
-		  net_set_ = true;
-		  is_scalar_ = false;
-	    }
-	    return;
-
+	    if (!net_.empty())
+		  return;
+	    net_ = rlist;
+	    break;
 	  case SR_BOTH:
-	    if (port_set_ || net_set_) {
-		  if (port_set_) {
-		        cerr << get_fileline() << ": error: Port ``" << name_
-		             << "'' has already been declared a port." << endl;
-		        error_cnt_ += 1;
-		  }
-		  if (net_set_) {
-		        cerr << get_fileline() << ": error: Net ``" << name_
-		             << "'' has already been declared." << endl;
-		        error_cnt_ += 1;
-		  }
-	    } else {
-		  port_ = rlist;
-		  port_set_ = true;
-		  net_ = rlist;
-		  net_set_ = true;
-		  is_scalar_ = false;
-	    }
-	    return;
+	    if (!port_.empty() || !net_.empty())
+		  return;
+	    port_ = rlist;
+	    net_ = rlist;
+	    break;
       }
 }
 
@@ -264,13 +178,11 @@ void PWire::set_unpacked_idx(const list<pform_range_t>&ranges)
 
 void PWire::set_data_type(data_type_t*type)
 {
-      assert(set_data_type_ == 0);
-      set_data_type_ = type;
+      if (set_data_type_.get() == type)
+	    return;
 
-      if (vector_type_t*tmp = dynamic_cast<vector_type_t*>(type)) {
-	    if (tmp->integer_flag)
-		  isint_ = true;
-      }
+      assert(!set_data_type_.get());
+      set_data_type_.reset(type);
 }
 
 void PWire::set_discipline(ivl_discipline_t d)
@@ -288,7 +200,6 @@ PNamedItem::SymbolType PWire::symbol_type() const
 {
       switch (type_) {
           case NetNet::IMPLICIT_REG:
-          case NetNet::INTEGER:
           case NetNet::REG:
             return VAR;
           default:
