@@ -1,7 +1,7 @@
 #ifndef IVL_PWire_H
 #define IVL_PWire_H
 /*
- * Copyright (c) 1998-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -28,7 +28,7 @@
 #ifdef HAVE_IOSFWD
 # include  <iosfwd>
 #else
-class ostream;
+# include  <iostream>
 #endif
 
 class PExpr;
@@ -57,7 +57,7 @@ class PWire : public PNamedItem {
       PWire(perm_string name,
 	    NetNet::Type t,
 	    NetNet::PortType pt,
-	    ivl_variable_type_t dt);
+	    PWSRType rt = SR_NET);
 
 	// Return a hierarchical name.
       perm_string basename() const;
@@ -70,39 +70,35 @@ class PWire : public PNamedItem {
 
       void set_signed(bool flag);
       bool get_signed() const;
-      bool get_isint() const;
-      bool get_scalar() const;
 
-      bool set_data_type(ivl_variable_type_t dt);
-      ivl_variable_type_t get_data_type() const;
-
-      void set_range_scalar(PWSRType type);
       void set_range(const std::list<pform_range_t>&ranges, PWSRType type);
 
       void set_unpacked_idx(const std::list<pform_range_t>&ranges);
-      void set_uarray_type(uarray_type_t*type) { uarray_type_ = type; }
 
       void set_data_type(data_type_t*type);
 
       void set_discipline(ivl_discipline_t);
       ivl_discipline_t get_discipline(void) const;
 
-      map<perm_string,PExpr*> attributes;
+      std::map<perm_string,PExpr*> attributes;
 
 	// Write myself to the specified stream.
-      void dump(ostream&out, unsigned ind=4) const;
+      void dump(std::ostream&out, unsigned ind=4) const;
 
       NetNet* elaborate_sig(Design*, NetScope*scope) const;
 
       SymbolType symbol_type() const;
 
+      bool is_net() const { return net_set_; };
+      bool is_port() const { return port_set_; };
+      void set_net(NetNet::Type t);
+      void set_port(NetNet::PortType pt);
+
     private:
       perm_string name_;
       NetNet::Type type_;
       NetNet::PortType port_type_;
-      ivl_variable_type_t data_type_;
       bool signed_;
-      bool isint_;		// original type of integer
 
 	// These members hold expressions for the bit width of the
 	// wire. If they do not exist, the wire is 1 bit wide. If they
@@ -120,17 +116,23 @@ class PWire : public PNamedItem {
 	// If this wire is actually a memory, these indices will give
 	// me the size and address ranges of the memory.
       std::list<pform_range_t>unpacked_;
-      uarray_type_t*uarray_type_;
 
 	// This is the complex type of the wire. the data_type_ may
 	// modify how this is interpreted.
-      data_type_t*set_data_type_;
+      std::unique_ptr<data_type_t> set_data_type_;
 
       ivl_discipline_t discipline_;
 
     private: // not implemented
       PWire(const PWire&);
       PWire& operator= (const PWire&);
+
+      ivl_type_t elaborate_type(Design*des, NetScope*scope,
+			        const std::vector<netrange_t>&packed_dimensions) const;
+      ivl_type_t elaborate_darray_type(Design*des, NetScope*scope,
+				       const char *darray_type,
+				       const std::vector<netrange_t>&packed_dimensions)
+				       const;
 };
 
 #endif /* IVL_PWire_H */
