@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -31,10 +31,13 @@
 # include  "netclass.h"
 # include  "netdarray.h"
 # include  "netqueue.h"
+# include  "netscalar.h"
 # include  "netvector.h"
 # include  "ivl_assert.h"
 # include  "PExpr.h"
 
+using namespace std;
+
 static ostream& operator<< (ostream&o, NetBlock::Type t)
 {
       switch (t) {
@@ -238,6 +241,17 @@ ostream& netqueue_t::debug_dump(ostream&fd) const
       return fd;
 }
 
+ostream& netreal_t::debug_dump(ostream&fd) const
+{
+      fd << "real";
+      return fd;
+}
+ostream& netstring_t::debug_dump(ostream&fd) const
+{
+      fd << "string";
+      return fd;
+}
+
 ostream& netvector_t::debug_dump(ostream&o) const
 {
       o << "netvector_t:" << type_ << (signed_? " signed" : " unsigned") << packed_dims_;
@@ -1110,7 +1124,10 @@ void NetAssign::dump(ostream&o, unsigned ind) const
       if (const NetExpr*de = get_delay())
 	    o << "#(" << *de << ") ";
 
-      o << *rval() << ";" << endl;
+      if (rval())
+	    o << *rval() << ";" << endl;
+      else
+	    o << "<rval elaboration error>;" << endl;
 }
 
 void NetAssignNB::dump(ostream&o, unsigned ind) const
@@ -1128,7 +1145,10 @@ void NetAssignNB::dump(ostream&o, unsigned ind) const
 	    o << *event_;
       }
 
-      o << *rval() << ";" << endl;
+      if (rval())
+	    o << *rval() << ";" << endl;
+      else
+	    o << "rval elaboration error>;" << endl;
 
 }
 
@@ -1273,6 +1293,9 @@ void NetEvProbe::dump_node(ostream&o, unsigned ind) const
 	  case NEGEDGE:
 	    o << "negedge ";
 	    break;
+	  case EDGE:
+	    o << "edge ";
+	    break;
       }
       o << setw(ind) << "" << "-> " << event_->name() << "; " << endl;
       dump_node_pins(o, ind+4);
@@ -1285,6 +1308,13 @@ void NetEvTrig::dump(ostream&o, unsigned ind) const
 	<< "// " << get_fileline() << endl;
 }
 
+void NetEvNBTrig::dump(ostream&o, unsigned ind) const
+{
+      o << setw(ind) << "" << "->> " ;
+      if (dly_) o << "#" << *dly_ << " ";
+      o << event_->name() << "; " << "// " << get_fileline() << endl;
+}
+
 void NetEvWait::dump(ostream&o, unsigned ind) const
 {
       o << setw(ind) << "";
@@ -1351,9 +1381,15 @@ void NetForever::dump(ostream&o, unsigned ind) const
 
 void NetForLoop::dump(ostream&fd, unsigned ind) const
 {
-      fd << setw(ind) << "" << "FOR LOOP index=" << index_->name() << endl;
+      fd << setw(ind) << "" << "FOR LOOP index=";
+      if (index_)
+	    fd << index_->name();
+      else
+	    fd << "<nil>";
+      fd << endl;
       statement_->dump(fd, ind+4);
-      step_statement_->dump(fd, ind+4);
+      if (step_statement_)
+	    step_statement_->dump(fd, ind+4);
 }
 
 void NetFree::dump(ostream&o, unsigned ind) const
@@ -1465,14 +1501,8 @@ void NetScope::dump(ostream&o) const
 		  else
 			o << "    parameter ";
 
-		  o << pp->second.type << " ";
-
-		  if ((*pp).second.signed_flag)
-			o << "signed ";
-
-		  if ((*pp).second.msb)
-			o << "[" << *(*pp).second.msb
-			  << ":" << *(*pp).second.lsb << "] ";
+		  if (pp->second.ivl_type)
+			pp->second.ivl_type->debug_dump(o);
 
 		  o << (*pp).first << " = ";
 		  if (pp->second.val)
