@@ -1,6 +1,6 @@
 /*
+ * Copyright (c) 2015-2022 Martin Whitaker
  * Copyright (c) 2002 Gus Baldauf (gus@picturel.com)
- * Copyright (c) 2015 Martin Whitaker
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -241,13 +241,13 @@ static int GetRegistryKey(char *key, char **value)
 
 	lrv = RegQueryValueEx(hkKey, key, NULL, &regKeyType, NULL, &regKeySize);
 	if ((lrv != ERROR_SUCCESS) || (regKeyType != REG_SZ) || (!regKeySize)) {
-		lrv = RegCloseKey(hkKey);
+		RegCloseKey(hkKey);
 		return 0;
 	}
 
 	regKeyBuffer = (char *) malloc(regKeySize+1);
 	if (!regKeyBuffer) {
-		lrv = RegCloseKey(hkKey);
+		RegCloseKey(hkKey);
 		fprintf(stderr, "error: out of memory\n");
 		myExit(4);
 	}
@@ -257,7 +257,7 @@ static int GetRegistryKey(char *key, char **value)
 	lrv = RegQueryValueEx(hkKey, key, NULL, &regKeyType,
 	                      (unsigned char *) regKeyBuffer, &regKeySize);
 	if ((lrv != ERROR_SUCCESS) || (regKeyType != REG_SZ) || (!regKeySize)) {
-		lrv = RegCloseKey(hkKey);
+		RegCloseKey(hkKey);
 		free(regKeyBuffer);
 		return 0;
 	}
@@ -564,10 +564,10 @@ static void compile(char *pSource, char *pFlags, char **pObject, int *compile_er
 		assignn(&src, ptr1, len);
 
 		  /* Build the object file name */
-		ostart = strrchr(ptr1, '/');
-		if (ostart == NULL) ostart = ptr1;
+		ostart = strrchr(src, '/');
+		if (ostart == NULL) ostart = src;
 		else ostart += 1;
-		olen = strrchr(ptr1, '.') - ostart;
+		olen = strrchr(ostart, '.') - ostart;
 		assignn(&obj, ostart, olen);
 		append(&obj, ".o");
 
