@@ -1,6 +1,6 @@
+const char COPYRIGHT[] =
+  "Copyright (c) 2000-2021 Stephen Williams (steve@icarus.com)";
 /*
- * Copyright (c) 2000-2020 Stephen Williams (steve@icarus.com)
- *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
  *    General Public License as published by the Free Software
@@ -38,7 +38,7 @@ const char NOTICE[] =
 ;
 
 const char HELP[] =
-"Usage: iverilog [-EiSuvV] [-B base] [-c cmdfile|-f cmdfile]\n"
+"Usage: iverilog [-EiRSuvV] [-B base] [-c cmdfile|-f cmdfile]\n"
 "                [-g1995|-g2001|-g2005|-g2005-sv|-g2009|-g2012] [-g<feature>]\n"
 "                [-D macro[=defn]] [-I includedir] [-L moduledir]\n"
 "                [-M [mode=]depfile] [-m module]\n"
@@ -184,7 +184,7 @@ int verbose_flag = 0;
 FILE *fp;
 
 char line[MAXSIZE];
-char tmp[MAXSIZE];
+char tmp[MAXSIZE+24];
 
 static char ivl_root[MAXSIZE];
 
@@ -355,15 +355,15 @@ static int t_version_only(void)
 
 static void build_preprocess_command(int e_flag)
 {
-      snprintf(tmp, sizeof tmp, "%s%civlpp %s%s%s -F\"%s\" -f\"%s\" -p\"%s\"%s",
+      snprintf(tmp, sizeof tmp, "%s%civlpp%s%s%s -F\"%s\" -f\"%s\" -p\"%s\"%s",
 	       ivlpp_dir, sep,
                verbose_flag ? " -v" : "",
 	       e_flag ? "" : " -L",
-               strchr(warning_flags, 'r') ? " -Wredef-all " :
-               strchr(warning_flags, 'R') ? " -Wredef-chg " : "",
+               strchr(warning_flags, 'r') ? " -Wredef-all" :
+               strchr(warning_flags, 'R') ? " -Wredef-chg" : "",
                defines_path, source_path,
 	       compiled_defines_path,
-	       e_flag ? "" : " | ");
+	       e_flag ? "" : " |");
 }
 
 static int t_preprocess_only(void)
@@ -382,7 +382,6 @@ static int t_preprocess_only(void)
 	    snprintf(tmp, sizeof tmp, " > \"%s\"", opath);
 	    cmd = realloc(cmd, ncmd+strlen(tmp)+1);
 	    strcpy(cmd+ncmd, tmp);
-	    ncmd += strlen(tmp);
       }
 
       if (verbose_flag)
@@ -483,7 +482,6 @@ static int t_compile(void)
       rc = strlen(tmp);
       cmd = realloc(cmd, ncmd+rc+1);
       strcpy(cmd+ncmd, tmp);
-      ncmd += rc;
 
 
       if (verbose_flag)
@@ -1046,7 +1044,7 @@ static void find_ivl_root(void)
 	   if the user has overridden $(bindir) or $(libdir), but there's
 	   not a lot we can do in that case.
  */
-#ifdef __MINGW32__
+#if defined(__MINGW32__)
       char tmppath[MAXSIZE];
       len = GetModuleFileName(NULL, tmppath, sizeof tmppath);
       if (len >= (ssize_t) sizeof ivl_root) {
@@ -1054,6 +1052,9 @@ static void find_ivl_root(void)
       }
 	/* Convert to a short name to remove any embedded spaces. */
       len = GetShortPathName(tmppath, ivl_root, sizeof ivl_root);
+#elif defined(__APPLE__)
+	  uint32_t size = sizeof ivl_root;
+	  len = _NSGetExecutablePath(ivl_root, &size) + 1;
 #else
       len = readlink("/proc/self/exe", ivl_root, sizeof ivl_root);
 #endif
@@ -1061,7 +1062,11 @@ static void find_ivl_root(void)
 	    find_ivl_root_failed("command path exceeds size of string buffer.");
       }
       if (len <= 0) {
-	    find_ivl_root_failed("couldn't get command path from OS.");
+	      // We've failed, but we may yet find a -B option on the command line.
+	      // Use the built-in path so the user sees a sensible error message.
+	    assert(strlen(IVL_ROOT) < sizeof ivl_root);
+	    strcpy(ivl_root, IVL_ROOT);
+	    return;
       }
       s = strrchr(ivl_root, sep);
       if (s == 0) {
@@ -1083,6 +1088,10 @@ static void find_ivl_root(void)
       strcat(ivl_root, ivl_lib_suffix);
 }
 
+static void print_runtime_paths(void) {
+	printf("includedir: %s\n", IVL_INCLUDE_INSTALL_DIR);
+}
+
 int main(int argc, char **argv)
 {
       int e_flag = 0;
@@ -1116,8 +1125,6 @@ int main(int argc, char **argv)
       }
 
       fprintf(defines_file, "D:__ICARUS__=1\n");
-      if (strcmp(gen_verilog_ams,"verilog-ams") == 0)
-	    fprintf(defines_file, "D:__VAMS_ENABLE__=1\n");
 
 	/* Create another temporary file for passing configuration
 	   information to ivl. */
@@ -1154,7 +1161,7 @@ int main(int argc, char **argv)
 	}
       }
 
-      while ((opt = getopt(argc, argv, "B:c:D:d:Ef:g:hl:I:iL:M:m:N:o:P:p:Ss:T:t:uvVW:y:Y:")) != EOF) {
+      while ((opt = getopt(argc, argv, "B:c:D:d:Ef:g:hl:I:iL:M:m:N:o:P:p:RSs:T:t:uvVW:y:Y:")) != EOF) {
 
 	    switch (opt) {
 		case 'B':
@@ -1241,6 +1248,10 @@ int main(int argc, char **argv)
 		  opath = optarg;
 		  break;
 
+		case 'R':
+		  print_runtime_paths();
+		  return 0;
+
 		case 'S':
 		  synth_flag = 1;
 		  break;
@@ -1301,6 +1312,11 @@ int main(int argc, char **argv)
 	    }
       }
 
+      if (strcmp(gen_verilog_ams,"verilog-ams") == 0)
+	    fprintf(defines_file, "D:__VAMS_ENABLE__=1\n");
+      if (synth_flag)
+	    fprintf(defines_file, "D:__ICARUS_SYNTH__=1\n");
+
       if (vpi_dir == 0)
 	    vpi_dir = base;
       if (ivlpp_dir == 0)
@@ -1310,7 +1326,7 @@ int main(int argc, char **argv)
 
       if (version_flag || verbose_flag) {
 	    printf("Icarus Verilog version " VERSION " (" VERSION_TAG ")\n\n");
-	    printf("Copyright 1998-2020 Stephen Williams\n\n");
+	    printf("%s\n\n", COPYRIGHT);
 	    puts(NOTICE);
       }
 
@@ -1379,7 +1395,7 @@ int main(int argc, char **argv)
 
 	    cfreset(fp, command_filename);
 	    rc = cfparse();
-	    if (rc != 0) {
+	    if (rc != 0 || command_file_errors > 0) {
 		  fprintf(stderr, "%s: parsing failed in base command "
 		          "file %s.\n", argv[0], command_filename);
 		  return 1;
