@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -24,6 +24,8 @@
 # include  <cstdlib>
 # include  "ivl_assert.h"
 
+using namespace std;
+
 NetEAccess* NetEAccess::dup_expr() const
 {
       NetEAccess*tmp = new NetEAccess(branch_, nature_);
@@ -146,7 +148,7 @@ NetEConst* NetEConst::dup_expr() const
 
 NetEConstEnum* NetEConstEnum::dup_expr() const
 {
-      NetEConstEnum*tmp = new NetEConstEnum(scope_, name_, enum_set_, value());
+      NetEConstEnum*tmp = new NetEConstEnum(name_, enum_set_, value());
       ivl_assert(*this, tmp);
       tmp->set_line(*this);
       return tmp;
