@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2021 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2012-2013 / Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
@@ -36,6 +36,8 @@
 # include  <climits>
 # include  "ivl_assert.h"
 
+using namespace std;
+
 /*
  * These methods generate a NetAssign_ object for the l-value of the
  * assignment. This is common code for the = and <= statements.
@@ -157,9 +159,6 @@ NetAssign_* PEIdent::elaborate_lval(Design*des,
 				    bool is_cassign,
 				    bool is_force) const
 {
-      NetNet*       reg = 0;
-      const NetExpr*par = 0;
-      NetEvent*     eve = 0;
 
       if (debug_elaborate) {
 	    cerr << get_fileline() << ": PEIdent::elaborate_lval: "
@@ -180,25 +179,12 @@ NetAssign_* PEIdent::elaborate_lval(Design*des,
 	    ivl_assert(*this, use_scope);
       }
 
-	/* Try to find the base part of the path that names the
-	   variable. The remainer is the member path. For example, if
-	   the path is a.b.c.d, and a.b is the path to a variable,
-	   then a.b becomes the base_path and c.d becomes the
-	   member_path. If we cannot find the variable with any
-	   prefix, then the base_path will be empty after this loop
-	   and reg will remain nil. */
-      pform_name_t base_path = path_;
-      pform_name_t member_path;
-      while (reg == 0 && base_path.size() > 0) {
-	    symbol_search(this, des, use_scope, base_path, reg, par, eve);
-	      // Found it!
-	    if (reg != 0) break;
-	      // Not found. Try to pop another name off the base_path
-	      // and push it to the front of the member_path.
-	    member_path.push_front( base_path.back() );
-	    base_path.pop_back();
-      }
+      symbol_search_results sr;
+      symbol_search(this, des, use_scope, path_, &sr);
 
+      NetNet *reg = sr.net;
+      pform_name_t &base_path = sr.path_head;
+      pform_name_t &member_path = sr.path_tail;
 
 	/* The l-value must be a variable. If not, then give up and
 	   print a useful error message. */
@@ -222,18 +208,28 @@ NetAssign_* PEIdent::elaborate_lval(Design*des,
       ivl_assert(*this, reg);
 
       if (debug_elaborate) {
-	    cerr << get_fileline() << ": PEIdent::elaborate_lval: "
+	    cerr << get_fileline() << ": " << __func__ << ": "
 		 << "Found l-value path_=" << path_
-		 << " as reg=" << reg->name()
+		 << " as reg=" << reg->name() << endl;
+	    cerr << get_fileline() << ": " << __func__ << ": "
+		 << "reg->type()=" << reg->type()
+		 << ", reg->unpacked_dimensions()=" << reg->unpacked_dimensions()
+		 << endl;
+	    if (reg->net_type())
+		  cerr << get_fileline() << ": " << __func__ << ": "
+		       << "reg->net_type()=" << *reg->net_type() << endl;
+	    else
+		  cerr << get_fileline() << ": " << __func__ << ": "
+		       << "reg->net_type()=<nil>" << endl;
+	    cerr << get_fileline() << ": " << __func__ << ": "
 		 << " base_path=" << base_path
 		 << ", member_path=" << member_path
-		 << " unpacked_dimensions()=" << reg->unpacked_dimensions() << endl;
+		 << endl;
       }
 
 	// We are processing the tail of a string of names. For
 	// example, the Verilog may be "a.b.c", so we are processing
-	// "c" at this point. (Note that if method_name is not nil,
-	// then this is "a.b.c.method" and "a.b.c" is a struct or class.)
+	// "c" at this point.
       const name_component_t&name_tail = path_.back();
 
 	// Use the last index to determine what kind of select
@@ -247,12 +243,18 @@ NetAssign_* PEIdent::elaborate_lval(Design*des,
 	    use_sel = name_tail.index.back().sel;
 
 	// Special case: The l-value is an entire memory, or array
-	// slice. This is, in fact, an error in l-values. Detect the
-	// situation by noting if the index count is less than the
-	// array dimensions (unpacked).
+	// slice. Detect the situation by noting if the index count
+	// is less than the array dimensions (unpacked).
       if (reg->unpacked_dimensions() > name_tail.index.size()) {
-	    cerr << get_fileline() << ": error: Cannot assign to array "
-		 << path_ << ". Did you forget a word index?" << endl;
+	    if (gn_system_verilog()) {
+		  cerr << get_fileline() << ": sorry: Assignment to an entire"
+		        " array or to an array slice is not yet supported."
+		       << endl;
+	    } else {
+		  cerr << get_fileline() << ": error: Assignment to an entire"
+		        " array or to an array slice requires SystemVerilog."
+		       << endl;
+	    }
 	    des->errors += 1;
 	    return 0;
       }
@@ -275,7 +277,7 @@ NetAssign_* PEIdent::elaborate_lval(Design*des,
 
 	// If we find that the matched variable is a packed struct,
 	// then we can handled it with the net_packed_member_ method.
-      if (reg->struct_type() && member_path.size() > 0) {
+      if (reg->struct_type() && !member_path.empty()) {
 	    NetAssign_*lv = new NetAssign_(reg);
 	    elaborate_lval_net_packed_member_(des, use_scope, lv, member_path);
 	    return lv;
@@ -283,8 +285,9 @@ NetAssign_* PEIdent::elaborate_lval(Design*des,
 
 	// If the variable is a class object, then handle it with the
 	// net_class_member_ method.
-      if (reg->class_type() && member_path.size() > 0 && gn_system_verilog()) {
-	    NetAssign_*lv = elaborate_lval_net_class_member_(des, use_scope, reg, member_path);
+      const netclass_t *class_type = dynamic_cast<const netclass_t *>(sr.type);
+      if (class_type && !member_path.empty() && gn_system_verilog()) {
+	    NetAssign_*lv = elaborate_lval_net_class_member_(des, use_scope, class_type, reg, member_path);
 	    return lv;
       }
 
@@ -292,7 +295,7 @@ NetAssign_* PEIdent::elaborate_lval(Design*des,
 	// Past this point, we should have taken care of the cases
 	// where the name is a member/method of a struct/class.
 	// XXXX ivl_assert(*this, method_name.nil());
-      ivl_assert(*this, member_path.size() == 0);
+      ivl_assert(*this, member_path.empty());
 
       bool need_const_idx = is_cassign || is_force || (reg->type()==NetNet::UNRESOLVED_WIRE);
 
@@ -375,10 +378,17 @@ NetAssign_* PEIdent::elaborate_lval_method_class_member_(Design*des,
       if (pidx < 0)
 	    return 0;
 
+      property_qualifier_t qual = class_type->get_prop_qual(pidx);
+
+	// Static properties are handled as normal signals. Regular symbol
+	// search will find it.
+      if (qual.test_static())
+	    return 0;
+
       NetScope*scope_method = find_method_containing_scope(*this, scope);
       ivl_assert(*this, scope_method);
 
-      NetNet*this_net = scope_method->find_signal(perm_string::literal("@"));
+      NetNet*this_net = scope_method->find_signal(perm_string::literal(THIS_TOKEN));
       if (this_net == 0) {
 	    cerr << get_fileline() << ": internal error: "
 		 << "Unable to find 'this' port of " << scope_path(scope_method)
@@ -410,7 +420,6 @@ NetAssign_* PEIdent::elaborate_lval_method_class_member_(Design*des,
 	// Detect assignment to constant properties. Note that the
 	// initializer constructor MAY assign to constant properties,
 	// as this is how the property gets its value.
-      property_qualifier_t qual = class_type->get_prop_qual(pidx);
       if (qual.test_const()) {
 	    if (class_type->get_prop_initialized(pidx)) {
 		  cerr << get_fileline() << ": error: "
@@ -467,7 +476,7 @@ NetAssign_* PEIdent::elaborate_lval_method_class_member_(Design*des,
       }
 
       NetAssign_*this_lval = new NetAssign_(this_net);
-      this_lval->set_property(member_name);
+      this_lval->set_property(member_name, pidx);
       if (canon_index) this_lval->set_word(canon_index);
 
       return this_lval;
@@ -853,12 +862,20 @@ bool PEIdent::elaborate_lval_net_part_(Design*des,
 	      // If there are fewer indices then there are packed
 	      // dimensions, then this is a range of slices. Calculate
 	      // it into a big slice.
-	    bool lrc;
+	    bool lrc, mrc;
 	    unsigned long tmp_lwid, tmp_mwid;
-	    lrc = reg->sb_to_slice(prefix_indices,lsb, loff, tmp_lwid);
-	    ivl_assert(*this, lrc);
-	    lrc = reg->sb_to_slice(prefix_indices,msb, moff, tmp_mwid);
-	    ivl_assert(*this, lrc);
+	    lrc = reg->sb_to_slice(prefix_indices, lsb, loff, tmp_lwid);
+	    mrc = reg->sb_to_slice(prefix_indices, msb, moff, tmp_mwid);
+	    if (!mrc || !lrc) {
+		  cerr << get_fileline() << ": error: ";
+		  cerr << "Part-select [" << msb << ":" << lsb;
+		  cerr << "] exceeds the declared bounds for ";
+		  cerr << reg->name();
+		  if (reg->unpacked_dimensions() > 0) cerr << "[]";
+		  cerr << "." << endl;
+		  des->errors += 1;
+		  return 0;
+	    }
 
 	    if (loff < moff) {
 		  moff = moff + tmp_mwid - 1;
@@ -938,21 +955,58 @@ bool PEIdent::elaborate_lval_net_idx_(Design*des,
 	      // we will handle it in the code generator.
 	    if (base_c->value().is_defined()) {
 		  long lsv = base_c->value().as_long();
-		  long offset = 0;
+		  long rel_base = 0;
 		    // Get the signal range.
 		  const vector<netrange_t>&packed = reg->packed_dims();
-		  ivl_assert(*this, packed.size() == prefix_indices.size()+1);
+		  if (prefix_indices.size()+1 < reg->packed_dims().size()) {
+			  // Here we are selecting one or more sub-arrays.
+			  // Make this work by finding the indexed sub-arrays and
+			  // creating a generated slice that spans the whole range.
+			long loff, moff;
+			unsigned long lwid, mwid;
+			bool lrc, mrc;
+			mrc = reg->sb_to_slice(prefix_indices, lsv, moff, mwid);
+			if (use_sel == index_component_t::SEL_IDX_UP)
+			      lrc = reg->sb_to_slice(prefix_indices, lsv+wid-1, loff, lwid);
+			else
+			      lrc = reg->sb_to_slice(prefix_indices, lsv-wid+1, loff, lwid);
+			if (!mrc || !lrc) {
+			      cerr << get_fileline() << ": error: ";
+			      cerr << "Part-select [" << lsv;
+			      if (index_tail.sel == index_component_t::SEL_IDX_UP) {
+				    cerr << "+:";
+			      } else {
+				    cerr << "-:";
+			      }
+			      cerr << wid << "] exceeds the declared bounds for ";
+			      cerr << reg->name();
+			      if (reg->unpacked_dimensions() > 0) cerr << "[]";
+			      cerr << "." << endl;
+			      des->errors += 1;
+			      return 0;
+			}
+			ivl_assert(*this, lwid == mwid);
 
-		    // We want the last range, which is where we work.
-		  const netrange_t&rng = packed.back();
-		  if (((rng.get_msb() < rng.get_lsb()) &&
-                       use_sel == index_component_t::SEL_IDX_UP) ||
-		      ((rng.get_msb() > rng.get_lsb()) &&
-		       use_sel == index_component_t::SEL_IDX_DO)) {
-			offset = -wid + 1;
+			if (moff > loff) {
+			      rel_base = loff;
+			      wid = moff + mwid - loff;
+			} else {
+			      rel_base = moff;
+			      wid = loff + lwid - moff;
+			}
+		  } else {
+			long offset = 0;
+			  // We want the last range, which is where we work.
+			const netrange_t&rng = packed.back();
+			if (((rng.get_msb() < rng.get_lsb()) &&
+			     use_sel == index_component_t::SEL_IDX_UP) ||
+			    ((rng.get_msb() > rng.get_lsb()) &&
+			     use_sel == index_component_t::SEL_IDX_DO)) {
+			      offset = -wid + 1;
+			}
+			rel_base = reg->sb_to_idx(prefix_indices,lsv) + offset;
 		  }
 		  delete base;
-		  long rel_base = reg->sb_to_idx(prefix_indices,lsv) + offset;
 		    /* If we cover the entire lvalue just skip the select. */
 		  if (rel_base == 0 && wid == reg->vector_width()) return true;
 		  base = new NetEConst(verinum(rel_base));
@@ -980,7 +1034,7 @@ bool PEIdent::elaborate_lval_net_idx_(Design*des,
 			      cerr << wid << "] is selecting after vector." << endl;
 			}
 		  }
-	    } else {
+	    } else if (warn_ob_select) {
 		  cerr << get_fileline() << ": warning: L-value indexed part "
 		       << "select of " << reg->name();
 		  if (reg->unpacked_dimensions() > 0) cerr << "[]";
@@ -1025,7 +1079,8 @@ bool PEIdent::elaborate_lval_net_idx_(Design*des,
  * obj, and member_path=base.x.
  */
 NetAssign_* PEIdent::elaborate_lval_net_class_member_(Design*des, NetScope*scope,
-				    NetNet*sig, pform_name_t member_path) const
+				    const netclass_t *class_type, NetNet*sig,
+				    pform_name_t member_path) const
 {
       if (debug_elaborate) {
 	    cerr << get_fileline() << ": PEIdent::elaborate_lval_net_class_member_: "
@@ -1033,7 +1088,6 @@ NetAssign_* PEIdent::elaborate_lval_net_class_member_(Design*des, NetScope*scope
 		 << " of " << sig->name() << "." << endl;
       }
 
-      const netclass_t*class_type = sig->class_type();
       ivl_assert(*this, class_type);
 
 	// Iterate over the member_path. This handles nested class
@@ -1096,7 +1150,7 @@ NetAssign_* PEIdent::elaborate_lval_net_class_member_(Design*des, NetScope*scope
 	    }
 
 	    lv = lv? new NetAssign_(lv) : new NetAssign_(sig);
-	    lv->set_property(method_name);
+	    lv->set_property(method_name, pidx);
 
 	      // Now get the type of the property.
 	    ivl_type_t ptype = class_type->get_prop_type(pidx);
@@ -1115,7 +1169,7 @@ NetAssign_* PEIdent::elaborate_lval_net_class_member_(Design*des, NetScope*scope
 	      // class type.
 	    class_type = dynamic_cast<const netclass_t*>(ptype);
 
-      } while (member_path.size() > 0);
+      } while (!member_path.empty());
 
 
       return lv;
@@ -1279,7 +1333,7 @@ bool PEIdent::elaborate_lval_net_packed_member_(Design*des, NetScope*scope,
 		    // In any case, this should be the tail of the
 		    // member_path, because the array element of this
 		    // kind of array cannot be a struct.
-		  if (member_comp.index.size() > 0) {
+		  if (!member_comp.index.empty()) {
 			  // These are the dimensions defined by the type
 			const vector<netrange_t>&mem_packed_dims = mem_vec->packed_dims();
 
@@ -1346,7 +1400,7 @@ bool PEIdent::elaborate_lval_net_packed_member_(Design*des, NetScope*scope,
 		    // possibly iterate through more of the member_path.
 
 		  ivl_assert(*this, array->packed());
-		  ivl_assert(*this, member_comp.index.size() > 0);
+		  ivl_assert(*this, !member_comp.index.empty());
 
 		    // These are the dimensions defined by the type
 		  const vector<netrange_t>&mem_packed_dims = array->static_dimensions();
@@ -1438,7 +1492,6 @@ bool PEIdent::elaborate_lval_net_packed_member_(Design*des, NetScope*scope,
 		       << endl;
 		  des->errors += 1;
 		  return false;
-		  struct_type = 0;
 	    }
 
 	      // Complete this component of the path, mark it
@@ -1446,7 +1499,7 @@ bool PEIdent::elaborate_lval_net_packed_member_(Design*des, NetScope*scope,
 	    completed_path .push_back(member_comp);
 	    member_path.pop_front();
 
-      } while (member_path.size() > 0 && struct_type != 0);
+      } while (!member_path.empty() && struct_type != 0);
 
       if (debug_elaborate) {
 	    cerr << get_fileline() << ": PEIdent::elaborate_lval_net_packed_member_: "
