@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2021 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2013 / Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
@@ -21,6 +21,8 @@
 # include  "config.h"
 # include  "compiler.h"
 # include  "netmisc.h"
+# include  "netvector.h"
+# include  "netparray.h"
 # include  <cstring>
 # include  <iostream>
 # include  <cstdlib>
@@ -48,12 +50,14 @@
 # include  "netlist.h"
 # include  "netclass.h"
 # include  "netenum.h"
+# include  "netqueue.h"
 # include  "parse_api.h"
 # include  "util.h"
 # include  <typeinfo>
 # include  <cassert>
 # include  "ivl_assert.h"
 
+using namespace std;
 
 void set_scope_timescale(Design*des, NetScope*scope, PScope*pscope)
 {
@@ -65,11 +69,26 @@ void set_scope_timescale(Design*des, NetScope*scope, PScope*pscope)
 
 typedef map<perm_string,LexicalScope::param_expr_t*>::const_iterator mparm_it_t;
 
-static void collect_parm_item_(Design*des, NetScope*scope, perm_string name,
-			       const LexicalScope::param_expr_t&cur,
-                               bool is_annotatable,
-                               bool local_flag)
+static void collect_parm_item(Design*des, NetScope*scope, perm_string name,
+			      const LexicalScope::param_expr_t&cur,
+			      bool is_annotatable)
 {
+      if (debug_scopes) {
+	    cerr << cur.get_fileline() << ": " << __func__  << ": "
+		 << "parameter " << name << " ";
+	    if (cur.data_type)
+		  cerr << *cur.data_type;
+	    else
+		  cerr << "(nil type)";
+	    ivl_assert(cur, cur.expr);
+	    cerr << " = " << *cur.expr << "; ";
+	    if (cur.range)
+		  cerr << "with ranges ";
+	    else
+		  cerr << "without ranges ";
+	    cerr << "; in scope " << scope_path(scope) << endl;
+      }
+
       NetScope::range_t*range_list = 0;
       for (LexicalScope::range_t*range = cur.range ; range ; range = range->next) {
 	    NetScope::range_t*tmp = new NetScope::range_t;
@@ -104,38 +123,41 @@ static void collect_parm_item_(Design*des, NetScope*scope, perm_string name,
 	    range_list = tmp;
       }
 
-
-      scope->set_parameter(name, is_annotatable, cur.expr, cur.type, cur.msb,
-			   cur.lsb, cur.signed_flag, local_flag, range_list, cur);
+      // The type of the parameter, if unspecified in the source, will come
+      // from the type of the value assigned to it. Therefore, if the type is
+      // not yet known, don't try to guess here, put the type guess off. Also
+      // don't try to elaborate it here, because there may be references to
+      // other parameters still being located during scope elaboration.
+      scope->set_parameter(name, is_annotatable, cur, range_list);
 }
 
-static void collect_scope_parameters_(Design*des, NetScope*scope,
+static void collect_scope_parameters(Design*des, NetScope*scope,
       const map<perm_string,LexicalScope::param_expr_t*>&parameters)
 {
+      if (debug_scopes) {
+	    cerr << scope->get_fileline() << ": " << __func__ << ": "
+		 << "collect parameters for " << scope_path(scope) << "." << endl;
+      }
+
       for (mparm_it_t cur = parameters.begin()
 		 ; cur != parameters.end() ;  ++ cur ) {
 
-	    collect_parm_item_(des, scope, cur->first, *(cur->second), false, false);
+	    collect_parm_item(des, scope, cur->first, *(cur->second), false);
       }
 }
 
-static void collect_scope_localparams_(Design*des, NetScope*scope,
-      const map<perm_string,LexicalScope::param_expr_t*>&localparams)
-{
-      for (mparm_it_t cur = localparams.begin()
-		 ; cur != localparams.end() ;  ++ cur ) {
-
-	    collect_parm_item_(des, scope, cur->first, *(cur->second), false, true);
-      }
-}
-
-static void collect_scope_specparams_(Design*des, NetScope*scope,
+static void collect_scope_specparams(Design*des, NetScope*scope,
       const map<perm_string,LexicalScope::param_expr_t*>&specparams)
 {
+      if (debug_scopes) {
+	    cerr << scope->get_fileline() << ": " << __func__ << ": "
+		 << "collect specparams for " << scope_path(scope) << "." << endl;
+      }
+
       for (mparm_it_t cur = specparams.begin()
 		 ; cur != specparams.end() ;  ++ cur ) {
 
-	    collect_parm_item_(des, scope, cur->first, *(cur->second), true, false);
+	    collect_parm_item(des, scope, cur->first, *(cur->second), true);
       }
 }
 
@@ -146,46 +168,31 @@ static void elaborate_scope_enumeration(Design*des, NetScope*scope,
 					enum_type_t*enum_type)
 {
       bool rc_flag;
-      assert(enum_type->range->size() == 1);
-      pform_range_t&range = enum_type->range->front();
-      NetExpr*msb_ex = elab_and_eval(des, scope, range.first, -1);
-      NetExpr*lsb_ex = elab_and_eval(des, scope, range.second, -1);
 
-      long msb = 0;
-      rc_flag = eval_as_long(msb, msb_ex);
-      assert(rc_flag);
-      long lsb = 0;
-      rc_flag = eval_as_long(lsb, lsb_ex);
-      assert(rc_flag);
+      enum_type->elaborate_type(des, scope);
 
-      netenum_t*use_enum = new netenum_t(enum_type->base_type,
-					 enum_type->signed_flag,
-					 enum_type->integer_flag, msb, lsb,
-					 enum_type->names->size(),
-					 enum_type);
-
-      use_enum->set_line(enum_type->li);
-      scope->add_enumeration_set(enum_type, use_enum);
+      netenum_t *use_enum = scope->enumeration_for_key(enum_type);
 
       size_t name_idx = 0;
 	// Find the enumeration width.
       long raw_width = use_enum->packed_width();
       assert(raw_width > 0);
       unsigned enum_width = (unsigned)raw_width;
+      bool is_signed = use_enum->get_signed();
 	// Define the default start value and the increment value to be the
 	// correct type for this enumeration.
       verinum cur_value ((uint64_t)0, enum_width);
-      cur_value.has_sign(enum_type->signed_flag);
+      cur_value.has_sign(is_signed);
       verinum one_value ((uint64_t)1, enum_width);
-      one_value.has_sign(enum_type->signed_flag);
+      one_value.has_sign(is_signed);
 	// Find the maximum allowed enumeration value.
       verinum max_value (0);
-      if (enum_type->signed_flag) {
+      if (is_signed) {
 	    max_value = pow(verinum(2), verinum(enum_width-1)) - one_value;
       } else {
 	    max_value = pow(verinum(2), verinum(enum_width)) - one_value;
       }
-      max_value.has_sign(enum_type->signed_flag);
+      max_value.has_sign(is_signed);
 	// Variable to indicate when a defined value wraps.
       bool implicit_wrapped = false;
 	// Process the enumeration definition.
@@ -210,7 +217,7 @@ static void elaborate_scope_enumeration(Design*des, NetScope*scope,
 		  implicit_wrapped = false;
 
 		    // A 2-state value can not have a constant with X/Z bits.
-		  if (enum_type->base_type==IVL_VT_BOOL &&
+		  if (use_enum->base_type() == IVL_VT_BOOL &&
 		      ! cur_value.is_defined()) {
 			cerr << use_enum->get_fileline()
 			     << ": error: Enumeration name " << cur->name
@@ -235,7 +242,7 @@ static void elaborate_scope_enumeration(Design*des, NetScope*scope,
 		    // value does not have a defined width.
 		  if (((cur_value.len() != enum_width) ||
 		       ! cur_value.has_len()) &&
-		      ! enum_type->signed_flag && cur_value.is_negative()) {
+		      ! is_signed && cur_value.is_negative()) {
 			cerr << use_enum->get_fileline()
 			     << ": error: Enumeration name " << cur->name
 			     << " has a negative value." << endl;
@@ -258,7 +265,7 @@ static void elaborate_scope_enumeration(Design*des, NetScope*scope,
 			      if (cur_value[idx] != cur_value[check_width]) {
 				      // If this is an unsigned enumeration
 				      // then zero padding is okay.
-				    if (! enum_type->signed_flag &&
+				    if (!is_signed &&
 				        (idx == enum_width) &&
 				        (cur_value[idx] == verinum::V0)) {
 					  check_width += 1;
@@ -304,7 +311,7 @@ static void elaborate_scope_enumeration(Design*des, NetScope*scope,
 		    // At this point the value has the correct size and needs
 		    // to have the correct sign attribute set.
 		  cur_value.has_len(true);
-		  cur_value.has_sign(enum_type->signed_flag);
+		  cur_value.has_sign(is_signed);
 
 	    } else if (! cur_value.is_defined()) {
 		  cerr << use_enum->get_fileline()
@@ -354,9 +361,16 @@ static void elaborate_scope_enumeration(Design*des, NetScope*scope,
 }
 
 static void elaborate_scope_enumerations(Design*des, NetScope*scope,
-					 const set<enum_type_t*>&enum_types)
+					 const vector<enum_type_t*>&enum_types)
 {
-      for (set<enum_type_t*>::const_iterator cur = enum_types.begin()
+      if (debug_scopes) {
+	    cerr << scope->get_fileline() << ": " << __func__ << ": "
+		 << "Elaborate " << enum_types.size() << " enumerations"
+		 << " in scope " << scope_path(scope) << "."
+		 << endl;
+      }
+
+      for (vector<enum_type_t*>::const_iterator cur = enum_types.begin()
 		 ; cur != enum_types.end() ; ++ cur) {
 	    enum_type_t*curp = *cur;
 	    elaborate_scope_enumeration(des, scope, curp);
@@ -405,7 +419,7 @@ static void blend_class_constructors(PClass*pclass)
 
 	// If we do not have an explicit constructor chain, but there
 	// is a parent class, then create an implicit chain.
-      if (chain_new==0 && pclass->type->base_type!=0) {
+      if (chain_new==0 && pclass->type->base_type) {
 	    chain_new = new PChainConstructor(pclass->type->base_args);
 	    chain_new->set_line(*pclass);
       }
@@ -458,40 +472,34 @@ static void elaborate_scope_class(Design*des, NetScope*scope, PClass*pclass)
 		 << endl;
       }
 
-      class_type_t*base_class = dynamic_cast<class_type_t*> (use_type->base_type);
-      if (use_type->base_type && !base_class) {
-	    cerr << pclass->get_fileline() << ": error: "
-		 << "Base type of " << use_type->name
-		 << " is not a class." << endl;
-	    des->errors += 1;
-      }
 
-      netclass_t*use_base_class = 0;
-      if (base_class) {
-	    use_base_class = scope->find_class(base_class->name);
-	    if (use_base_class == 0) {
+      const netclass_t*use_base_class = 0;
+      if (use_type->base_type) {
+	    ivl_type_t base_type = use_type->base_type->elaborate_type(des, scope);
+	    use_base_class = dynamic_cast<const netclass_t *>(base_type);
+	    if (!use_base_class) {
 		  cerr << pclass->get_fileline() << ": error: "
-		       << "Base class " << base_class->name
-		       << " not found." << endl;
+		       << "Base type of " << use_type->name
+		       << " is not a class." << endl;
 		  des->errors += 1;
 	    }
       }
 
       netclass_t*use_class = new netclass_t(use_type->name, use_base_class);
 
-      ivl_assert(*pclass, use_type->save_elaborated_type == 0);
-      use_type->save_elaborated_type = use_class;
-
       NetScope*class_scope = new NetScope(scope, hname_t(pclass->pscope_name()),
 					  NetScope::CLASS, scope->unit());
       class_scope->set_line(pclass);
       class_scope->set_class_def(use_class);
       use_class->set_class_scope(class_scope);
       use_class->set_definition_scope(scope);
+      use_class->set_virtual(use_type->virtual_class);
       set_scope_timescale(des, class_scope, pclass);
 
       class_scope->add_typedefs(&pclass->typedefs);
 
+      collect_scope_parameters(des, class_scope, pclass->parameters);
+
 	// Elaborate enum types declared in the class. We need these
 	// now because enumeration constants can be used during scope
 	// elaboration.
@@ -504,28 +512,19 @@ static void elaborate_scope_class(Design*des, NetScope*scope, PClass*pclass)
       }
       elaborate_scope_enumerations(des, class_scope, pclass->enum_sets);
 
-	// Collect the properties, elaborate them, and add them to the
-	// elaborated class definition.
-      for (map<perm_string, class_type_t::prop_info_t>::iterator cur = use_type->properties.begin()
-		 ; cur != use_type->properties.end() ; ++ cur) {
-
-	    ivl_type_s*tmp = cur->second.type->elaborate_type(des, class_scope);
-	    ivl_assert(*pclass, tmp);
-	    if (debug_scopes) {
-		  cerr << pclass->get_fileline() << ": elaborate_scope_class: "
-		       << "  Property " << cur->first
-		       << " type=" << *tmp << endl;
-	    }
-	    use_class->set_property(cur->first, cur->second.qual, tmp);
-
-      }
-
       for (map<perm_string,PTask*>::iterator cur = pclass->tasks.begin()
 		 ; cur != pclass->tasks.end() ; ++cur) {
 
 	    hname_t use_name (cur->first);
 	    NetScope*method_scope = new NetScope(class_scope, use_name, NetScope::TASK);
+
 	      // Task methods are always automatic...
+	    if (!cur->second->is_auto()) {
+		  cerr << "error: Lifetime of method `"
+		       << scope_path(method_scope)
+		       << "` must not be static" << endl;
+		  des->errors += 1;
+	    }
 	    method_scope->is_auto(true);
 	    method_scope->set_line(cur->second);
 	    method_scope->add_imports(&cur->second->explicit_imports);
@@ -544,7 +543,14 @@ static void elaborate_scope_class(Design*des, NetScope*scope, PClass*pclass)
 
 	    hname_t use_name (cur->first);
 	    NetScope*method_scope = new NetScope(class_scope, use_name, NetScope::FUNC);
+
 	      // Function methods are always automatic...
+	    if (!cur->second->is_auto()) {
+		  cerr << "error: Lifetime of method `"
+		       << scope_path(method_scope)
+		       << "` must not be static" << endl;
+		  des->errors += 1;
+	    }
 	    method_scope->is_auto(true);
 	    method_scope->set_line(cur->second);
 	    method_scope->add_imports(&cur->second->explicit_imports);
@@ -564,15 +570,27 @@ static void elaborate_scope_class(Design*des, NetScope*scope, PClass*pclass)
 static void elaborate_scope_classes(Design*des, NetScope*scope,
 				    const vector<PClass*>&classes)
 {
+      if (debug_scopes) {
+	    cerr << scope->get_fileline() << ": " << __func__ << ": "
+		 << "Elaborate " << classes.size() << " classes"
+		 << " in scope " << scope_path(scope) << "."
+		 << endl;
+      }
+
       for (size_t idx = 0 ; idx < classes.size() ; idx += 1) {
 	    blend_class_constructors(classes[idx]);
 	    elaborate_scope_class(des, scope, classes[idx]);
       }
 }
 
-static void replace_scope_parameters_(NetScope*scope, const LineInfo&loc,
-			              const Module::replace_t&replacements)
+static void replace_scope_parameters(Design *des, NetScope*scope, const LineInfo&loc,
+				     const Module::replace_t&replacements)
 {
+      if (debug_scopes) {
+	    cerr << scope->get_fileline() << ": " << __func__ << ": "
+		 << "Replace scope parameters for " << scope_path(scope) << "." << endl;
+      }
+
       for (Module::replace_t::const_iterator cur = replacements.begin()
 		 ; cur != replacements.end() ;  ++ cur ) {
 
@@ -591,13 +609,7 @@ static void replace_scope_parameters_(NetScope*scope, const LineInfo&loc,
 		  cerr << loc.get_fileline() << ":      : "
 		       << "Type=" << val->expr_type() << endl;
 	    }
-	    bool flag = scope->replace_parameter((*cur).first, val,
-                                                 scope->parent());
-	    if (! flag) {
-		  cerr << val->get_fileline() << ": warning: parameter "
-		       << (*cur).first << " not found in "
-		       << scope_path(scope) << "." << endl;
-	    }
+	    scope->replace_parameter(des, (*cur).first, val, scope->parent());
       }
 }
 
@@ -652,7 +664,8 @@ static void elaborate_scope_func(Design*des, NetScope*scope, PFunction*task)
 
       if (debug_scopes) {
 	    cerr << task->get_fileline() << ": elaborate_scope_func: "
-		 << "Elaborate task scope " << scope_path(task_scope) << endl;
+		 << "Elaborate function scope " << scope_path(task_scope)
+		 << endl;
       }
 
       task->elaborate_scope(des, task_scope);
@@ -712,8 +725,7 @@ bool PPackage::elaborate_scope(Design*des, NetScope*scope)
 
       scope->add_typedefs(&typedefs);
 
-      collect_scope_parameters_(des, scope, parameters);
-      collect_scope_localparams_(des, scope, localparams);
+      collect_scope_parameters(des, scope, parameters);
 
       if (debug_scopes) {
 	    cerr << get_fileline() << ": PPackage::elaborate_scope: "
@@ -751,23 +763,15 @@ bool Module::elaborate_scope(Design*des, NetScope*scope,
 	// will be evaluated later, once all parameter overrides for this
 	// module have been done.
 
-      collect_scope_parameters_(des, scope, parameters);
+      collect_scope_parameters(des, scope, parameters);
 
-      collect_scope_localparams_(des, scope, localparams);
-
-      collect_scope_specparams_(des, scope, specparams);
+      collect_scope_specparams(des, scope, specparams);
 
 	// Run parameter replacements that were collected from the
 	// containing scope and meant for me.
 
-      replace_scope_parameters_(scope, *this, replacements);
+      replace_scope_parameters(des, scope, *this, replacements);
 
-      if (debug_scopes) {
-	    cerr << get_fileline() << ": Module::elaborate_scope: "
-		 << "Elaborate " << enum_sets.size() << " enumerations"
-		 << " in scope " << scope_path(scope) << "."
-		 << endl;
-      }
       elaborate_scope_enumerations(des, scope, enum_sets);
 
       assert(classes.size() == classes_lexical.size());
@@ -796,7 +800,7 @@ bool Module::elaborate_scope(Design*des, NetScope*scope,
 	// we can not do that until defparams are run, so push it off
 	// into an elaborate work item.
       if (debug_scopes)
-	    cerr << get_fileline() << ": debug: "
+	    cerr << get_fileline() << ": " << __func__ << ": "
 		 << "Schedule generates within " << scope_path(scope)
 		 << " for elaboration after defparams." << endl;
 
@@ -894,6 +898,18 @@ bool PGenerate::generate_scope(Design*des, NetScope*container)
       }
 }
 
+void PGenerate::check_for_valid_genvar_value_(long value)
+{
+      if (generation_flag < GN_VER2005 && value < 0) {
+	    cerr << get_fileline() << ": warning: A negative value (" << value
+		 << ") has been assigned to genvar '" << loop_index << "'."
+		 << endl;
+	    cerr << get_fileline() << ":        : This is illegal in "
+		    "Verilog-2001. Use at least -g2005 to remove this warning."
+		 << endl;
+      }
+}
+
 /*
  * This is the elaborate scope method for a generate loop.
  */
@@ -903,8 +919,16 @@ bool PGenerate::generate_scope_loop_(Design*des, NetScope*container)
 	      // Check that the loop_index variable was declared in a
 	      // genvar statement.
 	    NetScope*cscope = container;
-	    while (cscope && !cscope->find_genvar(loop_index))
+	    while (cscope && !cscope->find_genvar(loop_index)) {
+		  if (cscope->symbol_exists(loop_index)) {
+			cerr << get_fileline() << ": error: "
+			     << "generate loop variable '" << loop_index
+			     << "' is not a genvar in this scope." << endl;
+			des->errors += 1;
+			return false;
+		  }
 		  cscope = cscope->parent();
+            }
 	    if (!cscope) {
 		  cerr << get_fileline() << ": error: genvar is missing for "
 			  "generate \"loop\" variable '" << loop_index << "'."
@@ -915,7 +939,7 @@ bool PGenerate::generate_scope_loop_(Design*des, NetScope*container)
       }
 
 	// We're going to need a genvar...
-      int genvar;
+      long genvar;
 
 	// The initial value for the genvar does not need (nor can it
 	// use) the genvar itself, so we can evaluate this expression
@@ -930,6 +954,7 @@ bool PGenerate::generate_scope_loop_(Design*des, NetScope*container)
       }
 
       genvar = init->value().as_long();
+      check_for_valid_genvar_value_(genvar);
       delete init_ex;
 
       if (debug_scopes)
@@ -966,7 +991,11 @@ bool PGenerate::generate_scope_loop_(Design*des, NetScope*container)
 	      // block. Code within this scope thus has access to the
 	      // genvar as a constant.
 	    {
-		  verinum genvar_verinum(genvar);
+		  verinum genvar_verinum;
+		  if (gn_strict_expr_width_flag)
+			genvar_verinum = verinum(genvar, integer_width);
+		  else
+			genvar_verinum = verinum(genvar);
 		  genvar_verinum.has_sign(true);
 		  NetEConstParam*gp = new NetEConstParam(scope,
 							 loop_index,
@@ -996,6 +1025,7 @@ bool PGenerate::generate_scope_loop_(Design*des, NetScope*container)
 		       << genvar << " to " << step->value().as_long() << endl;
 
 	    genvar = step->value().as_long();
+	    check_for_valid_genvar_value_(genvar);
 	    container->genvar_tmp_val = genvar;
 	    delete step;
 	    delete test_ex;
@@ -1042,8 +1072,7 @@ bool PGenerate::generate_scope_condit_(Design*des, NetScope*container, bool else
 		 << " value=" << test->value() << ": Generate scope="
 		 << use_name << endl;
 
-      probe_for_direct_nesting_();
-      if (direct_nested_) {
+      if (directly_nested) {
 	    if (debug_scopes)
 		  cerr << get_fileline() << ": debug: Generate condition "
 		       << (else_flag? "(else)" : "(if)")
@@ -1141,8 +1170,7 @@ bool PGenerate::generate_scope_case_(Design*des, NetScope*container)
 	// The name of the scope to generate, whatever that item is.
       hname_t use_name (item->scope_name);
 
-      item->probe_for_direct_nesting_();
-      if (item->direct_nested_) {
+      if (item->directly_nested) {
 	    if (debug_scopes)
 		  cerr << get_fileline() << ": debug: Generate case item " << scope_name
 		       << " detected direct nesting." << endl;
@@ -1208,11 +1236,11 @@ void PGenerate::elaborate_subscope_(Design*des, NetScope*scope)
 	    scope->add_genvar((*cur).first, (*cur).second);
       }
 
-	// Scan the localparams in this scope, and store the information
+	// Scan the parameters in this scope, and store the information
         // needed to evaluate the parameter expressions. The expressions
 	// will be evaluated later, once all parameter overrides for this
 	// module have been done.
-      collect_scope_localparams_(des, scope, localparams);
+      collect_scope_parameters(des, scope, parameters);
 
 	// Run through the defparams for this scope and save the result
 	// in a table for later final override.
@@ -1355,7 +1383,7 @@ void PGModule::elaborate_scope_mod_(Design*des, Module*mod, NetScope*sc) const
 	    return;
       }
 
-      if (msb_ || lsb_) {
+      if (is_array()) {
 	      // If there are expressions to evaluate in order to know
 	      // the actual number of instances that will be
 	      // instantiated, then we have to delay further scope
@@ -1385,41 +1413,17 @@ void PGModule::elaborate_scope_mod_(Design*des, Module*mod, NetScope*sc) const
  */
 void PGModule::elaborate_scope_mod_instances_(Design*des, Module*mod, NetScope*sc) const
 {
-      NetExpr*mse = msb_ ? elab_and_eval(des, sc, msb_, -1, true) : 0;
-      NetExpr*lse = lsb_ ? elab_and_eval(des, sc, lsb_, -1, true) : 0;
-      NetEConst*msb = dynamic_cast<NetEConst*> (mse);
-      NetEConst*lsb = dynamic_cast<NetEConst*> (lse);
-
-      assert( (msb == 0) || (lsb != 0) );
-
       long instance_low  = 0;
       long instance_high = 0;
-      long instance_count  = 1;
-      bool instance_array = false;
-
-      if (msb) {
-	    instance_array = true;
-	    instance_high = msb->value().as_long();
-	    instance_low  = lsb->value().as_long();
-	    if (instance_high > instance_low)
-		  instance_count = instance_high - instance_low + 1;
-	    else
-		  instance_count = instance_low - instance_high + 1;
-
-	    delete mse;
-	    delete lse;
-      }
+      long instance_count = calculate_array_size_(des, sc, instance_high, instance_low);
+      if (instance_count == 0)
+	    return;
 
       NetScope::scope_vec_t instances (instance_count);
-      if (debug_scopes) {
-	    cerr << get_fileline() << ": debug: Create " << instance_count
-		 << " instances of " << get_name()
-		 << "." << endl;
-      }
 
-	    struct attrib_list_t*attrib_list;
-	    unsigned attrib_list_n = 0;
-	    attrib_list = evaluate_attributes(attributes, attrib_list_n, des, sc);
+      struct attrib_list_t*attrib_list;
+      unsigned attrib_list_n = 0;
+      attrib_list = evaluate_attributes(attributes, attrib_list_n, des, sc);
 
 	// Run through the module instances, and make scopes out of
 	// them. Also do parameter overrides that are done on the
@@ -1428,8 +1432,8 @@ void PGModule::elaborate_scope_mod_instances_(Design*des, Module*mod, NetScope*s
 
 	    hname_t use_name (get_name());
 
-	    if (instance_array) {
-		  int instance_idx = idx;
+	    if (is_array()) {
+		  int instance_idx;
 		  if (instance_low < instance_high)
 			instance_idx = instance_low + idx;
 		  else
@@ -1483,8 +1487,21 @@ void PGModule::elaborate_scope_mod_instances_(Design*des, Module*mod, NetScope*s
 		  for (;;) {
 			if (jdx == overrides_->end())
 			      break;
-			if (cur == mod->param_names.end())
+			  // If we reached here we have more overrides than
+			  // module parameters, so print a warning.
+			if (cur == mod->param_names.end()) {
+			      cerr << get_fileline() << ": warning: "
+			              "ignoring "
+			           << overrides_->size() -
+			              mod->param_names.size()
+			           << " extra parameter override(s) for "
+			              "instance '" << use_name
+			           << "' of module '" << mod->mod_name()
+			           << "' which expects "
+			           << mod->param_names.size()
+			           << " parameter(s)." << endl;
 			      break;
+			}
 
 		          // No expression means that the parameter is not
 		          // replaced at all.
@@ -1545,7 +1562,7 @@ void PEvent::elaborate_scope(Design*, NetScope*scope) const
 
 void PFunction::elaborate_scope(Design*des, NetScope*scope) const
 {
-      assert(scope->type() == NetScope::FUNC);
+      ivl_assert(*this, scope->type() == NetScope::FUNC);
 
         // Save a reference to the pform representation of the function
         // in case we need to perform early elaboration.
@@ -1560,9 +1577,7 @@ void PFunction::elaborate_scope(Design*des, NetScope*scope) const
 	// Scan the parameters in the function, and store the information
         // needed to evaluate the parameter expressions.
 
-      collect_scope_parameters_(des, scope, parameters);
-
-      collect_scope_localparams_(des, scope, localparams);
+      collect_scope_parameters(des, scope, parameters);
 
 	// Scan through all the named events in this scope.
       elaborate_scope_events_(des, scope, events);
@@ -1580,9 +1595,7 @@ void PTask::elaborate_scope(Design*des, NetScope*scope) const
 	// Scan the parameters in the task, and store the information
         // needed to evaluate the parameter expressions.
 
-      collect_scope_parameters_(des, scope, parameters);
-
-      collect_scope_localparams_(des, scope, localparams);
+      collect_scope_parameters(des, scope, parameters);
 
 	// Scan through all the named events in this scope.
       elaborate_scope_events_(des, scope, events);
@@ -1630,9 +1643,7 @@ void PBlock::elaborate_scope(Design*des, NetScope*scope) const
 	      // Scan the parameters in the scope, and store the information
 	      // needed to evaluate the parameter expressions.
 
-            collect_scope_parameters_(des, my_scope, parameters);
-
-            collect_scope_localparams_(des, my_scope, localparams);
+            collect_scope_parameters(des, my_scope, parameters);
 
               // Scan through all the named events in this scope.
             elaborate_scope_events_(des, my_scope, events);
@@ -1650,7 +1661,7 @@ void PBlock::elaborate_scope(Design*des, NetScope*scope) const
 void PCase::elaborate_scope(Design*des, NetScope*scope) const
 {
       assert(items_);
-      for (unsigned idx = 0 ;  idx < (*items_).count() ;  idx += 1) {
+      for (unsigned idx = 0 ;  idx < (*items_).size() ;  idx += 1) {
 	    assert( (*items_)[idx] );
 
 	    if (Statement*sp = (*items_)[idx]->stat)
