@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -29,6 +29,8 @@
 # include  "ivl_assert.h"
 # include  "netmisc.h"
 
+using namespace std;
+
 NetExpr* NetExpr::eval_tree()
 {
       return 0;
@@ -69,7 +71,7 @@ static bool get_real_arg_(const NetExpr*expr, verireal&val)
 	    return false;
 
 	  default:
-	    assert(0);
+	    ivl_assert(*expr, 0);
       }
 
       return true;
@@ -362,7 +364,7 @@ NetEConst* NetEBComp::must_be_leeq_(const NetExpr*le, const verinum&rv, bool eq_
 	// or 'z' values.
       if (le->expr_type() == IVL_VT_LOGIC) return 0;
 
-      assert(le->expr_width() > 0);
+      ivl_assert(*le, le->expr_width() > 0);
       verinum lv (verinum::V1, le->expr_width());
       if (le->has_sign() && rv.has_sign()) {
 	      // If the expression is signed, then the largest
@@ -404,7 +406,6 @@ NetEConst* NetEBComp::eval_leeq_(const NetExpr*le, const NetExpr*re) const
 {
       if (le->expr_type() == IVL_VT_REAL || re->expr_type() == IVL_VT_REAL)
 	    return eval_leeq_real_(le, re, true);
-//      assert(expr_type() == IVL_VT_LOGIC);
 
       const NetEConst*r = dynamic_cast<const NetEConst*>(re);
       if (r == 0) return 0;
@@ -416,12 +417,6 @@ NetEConst* NetEBComp::eval_leeq_(const NetExpr*le, const NetExpr*re) const
 	    return res;
       }
 
-      if (le->expr_width() == 0) {
-	    cerr << get_fileline() << ": internal error: Something wrong "
-		 << "with the left side width of <= ?" << endl;
-	    cerr << get_fileline() << ":               : " << *this << endl;
-      }
-
       if (NetEConst*tmp = must_be_leeq_(le, rv, true)) {
 	    return tmp;
       }
@@ -776,7 +771,7 @@ NetExpr* NetEBDiv::eval_tree_real_(const NetExpr*l, const NetExpr*r) const
 NetExpr* NetEBDiv::eval_arguments_(const NetExpr*l, const NetExpr*r) const
 {
       if (expr_type() == IVL_VT_REAL) return eval_tree_real_(l,r);
-      assert(expr_type() == IVL_VT_LOGIC);
+      ivl_assert(*this, expr_type() == IVL_VT_LOGIC);
 
       const NetEConst*lc = dynamic_cast<const NetEConst*>(l);
       const NetEConst*rc = dynamic_cast<const NetEConst*>(r);
@@ -812,10 +807,40 @@ NetEConst* NetEBLogic::eval_arguments_(const NetExpr*l, const NetExpr*r) const
 {
 	// NetEBLogic arguments should have already been reduced so real is not possible.
       ivl_assert(*this, (l->expr_type() != IVL_VT_REAL) && (r->expr_type() != IVL_VT_REAL));
-      assert(expr_type() == IVL_VT_LOGIC);
+      ivl_assert(*this, expr_type() == IVL_VT_LOGIC);
 
       const NetEConst*lc = dynamic_cast<const NetEConst*>(l);
       const NetEConst*rc = dynamic_cast<const NetEConst*>(r);
+
+      // If the left side is constant and the right side is short circuited
+      // replace the expression with a constant
+      if (rc == 0 && lc != 0) {
+	    verinum v = lc->value();
+	    verinum::V res = verinum::Vx;
+	    switch (op_) {
+		case 'a': // Logical AND (&&)
+		  if (v.is_zero())
+			res = verinum::V0;
+		  break;
+		case 'o': // Logical OR (||)
+		  if (! v.is_zero() && v.is_defined())
+			res = verinum::V1;
+		  break;
+		case 'q': // Logical implication (->)
+		  if (v.is_zero())
+			res = verinum::V1;
+		  break;
+		default:
+		  break;
+	    }
+	    if (res != verinum::Vx) {
+		  NetEConst*tmp = new NetEConst(verinum(res, 1));
+		  ivl_assert(*this, tmp);
+		  eval_debug(this, tmp, false);
+		  return tmp;
+	    }
+      }
+
       if (lc == 0 || rc == 0) return 0;
 
       verinum::V lv = verinum::V0;
@@ -918,7 +943,7 @@ NetExpr* NetEBMinMax::eval_tree_real_(const NetExpr*l, const NetExpr*r) const
 NetExpr* NetEBMinMax::eval_arguments_(const NetExpr*l, const NetExpr*r) const
 {
       if (expr_type() == IVL_VT_REAL) return eval_tree_real_(l,r);
-      assert(expr_type() == IVL_VT_LOGIC);
+      ivl_assert(*this, expr_type() == IVL_VT_LOGIC);
 
       const NetEConst*lc = dynamic_cast<const NetEConst*>(l);
       const NetEConst*rc = dynamic_cast<const NetEConst*>(r);
@@ -971,7 +996,7 @@ NetExpr* NetEBMult::eval_tree_real_(const NetExpr*l, const NetExpr*r) const
 NetExpr* NetEBMult::eval_arguments_(const NetExpr*l, const NetExpr*r) const
 {
       if (expr_type() == IVL_VT_REAL) return eval_tree_real_(l,r);
-      assert(expr_type() == IVL_VT_LOGIC);
+      ivl_assert(*this, expr_type() == IVL_VT_LOGIC);
 
       const NetEConst*lc = dynamic_cast<const NetEConst*>(l);
       const NetEConst*rc = dynamic_cast<const NetEConst*>(r);
@@ -1009,7 +1034,7 @@ NetExpr* NetEBPow::eval_tree_real_(const NetExpr*l, const NetExpr*r) const
 NetExpr* NetEBPow::eval_arguments_(const NetExpr*l, const NetExpr*r) const
 {
       if (expr_type() == IVL_VT_REAL) return eval_tree_real_(l,r);
-      assert(expr_type() == IVL_VT_LOGIC);
+      ivl_assert(*this, expr_type() == IVL_VT_LOGIC);
 
       const NetEConst*lc = dynamic_cast<const NetEConst*>(l);
       const NetEConst*rc = dynamic_cast<const NetEConst*>(r);
@@ -1092,7 +1117,7 @@ NetEConst* NetEConcat::eval_tree()
 	      // Finally, try to evaluate the parameter expression
 	      // that is here. If I succeed, reset the parameter to
 	      // the evaluated value.
-	    assert(parms_[idx]);
+	    ivl_assert(*this, parms_[idx]);
 	    NetExpr*expr = parms_[idx]->eval_tree();
 	    if (expr) {
 		  expr->set_line(*parms_[idx]);
@@ -1225,7 +1250,7 @@ static void print_ternary_cond(NetExpr*expr)
 	    cerr << c->value() << endl;
 	    return;
       }
-      assert(0);
+      ivl_assert(*expr, 0);
 }
 
 /*
@@ -1250,6 +1275,9 @@ NetExpr* NetETernary::eval_tree()
 		       << *false_val_ << endl;
 	    }
 
+	      // Elaborate the alternate expression to check for errors.
+	    eval_expr(true_val_);
+
 	    if (expr_type() == IVL_VT_REAL &&
 	        false_val_->expr_type() != IVL_VT_REAL) {
 		  verireal f;
@@ -1272,6 +1300,9 @@ NetExpr* NetETernary::eval_tree()
 		       << *true_val_ << endl;
 	    }
 
+	      // Elaborate the alternate expression to check for errors.
+	    eval_expr(false_val_);
+
 	    if (expr_type() == IVL_VT_REAL &&
 	        true_val_->expr_type() != IVL_VT_REAL) {
 		  verireal t;
@@ -1437,7 +1468,7 @@ NetExpr* NetEUnary::eval_arguments_(const NetExpr*ex) const
 	    break;
 
 	  case '!':
-	    assert(0);
+	    ivl_assert(*this, 0);
 	  default:
 	    return 0;
       }
@@ -2012,7 +2043,7 @@ NetEConst* NetESFunc::evaluate_dimensions_(const NetExpr*arg) const
 	       * make these routines work correct add one if this is a
 	       * string data type. */
 	    if (sig->data_type() == IVL_VT_STRING) {
-		  assert(sig->packed_dimensions() == 0);
+		  ivl_assert(*this, sig->packed_dimensions() == 0);
 		  res += 1;
 	    }
       }
@@ -2046,7 +2077,7 @@ NetEConst* NetESFunc::evaluate_isunknown_(const NetExpr* arg) const
       return res;
 }
 
-static bool is_onehot(verinum&value, bool zero_is_okay)
+static bool is_onehot(const verinum&value, bool zero_is_okay)
 {
       bool found_a_one = false;
 
@@ -2141,7 +2172,7 @@ static bool get_array_info(const NetExpr*arg, long dim,
                            long &left, long &right, bool&defer)
 {
       if (const NetEConstParam*param = dynamic_cast<const NetEConstParam*>(arg)) {
-	assert(dim == 1);
+	ivl_assert(*arg, dim == 1);
 	left = param->expr_width() - 1;
 	right = 0;
 	return false;
@@ -2226,7 +2257,7 @@ NetEConst* NetESFunc::evaluate_array_funcs_(ID id,
 	    break;
 	default:
 	    res = 0;
-	    assert(0);
+	    ivl_assert(*this, 0);
       }
 	/* Return the result as an integer sized constant. */
       return new NetEConst(verinum(verinum(res), integer_width));
