@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -29,6 +29,8 @@
  */
 
 # include  "netlist.h"
+# include  "netscalar.h"
+# include  "netvector.h"
 # include  "util.h"
 # include  "compiler.h"
 # include  "netmisc.h"
@@ -37,6 +39,8 @@
 # include  <sstream>
 # include  "ivl_assert.h"
 
+using namespace std;
+
 Design:: Design()
     : errors(0), nodes_(0), procs_(0), aprocs_(0)
 {
@@ -429,13 +433,7 @@ void NetScope::run_defparams(Design*des)
 		  continue;
 	    }
 
-	    bool flag = targ_scope->replace_parameter(perm_name, val, this);
-	    if (! flag) {
-		  cerr << val->get_fileline() << ": warning: parameter "
-		       << perm_name << " not found in "
-		       << scope_path(targ_scope) << "." << endl;
-	    }
-
+	    targ_scope->replace_parameter(des, perm_name, val, this, true);
       }
 
 	// If some of the defparams didn't find a scope in the name,
@@ -470,12 +468,7 @@ void NetScope::run_defparams_later(Design*des)
 		  continue;
 	    }
 
-	    bool flag = targ_scope->replace_parameter(name, val, this);
-	    if (! flag) {
-		  cerr << val->get_fileline() << ": warning: parameter "
-		       << name << " not found in "
-		       << scope_path(targ_scope) << "." << endl;
-	    }
+	    targ_scope->replace_parameter(des, name, val, this, true);
 
 	      // We'll need to re-evaluate parameters in this scope
 	    target_scopes.insert(targ_scope);
@@ -508,56 +501,71 @@ void Design::evaluate_parameters()
 
 void NetScope::evaluate_parameter_logic_(Design*des, param_ref_t cur)
 {
-      long msb = 0;
-      long lsb = 0;
-      bool range_flag = false;
-
-	/* Evaluate the msb expression, if it is present. */
-      PExpr*msb_expr = (*cur).second.msb_expr;
-      if (msb_expr) {
-            (*cur).second.msb = elab_and_eval(des, this, msb_expr, -1, true);
-	    if (! eval_as_long(msb, (*cur).second.msb)) {
-		  cerr << (*cur).second.val->get_fileline()
-		       << ": error: Unable to evaluate msb expression "
-		       << "for parameter " << (*cur).first << ": "
-		       << *(*cur).second.msb << endl;
-		  des->errors += 1;
-		  return;
-	    }
-
-	    range_flag = true;
-      }
-
-	/* Evaluate the lsb expression, if it is present. */
-      PExpr*lsb_expr = (*cur).second.lsb_expr;
-      if (lsb_expr) {
-            (*cur).second.lsb = elab_and_eval(des, this, lsb_expr, -1, true);
-	    if (! eval_as_long(lsb, (*cur).second.lsb)) {
-		  cerr << (*cur).second.val->get_fileline()
-		       << ": error: Unable to evaluate lsb expression "
-		       << "for parameter " << (*cur).first << ": "
-		       << *(*cur).second.lsb << endl;
-		  des->errors += 1;
-		  return;
-	    }
-
-	    range_flag = true;
-      }
-
 	/* Evaluate the parameter expression. */
       PExpr*val_expr = (*cur).second.val_expr;
       NetScope*val_scope = (*cur).second.val_scope;
 
+      // The param_type may be nil if the parameter has no declared type. In
+      // this case, we'll try to take our type from the r-value.
+      ivl_type_t param_type = cur->second.ivl_type;
+
+      // Most parameter declarations are packed vectors, of the form:
+      //   parameter [H:L] foo == bar;
+      // so get the netvector_t. Note that this may also be the special
+      // case of a netvector_t with no dimensions, that exists only to carry
+      // signed-ness, e.g.:
+      //  parameter signed foo = bar;
+      // These will be marked as scalar, but also have the implict flag set.
+      const netvector_t* param_vect = dynamic_cast<const netvector_t*> (param_type);
+
+      if (debug_elaborate) {
+	    cerr << val_expr->get_fileline() << ": " << __func__ << ": "
+		 << "parameter=" << cur->first << endl;
+	    if (param_type)
+		  cerr << val_expr->get_fileline() << ": " << __func__ << ": "
+		       << "param_type=" << *param_type << endl;
+	    else
+		  cerr << val_expr->get_fileline() << ": " << __func__ << ": "
+		       << "param_type=(nil)" << endl;
+	    cerr << val_expr->get_fileline() << ": " << __func__ << ": "
+		 << "val_expr=" << *val_expr << endl;
+      }
+
+      ivl_variable_type_t use_type;
       int lv_width = -2;
-      if (range_flag)
-	    lv_width = (msb >= lsb) ? 1 + msb - lsb : 1 + lsb - msb;
+      if (param_type) {
+	    use_type = param_type->base_type();
+	    // Is this an implicit netvector_t with no dimenions?
+	    if (param_vect && param_vect->get_implicit() &&
+	        param_vect->get_scalar())
+		  lv_width = -2;
+	    else if (param_type->packed())
+		  lv_width = param_type->packed_width();
+      } else {
+	    use_type = val_expr->expr_type();
+      }
+
+      if (debug_elaborate) {
+	    cerr << val_expr->get_fileline() << ": " << __func__ << ": "
+		 << "use_type = " << use_type << endl;
+      }
 
       NetExpr*expr = elab_and_eval(des, val_scope, val_expr, lv_width, true,
-                                   (*cur).second.is_annotatable,
-                                   (*cur).second.type);
+                                   cur->second.is_annotatable, use_type);
       if (! expr)
             return;
 
+      // Make sure to carry the signed-ness from a vector type.
+      if (param_vect)
+	    expr->cast_signed(param_vect->get_signed());
+
+      if (debug_elaborate) {
+	    cerr << val_expr->get_fileline() << ": " << __func__ << ": "
+		 << "expr = " << *expr << endl;
+	    cerr << val_expr->get_fileline() << ": " << __func__ << ": "
+		 << "expr type = " << expr->expr_type() << endl;
+      }
+
       switch (expr->expr_type()) {
 	  case IVL_VT_REAL:
 	    if (! dynamic_cast<const NetECReal*>(expr)) {
@@ -567,6 +575,13 @@ void NetScope::evaluate_parameter_logic_(Design*des, param_ref_t cur)
 		  des->errors += 1;
 		  return;
 	    }
+
+	    // If the parameter has no type, then infer its type from the
+	    // r-value expression.
+	    if (param_type==0) {
+		  param_type = &netreal_t::type_real;
+		  cur->second.ivl_type = param_type;
+	    }
 	    break;
 
 	  case IVL_VT_LOGIC:
@@ -579,17 +594,20 @@ void NetScope::evaluate_parameter_logic_(Design*des, param_ref_t cur)
 		  return;
 	    }
 
-	      // If the parameter has type or range information, then
-	      // make sure the type is set right. Note that if the
-	      // parameter doesn't have an explicit type or range,
-	      // then it will get the signedness from the expression itself.
-	    if (cur->second.type != IVL_VT_NO_TYPE) {
-		  expr->cast_signed(cur->second.signed_flag);
-	    } else if (cur->second.signed_flag) {
-		  expr->cast_signed(true);
+	    // If the parameter has no type, then infer its type from the
+	    // r-value expression.
+	    if (param_type==0) {
+		  param_type = new netvector_t(expr->expr_type(), expr->expr_width()-1,
+					       0, expr->has_sign());
+		  cur->second.ivl_type = param_type;
 	    }
 
-	    if (!range_flag && !expr->has_width()) {
+	    if (param_type->base_type() != IVL_VT_NO_TYPE)
+		  expr->cast_signed(param_type->get_signed());
+
+	    if (!expr->has_width()) {
+		  expr = pad_to_width(expr, integer_width, *expr);
+	    } else if (param_type->slice_dimensions().size()==0 && !expr->has_width()) {
 		  expr = pad_to_width(expr, integer_width, *expr);
 	    }
 	    break;
@@ -597,22 +615,28 @@ void NetScope::evaluate_parameter_logic_(Design*des, param_ref_t cur)
 	  default:
 	    cerr << expr->get_fileline()
 		 << ": internal error: "
-		 << "Unhandled expression type?" << endl;
+		 << "Unhandled expression type "
+		 << expr->expr_type() << "?" << endl;
+	    cerr << expr->get_fileline()
+		 << ":               : "
+		 << "param_type: " << *param_type << endl;
 	    des->errors += 1;
 	    return;
       }
 
+      // By the time we're done with the above, we should certainly know the
+      // type of the parameter.
+      ivl_assert(*expr, cur->second.ivl_type);
+
       cur->second.val = expr;
 
 	// If there are no value ranges to test the value against,
 	// then we are done.
-      if ((*cur).second.range == 0) {
+      if ((*cur).second.range == 0)
 	    return;
-      }
 
       NetEConst*val = dynamic_cast<NetEConst*>((*cur).second.val);
-      ivl_assert(*(*cur).second.val, (*cur).second.val);
-      ivl_assert(*(*cur).second.val, val);
+      ivl_assert(*expr, val);
 
       verinum value = val->value();
 
@@ -671,10 +695,12 @@ void NetScope::evaluate_parameter_real_(Design*des, param_ref_t cur)
 {
       PExpr*val_expr = (*cur).second.val_expr;
       NetScope*val_scope = (*cur).second.val_scope;
+      ivl_type_t param_type = cur->second.ivl_type;
 
+      ivl_assert(*val_expr, param_type);
       NetExpr*expr = elab_and_eval(des, val_scope, val_expr, -1, true,
-                                   (*cur).second.is_annotatable,
-                                   (*cur).second.type);
+                                   cur->second.is_annotatable,
+                                   param_type->base_type());
       if (! expr)
             return;
 
@@ -753,20 +779,111 @@ void NetScope::evaluate_parameter_real_(Design*des, param_ref_t cur)
       }
 }
 
+/*
+ * Evaluate a parameter that is forced to type string. This comes to pass when
+ * the input is something like this:
+ *
+ *     parameter string foo = <expr>;
+ *
+ * The param_type should be netstring_t, the val_expr is the pform of the
+ * input <expr>, and we try to elaborate/evaluate down to a IVL_VT_STRING
+ * expression.
+ */
+void NetScope::evaluate_parameter_string_(Design*des, param_ref_t cur)
+{
+      PExpr*val_expr = (*cur).second.val_expr;
+      NetScope*val_scope = (*cur).second.val_scope;
+      ivl_type_t param_type = cur->second.ivl_type;
+
+      ivl_assert(cur->second, val_expr);
+      ivl_assert(cur->second, param_type);
+
+      NetExpr*expr = elab_and_eval(des, val_scope, val_expr, param_type, true);
+      if (! expr)
+	    return;
+
+      cur->second.val = expr;
+
+      if (debug_elaborate) {
+	    cerr << cur->second.get_fileline() << ": " << __func__ << ": "
+		 << "Parameter type: " << *param_type << endl;
+	    cerr << cur->second.get_fileline() << ": " << __func__ << ": "
+		 << "Parameter value: " << *val_expr << endl;
+	    cerr << cur->second.get_fileline() << ": " << __func__ << ": "
+		 << "Elaborated value: " << *expr << endl;
+      }
+}
+
+void NetScope::evaluate_type_parameter_(Design *des, param_ref_t cur)
+{
+      const PETypename *type_expr = dynamic_cast<const PETypename*>(cur->second.val_expr);
+      if (!type_expr) {
+	    cerr << this->get_fileline() << ": error: "
+		 << "Type parameter `" << cur->first << "` value `"
+	         << *cur->second.val_expr << "` is not a type."
+		 << endl;
+	    des->errors++;
+
+	    // Recover
+	    cur->second.ivl_type = netvector_t::integer_type();
+	    return;
+      }
+
+      data_type_t *type = type_expr->get_type();
+      NetScope *type_scope = cur->second.val_scope;
+      cur->second.ivl_type = type->elaborate_type(des, type_scope);
+}
+
 void NetScope::evaluate_parameter_(Design*des, param_ref_t cur)
 {
-	// If the parameter has already been evaluated, quietly return.
-      if (cur->second.val_expr == 0)
+
+      // If the parameter has already been evaluated, quietly return.
+      if (cur->second.val || cur->second.ivl_type)
             return;
 
+      if (cur->second.val_expr == 0) {
+	    cerr << this->get_fileline() << ": error: "
+	         << "Missing value for parameter `"
+	         << cur->first << "`." << endl;
+	    des->errors += 1;
+
+	    cur->second.val = new NetEConst(verinum(verinum::Vx));
+	    return;
+      }
+
+      if (cur->second.type_flag) {
+	    evaluate_type_parameter_(des, cur);
+	    return;
+      }
+
+      ivl_type_t param_type = cur->second.ivl_type;
+
+      // If the parameter type is present, then elaborate it now. Elaborate
+      // the type in the current scope, and not the scope of the expression.
+      if (cur->second.val_type) {
+	    param_type = cur->second.val_type->elaborate_type(des, this);
+	    cur->second.ivl_type = param_type;
+	    cur->second.val_type = 0;
+      }
+
+      // Guess the varaiable type of the parameter. If the parameter has no
+      // given type, then guess the type from the expression and use that to
+      // evaluate (this is currently handled in evaluate_parameter_logic_()).
+      ivl_variable_type_t use_type;
+      if (param_type)
+	    use_type = param_type->base_type();
+      else
+	    use_type = IVL_VT_NO_TYPE;
+
       if (cur->second.solving) {
             cerr << cur->second.get_fileline() << ": error: "
 	         << "Recursive parameter reference found involving "
                  << cur->first << "." << endl;
 	    des->errors += 1;
+
       } else {
             cur->second.solving = true;
-            switch (cur->second.type) {
+            switch (use_type) {
                 case IVL_VT_NO_TYPE:
                 case IVL_VT_BOOL:
                 case IVL_VT_LOGIC:
@@ -777,12 +894,19 @@ void NetScope::evaluate_parameter_(Design*des, param_ref_t cur)
                   evaluate_parameter_real_(des, cur);
                   break;
 
+		case IVL_VT_STRING:
+		  evaluate_parameter_string_(des, cur);
+		  break;
+
                 default:
                   cerr << cur->second.get_fileline() << ": internal error: "
-                       << "Unexpected expression type " << cur->second.type
+                       << "Unexpected parameter type " << use_type
                        << "." << endl;
                   cerr << cur->second.get_fileline() << ":               : "
                        << "Parameter name: " << cur->first << endl;
+		  if (param_type)
+			cerr << cur->second.get_fileline() << ":               : "
+			     << "Parameter ivl_type: " << *param_type << endl;
                   cerr << cur->second.get_fileline() << ":               : "
                        << "Expression is: " << *cur->second.val_expr << endl;
                   ivl_assert(cur->second, 0);
