@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2017 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -22,6 +22,8 @@
 # include  "netlist.h"
 # include  "ivl_assert.h"
 
+using namespace std;
+
 /*
  * NOTE: The name_ is perm-allocated by the caller.
  */
@@ -264,6 +266,39 @@ const NetEvent* NetEvTrig::event() const
       return event_;
 }
 
+NetEvNBTrig::NetEvNBTrig(NetEvent*ev, NetExpr*dly)
+: event_(ev), dly_(dly)
+{
+      enext_ = event_->nb_trig_;
+      event_->nb_trig_ = this;
+}
+
+NetEvNBTrig::~NetEvNBTrig()
+{
+      if (event_->nb_trig_ == this) {
+	    event_->nb_trig_ = enext_;
+
+      } else {
+	    NetEvNBTrig*cur = event_->nb_trig_;
+	    while (cur->enext_ != this) {
+		  assert(cur->enext_);
+		  cur = cur->enext_;
+	    }
+
+	    cur->enext_ = this->enext_;
+      }
+}
+
+const NetExpr* NetEvNBTrig::delay() const
+{
+      return dly_;
+}
+
+const NetEvent* NetEvNBTrig::event() const
+{
+      return event_;
+}
+
 NetEvProbe::NetEvProbe(NetScope*s, perm_string n, NetEvent*tgt,
 		       edge_t t, unsigned p)
 : NetNode(s, n, p), event_(tgt), edge_(t)
