@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2002-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -22,11 +22,14 @@
 # include  "netenum.h"
 # include  "netclass.h"
 # include  "netdarray.h"
+# include  "netscalar.h"
 # include  "compiler.h"
 # include  "netmisc.h"
 # include  <iostream>
 # include  "ivl_assert.h"
 
+using namespace std;
+
 NetExpr::NetExpr(unsigned w)
 : net_type_(0), width_(w), signed_flag_(false)
 {
@@ -247,11 +250,6 @@ ivl_variable_type_t NetEConcat::expr_type() const
       return expr_type_;
 }
 
-bool NetEConcat::has_width() const
-{
-      return true;
-}
-
 void NetEConcat::set(unsigned idx, NetExpr*e)
 {
       assert(idx < parms_.size());
@@ -260,8 +258,8 @@ void NetEConcat::set(unsigned idx, NetExpr*e)
       expr_width( expr_width() + repeat_ * e->expr_width() );
 }
 
-NetEConstEnum::NetEConstEnum(Definitions*s, perm_string n, const netenum_t*eset, const verinum&v)
-: NetEConst(v), scope_(s), enum_set_(eset), name_(n)
+NetEConstEnum::NetEConstEnum(perm_string n, const netenum_t*eset, const verinum&v)
+: NetEConst(v), enum_set_(eset), name_(n)
 {
       assert(has_width());
 }
@@ -291,17 +289,12 @@ const verireal& NetECReal::value() const
       return value_;
 }
 
-bool NetECReal::has_width() const
-{
-      return true;
-}
-
 ivl_variable_type_t NetECReal::expr_type() const
 {
       return IVL_VT_REAL;
 }
 
-NetECRealParam::NetECRealParam(NetScope*s, perm_string n, const verireal&v)
+NetECRealParam::NetECRealParam(const NetScope*s, perm_string n, const verireal&v)
 : NetECReal(v), scope_(s), name_(n)
 {
 }
@@ -320,6 +313,19 @@ const NetScope* NetECRealParam::scope() const
       return scope_;
 }
 
+NetECString::NetECString(const std::string& val)
+: NetEConst(verinum(val))
+{
+}
+
+NetECString::~NetECString()
+{
+}
+
+ivl_variable_type_t NetECString::expr_type() const
+{
+      return IVL_VT_STRING;
+}
 
 NetELast::NetELast(NetNet*s)
 : sig_(s)
@@ -376,13 +382,12 @@ NetENull::~NetENull()
 {
 }
 
-NetEProperty::NetEProperty(NetNet*net, perm_string pnam, NetExpr*idx)
-: net_(net), index_(idx)
+NetEProperty::NetEProperty(NetNet*net, size_t pidx, NetExpr*idx)
+: net_(net), pidx_(pidx), index_(idx)
 {
       const netclass_t*use_type = dynamic_cast<const netclass_t*>(net->net_type());
       assert(use_type);
 
-      pidx_ = use_type->property_idx_from_name(pnam);
       ivl_type_t prop_type = use_type->get_prop_type(pidx_);
       expr_width(prop_type->packed_width());
       cast_signed(prop_type->get_signed());
@@ -403,7 +408,14 @@ ivl_variable_type_t NetEProperty::expr_type() const
 
 NetESelect::NetESelect(NetExpr*exp, NetExpr*base, unsigned wid,
                        ivl_select_type_t sel_type)
-: expr_(exp), base_(base), sel_type_(sel_type)
+: expr_(exp), base_(base), use_type_(0), sel_type_(sel_type)
+{
+      expr_width(wid);
+}
+
+NetESelect::NetESelect(NetExpr*exp, NetExpr*base, unsigned wid,
+                       ivl_type_t use_type)
+: expr_(exp), base_(base), use_type_(use_type), sel_type_(IVL_SEL_OTHER)
 {
       expr_width(wid);
 }
@@ -431,6 +443,9 @@ ivl_select_type_t NetESelect::select_type() const
 
 ivl_variable_type_t NetESelect::expr_type() const
 {
+      if (use_type_)
+	    return use_type_->base_type();
+
       ivl_variable_type_t type = expr_->expr_type();
 
 	// Special case: If the sub-expression is an IVL_VT_STRING,
@@ -439,25 +454,12 @@ ivl_variable_type_t NetESelect::expr_type() const
       if (type == IVL_VT_STRING && expr_width()==8)
 	    return IVL_VT_BOOL;
 
-      if (type != IVL_VT_DARRAY)
-	    return type;
-
-      ivl_assert(*this, type == IVL_VT_DARRAY);
-
-	// Special case: If the expression is a DARRAY, then the
-	// sub-expression must be a NetESignal and the type of the
-	// NetESelect expression is the element type of the arrayed signal.
-      NetESignal*sig = dynamic_cast<NetESignal*>(expr_);
-      ivl_assert(*this, sig);
-      const netarray_t*array_type = dynamic_cast<const netarray_t*> (sig->sig()->net_type());
-      ivl_assert(*this, array_type);
-
-      return array_type->element_type()->base_type();
+      return type;
 }
 
-bool NetESelect::has_width() const
+const netenum_t* NetESelect::enumeration() const
 {
-      return true;
+      return dynamic_cast<const netenum_t*> (use_type_);
 }
 
 NetESFunc::NetESFunc(const char*n, ivl_variable_type_t t,
@@ -469,26 +471,13 @@ NetESFunc::NetESFunc(const char*n, ivl_variable_type_t t,
 }
 
 NetESFunc::NetESFunc(const char*n, ivl_type_t rtype, unsigned np)
-: NetExpr(rtype), name_(0), type_(IVL_VT_NO_TYPE), enum_type_(0), parms_(np), is_overridden_(false)
+: NetExpr(rtype), name_(0), type_(rtype->base_type()),
+  enum_type_(dynamic_cast<const netenum_t*>(rtype)), parms_(np),
+  is_overridden_(false)
 {
       name_ = lex_strings.add(n);
       expr_width(rtype->packed_width());
-	// FIXME: For now, assume that all uses of this constructor
-	// are for the IVL_VT_DARRAY type. Eventually, the type_
-	// member will go away.
-      if (dynamic_cast<const netdarray_t*>(rtype))
-	    type_ = IVL_VT_DARRAY;
-      else if (dynamic_cast<const netclass_t*>(rtype))
-	    type_ = IVL_VT_CLASS;
-      else
-	    ivl_assert(*this, 0);
-}
-
-NetESFunc::NetESFunc(const char*n, const netenum_t*enum_type, unsigned np)
-: name_(0), type_(enum_type->base_type()), enum_type_(enum_type), parms_(np), is_overridden_(false)
-{
-      name_ = lex_strings.add(n);
-      expr_width(enum_type->packed_width());
+      cast_signed_base_(rtype->get_signed());
 }
 
 NetESFunc::~NetESFunc()
@@ -556,6 +545,7 @@ ivl_variable_type_t NetEShallowCopy::expr_type() const
 NetEAccess::NetEAccess(NetBranch*br, ivl_nature_t nat)
 : branch_(br), nature_(nat)
 {
+      cast_signed_base_(true);
 }
 
 NetEAccess::~NetEAccess()
