@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2012-2022 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -23,10 +23,6 @@
 # include  <typeinfo>
 # include  "ivl_assert.h"
 
-#if __cplusplus < 201103L
-#define unique_ptr auto_ptr
-#endif
-
 using namespace std;
 
 /*
@@ -218,7 +214,7 @@ bool NetProc::evaluate_function(const LineInfo&,
 }
 
 void NetAssign::eval_func_lval_op_real_(const LineInfo&loc,
-					verireal&lv, verireal&rv) const
+					verireal&lv, const verireal&rv) const
 {
       switch (op_) {
 	  case '+':
@@ -339,7 +335,7 @@ bool NetAssign::eval_func_lval_(const LineInfo&loc,
 
 	    word = word_const->value().as_long();
 
-	    if (word >= var->nwords)
+	    if (word < 0 || word >= var->nwords)
 		  return true;
 
 	    old_lval = var->array[word];
@@ -360,14 +356,9 @@ bool NetAssign::eval_func_lval_(const LineInfo&loc,
 
 	    long base = base_const->value().as_long();
 
-	    list<long>prefix (0);
-	    base = lval->sig()->sb_to_idx(prefix, base);
-
 	    if (old_lval == 0)
 		  old_lval = make_const_x(lval->sig()->vector_width());
 
-	    ivl_assert(loc, base + lval->lwidth() <= old_lval->expr_width());
-
 	    NetEConst*lval_const = dynamic_cast<NetEConst*>(old_lval);
 	    ivl_assert(loc, lval_const);
 	    verinum lval_v = lval_const->value();
@@ -377,15 +368,20 @@ bool NetAssign::eval_func_lval_(const LineInfo&loc,
 
 	    verinum lpart(verinum::Vx, lval->lwidth());
 	    if (op_) {
-		  for (unsigned idx = 0 ; idx < lpart.len() ; idx += 1)
-			lpart.set(idx, lval_v[base+idx]);
-
+		  for (unsigned idx = 0 ; idx < lpart.len() ; idx += 1) {
+			long ldx = base + idx;
+			if (ldx >= 0 && ldx < lval_v.len())
+			      lpart.set(idx, lval_v[ldx]);
+		  }
 		  eval_func_lval_op_(loc, lpart, rval_v);
 	    } else {
 		  lpart = cast_to_width(rval_v, lval->lwidth());
 	    }
-	    for (unsigned idx = 0 ; idx < lpart.len() ; idx += 1)
-		  lval_v.set(idx+base, lpart[idx]);
+	    for (unsigned idx = 0 ; idx < lpart.len() ; idx += 1) {
+		  long ldx = base + idx;
+		  if (ldx >= 0 && ldx < lval_v.len())
+			lval_v.set(idx+base, lpart[idx]);
+	    }
 
 	    delete base_result;
 	    delete rval_result;
@@ -441,8 +437,11 @@ bool NetAssign::eval_func_lval_(const LineInfo&loc,
 bool NetAssign::evaluate_function(const LineInfo&loc,
 				  map<perm_string,LocalVar>&context_map) const
 {
-	// Evaluate the r-value expression.
-      NetExpr*rval_result = rval()->evaluate_function(loc, context_map);
+      // Evaluate the r-value expression.
+      const NetExpr*use_rval = rval();
+      if (use_rval == 0)
+	    return false;
+      NetExpr*rval_result = use_rval->evaluate_function(loc, context_map);
       if (rval_result == 0)
 	    return false;
 
@@ -960,8 +959,11 @@ NetExpr* NetESelect::evaluate_function(const LineInfo&loc,
       }
 
       verinum res (verinum::Vx, expr_width());
-      for (unsigned idx = 0 ; idx < res.len() ; idx += 1)
-	    res.set(idx, sub[base+idx]);
+      for (unsigned idx = 0 ; idx < res.len() ; idx += 1) {
+	    long sdx = base + idx;
+	    if (sdx >= 0 && sdx < sub.len())
+		  res.set(idx, sub[sdx]);
+      }
 
       NetEConst*res_const = new NetEConst(res);
       return res_const;
@@ -996,7 +998,7 @@ NetExpr* NetESignal::evaluate_function(const LineInfo&loc,
 
 	    int word = word_const->value().as_long();
 
-	    if (word_const->value().is_defined() && (word < var->nwords))
+	    if (word_const->value().is_defined() && (word >= 0) && (word < var->nwords))
 		  value = var->array[word];
       } else {
 	    value = var->value;
