@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2017 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2002-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -26,6 +26,8 @@
 # include  "netlist.h"
 # include  "netmisc.h"
 
+using namespace std;
+
 void NetProc::nex_output(NexusSet&)
 {
       cerr << get_fileline()
@@ -134,6 +136,10 @@ void NetEvTrig::nex_output(NexusSet&)
 {
 }
 
+void NetEvNBTrig::nex_output(NexusSet&)
+{
+}
+
 void NetEvWait::nex_output(NexusSet&out)
 {
       if (statement_) statement_->nex_output(out);
