@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2021 Stephen Williams (steve@icarus.com)
  * Copyright (c) 2016 CERN Michele Castellana (michele.castellana@cern.ch)
  *
  *    This source code is free software; you can redistribute it
@@ -25,12 +25,15 @@
 # include  "netclass.h"
 # include  "netenum.h"
 # include  "netvector.h"
+# include  "PExpr.h"
 # include  "PPackage.h"
 # include  <cstring>
 # include  <cstdlib>
 # include  <sstream>
 # include  "ivl_assert.h"
 
+using namespace std;
+
 class PExpr;
 
 Definitions::Definitions()
@@ -53,7 +56,7 @@ bool Definitions::add_enumeration_name(netenum_t*enum_set, perm_string name)
       netenum_t::iterator enum_val = enum_set->find_name(name);
       assert(enum_val != enum_set->end_name());
 
-      NetEConstEnum*val = new NetEConstEnum(this, name, enum_set, enum_val->second);
+      NetEConstEnum*val = new NetEConstEnum(name, enum_set, enum_val->second);
 
       pair<map<perm_string,NetEConstEnum*>::iterator, bool> cur;
       cur = enum_names_.insert(make_pair(name,val));
@@ -73,17 +76,6 @@ netenum_t* Definitions::enumeration_for_key(const enum_type_t*key) const
 	    return 0;
 }
 
-/*
- * This locates the enumeration TYPE for the given enumeration literal.
- */
-const netenum_t*Definitions::enumeration_for_name(perm_string name)
-{
-      NetEConstEnum*tmp = enum_names_[name];
-      assert(tmp != 0);
-
-      return tmp->enumeration();
-}
-
 /*
  * This locates the VALUE for the given enumeration literal.
  */
@@ -119,7 +111,6 @@ NetScope::NetScope(NetScope*up, const hname_t&n, NetScope::TYPE t, NetScope*in_u
   is_interface_(interface), is_unit_(compilation_unit), unit_(in_unit), up_(up)
 {
       imports_ = 0;
-      typedefs_ = 0;
       events_ = 0;
       lcounter_ = 0;
       is_auto_ = false;
@@ -198,6 +189,22 @@ void NetScope::set_line(perm_string file, unsigned lineno)
       def_lineno_ = lineno;
 }
 
+string NetScope::get_fileline() const
+{
+      ostringstream buf;
+      buf << (file_? file_ : "") << ":" << lineno_;
+      string res = buf.str();
+      return res;
+}
+
+string NetScope::get_def_fileline() const
+{
+      ostringstream buf;
+      buf << (def_file_? def_file_ : "") << ":" << def_lineno_;
+      string res = buf.str();
+      return res;
+}
+
 void NetScope::set_line(perm_string file, perm_string def_file,
                         unsigned lineno, unsigned def_lineno)
 {
@@ -225,19 +232,20 @@ NetScope*NetScope::find_import(const Design*des, perm_string name)
             return 0;
 }
 
-void NetScope::add_typedefs(const map<perm_string,data_type_t*>*typedefs)
+void NetScope::add_typedefs(const map<perm_string,typedef_t*>*typedefs)
 {
       if (!typedefs->empty())
-	    typedefs_ = typedefs;
+	    typedefs_ = *typedefs;
 }
 
-NetScope*NetScope::find_typedef_scope(const Design*des, data_type_t*type)
+NetScope*NetScope::find_typedef_scope(const Design*des, const typedef_t*type)
 {
       assert(type);
 
       NetScope *cur_scope = this;
       while (cur_scope) {
-	    if (cur_scope->typedefs_ && cur_scope->typedefs_->find(type->name) != cur_scope->typedefs_->end())
+	    auto it = cur_scope->typedefs_.find(type->name);
+	    if (it != cur_scope->typedefs_.end() && it->second == type)
 		  return cur_scope;
 	    NetScope*import_scope = cur_scope->find_import(des, type->name);
 	    if (import_scope)
@@ -255,52 +263,28 @@ NetScope*NetScope::find_typedef_scope(const Design*des, data_type_t*type)
 }
 
 /*
- * Look for the enumeration in the current scope and any parent scopes.
+ * Attach to the a parameter name in the scope a value and a type. The value
+ * (val_expr) is the PExpr form that is not yet elaborated. Later, when
+ * elaboration happens, the val_expr is elaborated and written to the val
+ * member.
  */
-const netenum_t*NetScope::find_enumeration_for_name(const Design*des, perm_string name)
-{
-      NetScope *cur_scope = this;
-      while (cur_scope) {
-	    NetEConstEnum*tmp = cur_scope->enum_names_[name];
-	    if (tmp) break;
-	    NetScope*import_scope = cur_scope->find_import(des, name);
-	    if (import_scope)
-		  cur_scope = import_scope;
-	    else if (cur_scope == unit_)
-		  return 0;
-	    else
-		  cur_scope = cur_scope->parent();
-
-	    if (cur_scope == 0)
-		  cur_scope = unit_;
-      }
-
-      assert(cur_scope);
-      return cur_scope->enum_names_[name]->enumeration();
-}
-
 void NetScope::set_parameter(perm_string key, bool is_annotatable,
-			     PExpr*val, ivl_variable_type_t type__,
-			     PExpr*msb, PExpr*lsb, bool signed_flag,
-			     bool local_flag,
-			     NetScope::range_t*range_list,
-			     const LineInfo&file_line)
+			     const LexicalScope::param_expr_t &param,
+			     NetScope::range_t *range_list)
 {
       param_expr_t&ref = parameters[key];
       ref.is_annotatable = is_annotatable;
-      ref.msb_expr = msb;
-      ref.lsb_expr = lsb;
-      ref.val_expr = val;
+      ref.val_expr = param.expr;
+      ref.val_type = param.data_type;
       ref.val_scope = this;
-      ref.type = type__;
-      ref.msb = 0;
-      ref.lsb = 0;
-      ref.signed_flag = signed_flag;
-      ref.local_flag = local_flag;
-      ivl_assert(file_line, ref.range == 0);
+      ref.local_flag = param.local_flag;
+      ref.overridable = param.overridable;
+      ref.type_flag = param.type_flag;
+      ivl_assert(param, !ref.range);
       ref.range = range_list;
       ref.val = 0;
-      ref.set_line(file_line);
+      ref.ivl_type = 0;
+      ref.set_line(param);
 }
 
 /*
@@ -313,14 +297,11 @@ void NetScope::set_parameter(perm_string key, NetExpr*val,
 {
       param_expr_t&ref = parameters[key];
       ref.is_annotatable = false;
-      ref.msb_expr = 0;
-      ref.lsb_expr = 0;
       ref.val_expr = 0;
+      ref.val_type = 0;
       ref.val_scope = this;
-      ref.type = IVL_VT_BOOL;
-      ref.msb = 0;
-      ref.lsb = 0;
-      ref.signed_flag = false;
+      ref.ivl_type = netvector_t::integer_type();
+      ivl_assert(file_line, ref.ivl_type);
       ref.val = val;
       ref.set_line(file_line);
 }
@@ -345,8 +326,9 @@ bool NetScope::auto_name(const char*prefix, char pad, const char* suffix)
       while (use_prefix.size() <= max_pad_attempts) {
 	      // Try this name...
 	    string tmp = use_prefix + suffix;
-	    hname_t new_name(lex_strings.make(tmp.c_str()), name_.peek_numbers());
-	    if (!up_->child(new_name)) {
+	    perm_string base_name = lex_strings.make(tmp.c_str());
+	    hname_t new_name(base_name, name_.peek_numbers());
+	    if (!up_->child(new_name) && !up_->symbol_exists(base_name)) {
 		    // Ah, this name is unique. Rename myself, and
 		    // change my name in the parent scope.
 		  name_ = new_name;
@@ -365,18 +347,47 @@ bool NetScope::auto_name(const char*prefix, char pad, const char* suffix)
  * Return false if the parameter does not already exist.
  * A parameter is not automatically created.
  */
-bool NetScope::replace_parameter(perm_string key, PExpr*val, NetScope*scope)
+void NetScope::replace_parameter(Design *des, perm_string key, PExpr*val,
+			         NetScope*scope, bool defparam)
 {
-      if (parameters.find(key) == parameters.end())
-	    return false;
+      if (parameters.find(key) == parameters.end()) {
+	    cerr << val->get_fileline() << ": error: parameter `"
+	         << key << "` not found in `"
+	         << scope_path(this) << "`." << endl;
+	    des->errors++;
+	    return;
+      }
 
       param_expr_t&ref = parameters[key];
-      if (ref.local_flag)
-	    return false;
+      if (ref.local_flag) {
+	    cerr << val->get_fileline() << ": error: "
+	         << "Cannot override localparam `" << key << "` in `"
+	         << scope_path(this) << "`." << endl;
+	    des->errors++;
+	    return;
+      }
+      if (!ref.overridable) {
+	    cerr << val->get_fileline() << ": error: "
+		 << "Cannot override parameter `" << key << "` in `"
+		 << scope_path(this) << "`. Parameter cannot be overriden "
+		 << "in the scope it has been declared in."
+		 << endl;
+	    des->errors++;
+	    return;
+      }
+
+      if (ref.type_flag && defparam) {
+	    cerr << val->get_fileline() << ": error: "
+		 << "Cannot override type parameter `" << key << "` in `"
+		 << scope_path(this) << "`. It is not allowed to override type"
+		 << " parameters using a defparam statement."
+		 << endl;
+	    des->errors++;
+	    return;
+      }
 
       ref.val_expr = val;
       ref.val_scope = scope;
-      return true;
 }
 
 bool NetScope::make_parameter_unannotatable(perm_string key)
@@ -399,18 +410,14 @@ bool NetScope::make_parameter_unannotatable(perm_string key)
  * perm_string::literal method to fake the compiler into doing the
  * compare without actually creating a perm_string.
  */
-const NetExpr* NetScope::get_parameter(Design*des,
-				       const char* key,
-				       const NetExpr*&msb,
-				       const NetExpr*&lsb)
+const NetExpr* NetScope::get_parameter(Design*des, const char* key,
+				       ivl_type_t&ivl_type)
 {
-      return get_parameter(des, perm_string::literal(key), msb, lsb);
+      return get_parameter(des, perm_string::literal(key), ivl_type);
 }
 
-const NetExpr* NetScope::get_parameter(Design*des,
-				       perm_string key,
-				       const NetExpr*&msb,
-				       const NetExpr*&lsb)
+const NetExpr* NetScope::get_parameter(Design*des, perm_string key,
+				       ivl_type_t&ivl_type)
 {
       map<perm_string,param_expr_t>::iterator idx;
 
@@ -419,29 +426,27 @@ const NetExpr* NetScope::get_parameter(Design*des,
             if (idx->second.val_expr)
                   evaluate_parameter_(des, idx);
 
-	    msb = idx->second.msb;
-	    lsb = idx->second.lsb;
+	    ivl_type = idx->second.ivl_type;
 	    return idx->second.val;
       }
 
-      msb = 0;
-      lsb = 0;
+      ivl_type = 0;
       const NetExpr*tmp = enumeration_expr(key);
       return tmp;
 }
 
-NetScope::param_ref_t NetScope::find_parameter(perm_string key)
+LineInfo NetScope::get_parameter_line_info(perm_string key) const
 {
-      map<perm_string,param_expr_t>::iterator idx;
+      map<perm_string,param_expr_t>::const_iterator idx;
 
       idx = parameters.find(key);
-      if (idx != parameters.end()) return idx;
+      if (idx != parameters.end()) return idx->second;
 
 	// To get here the parameter must already exist, so we should
 	// never get here.
       assert(0);
 	// But return something to avoid a compiler warning.
-      return idx;
+      return LineInfo();
 }
 
 void NetScope::print_type(ostream&stream) const
@@ -702,7 +707,7 @@ NetNet* NetScope::find_signal(perm_string key)
 	    return 0;
 }
 
-netclass_t*NetScope::find_class(perm_string name)
+netclass_t*NetScope::find_class(const Design*des, perm_string name)
 {
 	// Special case: The scope itself is the class that we are
 	// looking for. This may happen for example when elaborating
@@ -715,20 +720,25 @@ netclass_t*NetScope::find_class(perm_string name)
       if (cur != classes_.end())
 	    return cur->second;
 
+        // Try the imports.
+      NetScope*import_scope = find_import(des, name);
+      if (import_scope)
+            return import_scope->find_class(des, name);
+
       if (up_==0 && type_==CLASS) {
 	    assert(class_def_);
 
 	    NetScope*def_parent = class_def_->definition_scope();
-	    return def_parent->find_class(name);
+	    return def_parent->find_class(des, name);
       }
 
 	// Try looking up for the class.
       if (up_!=0 && type_!=MODULE)
-	    return up_->find_class(name);
+	    return up_->find_class(des, name);
 
 	// Try the compilation unit.
-      if (unit_ != 0)
-	    return unit_->find_class(name);
+      if (unit_ != 0 && this != unit_)
+	    return unit_->find_class(des, name);
 
 	// Nowhere left to try...
       return 0;
@@ -797,6 +807,24 @@ const NetScope* NetScope::child_byname(perm_string name) const
 }
 
 
+bool NetScope::symbol_exists(perm_string sym)
+{
+      if (signals_map_.find(sym) != signals_map_.end())
+            return true;
+      if (parameters.find(sym) != parameters.end())
+            return true;
+      if (genvars_.find(sym) != genvars_.end())
+            return true;
+      if (classes_.find(sym) != classes_.end())
+          return true;
+      if (typedefs_.find(sym) != typedefs_.end())
+          return true;
+      if (find_event(sym))
+          return true;
+
+      return false;
+}
+
 perm_string NetScope::local_symbol()
 {
       perm_string sym;
@@ -805,16 +833,8 @@ perm_string NetScope::local_symbol()
 	    res << "_ivl_" << (lcounter_++);
 	    perm_string sym_tmp = lex_strings.make(res.str());
 
-	      // If the name already exists as a signal, try again.
-	    if (signals_map_.find(sym_tmp) != signals_map_.end())
-		  continue;
-	      // If the name already exists as a parameter, try again.
-	    if (parameters.find(sym_tmp) != parameters.end())
-		  continue;
-	    if (genvars_.find(sym_tmp) != genvars_.end())
-		  continue;
-	      // If the name already exists as a class, try again.
-	    if (classes_.find(sym_tmp) != classes_.end())
+	      // If the name already exists, try again.
+	    if (symbol_exists(sym_tmp))
 		  continue;
 
 	      // No collisions, this is the one.
