@@ -23,8 +23,8 @@
 
 using namespace std;
 
-netclass_t::netclass_t(perm_string name, netclass_t*sup)
-: name_(name), super_(sup), class_scope_(0), definition_scope_(0)
+netclass_t::netclass_t(perm_string name, const netclass_t*super)
+: name_(name), super_(super), class_scope_(0), definition_scope_(0)
 {
 }
 
@@ -32,7 +32,8 @@ netclass_t::~netclass_t()
 {
 }
 
-bool netclass_t::set_property(perm_string pname, property_qualifier_t qual, ivl_type_s*ptype)
+bool netclass_t::set_property(perm_string pname, property_qualifier_t qual,
+			      ivl_type_t ptype)
 {
       map<perm_string,size_t>::const_iterator cur;
       cur = properties_.find(pname);
@@ -188,3 +189,20 @@ bool netclass_t::test_scope_is_method(const NetScope*scope) const
       else
 	    return true;
 }
+
+const NetExpr* netclass_t::get_parameter(Design *des, perm_string name,
+					 ivl_type_t &par_type) const
+{
+      return class_scope_->get_parameter(des, name, par_type);
+}
+
+bool netclass_t::test_compatibility(ivl_type_t that) const
+{
+      for (const netclass_t *class_type = dynamic_cast<const netclass_t *>(that);
+	    class_type; class_type = class_type->get_super()) {
+	    if (class_type == this)
+		  return true;
+      }
+
+      return false;
+}
