@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -32,11 +32,13 @@
 # include  "netdarray.h"
 # include  "netenum.h"
 # include  "netparray.h"
+# include  "netscalar.h"
 # include  "netqueue.h"
 # include  "netstruct.h"
 # include  "netvector.h"
 # include  "ivl_assert.h"
 
+using namespace std;
 
 ostream& operator<< (ostream&o, NetNet::Type t)
 {
@@ -50,9 +52,6 @@ ostream& operator<< (ostream&o, NetNet::Type t)
 	  case NetNet::IMPLICIT_REG:
 	    o << "reg /*implicit*/";
 	    break;
-	  case NetNet::INTEGER:
-	    o << "integer";
-	    break;
 	  case NetNet::REG:
 	    o << "reg";
 	    break;
@@ -224,7 +223,7 @@ void NetPins::set_default_dir(Link::DIR d)
        default_dir_ = d;
 }
 
-bool NetPins::is_linked(void)
+bool NetPins::is_linked(void) const
 {
       bool linked_flag = false;
       if (pins_ == NULL) return false;
@@ -550,6 +549,9 @@ template <class T> static unsigned calculate_count(T*type)
 void NetNet::calculate_slice_widths_from_packed_dims_(void)
 {
       ivl_assert(*this, net_type_);
+      if (!net_type_->packed())
+	    return;
+
       slice_dims_ = net_type_->slice_dimensions();
 
 	// Special case: There are no actual packed dimensions, so
@@ -599,42 +601,10 @@ NetNet::NetNet(NetScope*s, perm_string n, Type t,
       s->add_signal(this);
 }
 
-/*
- * When we create a netnet for a packed struct, create a single
- * vector with the msb_/lsb_ chosen to name enough bits for the entire
- * packed structure.
- */
-NetNet::NetNet(NetScope*s, perm_string n, Type t, netstruct_t*ty)
+NetNet::NetNet(NetScope*s, perm_string n, Type t, ivl_type_t type)
 : NetObj(s, n, 1),
     type_(t), port_type_(NOT_A_PORT),
-    local_flag_(false), net_type_(ty),
-    discipline_(0),
-    eref_count_(0), lref_count_(0)
-{
-	//XXXX packed_dims_.push_back(netrange_t(calculate_count(ty)-1, 0));
-      calculate_slice_widths_from_packed_dims_();
-
-      initialize_dir_();
-
-      s->add_signal(this);
-}
-
-NetNet::NetNet(NetScope*s, perm_string n, Type t, netdarray_t*ty)
-: NetObj(s, n, 1),
-    type_(t), port_type_(NOT_A_PORT),
-    local_flag_(false), net_type_(ty),
-    discipline_(0),
-    eref_count_(0), lref_count_(0)
-{
-      initialize_dir_();
-
-      s->add_signal(this);
-}
-
-NetNet::NetNet(NetScope*s, perm_string n, Type t, netvector_t*ty)
-: NetObj(s, n, 1),
-    type_(t), port_type_(NOT_A_PORT),
-    local_flag_(false), net_type_(ty),
+    local_flag_(false), net_type_(type),
     discipline_(0),
     eref_count_(0), lref_count_(0)
 {
@@ -715,20 +685,10 @@ bool NetNet::get_signed() const
       return net_type_->get_signed();
 }
 
-bool NetNet::get_isint() const
-{
-      if (const netvector_t*vec = dynamic_cast<const netvector_t*> (net_type_))
-	    return vec->get_isint();
-      else
-	    return false;
-}
-
 bool NetNet::get_scalar() const
 {
-      if (const netvector_t*vec = dynamic_cast<const netvector_t*> (net_type_))
-	    return vec->get_scalar();
-      else
-	    return false;
+      ivl_assert(*this, net_type_);
+      return net_type_->get_scalar();
 }
 
 const netenum_t*NetNet::enumeration(void) const
@@ -2199,6 +2159,7 @@ NetEUFunc::NetEUFunc(NetScope*scope, NetScope*def, NetESignal*res,
 : scope_(scope), func_(def), result_sig_(res), parms_(p), need_const_(nc)
 {
       expr_width(result_sig_->expr_width());
+      cast_signed_base_(result_sig_->has_sign());
 }
 
 NetEUFunc::~NetEUFunc()
@@ -2390,7 +2351,7 @@ void NetEConst::trim()
       expr_width(value_.len());
 }
 
-NetEConstParam::NetEConstParam(NetScope*s, perm_string n, const verinum&v)
+NetEConstParam::NetEConstParam(const NetScope*s, perm_string n, const verinum&v)
 : NetEConst(v), scope_(s), name_(n)
 {
       cast_signed_base_(v.has_sign());
@@ -2974,7 +2935,12 @@ DelayType NetTaskDef::delay_type(bool print_delay) const
 
 DelayType NetUTask::delay_type(bool print_delay) const
 {
-      return task()->task_def()->delay_type(print_delay);
+	// Is this a void function call in a final block?
+      if (task()->type() == NetScope::FUNC) {
+	    return NO_DELAY;
+      } else {
+	    return task()->task_def()->delay_type(print_delay);
+      }
 }
 
 static bool do_expr_event_match(const NetExpr*expr, const NetEvWait*evwt)
@@ -3184,7 +3150,6 @@ bool NetBlock::check_synth(ivl_process_type_t pr_type,
       for (const NetProc*cur = proc_first(); cur; cur = proc_next(cur)) {
 	    result |= cur->check_synth(pr_type, scope);
       }
-      scope = save_scope;
       return result;
 }
 
@@ -3256,6 +3221,13 @@ bool NetEvTrig::check_synth(ivl_process_type_t pr_type,
       return false;
 }
 
+bool NetEvNBTrig::check_synth(ivl_process_type_t pr_type,
+                              const NetScope* /* scope */ ) const
+{
+      print_synth_warning(this, "A non-blocking event trigger", pr_type);
+      return false;
+}
+
 // The delay check above has already marked this as an error.
 bool NetEvWait::check_synth(ivl_process_type_t pr_type,
                             const NetScope* scope) const
@@ -3490,7 +3462,14 @@ bool NetSTask::check_synth(ivl_process_type_t pr_type,
       return false;
 }
 
-bool NetTaskDef::check_synth(ivl_process_type_t pr_type,
+/*
+ * This function is called to make sure the task/function can be used
+ * in a context where it must be synthesizable, such as in an always_comb
+ * or always_ff.
+ *
+ * If this is a function, then the function must be void.
+ */
+bool NetBaseDef::check_synth(ivl_process_type_t pr_type,
                              const NetScope* /* scope */) const
 {
       bool result = false;
@@ -3510,7 +3489,13 @@ bool NetTaskDef::check_synth(ivl_process_type_t pr_type,
 bool NetUTask::check_synth(ivl_process_type_t pr_type,
                            const NetScope* scope) const
 {
-      return task()->task_def()->check_synth(pr_type, scope);
+      const NetScope* task_scope = task();
+      if (task_scope->type() == NetScope::FUNC) {
+	    // This can happen if this a void function.
+	    return task_scope->func_def()->check_synth(pr_type, scope);
+      } else {
+	    return task_scope->task_def()->check_synth(pr_type, scope);
+      }
 }
 
 bool NetWhile::check_synth(ivl_process_type_t pr_type,
