@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -30,6 +30,7 @@
 # include  "compiler.h"
 # include  "ivl_assert.h"
 
+using namespace std;
 
 NetNet* sub_net_from(Design*des, NetScope*scope, long val, NetNet*sig)
 {
@@ -543,7 +544,7 @@ void indices_to_expressions(Design*des, NetScope*scope,
 	    }
 	    ivl_assert(*loc, cur->msb);
 
-	    NetExpr*word_index = elab_and_eval_lossless(des, scope, cur->msb, -2, need_const);
+	    NetExpr*word_index = elab_and_eval(des, scope, cur->msb, -1, need_const);
 
 	    if (word_index == 0)
 		  flags.invalid = true;
@@ -844,20 +845,23 @@ NetExpr* condition_reduce(NetExpr*expr)
       return cmp;
 }
 
-static NetExpr* do_elab_and_eval(Design*des, NetScope*scope, PExpr*pe,
-				 int context_width, bool need_const,
-				 bool annotatable, bool force_expand,
-				 ivl_variable_type_t cast_type,
-				 bool force_unsigned)
+NetExpr* elab_and_eval(Design*des, NetScope*scope, PExpr*pe,
+		       int context_width, bool need_const, bool annotatable,
+		       ivl_variable_type_t cast_type, bool force_unsigned)
 {
       PExpr::width_mode_t mode = PExpr::SIZED;
       if ((context_width == -2) && !gn_strict_expr_width_flag)
             mode = PExpr::EXPAND;
-      if (force_expand)
-	    mode = PExpr::EXPAND;
 
       pe->test_width(des, scope, mode);
 
+      if (pe->expr_type() == IVL_VT_CLASS) {
+	    cerr << pe->get_fileline() << ": Error: "
+	         << "Class/null r-value not allowed in this context." << endl;
+	    des->errors += 1;
+	    return 0;
+      }
+
         // Get the final expression width. If the expression is unsized,
         // this may be different from the value returned by test_width().
       unsigned expr_width = pe->expr_width();
@@ -880,7 +884,6 @@ static NetExpr* do_elab_and_eval(Design*des, NetScope*scope, PExpr*pe,
                  << "returns type=" << pe->expr_type()
 		 << ", context_width=" << context_width
                  << ", signed=" << pe->has_sign()
-		 << ", force_expand=" << force_expand
                  << ", expr_width=" << expr_width
                  << ", mode=" << PExpr::width_mode_name(mode) << endl;
 	    cerr << pe->get_fileline() << ":              : "
@@ -953,13 +956,6 @@ static NetExpr* do_elab_and_eval(Design*des, NetScope*scope, PExpr*pe,
             }
       }
 
-	// If the context_width sent is is actually the minimum width,
-	// then raise the context_width to be big enough for the
-	// lossless expression.
-      if (force_expand && context_width > 0) {
-	    context_width = max(context_width, (int)expr_width);
-      }
-
       eval_expr(tmp, context_width);
 
       if (NetEConst*ce = dynamic_cast<NetEConst*>(tmp)) {
@@ -970,34 +966,11 @@ static NetExpr* do_elab_and_eval(Design*des, NetScope*scope, PExpr*pe,
       return tmp;
 }
 
-NetExpr* elab_and_eval(Design*des, NetScope*scope, PExpr*pe,
-		       int context_width, bool need_const, bool annotatable,
-		       ivl_variable_type_t cast_type, bool force_unsigned)
-{
-      return do_elab_and_eval(des, scope, pe, context_width,
-			      need_const, annotatable, false,
-			      cast_type, force_unsigned);
-}
-
-/*
- * This variant of elab_and_eval does the expression losslessly, no
- * matter what the generation of Verilog. This is in support of
- * certain special contexts, notably index expressions.
- */
-NetExpr* elab_and_eval_lossless(Design*des, NetScope*scope, PExpr*pe,
-				 int context_width, bool need_const, bool annotatable,
-				 ivl_variable_type_t cast_type)
-{
-      return do_elab_and_eval(des, scope, pe, context_width,
-			      need_const, annotatable, true,
-			      cast_type, false);
-}
-
 NetExpr* elab_and_eval(Design*des, NetScope*scope, PExpr*pe,
 		       ivl_type_t lv_net_type, bool need_const)
 {
       if (debug_elaborate) {
-	    cerr << pe->get_fileline() << ": elab_and_eval: "
+	    cerr << pe->get_fileline() << ": " << __func__ << ": "
 		 << "pe=" << *pe
 		 << ", lv_net_type=" << *lv_net_type << endl;
       }
@@ -1012,16 +985,17 @@ NetExpr* elab_and_eval(Design*des, NetScope*scope, PExpr*pe,
       if (tmp == 0) return 0;
 
       ivl_variable_type_t cast_type = ivl_type_base(lv_net_type);
-      if ((cast_type != IVL_VT_NO_TYPE) && (cast_type != tmp->expr_type())) {
+      ivl_variable_type_t expr_type = tmp->expr_type();
+      if ((cast_type != IVL_VT_NO_TYPE) && (cast_type != expr_type)) {
 	      // Catch some special cases.
 	    switch (cast_type) {
 		case IVL_VT_DARRAY:
 		case IVL_VT_QUEUE:
-		  if (NetESignal*net = dynamic_cast<NetESignal*>(tmp)) {
-			ivl_variable_type_t type = net->expr_type();
-			if ((type == IVL_VT_DARRAY) || (type == IVL_VT_QUEUE))
-			      return tmp;
-		  }
+		  if ((expr_type == IVL_VT_DARRAY) || (expr_type == IVL_VT_QUEUE))
+			return tmp;
+
+		  // This is needed to handle the special case of `'{}` which
+		  // gets elaborated to NetENull.
 		  if (dynamic_cast<PEAssignPattern*>(pe))
 			return tmp;
 		  // fall through
@@ -1055,8 +1029,9 @@ NetExpr* elab_sys_task_arg(Design*des, NetScope*scope, perm_string name,
       pe->test_width(des, scope, mode);
 
       if (debug_elaborate) {
-            cerr << pe->get_fileline() << ": debug: test_width of "
-                 << name << " argument " << (arg_idx+1) << " " << *pe << endl;
+	    cerr << pe->get_fileline() << ": " << __func__ << ": "
+		 << "test_width of " << name
+                 << " argument " << (arg_idx+1) << " " << *pe << endl;
             cerr << pe->get_fileline() << ":        "
                  << "returns type=" << pe->expr_type()
                  << ", width=" << pe->expr_width()
@@ -1124,8 +1099,8 @@ bool evaluate_range(Design*des, NetScope*scope, const LineInfo*li,
                   if (!dimension_ok) {
                         // bail out
                   } else if (index_l > 0) {
-                        index_l = index_l - 1;
-                        index_r = 0;
+                        index_r = index_l - 1;
+                        index_l = 0;
                   } else {
                         cerr << range.first->get_fileline() << ": error: "
                                 "Dimension size must be greater than zero." << endl;
@@ -1802,3 +1777,52 @@ void check_for_inconsistent_delays(NetScope*scope)
 	    display_ts_dly_warning = false;
       }
 }
+
+
+/*
+ * Calculate the bit vector range for a parameter, from the type of the
+ * parameter. This is expecting that the type is a vector type. The parameter
+ * is presumably declared something like this:
+ *
+ *    parameter [4:1] foo = <value>;
+ *
+ * In this case, the par_type is a netvector with a single dimension. The
+ * par_msv gets 4, and par_lsv get 1. The caller uses these values to
+ * interpret things like bit selects.
+ */
+bool calculate_param_range(const LineInfo&line, ivl_type_t par_type,
+			   long&par_msv, long&par_lsv, long length)
+{
+      const netvector_t*vector_type = dynamic_cast<const netvector_t*> (par_type);
+      if (vector_type == 0) {
+	    // If the parameter doesn't have an explicit range, then
+	    // just return range values of [length-1:0].
+	    par_msv = length-1;
+	    par_lsv = 0;
+	    return true;
+      }
+
+      ivl_assert(line, vector_type->packed());
+      const std::vector<netrange_t>& packed_dims = vector_type->packed_dims();
+
+      // This is a netvector_t with 0 dimensions, then the parameter was
+      // declared with a statement like this:
+      //
+      //    parameter signed foo = <value>;
+      //
+      // The netvector_t is just here to carry the signed-ness, which we don't
+      // even need here. So act like the type is defined by the r-value
+      // length.
+      if (packed_dims.size() == 0) {
+	    par_msv = length-1;
+	    par_lsv = 0;
+	    return true;
+      }
+      ivl_assert(line, packed_dims.size() == 1);
+
+      netrange_t use_range = packed_dims[0];
+      par_msv = use_range.get_msb();
+      par_lsv = use_range.get_lsb();
+
+      return true;
+}
