@@ -1,7 +1,7 @@
 #ifndef IVL_netmisc_H
 #define IVL_netmisc_H
 /*
- * Copyright (c) 1999-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -23,6 +23,97 @@
 
 class netsarray_t;
 
+/*
+ * Search for a hierarchical name. The input path is one or more name
+ * components (name_component_t) which describe a path to the object. The
+ * simplest case is the path is a single name_component_t. This is the most
+ * usual case. More complex cases might include a string of name components
+ * that end in an item or scope, like this:
+ *
+ *   a.b[1].c
+ *
+ * In this case, the "path input would include a.b.c, with index expressions
+ * on name_component_t for "b". In this case, usually "c" is the found item
+ * and "a" and "b" are scopes that lead up to the item.
+ *
+ * The search will stop when it finds a component in the path that is an
+ * object of some sort (other then a scope. So for example, if a.b is an
+ * array, then the search for a.b[1].c will stop at a.b, leave a.b[1] in
+ * path_head, and "c" in path_tail. It is up to the caller to then note that
+ * "c" must be a method of some sort.
+ */
+struct symbol_search_results {
+      inline symbol_search_results() {
+	    scope = 0;
+	    net = 0;
+	    cls_val = 0;
+	    par_val = 0;
+	    type = 0;
+	    eve = 0;
+      }
+
+      inline bool is_scope() const {
+	    if (net) return false;
+	    if (eve) return false;
+	    if (cls_val) return false;
+	    if (par_val) return false;
+	    if (scope) return true;
+	    return false;
+      }
+
+      inline bool is_found() const {
+	    if (net) return true;
+	    if (eve) return true;
+	    if (cls_val) return true;
+	    if (par_val) return true;
+	    if (scope) return true;
+	    return false;
+      }
+
+	// Scope where symbol was located. This is set in all cases,
+	// assuming the search succeeded.
+      NetScope*scope;
+	// If this was a net, the signal itself.
+      NetNet*net;
+	// For a class property we only have type information.
+      ivl_type_t cls_val;
+	// If this was a parameter, the value expression and the
+	// optional value dimensions.
+      const NetExpr*par_val;
+      ivl_type_t type;
+	// If this is a named event, ...
+      NetEvent*eve;
+
+        // Store bread crumbs of the search here. The path_tail is the parts
+        // of the original path that were not found, or are after an object
+        // (and so are probably members or methods).
+      pform_name_t path_tail;
+        // The path_head is the parts of the original path that were found.
+        // The last item in path_head is the final name (possibly before the
+        // path_tail items) that identifies the object. This name may contain
+        // index expressions. If the search result is a scope, then this name
+        // is also the name of the scope identified.
+      pform_name_t path_head;
+};
+
+/*
+ * Test the search results and return true if this represents a function
+ * return value. That will be the case if the object is a net, the scope
+ * containing the object is a FUNCtion, and the containing scope and the
+ * object have the same name.
+ */
+static inline bool test_function_return_value(const symbol_search_results&search_results)
+{
+      if (!search_results.net) return false;
+      if (search_results.scope->type()!=NetScope::FUNC) return false;
+      if (search_results.net->name() != search_results.scope->basename()) return false;
+      return true;
+}
+
+extern bool symbol_search(const LineInfo*li, Design*des, NetScope*scope,
+			  pform_name_t path, struct symbol_search_results*res,
+			  NetScope*start_scope = 0);
+
 /*
  * Search for a symbol using the "start" scope as the starting
  * point. If the path includes a scope part, then locate the
@@ -41,11 +132,12 @@ class netsarray_t;
 extern NetScope* symbol_search(const LineInfo*li,
                                Design*des,
 			       NetScope*start,
-                               pform_name_t path,
+                               const pform_name_t&path,
 			       NetNet*&net,       /* net/reg */
 			       const NetExpr*&par,/* parameter/expr */
 			       NetEvent*&eve,     /* named event */
-			       const NetExpr*&ex1, const NetExpr*&ex2);
+			       ivl_type_t&par_type,
+			       ivl_type_t&cls_val);
 
 inline NetScope* symbol_search(const LineInfo*li,
                                Design*des,
@@ -55,8 +147,10 @@ inline NetScope* symbol_search(const LineInfo*li,
 			       const NetExpr*&par,/* parameter/expr */
 			       NetEvent*&eve      /* named event */)
 {
-      const NetExpr*ex1, *ex2;
-      return symbol_search(li, des, start, path, net, par, eve, ex1, ex2);
+      ivl_type_t par_type;
+      ivl_type_t cls_val;
+      return symbol_search(li, des, start, path, net, par, eve,
+                           par_type, cls_val);
 }
 
 /*
@@ -67,13 +161,13 @@ inline NetScope* symbol_search(const LineInfo*li,
  * signed_flag.
  */
 extern NetExpr*pad_to_width(NetExpr*expr, unsigned wid, bool signed_flag,
-			    const LineInfo&info);
+			    const LineInfo&info, ivl_type_t use_type = 0);
 /*
  * This version determines the extension method from the base expression type.
  */
-inline NetExpr*pad_to_width(NetExpr*expr, unsigned wid, const LineInfo&info)
+inline NetExpr*pad_to_width(NetExpr*expr, unsigned wid, const LineInfo&info, ivl_type_t use_type = 0)
 {
-      return pad_to_width(expr, wid, expr->has_sign(), info);
+      return pad_to_width(expr, wid, expr->has_sign(), info, use_type);
 }
 
 /*
@@ -130,7 +224,7 @@ extern NetExpr*normalize_variable_base(NetExpr *base, long msb, long lsb,
                                        unsigned long wid, bool is_up,
 				       long slice_off =0);
 extern NetExpr*normalize_variable_base(NetExpr *base,
-				       const list<netrange_t>&dims,
+				       const std::list<netrange_t>&dims,
 				       unsigned long wid, bool is_up);
 
 /*
@@ -141,7 +235,7 @@ extern NetExpr*normalize_variable_base(NetExpr *base,
  *   ... foo[1][x] ...
  * base is (x) and the generated expression will be (x+8).
  */
-extern NetExpr*normalize_variable_bit_base(const list<long>&indices, NetExpr *base,
+extern NetExpr*normalize_variable_bit_base(const std::list<long>&indices, NetExpr *base,
 					   const NetNet*reg);
 
 /*
@@ -153,7 +247,7 @@ extern NetExpr*normalize_variable_bit_base(const list<long>&indices, NetExpr *ba
  * base is (x), wid input is (2), and is_up is (true). The output
  * expression is (x+8).
  */
-extern NetExpr *normalize_variable_part_base(const list<long>&indices, NetExpr*base,
+extern NetExpr *normalize_variable_part_base(const std::list<long>&indices, NetExpr*base,
 					     const NetNet*reg,
 					     unsigned long wid, bool is_up);
 /*
@@ -167,7 +261,7 @@ extern NetExpr *normalize_variable_part_base(const list<long>&indices, NetExpr*b
  * base is (x) and the generated expression will be (x*8 - 8), with
  * lwid set to (8).
  */
-extern NetExpr*normalize_variable_slice_base(const list<long>&indices, NetExpr *base,
+extern NetExpr*normalize_variable_slice_base(const std::list<long>&indices, NetExpr *base,
 					     const NetNet*reg, unsigned long&lwid);
 
 /*
@@ -181,8 +275,8 @@ template <class TYPE> struct __IndicesManip {
 template <class TYPE> inline __IndicesManip<TYPE> as_indices(const std::list<TYPE>&indices)
 { return __IndicesManip<TYPE>(indices); }
 
-extern ostream& operator << (ostream&o, __IndicesManip<long>);
-extern ostream& operator << (ostream&o, __IndicesManip<NetExpr*>);
+extern std::ostream& operator << (std::ostream&o, __IndicesManip<long>);
+extern std::ostream& operator << (std::ostream&o, __IndicesManip<NetExpr*>);
 
 /*
  * Given a list of index expressions, generate elaborated expressions
@@ -198,18 +292,18 @@ extern void indices_to_expressions(Design*des, NetScope*scope,
 				   const LineInfo*loc,
 				     // src is the index list, and count is
 				     // the number of items in the list to use.
-				   const list<index_component_t>&src, unsigned count,
+				   const std::list<index_component_t>&src, unsigned count,
 				     // True if the expression MUST be constant.
 				   bool need_const,
 				     // These are the outputs.
 				   indices_flags&flags,
-				   list<NetExpr*>&indices,list<long>&indices_const);
+				   std::list<NetExpr*>&indices,std::list<long>&indices_const);
 
-extern NetExpr*normalize_variable_unpacked(const NetNet*net, list<long>&indices);
-extern NetExpr*normalize_variable_unpacked(const netsarray_t*net, list<long>&indices);
+extern NetExpr*normalize_variable_unpacked(const NetNet*net, std::list<long>&indices);
+extern NetExpr*normalize_variable_unpacked(const netsarray_t*net, std::list<long>&indices);
 
-extern NetExpr*normalize_variable_unpacked(const NetNet*net, list<NetExpr*>&indices);
-extern NetExpr*normalize_variable_unpacked(const LineInfo&loc, const netsarray_t*net, list<NetExpr*>&indices);
+extern NetExpr*normalize_variable_unpacked(const NetNet*net, std::list<NetExpr*>&indices);
+extern NetExpr*normalize_variable_unpacked(const LineInfo&loc, const netsarray_t*net, std::list<NetExpr*>&indices);
 
 extern NetExpr*make_canonical_index(Design*des, NetScope*scope,
 				      // loc for error messages
@@ -281,12 +375,6 @@ extern NetExpr* elab_and_eval(Design*des, NetScope*scope,
 			      ivl_variable_type_t cast_type =IVL_VT_NO_TYPE,
 			      bool force_unsigned =false);
 
-extern NetExpr* elab_and_eval_lossless(Design*des, NetScope*scope,
-			      PExpr*pe, int context_width,
-                              bool need_const =false,
-                              bool annotatable =false,
-			      ivl_variable_type_t cast_type =IVL_VT_NO_TYPE);
-
 /*
  * This form of elab_and_eval uses the ivl_type_t to carry type
  * information instead of the piecemeal form. We should transition to
@@ -320,6 +408,13 @@ extern NetExpr* elaborate_rval_expr(Design*des, NetScope*scope,
 				    unsigned lv_width, PExpr*expr,
 				    bool need_const =false,
 				    bool force_unsigned =false);
+/*
+ * Same as above, but lv_width and lv_type are derived from the lv_net_type.
+ */
+extern NetExpr* elaborate_rval_expr(Design *des, NetScope *scope,
+				    ivl_type_t lv_net_type, PExpr *expr,
+				    bool need_const = false,
+				    bool force_unsigned = false);
 
 extern bool evaluate_range(Design*des, NetScope*scope, const LineInfo*li,
                            const pform_range_t&range,
@@ -393,15 +488,15 @@ extern uint64_t get_scaled_time_from_real(Design*des,
 extern void collapse_partselect_pv_to_concat(Design*des, NetNet*sig);
 
 extern bool evaluate_index_prefix(Design*des, NetScope*scope,
-				  list<long>&prefix_indices,
-				  const list<index_component_t>&indices);
+				  std::list<long>&prefix_indices,
+				  const std::list<index_component_t>&indices);
 
 extern NetExpr*collapse_array_indices(Design*des, NetScope*scope, NetNet*net,
 				      const std::list<index_component_t>&indices);
 
 extern NetExpr*collapse_array_exprs(Design*des, NetScope*scope,
 				    const LineInfo*loc, NetNet*net,
-				    const list<index_component_t>&indices);
+				    const std::list<index_component_t>&indices);
 
 extern void assign_unpacked_with_bufz(Design*des, NetScope*scope,
 				      const LineInfo*loc,
