@@ -28,6 +28,10 @@ class netreal_t : public ivl_type_s {
       ~netreal_t();
 
       ivl_variable_type_t base_type() const;
+      bool get_signed() const { return true; }
+      bool get_scalar() const { return true; }
+
+      std::ostream& debug_dump(std::ostream&) const;
 
     public:
       static netreal_t type_real;
@@ -42,6 +46,8 @@ class netstring_t : public ivl_type_s {
 
       ivl_variable_type_t base_type() const;
 
+      std::ostream& debug_dump(std::ostream&) const;
+
     public:
       static netstring_t type_string;
 };
