@@ -1,7 +1,7 @@
 #ifndef IVL_pform_H
 #define IVL_pform_H
 /*
- * Copyright (c) 1998-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -62,6 +62,7 @@ class PSpecPath;
 class PClass;
 class PPackage;
 struct vlltype;
+struct lgate;
 
 /*
  * The min:typ:max expression s selected at parse time using the
@@ -89,42 +90,16 @@ extern bool pform_library_flag;
 
 
 struct parmvalue_t {
-      list<PExpr*>*by_order;
-      list<named_pexpr_t>*by_name;
+      std::list<PExpr*>*by_order;
+      std::list<named_pexpr_t>*by_name;
 };
 
 struct str_pair_t { ivl_drive_t str0, str1; };
 
-
-struct net_decl_assign_t {
-      perm_string name;
-      PExpr*expr;
-      struct net_decl_assign_t*next;
-};
-
-/* The lgate is gate instantiation information. */
-struct lgate {
-      explicit inline lgate(int =0)
-      : parms(0), parms_by_name(0), file(NULL), lineno(0)
-      { }
-
-      string name;
-      list<PExpr*>*parms;
-      list<named_pexpr_t>*parms_by_name;
-
-      pform_range_t range;
-
-      const char* file;
-      unsigned lineno;
-};
-
-extern std::list<pform_range_t>* make_range_from_width(uint64_t wid);
-extern std::list<pform_range_t>* copy_range(std::list<pform_range_t>* orig);
-
   /* Use this function to transform the parted form of the attribute
      list to the attribute map that is used later. */
-extern void pform_bind_attributes(map<perm_string,PExpr*>&attributes,
-				  list<named_pexpr_t>*attr,
+extern void pform_bind_attributes(std::map<perm_string,PExpr*>&attributes,
+				  std::list<named_pexpr_t>*attr,
 				  bool keep_attr =false);
 
   /* The lexor calls this function to change the default nettype. */
@@ -150,7 +125,6 @@ extern PWire* pform_get_wire_in_scope(perm_string name);
 extern PWire* pform_get_make_wire_in_scope(const struct vlltype&li,
                                            perm_string name,
                                            NetNet::Type net_type,
-                                           NetNet::PortType port_type,
                                            ivl_variable_type_t vt_type);
 
 /*
@@ -168,8 +142,8 @@ extern PWire* pform_get_make_wire_in_scope(const struct vlltype&li,
 extern void pform_startmodule(const struct vlltype&loc, const char*name,
 			      bool program_block, bool is_interface,
 			      LexicalScope::lifetime_t lifetime,
-			      list<named_pexpr_t>*attr);
-extern void pform_module_set_ports(vector<Module::port_t*>*);
+			      std::list<named_pexpr_t>*attr);
+extern void pform_module_set_ports(std::vector<Module::port_t*>*);
 extern void pform_set_scope_timescale(const struct vlltype&loc);
 
 /* These functions are used when we have a complete port definition, either
@@ -180,18 +154,18 @@ extern void pform_module_define_port(const struct vlltype&li,
 				     NetNet::PortType,
 				     NetNet::Type type,
 				     data_type_t*vtype,
-				     list<named_pexpr_t>*attr,
+				     std::list<pform_range_t>*urange,
+				     std::list<named_pexpr_t>*attr,
 				     bool keep_attr =false);
 extern void pform_module_define_port(const struct vlltype&li,
-				     list<pform_port_t>*ports,
+				     std::list<pform_port_t>*ports,
 				     NetNet::PortType,
 				     NetNet::Type type,
 				     data_type_t*vtype,
-				     list<named_pexpr_t>*attr);
+				     std::list<named_pexpr_t>*attr);
 
-extern Module::port_t* pform_module_port_reference(perm_string name,
-						   const char*file,
-						   unsigned lineno);
+extern Module::port_t* pform_module_port_reference(const struct vlltype&loc,
+						   perm_string name);
 extern void pform_endmodule(const char*, bool inside_celldefine,
                             Module::UCDriveType uc_drive_def);
 
@@ -199,7 +173,7 @@ extern void pform_start_class_declaration(const struct vlltype&loc,
 					  class_type_t*type,
 					  data_type_t*base_type,
 					  std::list<PExpr*>*base_exprs,
-					  LexicalScope::lifetime_t lifetime);
+					  bool virtual_class);
 extern void pform_class_property(const struct vlltype&loc,
 				 property_qualifier_t pq,
 				 data_type_t*data_type,
@@ -208,18 +182,18 @@ extern void pform_set_this_class(const struct vlltype&loc, PTaskFunc*net);
 extern void pform_set_constructor_return(PFunction*net);
 
 extern void pform_end_class_declaration(const struct vlltype&loc);
+extern bool pform_in_class();
 
-extern void pform_make_udp(perm_string name, list<perm_string>*parms,
-			   std::vector<PWire*>*decl, list<string>*table,
-			   Statement*init,
-			   const char*file, unsigned lineno);
+extern void pform_make_udp(const struct vlltype&loc, perm_string name,
+			   std::list<perm_string>*parms,
+			   std::vector<PWire*>*decl, std::list<std::string>*table,
+			   Statement*init);
 
-extern void pform_make_udp(perm_string name,
+extern void pform_make_udp(const struct vlltype&loc, perm_string name,
 			   bool sync_flag, perm_string out_name,
 			   PExpr*sync_init,
-			   list<perm_string>*parms,
-			   list<string>*table,
-			   const char*file, unsigned lineno);
+			   std::list<perm_string>*parms,
+			   std::list<std::string>*table);
 /*
  * Package related functions.
  */
@@ -250,6 +224,9 @@ extern PEIdent* pform_new_ident(const struct vlltype&loc, const pform_name_t&nam
 
 extern PTrigger* pform_new_trigger(const struct vlltype&loc, PPackage*pkg,
 				   const pform_name_t&name);
+extern PNBTrigger* pform_new_nb_trigger(const struct vlltype&loc,
+				        const std::list<PExpr*>*dly,
+				        const pform_name_t&name);
 
 /*
  * Enter/exit name scopes. The push_scope function pushes the scope
@@ -263,8 +240,7 @@ extern void pform_pop_scope();
  */
 extern LexicalScope* pform_peek_scope();
 
-extern PClass* pform_push_class_scope(const struct vlltype&loc, perm_string name,
-				      LexicalScope::lifetime_t lifetime);
+extern PClass* pform_push_class_scope(const struct vlltype&loc, perm_string name);
 
 extern PFunction*pform_push_constructor_scope(const struct vlltype&loc);
 
@@ -289,7 +265,14 @@ extern verinum* pform_verinum_with_size(verinum*s, verinum*val,
  * This function takes the list of names as new genvars to declare in
  * the current module or generate scope.
  */
-extern void pform_genvars(const struct vlltype&li, list<perm_string>*names);
+extern void pform_genvars(const struct vlltype&li, std::list<perm_string>*names);
+
+/*
+ * This flag is set by the parser to indicate the current generate block
+ * was not enclosed in a begin-end pair. This is a pre-requisite for
+ * directly nesting generate constructs.
+ */
+extern bool pform_generate_single_item;
 
 extern void pform_start_generate_for(const struct vlltype&li,
 				     bool local_index,
@@ -302,7 +285,7 @@ extern void pform_start_generate_if(const struct vlltype&li, PExpr*test);
 extern void pform_start_generate_else(const struct vlltype&li);
 extern void pform_start_generate_case(const struct vlltype&lp, PExpr*test);
 extern void pform_start_generate_nblock(const struct vlltype&lp, char*name);
-extern void pform_generate_case_item(const struct vlltype&lp, list<PExpr*>*test);
+extern void pform_generate_case_item(const struct vlltype&lp, std::list<PExpr*>*test);
 extern void pform_generate_block_name(char*name);
 extern void pform_endgenerate(bool end_conditional);
 
@@ -313,8 +296,17 @@ extern void pform_endgenerate(bool end_conditional);
  */
 extern PGenerate* pform_parent_generate(void);
 
-extern void pform_set_typedef(perm_string name, data_type_t*data_type,
-			      std::list<pform_range_t>*unp_ranges);
+bool pform_error_in_generate(const vlltype&loc, const char *type);
+
+extern void pform_make_elab_task(const struct vlltype&li,
+                                 perm_string name,
+                                 const std::list<PExpr*>&params);
+
+extern void pform_set_typedef(const struct vlltype&loc, perm_string name,
+			      data_type_t*data_type,
+			      std::list<pform_range_t>*unp_ranges = nullptr);
+extern void pform_forward_typedef(const struct vlltype&loc, perm_string name,
+			      enum typedef_t::basic_type basic_type);
 
 extern void pform_set_type_referenced(const struct vlltype&loc, const char*name);
 
@@ -339,44 +331,24 @@ extern PForeach* pform_make_foreach(const struct vlltype&loc,
  * The makewire functions announce to the pform code new wires. These
  * go into a module that is currently opened.
  */
-extern void pform_makewire(const struct vlltype&li, perm_string name,
-			   NetNet::Type type,
-			   NetNet::PortType pt,
-			   ivl_variable_type_t,
-			   list<named_pexpr_t>*attr);
-
-/* This form handles simple declarations */
-extern void pform_makewire(const struct vlltype&li,
-			   list<pform_range_t>*range,
-			   bool signed_flag,
-			   list<perm_string>*names,
-			   NetNet::Type type,
-			   NetNet::PortType,
-			   ivl_variable_type_t,
-			   list<named_pexpr_t>*attr,
-			   PWSRType rt = SR_NET);
+extern PWire *pform_makewire(const struct vlltype&li, perm_string name,
+			     NetNet::Type type,
+			     std::list<pform_range_t> *indices);
 
 /* This form handles assignment declarations. */
-extern void pform_makewire(const struct vlltype&li,
-			   list<PExpr*>*delay,
-			   str_pair_t str,
-			   net_decl_assign_t*assign_list,
-			   NetNet::Type type,
-			   data_type_t*data_type);
 
 extern void pform_makewire(const struct vlltype&li,
 			   std::list<PExpr*>*delay,
 			   str_pair_t str,
 			   std::list<decl_assignment_t*>*assign_list,
 			   NetNet::Type type,
-			   data_type_t*data_type);
+			   data_type_t*data_type,
+			   std::list<named_pexpr_t>*attr = 0);
 
-/* This form handles nets declared as structures. (See pform_struct_type.cc) */
-extern void pform_makewire(const struct vlltype&li,
-			   struct_type_t*struct_type,
-			   NetNet::PortType,
-			   list<perm_string>*names,
-			   list<named_pexpr_t>*attr);
+extern void pform_make_var(const struct vlltype&loc,
+			   std::list<decl_assignment_t*>*assign_list,
+			   data_type_t*data_type,
+			   std::list<named_pexpr_t>*attr = 0);
 
 extern void pform_make_var_init(const struct vlltype&li,
 				perm_string name, PExpr*expr);
@@ -386,17 +358,16 @@ extern void pform_make_var_init(const struct vlltype&li,
    the port type, i.e. input, output or inout, and, if specified, the
    range and signedness. If the wire does not exist, create it. */
 extern void pform_set_port_type(const struct vlltype&li,
-				list<pform_port_t>*ports,
+				std::list<pform_port_t>*ports,
 				NetNet::PortType,
 				data_type_t*dt,
-				list<named_pexpr_t>*attr);
+				std::list<named_pexpr_t>*attr);
 
-extern void pform_set_reg_idx(perm_string name,
-			      std::list<pform_range_t>*indices);
-
-extern void pform_set_data_type(const struct vlltype&li, data_type_t*, list<perm_string>*names, NetNet::Type net_type, list<named_pexpr_t>*attr);
-
-extern void pform_set_struct_type(const struct vlltype&li, struct_type_t*struct_type, std::list<perm_string>*names, NetNet::Type net_type, std::list<named_pexpr_t>*attr);
+extern void pform_set_data_type(const struct vlltype&li,
+				data_type_t *data_type,
+				std::vector<PWire*> *wires,
+				NetNet::Type net_type,
+				std::list<named_pexpr_t>*attr);
 
 extern void pform_set_string_type(const struct vlltype&li, const string_type_t*string_type, std::list<perm_string>*names, NetNet::Type net_type, std::list<named_pexpr_t>*attr);
 
@@ -409,7 +380,7 @@ extern void pform_set_class_type(const struct vlltype&li, class_type_t*class_typ
      passed in. */
 extern void pform_set_attrib(perm_string name, perm_string key,
 			     char*value);
-extern void pform_set_type_attrib(perm_string name, const string&key,
+extern void pform_set_type_attrib(perm_string name, const std::string&key,
 				  char*value);
 
 extern LexicalScope::range_t* pform_parameter_value_range(bool exclude_flag,
@@ -418,41 +389,37 @@ extern LexicalScope::range_t* pform_parameter_value_range(bool exclude_flag,
 
 extern void pform_set_parameter(const struct vlltype&loc,
 				perm_string name,
-				ivl_variable_type_t type,
-				bool signed_flag,
-				list<pform_range_t>*range,
+				bool is_local, bool is_type,
+				data_type_t*data_type,
 				PExpr*expr, LexicalScope::range_t*value_range);
-extern void pform_set_localparam(const struct vlltype&loc,
-				 perm_string name,
-				 ivl_variable_type_t type,
-				 bool signed_flag,
-				 list<pform_range_t>*range,
-				 PExpr*expr);
 extern void pform_set_specparam(const struct vlltype&loc,
 				 perm_string name,
-				 list<pform_range_t>*range,
+				 std::list<pform_range_t>*range,
 				 PExpr*expr);
 extern void pform_set_defparam(const pform_name_t&name, PExpr*expr);
 
-extern void pform_set_param_from_type(const struct vlltype&loc,
-                                      const data_type_t *data_type,
-                                      const char *name,
-                                      list<pform_range_t> *&param_range,
-                                      bool &param_signed,
-                                      ivl_variable_type_t &param_type);
+extern void pform_make_let(const struct vlltype&loc,
+                           perm_string name,
+                           std::list<PLet::let_port_t*>*ports,
+                           PExpr*expr);
+
+extern PLet::let_port_t* pform_make_let_port(data_type_t*data_type,
+                                             perm_string name,
+                                             std::list<pform_range_t>*range,
+                                             PExpr*def);
 
 /*
  * Functions related to specify blocks.
  */
 extern PSpecPath*pform_make_specify_path(const struct vlltype&li,
-					 list<perm_string>*src, char pol,
-					 bool full_flag, list<perm_string>*dst);
+					 std::list<perm_string>*src, char pol,
+					 bool full_flag, std::list<perm_string>*dst);
 extern PSpecPath*pform_make_specify_edge_path(const struct vlltype&li,
 					 int edge_flag, /*posedge==true */
-					 list<perm_string>*src, char pol,
-					 bool full_flag, list<perm_string>*dst,
+					 std::list<perm_string>*src, char pol,
+					 bool full_flag, std::list<perm_string>*dst,
 					 PExpr*data_source_expression);
-extern PSpecPath*pform_assign_path_delay(PSpecPath*obj, list<PExpr*>*delays);
+extern PSpecPath*pform_assign_path_delay(PSpecPath*obj, std::list<PExpr*>*delays);
 
 extern void pform_module_specify_path(PSpecPath*obj);
 
@@ -461,19 +428,13 @@ extern void pform_module_specify_path(PSpecPath*obj);
  * or initial items.
  */
 extern PProcess*  pform_make_behavior(ivl_process_type_t, Statement*,
-				      list<named_pexpr_t>*attr);
+				      std::list<named_pexpr_t>*attr);
 extern void pform_mc_translate_on(bool flag);
 
-extern std::vector<PWire*>* pform_make_udp_input_ports(list<perm_string>*);
-
-extern void pform_make_events(list<perm_string>*names,
-			      const char*file, unsigned lineno);
-/*
- * Make real datum objects.
- */
-extern void pform_make_reals(list<perm_string>*names,
-			     const char*file, unsigned lineno);
+extern std::vector<PWire*>* pform_make_udp_input_ports(std::list<perm_string>*);
 
+extern void pform_make_events(const struct vlltype&loc,
+			      std::list<perm_string>*names);
 /*
  * The makegate function creates a new gate (which need not have a
  * name) and connects it to the specified wires.
@@ -481,36 +442,27 @@ extern void pform_make_reals(list<perm_string>*names,
 extern void pform_makegates(const struct vlltype&loc,
 			    PGBuiltin::Type type,
 			    struct str_pair_t str,
-			    list<PExpr*>*delay,
-			    svector<lgate>*gates,
-			    list<named_pexpr_t>*attr);
+			    std::list<PExpr*>*delay,
+			    std::vector<lgate>*gates,
+			    std::list<named_pexpr_t>*attr);
 
 extern void pform_make_modgates(const struct vlltype&loc,
 				perm_string type,
 				struct parmvalue_t*overrides,
-				svector<lgate>*gates,
-				list<named_pexpr_t>*attr);
+				std::vector<lgate>*gates,
+				std::list<named_pexpr_t>*attr);
 
 /* Make a continuous assignment node, with optional bit- or part- select. */
-extern void pform_make_pgassign_list(list<PExpr*>*alist,
-				     list<PExpr*>*del,
-				     struct str_pair_t str,
-				     const char* fn, unsigned lineno);
-
-/* Given a port type and a list of names, make a list of wires that
-   can be used as task port information. */
-extern std::vector<pform_tf_port_t>*pform_make_task_ports(const struct vlltype&loc,
-					     NetNet::PortType pt,
-					     ivl_variable_type_t vtype,
-					     bool signed_flag,
-					     list<pform_range_t>*range,
-					     list<perm_string>*names,
-					     bool isint = false);
+extern void pform_make_pgassign_list(const struct vlltype&loc,
+				     std::list<PExpr*>*alist,
+				     std::list<PExpr*>*del,
+				     struct str_pair_t str);
 
 extern std::vector<pform_tf_port_t>*pform_make_task_ports(const struct vlltype&loc,
 					     NetNet::PortType pt,
 					     data_type_t*vtype,
-					     list<perm_string>*names);
+					     std::list<pform_port_t>*ports,
+					     bool allow_implicit = false);
 
 /*
  * The parser uses this function to convert a unary
@@ -527,13 +479,16 @@ extern PAssign* pform_compressed_assign_from_inc_dec(const struct vlltype&loc,
 extern PExpr* pform_genvar_inc_dec(const struct vlltype&loc, const char*name,
                                    bool inc_flag);
 
+extern PExpr* pform_genvar_compressed(const struct vlltype &loc,
+				      const char *name, char op, PExpr *rval);
+
 /*
  * These are functions that the outside-the-parser code uses the do
  * interesting things to the Verilog. The parse function reads and
  * parses the source file and places all the modules it finds into the
  * mod list. The dump function dumps a module to the output stream.
  */
-extern void pform_dump(ostream&out, Module*mod);
+extern void pform_dump(std::ostream&out, Module*mod);
 
 /* ** pform_discipline.cc
  * Functions for handling the parse of natures and disciplines. These
@@ -553,10 +508,10 @@ extern void pform_discipline_potential(const struct vlltype&loc, const char*name
 extern void pform_discipline_flow(const struct vlltype&loc, const char*name);
 
 extern void pform_attach_discipline(const struct vlltype&loc,
-				    ivl_discipline_t discipline, list<perm_string>*names);
+				    ivl_discipline_t discipline, std::list<perm_string>*names);
 
-extern void pform_dump(ostream&out, const ivl_nature_s*);
-extern void pform_dump(ostream&out, const ivl_discipline_s*);
+extern void pform_dump(std::ostream&out, const ivl_nature_s*);
+extern void pform_dump(std::ostream&out, const ivl_discipline_s*);
 
 /* ** pform_analog.cc
 */
@@ -577,13 +532,14 @@ extern PExpr* pform_make_branch_probe_expression(const struct vlltype&loc,
  * is the hierarchical name of a valid parameter name and value
  * is the value user wants to assign to. The value should be constant.
  */
-extern void parm_to_defparam_list(const string&param);
+extern void parm_to_defparam_list(const std::string&param);
 
 /*
  * Tasks to set the timeunit or timeprecision for SystemVerilog.
  */
 extern bool get_time_unit(const char*cp, int &unit);
 extern int  pform_get_timeunit();
+extern int  pform_get_timeprec();
 extern void pform_set_timeunit(const char*txt, bool initial_decl);
 extern void pform_set_timeprec(const char*txt, bool initial_decl);
 /*
@@ -592,4 +548,14 @@ extern void pform_set_timeprec(const char*txt, bool initial_decl);
 extern bool allow_timeunit_decl;
 extern bool allow_timeprec_decl;
 
+void pform_put_enum_type_in_scope(enum_type_t*enum_set);
+
+bool pform_requires_sv(const struct vlltype&loc, const char *feature);
+
+void pform_start_parameter_port_list();
+void pform_end_parameter_port_list();
+
+void pform_check_net_data_type(const struct vlltype&loc, NetNet::Type net_type,
+			       const data_type_t *data_type);
+
 #endif /* IVL_pform_H */
