@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1998-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -38,6 +38,8 @@
 # include  <iomanip>
 # include  <typeinfo>
 
+using namespace std;
+
 ostream& operator << (ostream&out, const PExpr&obj)
 {
       obj.dump(out);
@@ -85,7 +87,12 @@ ostream& operator<< (ostream&out, const index_component_t&that)
 
 ostream& operator<< (ostream&out, const name_component_t&that)
 {
-      out << that.name.str();
+      if (that.name == THIS_TOKEN)
+	    out << "this";
+      else if (that.name == SUPER_TOKEN)
+	    out << "super";
+      else
+	    out << that.name.str();
 
       typedef std::list<index_component_t>::const_iterator index_it_t;
       for (index_it_t idx = that.index.begin()
@@ -99,7 +106,7 @@ ostream& operator<< (ostream&out, const name_component_t&that)
 ostream& operator<< (ostream&o, const pform_name_t&that)
 {
       pform_name_t::const_iterator cur;
-      if (that.size() == 0) {
+      if (that.empty()) {
 	    o << "<nil>";
 	    return o;
       }
@@ -165,6 +172,54 @@ void data_type_t::pform_dump(ostream&out, unsigned indent) const
       out << setw(indent) << "" << typeid(*this).name() << endl;
 }
 
+ostream& data_type_t::debug_dump(ostream&out) const
+{
+      out << typeid(*this).name();
+      return out;
+}
+
+std::ostream& typeref_t::debug_dump(ostream&out) const
+{
+      if (scope)
+	    out << scope->pscope_name() << "::";
+      out << type->name;
+
+      return out;
+}
+
+ostream& atom_type_t::debug_dump(ostream&out) const
+{
+      if (signed_flag)
+	    out << "signed ";
+      else
+	    out << "unsigned ";
+
+      switch (type_code) {
+      case INTEGER:
+	    out << "integer";
+	    break;
+      case TIME:
+	    out << "time";
+	    break;
+      case LONGINT:
+	    out << "longint";
+	    break;
+      case INT:
+	    out << "int";
+	    break;
+      case SHORTINT:
+	    out << "shortint";
+	    break;
+      case BYTE:
+	    out << "byte";
+	    break;
+      default:
+	    assert(0);
+	    break;
+      }
+      return out;
+}
+
 void void_type_t::pform_dump(ostream&out, unsigned indent) const
 {
       out << setw(indent) << "" << "void" << endl;
@@ -177,6 +232,19 @@ void parray_type_t::pform_dump(ostream&out, unsigned indent) const
       base_type->pform_dump(out, indent+4);
 }
 
+ostream& real_type_t::debug_dump(ostream&out) const
+{
+      switch (type_code_) {
+	  case REAL:
+	    out << "real";
+	    break;
+	  case SHORTREAL:
+	    out << "shortreal";
+	    break;
+      }
+      return out;
+}
+
 void struct_type_t::pform_dump(ostream&out, unsigned indent) const
 {
       out << setw(indent) << "" << "Struct " << (packed_flag?"packed":"unpacked")
@@ -213,6 +281,25 @@ void vector_type_t::pform_dump(ostream&fd, unsigned indent) const
       fd << endl;
 }
 
+ostream& vector_type_t::debug_dump(ostream&fd) const
+{
+      if (signed_flag)
+	    fd << "signed ";
+      if (!pdims.get()) {
+	    fd << "/* vector_type_t nil */";
+	    return fd;
+      }
+
+      for (list<pform_range_t>::iterator cur = pdims->begin()
+		 ; cur != pdims->end() ; ++cur) {
+	    fd << "[";
+	    if (cur->first)  fd << *(cur->first);
+	    if (cur->second) fd << ":" << *(cur->second);
+	    fd << "]";
+      }
+      return fd;
+}
+
 void class_type_t::pform_dump(ostream&out, unsigned indent) const
 {
       out << setw(indent) << "" << "class " << name;
@@ -347,6 +434,15 @@ void PECastType::dump(ostream &out) const
       out << ")";
 }
 
+void PECastSign::dump(ostream &out) const
+{
+      if (!signed_flag_)
+	    out << "un";
+      out << "signed'(";
+      base_->dump(out);
+      out << ")";
+}
+
 void PEEvent::dump(ostream&out) const
 {
       switch (type_) {
@@ -358,6 +454,9 @@ void PEEvent::dump(ostream&out) const
 	  case PEEvent::NEGEDGE:
 	    out << "negedge ";
 	    break;
+	  case PEEvent::EDGE:
+	    out << "edge ";
+	    break;
 	  case PEEvent::POSITIVE:
 	    out << "positive ";
 	    break;
@@ -526,19 +625,14 @@ void PWire::dump(ostream&out, unsigned ind) const
 	    break;
       }
 
-      out << " " << data_type_;
-
       if (signed_) {
 	    out << " signed";
       }
-      if (get_isint()) {
-	    out << " integer";
-      }
-      if (get_scalar()) {
+      if (is_scalar_) {
 	    out << " scalar";
       }
       if (set_data_type_) {
-	    out << " set_data_type_=" << typeid(*set_data_type_).name();
+	    out << " set_data_type_=" << *set_data_type_;
       }
 
       if (discipline_) {
@@ -624,6 +718,17 @@ void PGate::dump_delays(ostream&out) const
       delay_.dump_delays(out);
 }
 
+void PGate::dump_ranges(ostream&out) const
+{
+      for (list<pform_range_t>::iterator cur = ranges_->begin()
+		 ; cur != ranges_->end() ; ++cur) {
+	    out << "[";
+	    if (cur->first)  out << *(cur->first);
+	    if (cur->second) out << ":" << *(cur->second);
+	    out << "]";
+      }
+}
+
 void PGate::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << typeid(*this).name() << " ";
@@ -686,11 +791,7 @@ void PGBuiltin::dump(ostream&out, unsigned ind) const
       out << "(" << strength0() << "0 " << strength1() << "1) ";
       dump_delays(out);
       out << " " << get_name();
-
-      if (msb_) {
-	    out << " [" << *msb_ << ":" << *lsb_ << "]";
-      }
-
+      dump_ranges(out);
       out << "(";
       dump_pins(out);
       out << ");" << endl;
@@ -731,16 +832,7 @@ void PGModule::dump(ostream&out, unsigned ind) const
       }
 
       out << get_name();
-
-	// If the module is arrayed, print the index expressions.
-      if (msb_ || lsb_) {
-	    out << "[";
-	    if (msb_) out << *msb_;
-	    out << ":";
-	    if (lsb_) out << *lsb_;
-	    out << "]";
-      }
-
+      dump_ranges(out);
       out << "(";
       if (pins_) {
 	    out << "." << pins_[0].name << "(";
@@ -812,8 +904,6 @@ void PBlock::dump(ostream&out, unsigned ind) const
       if (pscope_name() != 0) {
             dump_parameters_(out, ind+2);
 
-            dump_localparams_(out, ind+2);
-
             dump_events_(out, ind+2);
 
 	    dump_wires_(out, ind+2);
@@ -881,18 +971,21 @@ void PCase::dump(ostream&out, unsigned ind) const
       out << " (" << *expr_ << ") /* " << get_fileline() << " */" << endl;
       dump_attributes_map(out, attributes, ind+2);
 
-      for (unsigned idx = 0 ;  idx < items_->count() ;  idx += 1) {
+      for (unsigned idx = 0 ;  idx < items_->size() ;  idx += 1) {
 	    PCase::Item*cur = (*items_)[idx];
 
-	    if (! cur->expr.empty()) {
+	    if (cur->expr.empty()) {
 		  out << setw(ind+2) << "" << "default:";
 
 	    } else {
 		  list<PExpr*>::iterator idx_exp = cur->expr.begin();
-		  out << setw(ind+2) << "" << *idx_exp;
+		  out << setw(ind+2) << "";
+		  (*idx_exp)->dump(out);
 
-		  for( ; idx_exp != cur->expr.end() ; ++idx_exp)
-			out << ", " << *idx_exp;
+		  for (++idx_exp ; idx_exp != cur->expr.end() ; ++idx_exp) {
+			out << ", ";
+			(*idx_exp)->dump(out);
+		  }
 
 		  out << ":";
 	    }
@@ -978,16 +1071,16 @@ void PDoWhile::dump(ostream&out, unsigned ind) const
 
 void PEventStatement::dump(ostream&out, unsigned ind) const
 {
-      if (expr_.count() == 0) {
+      if (expr_.size() == 0) {
 	    out << setw(ind) << "" << "@* ";
 
-      } else if ((expr_.count() == 1) && (expr_[0] == 0)) {
+      } else if ((expr_.size() == 1) && (expr_[0] == 0)) {
 	    out << setw(ind) << "" << "wait fork ";
 
       } else {
 	    out << setw(ind) << "" << "@(" << *(expr_[0]);
-	    if (expr_.count() > 1)
-		  for (unsigned idx = 1 ;  idx < expr_.count() ;  idx += 1)
+	    if (expr_.size() > 1)
+		  for (unsigned idx = 1 ;  idx < expr_.size() ;  idx += 1)
 			out << " or " << *(expr_[idx]);
 
 	    out << ")";
@@ -1005,13 +1098,13 @@ void PEventStatement::dump_inline(ostream&out) const
 {
       assert(statement_ == 0);
 
-      if (expr_.count() == 0) {
+      if (expr_.size() == 0) {
 	    out << "@* ";
 
       } else {
 	    out << "@(" << *(expr_[0]);
-	    if (expr_.count() > 1)
-		  for (unsigned idx = 1 ;  idx < expr_.count() ;  idx += 1)
+	    if (expr_.size() > 1)
+		  for (unsigned idx = 1 ;  idx < expr_.size() ;  idx += 1)
 			out << " or " << *(expr_[idx]);
 
 	    out << ")";
@@ -1052,9 +1145,26 @@ void PForever::dump(ostream&out, unsigned ind) const
 
 void PForStatement::dump(ostream&out, unsigned ind) const
 {
-      out << setw(ind) << "" << "for (" << *name1_ << " = " << *expr1_
-	  << "; " << *cond_ << "; <for_step>)" << endl;
-      step_->dump(out, ind+6);
+      out << setw(ind) << "" << "for (";
+      if (name1_)
+	    out << *name1_;
+      else
+	    out << "<no-name1>";
+      out << " = ";
+      if (expr1_)
+	    out << *expr1_;
+      else
+	    out << "<no-expr1>";
+      out << "; ";
+      if (cond_)
+	    out << *cond_;
+      else
+	    out << "<no-cond>";
+      out << "; <for_step>)" << endl;
+      if (step_)
+	    step_->dump(out, ind+6);
+      else
+	    out << setw(ind+6) << "" << "<no for_step statement>" << endl;
       if (statement_)
 	    statement_->dump(out, ind+3);
       else
@@ -1079,8 +1189,6 @@ void PFunction::dump(ostream&out, unsigned ind) const
 
       dump_parameters_(out, ind+2);
 
-      dump_localparams_(out, ind+2);
-
       dump_events_(out, ind+2);
 
       dump_wires_(out, ind+2);
@@ -1093,6 +1201,33 @@ void PFunction::dump(ostream&out, unsigned ind) const
 	    out << setw(ind+2) << "" << "/* NOOP */" << endl;
 }
 
+void PLet::let_port_t::dump(ostream&out, unsigned) const
+{
+      if (type_) out << *type_ << " ";
+      out << name_;
+	// FIXME: This has not been tested and is likely wrong!
+      if (range_) out << " " << range_;
+      if (def_) out << "=" << *def_;
+}
+
+void PLet::dump(ostream&out, unsigned ind) const
+{
+      out << setw(ind) << "" << "let ";
+      out << pscope_name();
+      if (ports_) {
+	    out << "(";
+	    typedef std::list<let_port_t*>::const_iterator port_itr_t;
+	    port_itr_t idx = ports_->begin();
+	    (*idx)->dump(out, 0);
+	    for (++idx; idx != ports_->end(); ++idx ) {
+		  out << ", ";
+		  (*idx)->dump(out, 0);
+	    }
+	    out << ")";
+      }
+      out << " = " << *expr_ << ";" << endl;
+}
+
 void PRelease::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "release " << *lval_ << "; /* "
@@ -1126,8 +1261,6 @@ void PTask::dump(ostream&out, unsigned ind) const
 
       dump_parameters_(out, ind+2);
 
-      dump_localparams_(out, ind+2);
-
       dump_events_(out, ind+2);
 
       dump_wires_(out, ind+2);
@@ -1185,6 +1318,13 @@ void PTrigger::dump(ostream&out, unsigned ind) const
       out << setw(ind) << "" << "-> " << event_ << ";" << endl;
 }
 
+void PNBTrigger::dump(ostream&out, unsigned ind) const
+{
+      out << setw(ind) << "" << "->> ";
+      if (dly_) out << "#" << *dly_ << " ";
+      out << event_ << ";" << endl;
+}
+
 void PWhile::dump(ostream&out, unsigned ind) const
 {
       out << setw(ind) << "" << "while (" << *cond_ << ")" << endl;
@@ -1324,7 +1464,7 @@ void PGenerate::dump(ostream&out, unsigned indent) const
 
       out << endl;
 
-      dump_localparams_(out, indent+2);
+      dump_parameters_(out, indent+2);
 
       typedef list<PGenerate::named_expr_t>::const_iterator parm_hiter_t;
       for (parm_hiter_t cur = defparms.begin()
@@ -1357,6 +1497,11 @@ void PGenerate::dump(ostream&out, unsigned indent) const
 	    (*idx)->dump(out, indent+2);
       }
 
+      for (list<PCallTask*>::const_iterator idx = elab_tasks.begin()
+		 ; idx != elab_tasks.end() ; ++ idx ) {
+	    (*idx)->dump(out, indent+2);
+      }
+
       typedef map<perm_string,LineInfo*>::const_iterator genvar_iter_t;
       for (genvar_iter_t cur = genvars.begin()
 		 ; cur != genvars.end() ; ++ cur ) {
@@ -1377,10 +1522,10 @@ void PGenerate::dump(ostream&out, unsigned indent) const
 
 void LexicalScope::dump_typedefs_(ostream&out, unsigned indent) const
 {
-      typedef map<perm_string,data_type_t*>::const_iterator iter_t;
+      typedef typedef_map_t::const_iterator iter_t;
       for (iter_t cur = typedefs.begin() ; cur != typedefs.end() ; ++ cur) {
 	    out << setw(indent) << "" << "typedef of " << cur->first << ":" << endl;
-	    cur->second->pform_dump(out, indent+4);
+	    cur->second->get_data_type()->pform_dump(out, indent+4);
       }
 }
 
@@ -1389,18 +1534,18 @@ void LexicalScope::dump_parameters_(ostream&out, unsigned indent) const
       typedef map<perm_string,param_expr_t*>::const_iterator parm_iter_t;
       for (parm_iter_t cur = parameters.begin()
 		 ; cur != parameters.end() ; ++ cur ) {
-	    out << setw(indent) << "" << "parameter "
-                << (*cur).second->type << " ";
-	    if ((*cur).second->signed_flag)
-		  out << "signed ";
-	    if ((*cur).second->msb)
-		  out << "[" << *(*cur).second->msb << ":"
-		      << *(*cur).second->lsb << "] ";
-	    out << (*cur).first << " = ";
-	    if ((*cur).second->expr)
-		  out << *(*cur).second->expr;
+	    out << setw(indent) << "";
+	    if (cur->second->local_flag)
+		  out << "localparam ";
 	    else
-		  out << "/* ERROR */";
+		  out << "parameter ";
+	    if (cur->second->data_type)
+	          cur->second->data_type->debug_dump(out);
+	    else
+		  out << "(nil type)";
+	    if ((*cur).second->expr)
+		  out << " " << (*cur).first << " = "
+		      << *(*cur).second->expr;
 	    for (LexicalScope::range_t*tmp = (*cur).second->range
 		       ; tmp ; tmp = tmp->next) {
 		  if (tmp->exclude_flag)
@@ -1433,26 +1578,9 @@ void LexicalScope::dump_parameters_(ostream&out, unsigned indent) const
       }
 }
 
-void LexicalScope::dump_localparams_(ostream&out, unsigned indent) const
-{
-      typedef map<perm_string,param_expr_t*>::const_iterator parm_iter_t;
-      for (parm_iter_t cur = localparams.begin()
-		 ; cur != localparams.end() ; ++ cur ) {
-	    out << setw(indent) << "" << "localparam ";
-	    if ((*cur).second->msb)
-		  out << "[" << *(*cur).second->msb << ":"
-		      << *(*cur).second->lsb << "] ";
-	    out << (*cur).first << " = ";
-	    if ((*cur).second->expr)
-		  out << *(*cur).second->expr << ";" << endl;
-	    else
-		  out << "/* ERROR */;" << endl;
-      }
-}
-
 void LexicalScope::dump_enumerations_(ostream&out, unsigned indent) const
 {
-      for (set<enum_type_t*>::const_iterator cur = enum_sets.begin()
+      for (vector<enum_type_t*>::const_iterator cur = enum_sets.begin()
 		 ; cur != enum_sets.end() ; ++ cur) {
 	    out << setw(indent) << "" << "enum {" << endl;
 
@@ -1544,9 +1672,11 @@ void Module::dump_specparams_(ostream&out, unsigned indent) const
       for (parm_iter_t cur = specparams.begin()
 		 ; cur != specparams.end() ; ++ cur ) {
 	    out << setw(indent) << "" << "specparam ";
-	    if ((*cur).second->msb)
-		  out << "[" << *(*cur).second->msb << ":"
-		      << *(*cur).second->lsb << "] ";
+	    if (cur->second->data_type)
+		  cur->second->data_type->debug_dump(out);
+	    else
+		  out << "(nil type)";
+
 	    out << (*cur).first << " = ";
 	    if ((*cur).second->expr)
 		  out << *(*cur).second->expr << ";" << endl;
@@ -1601,8 +1731,6 @@ void Module::dump(ostream&out) const
 
       dump_parameters_(out, 4);
 
-      dump_localparams_(out, 4);
-
       dump_specparams_(out, 4);
 
       dump_enumerations_(out, 4);
@@ -1663,6 +1791,11 @@ void Module::dump(ostream&out) const
 	    (*idx)->dump(out, 4);
       }
 
+      for (list<PCallTask*>::const_iterator idx = elab_tasks.begin()
+		 ; idx != elab_tasks.end() ; ++ idx ) {
+	    (*idx)->dump(out, 4);
+      }
+
       for (list<PSpecPath*>::const_iterator spec = specify_paths.begin()
 		 ; spec != specify_paths.end() ; ++ spec ) {
 
@@ -1680,7 +1813,7 @@ void pform_dump(ostream&out, Module*mod)
 void PUdp::dump(ostream&out) const
 {
       out << "primitive " << name_ << "(" << ports[0];
-      for (unsigned idx = 1 ;  idx < ports.count() ;  idx += 1)
+      for (unsigned idx = 1 ;  idx < ports.size() ;  idx += 1)
 	    out << ", " << ports[idx];
       out << ");" << endl;
 
@@ -1688,7 +1821,7 @@ void PUdp::dump(ostream&out) const
 	    out << "    reg " << ports[0] << ";" << endl;
 
       out << "    table" << endl;
-      for (unsigned idx = 0 ;  idx < tinput.count() ;  idx += 1) {
+      for (unsigned idx = 0 ;  idx < tinput.size() ;  idx += 1) {
 	    out << "     ";
 	    for (unsigned chr = 0 ;  chr < tinput[idx].length() ;  chr += 1)
 		  out << " " << tinput[idx][chr];
@@ -1748,7 +1881,6 @@ void pform_dump(std::ostream&out, const PPackage*pac)
 void PPackage::pform_dump(std::ostream&out) const
 {
       out << "package " << pscope_name() << endl;
-      dump_localparams_(out, 4);
       dump_parameters_(out, 4);
       dump_typedefs_(out, 4);
       dump_enumerations_(out, 4);
