@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2012-2021 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2013 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
@@ -18,6 +18,7 @@
  *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+# include  <cstdarg>
 # include  "pform.h"
 # include  "PPackage.h"
 # include  "parse_misc.h"
@@ -29,9 +30,14 @@
 using namespace std;
 
 /*
- * This is a map of packages that have been defined.
+ * This is a list of packages in the order that they were defined.
  */
-map<perm_string,PPackage*> pform_packages;
+vector<PPackage*> pform_packages;
+
+/*
+ * This allows us to easily check for name collisions.
+ */
+static map<perm_string,PPackage*> packages_by_name;
 
 static PPackage*pform_cur_package = 0;
 
@@ -51,18 +57,17 @@ void pform_end_package_declaration(const struct vlltype&loc)
       ivl_assert(loc, pform_cur_package);
       perm_string use_name = pform_cur_package->pscope_name();
 
-      map<perm_string,PPackage*>::const_iterator test = pform_packages.find(use_name);
-      if (test != pform_packages.end()) {
+      map<perm_string,PPackage*>::const_iterator test = packages_by_name.find(use_name);
+      if (test != packages_by_name.end()) {
 	    ostringstream msg;
-	    msg << "Package " << use_name << " was already declared here: "
+	    msg << "error: Package " << use_name << " was already declared here: "
 		<< test->second->get_fileline() << ends;
-	    VLerror(msg.str().c_str());
-      } else {
-	    pform_packages[use_name] = pform_cur_package;
+	    VLerror(loc, msg.str().c_str());
       }
 
 
-      pform_packages[use_name] = pform_cur_package;
+      packages_by_name[use_name] = pform_cur_package;
+      pform_packages.push_back(pform_cur_package);
       pform_cur_package = 0;
       pform_pop_scope();
 }
@@ -120,10 +125,12 @@ void pform_package_import(const struct vlltype&loc, PPackage*pkg, const char*ide
 	    scope->explicit_imports[use_ident] = pkg;
 
       } else {
-	    set<PPackage*>::const_iterator cur_pkg
-		  = scope->potential_imports.find(pkg);
+	    list<PPackage*>::const_iterator cur_pkg
+		  = find(scope->potential_imports.begin(),
+                         scope->potential_imports.end(),
+                         pkg);
 	    if (cur_pkg == scope->potential_imports.end())
-		  scope->potential_imports.insert(pkg);
+		  scope->potential_imports.push_back(pkg);
       }
 }
 
@@ -136,10 +143,12 @@ PExpr* pform_package_ident(const struct vlltype&loc,
       return tmp;
 }
 
-data_type_t* pform_test_type_identifier(PPackage*pkg, const char*txt)
+typedef_t* pform_test_type_identifier(PPackage*pkg, const char*txt)
 {
       perm_string use_name = lex_strings.make(txt);
-      map<perm_string,data_type_t*>::const_iterator cur = pkg->typedefs.find(use_name);
+      LexicalScope::typedef_map_t::const_iterator cur;
+
+      cur = pkg->typedefs.find(use_name);
       if (cur != pkg->typedefs.end())
 	    return cur->second;
 
@@ -154,8 +163,8 @@ data_type_t* pform_test_type_identifier(PPackage*pkg, const char*txt)
 PPackage* pform_test_package_identifier(const char*pkg_name)
 {
       perm_string use_name = lex_strings.make(pkg_name);
-      map<perm_string,PPackage*>::const_iterator pcur = pform_packages.find(use_name);
-      if (pcur == pform_packages.end())
+      map<perm_string,PPackage*>::const_iterator pcur = packages_by_name.find(use_name);
+      if (pcur == packages_by_name.end())
 	    return 0;
 
       assert(pcur->second);
