@@ -1,7 +1,7 @@
 #ifndef IVL_pform_types_H
 #define IVL_pform_types_H
 /*
- * Copyright (c) 2007-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2007-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -33,10 +33,6 @@
 # include  <map>
 # include  <memory>
 
-#if __cplusplus < 201103L
-#define unique_ptr auto_ptr
-#endif
-
 /*
  * parse-form types.
  */
@@ -45,9 +41,9 @@ class Design;
 class NetScope;
 class Definitions;
 class PExpr;
+class PScope;
 class PWire;
 class Statement;
-class ivl_type_s;
 class netclass_t;
 class netenum_t;
 typedef named<verinum> named_number_t;
@@ -76,18 +72,29 @@ typedef named<PExpr*> named_pexpr_t;
  */
 typedef std::pair<PExpr*,PExpr*> pform_range_t;
 
+/* The lgate is gate instantiation information. */
+struct lgate : public LineInfo {
+      explicit lgate() : parms(0), parms_by_name(0), ranges(0) { }
+
+      std::string name;
+      std::list<PExpr*>*parms;
+      std::list<named_pexpr_t>*parms_by_name;
+
+      std::list<pform_range_t>*ranges;
+};
+
 /*
  * The pform_port_t holds the name and optional unpacked dimensions
  * and initialization expression for a single port in a list of port
  * declarations.
  */
 struct pform_port_t {
-      pform_port_t(perm_string n, list<pform_range_t>*ud, PExpr*e)
+      pform_port_t(perm_string n, std::list<pform_range_t>*ud, PExpr*e)
 	: name(n), udims(ud), expr(e) { }
       ~pform_port_t() { }
 
       perm_string name;
-      list<pform_range_t>*udims;
+      std::list<pform_range_t>*udims;
       PExpr*expr;
 };
 
@@ -111,9 +118,13 @@ struct index_component_t {
 };
 
 struct name_component_t {
-      explicit name_component_t(perm_string n) : name(n) { }
+      inline name_component_t() { }
+      inline explicit name_component_t(perm_string n) : name(n) { }
       ~name_component_t() { }
 
+      // Return true if this component is nil.
+      inline bool empty() const { return name.nil(); }
+
       perm_string name;
       std::list<index_component_t>index;
 };
@@ -141,24 +152,71 @@ class data_type_t : public PNamedItem {
     public:
       inline explicit data_type_t() { }
       virtual ~data_type_t() = 0;
-	// This method is used to figure out the base type of a packed
-	// compound object. Return IVL_VT_NO_TYPE if the type is not packed.
-      virtual ivl_variable_type_t figure_packed_base_type(void)const;
-	// This method is used by the pform dumper to diagnostic dump.
+      // This method is used by the pform dumper to diagnostic dump. The
+      //  pform_dump dumps type type in pform format, and the debug_dump
+      // prints the output in a linear form.
       virtual void pform_dump(std::ostream&out, unsigned indent) const;
+      virtual std::ostream& debug_dump(std::ostream&out) const;
 
-      ivl_type_s* elaborate_type(Design*des, NetScope*scope);
+      ivl_type_t elaborate_type(Design*des, NetScope*scope);
 
       virtual SymbolType symbol_type() const;
 
-      perm_string name;
-
     private:
 	// Elaborate the type to an ivl_type_s type.
-      virtual ivl_type_s* elaborate_type_raw(Design*des, NetScope*scope) const;
+      virtual ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
+      virtual NetScope *find_scope(Design* des, NetScope *scope) const;
+
+      bool elaborating = false;
 
 	// Keep per-scope elaboration results cached.
-      std::map<Definitions*,ivl_type_s*> cache_type_elaborate_;
+      std::map<Definitions*,ivl_type_t> cache_type_elaborate_;
+};
+
+struct typedef_t : public PNamedItem {
+      explicit typedef_t(perm_string n) : basic_type(ANY), name(n) { };
+
+      ivl_type_t elaborate_type(Design*des, NetScope*scope);
+
+      enum basic_type {
+	    ANY,
+	    ENUM,
+	    STRUCT,
+	    UNION,
+	    CLASS
+      };
+
+      bool set_data_type(data_type_t *t);
+      const data_type_t *get_data_type() const { return data_type.get(); }
+
+      bool set_basic_type(basic_type bt);
+      enum basic_type get_basic_type() const { return basic_type; }
+
+protected:
+      enum basic_type basic_type;
+      std::unique_ptr<data_type_t> data_type;
+public:
+      perm_string name;
+};
+
+struct typeref_t : public data_type_t {
+      explicit typeref_t(typedef_t *t, PScope *s = 0) : scope(s), type(t) {}
+
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
+      NetScope *find_scope(Design* des, NetScope *scope) const;
+
+      std::ostream& debug_dump(std::ostream&out) const;
+
+private:
+      PScope *scope;
+      typedef_t *type;
+};
+
+struct type_parameter_t : data_type_t {
+      explicit type_parameter_t(perm_string n) : name(n) { }
+      ivl_type_t elaborate_type_raw(Design *des, NetScope *scope) const;
+
+      perm_string name;
 };
 
 struct void_type_t : public data_type_t {
@@ -172,45 +230,55 @@ struct void_type_t : public data_type_t {
  * until it is elaborated in a scope.
  */
 struct enum_type_t : public data_type_t {
+      explicit enum_type_t(data_type_t *btype) : base_type(btype) { }
+
 	// Return the elaborated version of the type.
-      virtual ivl_type_s*elaborate_type_raw(Design*des, NetScope*scope) const;
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
 
       SymbolType symbol_type() const;
 
-      ivl_variable_type_t base_type;
-      bool signed_flag;
-      bool integer_flag; // True if "integer" was used
-      std::unique_ptr< list<pform_range_t> > range;
-      std::unique_ptr< list<named_pexpr_t> > names;
-      LineInfo li;
+      std::unique_ptr<data_type_t> base_type;
+      std::unique_ptr< std::list<named_pexpr_t> > names;
 };
 
 struct struct_member_t : public LineInfo {
       std::unique_ptr<data_type_t> type;
-      std::unique_ptr< list<decl_assignment_t*> > names;
+      std::unique_ptr< std::list<decl_assignment_t*> > names;
       void pform_dump(std::ostream&out, unsigned indent) const;
 };
 
 struct struct_type_t : public data_type_t {
-      virtual ivl_variable_type_t figure_packed_base_type(void)const;
       virtual void pform_dump(std::ostream&out, unsigned indent) const;
-      virtual netstruct_t* elaborate_type_raw(Design*des, NetScope*scope) const;
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
 
       bool packed_flag;
       bool union_flag;
-      std::unique_ptr< list<struct_member_t*> > members;
+      bool signed_flag;
+      std::unique_ptr< std::list<struct_member_t*> > members;
 };
 
-struct atom2_type_t : public data_type_t {
-      inline explicit atom2_type_t(int tc, bool flag)
-      : type_code(tc), signed_flag(flag) { }
-      int type_code;
+struct atom_type_t : public data_type_t {
+      enum type_code {
+	    INTEGER,
+	    TIME,
+	    BYTE,
+	    SHORTINT,
+	    INT,
+	    LONGINT
+      };
+
+      explicit atom_type_t(enum type_code tc, bool flag) : type_code(tc),
+							   signed_flag(flag) { }
+
+      enum type_code type_code;
       bool signed_flag;
 
-      ivl_type_s* elaborate_type_raw(Design*des, NetScope*scope) const;
+      virtual std::ostream& debug_dump(std::ostream&out) const;
+
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
 };
 
-extern atom2_type_t size_type;
+extern atom_type_t size_type;
 
 /*
  * The vector_type_t class represents types in the old Verilog
@@ -220,12 +288,7 @@ extern atom2_type_t size_type;
  *   bit unsigned foo
  *   reg foo
  *
- * There are a few special cases:
- *
- * For the most part, Verilog treats "logic" and "reg" as synonyms,
- * but there are a few cases where the parser needs to know the
- * difference. So "reg_flag" is set to true if the IVL_VT_LOGIC type
- * is due to the "reg" keyword.
+ * There is one special case:
  *
  * If there are no reg/logic/bit/bool keywords, then Verilog will
  * assume the type is logic, but the context may need to know about
@@ -234,17 +297,16 @@ extern atom2_type_t size_type;
 struct vector_type_t : public data_type_t {
       inline explicit vector_type_t(ivl_variable_type_t bt, bool sf,
 				    std::list<pform_range_t>*pd)
-      : base_type(bt), signed_flag(sf), reg_flag(false), integer_flag(false), implicit_flag(false), pdims(pd) { }
-      virtual ivl_variable_type_t figure_packed_base_type(void)const;
+      : base_type(bt), signed_flag(sf), integer_flag(false), implicit_flag(false), pdims(pd) { }
       virtual void pform_dump(std::ostream&out, unsigned indent) const;
-      ivl_type_s* elaborate_type_raw(Design*des, NetScope*scope) const;
+      virtual std::ostream& debug_dump(std::ostream&out) const;
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
 
       ivl_variable_type_t base_type;
       bool signed_flag;
-      bool reg_flag; // True if "reg" was used
       bool integer_flag; // True if "integer" was used
       bool implicit_flag; // True if this type is implicitly logic/reg
-      std::unique_ptr< list<pform_range_t> > pdims;
+      std::unique_ptr< std::list<pform_range_t> > pdims;
 };
 
 struct array_base_t : public data_type_t {
@@ -252,8 +314,8 @@ struct array_base_t : public data_type_t {
       inline explicit array_base_t(data_type_t*btype, std::list<pform_range_t>*pd)
       : base_type(btype), dims(pd) { }
 
-      data_type_t*base_type;
-      std::unique_ptr< list<pform_range_t> > dims;
+      std::unique_ptr<data_type_t> base_type;
+      std::unique_ptr< std::list<pform_range_t> > dims;
 };
 
 /*
@@ -266,9 +328,8 @@ struct parray_type_t : public array_base_t {
       inline explicit parray_type_t(data_type_t*btype, std::list<pform_range_t>*pd)
       : array_base_t(btype, pd) { }
 
-      virtual ivl_variable_type_t figure_packed_base_type(void)const;
       virtual void pform_dump(std::ostream&out, unsigned indent) const;
-      virtual ivl_type_s* elaborate_type_raw(Design*des, NetScope*scope) const;
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
 };
 
 /*
@@ -280,28 +341,33 @@ struct uarray_type_t : public array_base_t {
 
     public:
       virtual void pform_dump(std::ostream&out, unsigned indent) const;
-      virtual ivl_type_s* elaborate_type_raw(Design*des, NetScope*scope) const;
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
 };
 
 struct real_type_t : public data_type_t {
+ public:
       enum type_t { REAL, SHORTREAL };
-      inline explicit real_type_t(type_t tc) : type_code(tc) { }
-      type_t type_code;
+      inline explicit real_type_t(type_t tc) : type_code_(tc) { }
+      virtual std::ostream& debug_dump(std::ostream&out) const;
 
-      ivl_type_s* elaborate_type_raw(Design*des, NetScope*scope) const;
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
+
+      inline type_t type_code() const { return type_code_; }
+
+ private:
+      type_t type_code_;
 };
 
 struct string_type_t : public data_type_t {
       inline explicit string_type_t() { }
       ~string_type_t();
 
-      ivl_type_s* elaborate_type_raw(Design*des, NetScope*scope) const;
+      ivl_type_t elaborate_type_raw(Design*des, NetScope*scope) const;
 };
 
 struct class_type_t : public data_type_t {
 
-      inline explicit class_type_t(perm_string n)
-      : base_type(0), save_elaborated_type(0) { name = n; }
+      inline explicit class_type_t(perm_string n) : name(n) { }
 
       void pform_dump(std::ostream&out, unsigned indent) const;
       void pform_dump_init(std::ostream&out, unsigned indent) const;
@@ -310,15 +376,19 @@ struct class_type_t : public data_type_t {
 	// class that we are extending. This is nil if there is no
 	// hierarchy. If there are arguments to the base class, then
 	// put them in the base_args vector.
-      data_type_t*base_type;
+      std::unique_ptr<data_type_t> base_type;
       std::list<PExpr*>base_args;
 
+      bool virtual_class;
+
 	// This is a map of the properties. Map the name to the type.
-      struct prop_info_t {
-	    inline prop_info_t() : qual(property_qualifier_t::make_none()), type(0) { }
+      struct prop_info_t : public LineInfo {
+	    inline prop_info_t() : qual(property_qualifier_t::make_none()) { }
 	    inline prop_info_t(property_qualifier_t q, data_type_t*t) : qual(q), type(t) { }
+	    prop_info_t(prop_info_t&&) = default;
+	    prop_info_t& operator=(prop_info_t&&) = default;
 	    property_qualifier_t qual;
-	    data_type_t* type;
+	    std::unique_ptr<data_type_t> type;
       };
       std::map<perm_string, struct prop_info_t> properties;
 
@@ -332,16 +402,17 @@ struct class_type_t : public data_type_t {
 	// without waiting for any constructor.
       std::vector<Statement*> initialize_static;
 
-      ivl_type_s* elaborate_type_raw(Design*, NetScope*) const;
-	// The save_elaborated_type member must be set to the pointer
-	// to the netclass_t object that is created to represent this
-	// type. The elaborate_type_raw() method uses this pointer,
-	// and it is used in some other situations as well.
-      netclass_t* save_elaborated_type;
+      ivl_type_t elaborate_type_raw(Design*, NetScope*) const;
+
+      perm_string name;
 
       virtual SymbolType symbol_type() const;
 };
 
+ivl_type_t elaborate_array_type(Design *des, NetScope *scope,
+			        const LineInfo &li, ivl_type_t base_type,
+			        const std::list<pform_range_t> &dims);
+
 /*
  * The pform_name_t is the general form for a hierarchical
  * identifier. It is an ordered list of name components. Each name
@@ -381,12 +452,22 @@ inline perm_string peek_tail_name(const pform_name_t&that)
       return that.back().name;
 }
 
+/*
+ * In pform names, the "super" and "this" keywords are converted to
+ * These tokens so that they don't interfere with the namespace and
+ * are handled specially.
+ */
+# define SUPER_TOKEN "#"
+# define THIS_TOKEN  "@"
+
+static inline std::ostream& operator<< (std::ostream&out, const data_type_t&that)
+{
+      return that.debug_dump(out);
+}
+
 extern std::ostream& operator<< (std::ostream&out, const pform_name_t&);
 extern std::ostream& operator<< (std::ostream&out, const name_component_t&that);
 extern std::ostream& operator<< (std::ostream&out, const index_component_t&that);
-
-#if __cplusplus < 201103L
-#undef unique_ptr
-#endif
+extern std::ostream& operator<< (std::ostream&out, enum typedef_t::basic_type bt);
 
 #endif /* IVL_pform_types_H */
