@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2002-2022 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -1453,6 +1453,17 @@ bool NetForLoop::synth_async(Design*des, NetScope*scope,
 			     NexusSet&nex_map, NetBus&nex_out,
 			     NetBus&enables, vector<mask_t>&bitmasks)
 {
+      if (!index_) {
+	    cerr << get_fileline() << ": sorry: Unable to synthesize for-loop without explicit index variable." << endl;
+	    return false;
+      }
+
+      if (!step_statement_) {
+	    cerr << get_fileline() << ": sorry: Unable to synthesize for-loop without for_step statement." << endl;
+	    return false;
+      }
+
+      ivl_assert(*this, index_ && init_expr_);
       if (debug_synth2) {
 	    cerr << get_fileline() << ": NetForLoop::synth_async: "
 		 << "Index variable is " << index_->name() << endl;
@@ -1847,7 +1858,7 @@ bool NetCondit::synth_sync(Design*des, NetScope*scope,
 
 		  if (!all_bits_driven(tmp_masks[pin])) {
 			cerr << get_fileline() << ": sorry: Not all bits of '"
-			     << nex_map[idx].lnk.nexus()->pick_any_net()->name()
+			     << nex_map[pin].lnk.nexus()->pick_any_net()->name()
 			     << "' are asynchronously set or reset. This is "
 			     << "not currently supported in synthesis." << endl;
 			des->errors += 1;
@@ -1965,7 +1976,7 @@ bool NetCondit::synth_sync(Design*des, NetScope*scope,
 
 		  assert(else_ != 0);
 		  flag = else_->synth_sync(des, scope, ff, nex_map,
-					   nex_out, svector<NetEvProbe*>(0))
+					   nex_out, std::vector<NetEvProbe*>())
 			&& flag;
 		  DEBUG_SYNTH2_EXIT("NetCondit",flag)
 		  return flag;
