@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2021 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2013 / Stephen Williams (steve@icarus.com)
  * Copyright (c) 2016 CERN Michele Castellana (michele.castellana@cern.ch)
  *
@@ -32,6 +32,8 @@
 # include  <cstring>
 # include  "ivl_alloc.h"
 
+using namespace std;
+
 static StringHeap api_strings;
 
 /* THE FOLLOWING ARE FUNCTIONS THAT ARE CALLED FROM THE TARGET. */
@@ -366,6 +368,19 @@ extern "C" ivl_nexus_t ivl_event_any(ivl_event_t net, unsigned idx)
       return net->pins[idx];
 }
 
+extern "C" unsigned ivl_event_nedg(ivl_event_t net)
+{
+      assert(net);
+      return net->nedg;
+}
+
+extern "C" ivl_nexus_t ivl_event_edg(ivl_event_t net, unsigned idx)
+{
+      assert(net);
+      assert(idx < net->nedg);
+      return net->pins[net->nany + net->nneg + net->npos + idx];
+}
+
 extern "C" unsigned ivl_event_nneg(ivl_event_t net)
 {
       assert(net);
@@ -1862,6 +1877,12 @@ extern "C" int ivl_parameter_local(ivl_parameter_t net)
       return net->local;
 }
 
+extern "C" int ivl_parameter_is_type(ivl_parameter_t net)
+{
+      assert(net);
+      return net->is_type;
+}
+
 extern "C" int ivl_parameter_signed(ivl_parameter_t net)
 {
       assert(net);
@@ -2698,6 +2719,11 @@ extern "C" ivl_scope_t ivl_stmt_call(ivl_statement_t net)
       }
 }
 
+extern "C" bool ivl_stmt_flow_control(ivl_statement_t net)
+{
+      return net->u_.disable_.flow_control;
+}
+
 extern "C" unsigned ivl_stmt_case_count(ivl_statement_t net)
 {
       assert(net);
@@ -2821,6 +2847,9 @@ extern "C" ivl_expr_t ivl_stmt_delay_expr(ivl_statement_t net)
 	  case IVL_ST_DELAYX:
 	    return net->u_.delayx_.expr;
 
+	  case IVL_ST_NB_TRIGGER:
+	    return net->u_.wait_.delay;
+
 	  default:
 	    assert(0);
 	    return 0;
@@ -2851,12 +2880,15 @@ extern "C" unsigned ivl_stmt_nevent(ivl_statement_t net)
 	  case IVL_ST_ASSIGN_NB:
 	    return net->u_.assign_.nevent;
 
-	  case IVL_ST_WAIT:
-	    return net->u_.wait_.nevent;
+	  case IVL_ST_NB_TRIGGER:
+	    return 1;
 
 	  case IVL_ST_TRIGGER:
 	    return 1;
 
+	  case IVL_ST_WAIT:
+	    return net->u_.wait_.nevent;
+
 	  default:
 	    assert(0);
       }
@@ -2874,6 +2906,14 @@ extern "C" ivl_event_t ivl_stmt_events(ivl_statement_t net, unsigned idx)
 	    else
 		  return net->u_.assign_.events[idx];
 
+	  case IVL_ST_NB_TRIGGER:
+	    assert(idx == 0);
+	    return net->u_.wait_.event;
+
+	  case IVL_ST_TRIGGER:
+	    assert(idx == 0);
+	    return net->u_.wait_.event;
+
 	  case IVL_ST_WAIT:
 	    assert(idx < net->u_.wait_.nevent);
 	    if (net->u_.wait_.nevent == 1)
@@ -2881,10 +2921,6 @@ extern "C" ivl_event_t ivl_stmt_events(ivl_statement_t net, unsigned idx)
 	    else
 		  return net->u_.wait_.events[idx];
 
-	  case IVL_ST_TRIGGER:
-	    assert(idx == 0);
-	    return net->u_.wait_.event;
-
 	  default:
 	    assert(0);
       }
@@ -3172,6 +3208,11 @@ extern "C" ivl_type_t ivl_type_element(ivl_type_t net)
       return 0;
 }
 
+extern "C" unsigned ivl_type_packed_width(ivl_type_t net)
+{
+      return net->packed_width();
+}
+
 extern "C" unsigned ivl_type_packed_dimensions(ivl_type_t net)
 {
       assert(net);
