@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000-2017 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2022 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -179,8 +179,6 @@ bool dll_target::make_single_lval_(const LineInfo*li, struct ivl_lval_s*cur, con
 
       cur->width_ = asn->lwidth();
 
-      ivl_type_t nest_type = 0;
-
       if (asn->sig()) {
 	    cur->type_ = IVL_LVAL_REG;
 	    cur->n.sig = find_signal(des_, asn->sig());
@@ -188,7 +186,6 @@ bool dll_target::make_single_lval_(const LineInfo*li, struct ivl_lval_s*cur, con
       } else {
 	    const NetAssign_*asn_nest = asn->nest();
 	    ivl_assert(*li, asn_nest);
-	    nest_type = asn_nest->net_type();
 	    struct ivl_lval_s*cur_nest = new struct ivl_lval_s;
 	    make_single_lval_(li, cur_nest, asn_nest);
 
@@ -209,22 +206,7 @@ bool dll_target::make_single_lval_(const LineInfo*li, struct ivl_lval_s*cur, con
 	    expr_ = 0;
       }
 
-      cur->property_idx = -1;
-      perm_string pname = asn->get_property();
-      if (!pname.nil()) {
-	    const netclass_t*use_type;
-	    switch (cur->type_) {
-		case IVL_LVAL_LVAL:
-		  assert(nest_type);
-		  use_type = dynamic_cast<const netclass_t*> (nest_type);
-		  break;
-		default:
-		  use_type = dynamic_cast<const netclass_t*> (cur->n.sig->net_type);
-		  break;
-	    }
-	    assert(use_type);
-	    cur->property_idx = use_type->property_idx_from_name(pname);
-      }
+      cur->property_idx = asn->get_property_idx();
 
       return flag;
 }
@@ -370,6 +352,7 @@ void dll_target::proc_assign_nb(const NetAssignNB*net)
 			unsigned iany = 0;
 			unsigned ineg = ev_tmp->nany;
 			unsigned ipos = ineg + ev_tmp->nneg;
+			unsigned iedg = ipos + ev_tmp->npos;
 
 			for (unsigned idx = 0;  idx < ev->nprobe();  idx += 1) {
 			      const NetEvProbe*pr = ev->probe(idx);
@@ -388,6 +371,10 @@ void dll_target::proc_assign_nb(const NetAssignNB*net)
 				    base = ipos;
 				    ipos += pr->pin_count();
 				    break;
+				  case NetEvProbe::EDGE:
+				    base = iedg;
+				    iedg += pr->pin_count();
+				    break;
 			      }
 
 			      for (unsigned bit = 0; bit < pr->pin_count();
@@ -421,9 +408,11 @@ bool dll_target::proc_block(const NetBlock*net)
 	    return true;
       }
 
-	/* If there is exactly one statement, there is no need for the
-	   block wrapper, generate the contained statement instead. */
-      if ((count == 1) && (net->subscope() == 0)) {
+	/* If there is exactly one statement and the block is not a
+	   fork/join_none, there is no need for the block wrapper,
+	   generate the contained statement instead. */
+      if ((count == 1) && (net->subscope() == 0) &&
+	  (net->type() != NetBlock::PARA_JOIN_NONE)) {
 	    return net->proc_first()->emit_proc(this);
       }
 
@@ -652,6 +641,7 @@ bool dll_target::proc_disable(const NetDisable*net)
       FILE_NAME(stmt_cur_, net);
 
       stmt_cur_->type_ = IVL_ST_DISABLE;
+      stmt_cur_->u_.disable_.flow_control = net->flow_control();
       const NetScope* dis_scope = net->target();
 	/* A normal disable. */
       if (dis_scope) stmt_cur_->u_.disable_.scope = lookup_scope_(dis_scope);
@@ -820,6 +810,38 @@ bool dll_target::proc_trigger(const NetEvTrig*net)
 	    }
       }
 
+      return true;
+}
+
+bool dll_target::proc_nb_trigger(const NetEvNBTrig*net)
+{
+      assert(stmt_cur_);
+      assert(stmt_cur_->type_ == IVL_ST_NONE);
+      FILE_NAME(stmt_cur_, net);
+
+      stmt_cur_->type_ = IVL_ST_NB_TRIGGER;
+      stmt_cur_->u_.wait_.nevent = 1;
+      stmt_cur_->u_.wait_.delay = 0;
+
+      if (const NetExpr*expr = net->delay()) {
+	    assert(expr_ == 0);
+	    expr->expr_scan(this);
+	    stmt_cur_->u_.wait_.delay = expr_;
+	    expr_ = 0;
+      }
+
+	/* Locate the event by name. Save the ivl_event_t in the
+	   statement so that the generator can find it easily. */
+      const NetEvent*ev = net->event();
+      ivl_scope_t ev_scope = lookup_scope_(ev->scope());
+
+      for (unsigned idx = 0 ;  idx < ev_scope->nevent_ ;  idx += 1) {
+	    const char*ename = ivl_event_basename(ev_scope->event_[idx]);
+	    if (strcmp(ev->name(), ename) == 0) {
+		  stmt_cur_->u_.wait_.event = ev_scope->event_[idx];
+		  break;
+	    }
+      }
 
       return true;
 }
@@ -895,6 +917,7 @@ bool dll_target::proc_wait(const NetEvWait*net)
 		  unsigned iany = 0;
 		  unsigned ineg = ev_tmp->nany;
 		  unsigned ipos = ineg + ev_tmp->nneg;
+		  unsigned iedg = ipos + ev_tmp->npos;
 
 		  for (unsigned idx = 0 ;  idx < ev->nprobe() ;  idx += 1) {
 			const NetEvProbe*pr = ev->probe(idx);
@@ -913,14 +936,16 @@ bool dll_target::proc_wait(const NetEvWait*net)
 			      base = ipos;
 			      ipos += pr->pin_count();
 			      break;
+			    case NetEvProbe::EDGE:
+			      base = iedg;
+			      iedg += pr->pin_count();
+			      break;
 			}
 
-			for (unsigned bit = 0
-				   ; bit < pr->pin_count()
-				   ; bit += 1) {
+			for (unsigned bit = 0; bit < pr->pin_count(); bit += 1) {
 			      ivl_nexus_t nex = (ivl_nexus_t)
 				    pr->pin(bit).nexus()->t_cookie();
-			      assert(nex);
+			      ivl_assert(*ev, nex);
 			      ev_tmp->pins[base+bit] = nex;
 			}
 		  }
