@@ -1,7 +1,7 @@
 #ifndef IVL_t_dll_H
 #define IVL_t_dll_H
 /*
- * Copyright (c) 2000-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -128,6 +128,7 @@ struct dll_target  : public target_t, public expr_scan_t {
       void proc_repeat(const NetRepeat*);
       void proc_stask(const NetSTask*);
       bool proc_trigger(const NetEvTrig*);
+      bool proc_nb_trigger(const NetEvNBTrig*);
       void proc_utask(const NetUTask*);
       bool proc_wait(const NetEvWait*);
       void proc_while(const NetWhile*);
@@ -217,7 +218,7 @@ struct ivl_event_s {
       ivl_scope_t scope;
       perm_string file;
       unsigned lineno;
-      unsigned nany, nneg, npos;
+      unsigned nany, nneg, npos, nedg;
       ivl_nexus_t*pins;
 };
 
@@ -575,7 +576,7 @@ struct ivl_udp_s {
       ccharp_t*table; // zero terminated array of pointers
       perm_string file;
       unsigned lineno;
-      string*ports;
+      std::string*ports;
 };
 
 /*
@@ -616,7 +617,7 @@ struct ivl_nexus_ptr_s {
  */
 struct ivl_nexus_s {
       ivl_nexus_s() : ptrs_(1), nexus_(0), name_(0), private_data(0) { }
-      vector<ivl_nexus_ptr_s>ptrs_;
+      std::vector<ivl_nexus_ptr_s>ptrs_;
       const Nexus*nexus_;
       const char*name_;
       void*private_data;
@@ -637,6 +638,7 @@ struct ivl_parameter_s {
       long          lsb;
       bool  signed_flag;
       bool        local;
+      bool      is_type;
       perm_string file;
       unsigned lineno;
 };
@@ -842,6 +844,7 @@ struct ivl_statement_s {
 
 	    struct { /* IVL_ST_DISABLE */
 		  ivl_scope_t scope;
+		  bool flow_control;
 	    } disable_;
 
 	    struct { /* IVL_ST_FOREVER */
@@ -863,13 +866,14 @@ struct ivl_statement_s {
 		  ivl_scope_t def;
 	    } utask_;
 
-	    struct { /* IVL_ST_TRIGGER IVL_ST_WAIT */
+	    struct { /* IVL_ST_TRIGGER IVL_ST_NB_TRIGGER IVL_ST_WAIT */
 		  unsigned needs_t0_trigger;
 		  unsigned nevent;
 		  union {
 			ivl_event_t event;
 			ivl_event_t*events;
 		  };
+		  ivl_expr_t delay;
 		  ivl_statement_t stmt_;
 	    } wait_;
 
