@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2016 Stephen Williams <steve@icarus.com>
+ * Copyright (c) 1998-2021 Stephen Williams <steve@icarus.com>
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -24,6 +24,8 @@
 # include  "target.h"
 # include  <typeinfo>
 
+using namespace std;
+
 target_t::~target_t()
 {
 }
@@ -424,6 +426,13 @@ bool target_t::proc_trigger(const NetEvTrig*tr)
       return false;
 }
 
+bool target_t::proc_nb_trigger(const NetEvNBTrig*tr)
+{
+      cerr << tr->get_fileline() << ": error: target (" << typeid(*this).name()
+	   <<  "): Unhandled non-blocking event trigger." << endl;
+      return false;
+}
+
 void target_t::proc_stask(const NetSTask*)
 {
       cerr << "target (" << typeid(*this).name() <<  "): "
