@@ -1,7 +1,7 @@
 /*
  *  VHDL code generation for scopes.
  *
- *  Copyright (C) 2008-2014  Nick Gasson (nick@nickg.me.uk)
+ *  Copyright (C) 2008-2021  Nick Gasson (nick@nickg.me.uk)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 #include <cassert>
 #include <cstring>
 
+using namespace std;
+
 /*
  * This represents the portion of a nexus that is visible within
  * a VHDL scope. If that nexus portion does not contain a signal,
@@ -938,9 +940,15 @@ static void create_skeleton_entity_for(ivl_scope_t scope, int depth)
    unsigned nparams = ivl_scope_params(scope);
    for (unsigned i = 0; i < nparams; i++) {
       ivl_parameter_t param = ivl_scope_param(scope, i);
-      ss << "\n  " << ivl_parameter_basename(param) << " = ";
+
+      // Type parameter usages get replaced with their actual type
+      if (ivl_parameter_is_type(param))
+	    continue;
 
       ivl_expr_t value = ivl_parameter_expr(param);
+
+      ss << "\n  " << ivl_parameter_basename(param) << " = ";
+
       switch (ivl_expr_type(value)) {
          case IVL_EX_STRING:
             ss << "\"" << ivl_expr_string(value) << "\"";
