@@ -10,8 +10,6 @@
 
 #include <string>
 
-using namespace std;
-
 void error(const char *fmt, ...);
 void debug_msg(const char *fmt, ...);
 
@@ -28,7 +26,7 @@ vhdl_expr *translate_time_expr(ivl_expr_t e);
 ivl_design_t get_vhdl_design();
 vhdl_var_ref *nexus_to_var_ref(vhdl_scope *arch_scope, ivl_nexus_t nexus);
 vhdl_var_ref* readable_ref(vhdl_scope* scope, ivl_nexus_t nex);
-string make_safe_name(ivl_signal_t sig);
+std::string make_safe_name(ivl_signal_t sig);
 void require_support_function(support_function_t f);
 
 #endif /* #ifndef INC_VHDL_TARGET_H */
