@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2020 Cary R. (cygcary@yahoo.com)
+ * Copyright (C) 2010-2021 Cary R. (cygcary@yahoo.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -253,13 +253,27 @@ static void emit_net_def(ivl_scope_t scope, ivl_signal_t sig)
       }
 }
 
+static bool scope_is_unique(ivl_scope_t scope)
+{
+      unsigned int count = ivl_scope_params(scope);
+
+      for (unsigned int idx = 0; idx < count; idx++) {
+	    ivl_parameter_t par = ivl_scope_param(scope, idx);
+	    if (!ivl_parameter_local(par)) {
+		  return false;
+	    }
+      }
+
+      return true;
+}
+
 static void emit_mangled_name(ivl_scope_t scope, unsigned root)
 {
-	/* If the module has parameters and it's not a root module then it
-	 * may not be unique so we create a mangled name version instead.
-	 * The mangled name is of the form:
+	/* If the module has non-local parameters and it's not a root module then it
+	 * may not be unique so we create a mangled name version instead. The
+	 * mangled name is of the form:
 	 *   <module_name>[<full_instance_scope>]. */
-      if (ivl_scope_params(scope) && ! root) {
+      if (!root && !scope_is_unique(scope)) {
 	    char *name;
 	    size_t len = strlen(ivl_scope_name(scope)) +
 	                 strlen(ivl_scope_tname(scope)) + 3;
@@ -295,6 +309,13 @@ void emit_scope_variables(ivl_scope_t scope)
       count = ivl_scope_params(scope);
       for (idx = 0; idx < count; idx += 1) {
 	    ivl_parameter_t par = ivl_scope_param(scope, idx);
+	      // vlog95 does not support type parameters. Places where type
+	      // parameters have been used it will be replaced with the actual
+	      // type that the module was instantiated with. Similar to
+	      // typedefs.
+	    if (ivl_parameter_is_type(par))
+		  continue;
+
 	    ivl_expr_t pex = ivl_parameter_expr(par);
 	    fprintf(vlog_out, "%*cparameter ", indent, ' ');
 	    emit_id(ivl_parameter_basename(par));
@@ -696,7 +717,8 @@ static int find_tfb_process(ivl_process_t proc, ivl_scope_t scope)
 	       * processes that are used to set local variables. */
 	    assert(ivl_process_type(proc) == IVL_PR_INITIAL);
 	      /* Find the module scope for this task/function. */
-	    while (ivl_scope_type(mod_scope) != IVL_SCT_MODULE) {
+	    while (ivl_scope_type(mod_scope) != IVL_SCT_MODULE &&
+		   ivl_scope_type(mod_scope) != IVL_SCT_PACKAGE) {
 		  mod_scope = ivl_scope_parent(mod_scope);
 		  assert(mod_scope);
 	    }
@@ -863,7 +885,6 @@ static unsigned has_func_disable(ivl_scope_t scope, ivl_statement_t stmt)
 	case IVL_ST_FORK:
 	case IVL_ST_FORK_JOIN_ANY:
 	case IVL_ST_FORK_JOIN_NONE:
-	case IVL_ST_UTASK:
 	case IVL_ST_WAIT:
 	    assert(0);
 	    break;
@@ -877,7 +898,9 @@ static unsigned has_func_disable(ivl_scope_t scope, ivl_statement_t stmt)
 	case IVL_ST_FREE:
 	case IVL_ST_RELEASE:
 	case IVL_ST_STASK:
+	case IVL_ST_UTASK: // this will be generated for a SV void function
 	case IVL_ST_TRIGGER:
+	case IVL_ST_NB_TRIGGER:
 	    break;
 	  /* Look for a disable in each block statement. */
 	case IVL_ST_BLOCK:
@@ -1153,7 +1176,9 @@ int emit_scope(ivl_scope_t scope, ivl_scope_t parent)
 	    for (idx = 0; idx < count; idx += 1) {
 		  emit_tran(scope, ivl_scope_switch(scope, idx));
 	    }
+      }
 
+      if (sc_type == IVL_SCT_MODULE || sc_type == IVL_SCT_PACKAGE) {
 	      /* Output any initial blocks for tasks or functions or named
 	       * blocks defined in this module. Used to initialize local
 	       * variables. */
@@ -1240,7 +1265,7 @@ int emit_scope(ivl_scope_t scope, ivl_scope_t parent)
 			(void) emit_scope(scope_to_emit, 0);
 			  /* If we used a mangled name then the instance is
 			   * unique so don't add it to the list. */
-			if (ivl_scope_params(scope_to_emit)) continue;
+			if (!scope_is_unique(scope_to_emit)) continue;
 			add_scope_to_list(scope_to_emit);
 		  }
 		  free(scopes_to_emit);
