@@ -83,7 +83,7 @@ static void draw_lpm_mux_nest(ivl_lpm_t net, const char*muxz)
 	      net, select_input);
 
       for (idx = 0 ;  idx < ivl_lpm_size(net) ;  idx += 2) {
-	    fprintf(vvp_out, "L_%p/0/%d .functor %s %u",
+	    fprintf(vvp_out, "L_%p/0/%u .functor %s %u",
 		    net, idx/2, muxz, width);
 	    fprintf(vvp_out, ", %s", draw_net_input(ivl_lpm_data(net,idx+0)));
 	    fprintf(vvp_out, ", %s", draw_net_input(ivl_lpm_data(net,idx+1)));
@@ -95,10 +95,10 @@ static void draw_lpm_mux_nest(ivl_lpm_t net, const char*muxz)
 		    net, level, select_input, level, level);
 
 	    for (idx = 0 ;  idx < (ivl_lpm_size(net) >> level); idx += 2) {
-		  fprintf(vvp_out, "L_%p/%u/%d .functor %s %u",
+		  fprintf(vvp_out, "L_%p/%u/%u .functor %s %u",
 			  net, level, idx/2, muxz, width);
-		  fprintf(vvp_out, ", L_%p/%d/%d", net, level-1, idx+0);
-		  fprintf(vvp_out, ", L_%p/%d/%d", net, level-1, idx+1);
+		  fprintf(vvp_out, ", L_%p/%u/%u", net, level-1, idx+0);
+		  fprintf(vvp_out, ", L_%p/%u/%u", net, level-1, idx+1);
 		  fprintf(vvp_out, ", L_%p/%us",   net, level);
 		  fprintf(vvp_out, ", C4<>;\n");
 	    }
@@ -106,14 +106,14 @@ static void draw_lpm_mux_nest(ivl_lpm_t net, const char*muxz)
       }
 
 
-      fprintf(vvp_out, "L_%p/%ds .part %s, %d, 1; Bit %d of the select\n",
+      fprintf(vvp_out, "L_%p/%us .part %s, %u, 1; Bit %u of the select\n",
 	      net, swidth-1, select_input, swidth-1, swidth-1);
 
 
       fprintf(vvp_out, "L_%p .functor %s %u", net, muxz, width);
-      fprintf(vvp_out, ", L_%p/%d/0", net, swidth-2);
-      fprintf(vvp_out, ", L_%p/%d/1", net, swidth-2);
-      fprintf(vvp_out, ", L_%p/%ds",  net, swidth-1);
+      fprintf(vvp_out, ", L_%p/%u/0", net, swidth-2);
+      fprintf(vvp_out, ", L_%p/%u/1", net, swidth-2);
+      fprintf(vvp_out, ", L_%p/%us",  net, swidth-1);
       fprintf(vvp_out, ", C4<>;\n");
 
       free(select_input);
