@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2008-2020 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -73,6 +73,15 @@ void draw_switch_in_scope(ivl_switch_t sw)
       }
 
       switch (ivl_switch_type(sw)) {
+	  case IVL_SW_RTRAN:
+	    fprintf(vvp_out, " .rtran");
+	    break;
+	  case IVL_SW_RTRANIF0:
+	    fprintf(vvp_out, " .rtranif0");
+	    break;
+	  case IVL_SW_RTRANIF1:
+	    fprintf(vvp_out, " .rtranif1");
+	    break;
 	  case IVL_SW_TRAN:
 	    fprintf(vvp_out, " .tran");
 	    break;
@@ -88,8 +97,7 @@ void draw_switch_in_scope(ivl_switch_t sw)
 	    break;
 
 	  default:
-	    fprintf(stderr, "%s:%u: tgt-vvp sorry: resistive switch modeling "
-	                    "is not currently supported.\n",
+	    fprintf(stderr, "%s:%u: vvp.tgt error: unrecognised switch type.\n",
 		            ivl_switch_file(sw), ivl_switch_lineno(sw));
 	    vvp_errors += 1;
 	    return;
