@@ -199,6 +199,11 @@ static int get_vpi_taskfunc_signal_arg(struct args_info *result,
 	    if (ivl_expr_value(vexpr) == IVL_VT_DARRAY)
 		  return 0;
 
+	      /* Part select is always unsigned. If the expression is signed
+	       * fallback. */
+	    if (ivl_expr_signed(expr))
+		  return 0;
+
 	      /* The signal is part of an array. */
 	      /* Add &APV<> code here when it is finished. */
 	    bexpr = ivl_expr_oper2(expr);
@@ -384,6 +389,10 @@ static void draw_vpi_taskfunc_args(const char*call_string,
 			args[idx].stack = 0;
 			break;
 		  }
+		case IVL_EX_NULL:
+		  snprintf(buffer, sizeof buffer, "null");
+		  args[idx].text = strdup(buffer);
+		  continue;
 		    /* Everything else will need to be evaluated and
 		       passed as a constant to the vpi task. */
 		default:
@@ -424,6 +433,8 @@ static void draw_vpi_taskfunc_args(const char*call_string,
 		  buffer[0] = 0;
 		  break;
 		default:
+		  fprintf(stderr, "%s:%u: Sorry, cannot generate code for argument %u.\n",
+		                  ivl_expr_file(expr), ivl_expr_lineno(expr), idx+1);
 		  fprintf(vvp_out, "\nXXXX Unexpected argument: call_string=<%s>, arg=%u, type=%d\n",
 			  call_string, idx, ivl_expr_value(expr));
 		  fflush(vvp_out);
