@@ -409,11 +409,26 @@ static void draw_binary_vec4_compare(ivl_expr_t expr)
       }
 }
 
+/*
+ * Handle the logical implication:
+ *    <le> -> <re>
+ * which is the same as the expression
+ *    !<le> || <re>
+ *
+ */
 static void draw_binary_vec4_limpl(ivl_expr_t expr)
 {
-      fprintf(stderr, "vvp.tgt sorry: No support for logical implication (%s:%u).\n",
-                      ivl_expr_file(expr), ivl_expr_lineno(expr));
-      assert(0);
+      ivl_expr_t le = ivl_expr_oper1(expr);
+      ivl_expr_t re = ivl_expr_oper2(expr);
+
+      /* The arguments should have alreacy been reduced. */
+      assert(ivl_expr_width(le) == 1);
+      assert(ivl_expr_width(re) == 1);
+
+      draw_eval_vec4(le);
+      fprintf(vvp_out, "    %%inv;\n");
+      draw_eval_vec4(re);
+      fprintf(vvp_out, "    %%or;\n");
 }
 
 static void draw_binary_vec4_lequiv(ivl_expr_t expr)
@@ -432,25 +447,45 @@ static void draw_binary_vec4_lequiv(ivl_expr_t expr)
       assert(ivl_expr_width(expr) == 1);
 }
 
-static void draw_binary_vec4_land(ivl_expr_t expr)
+static void draw_binary_vec4_logical(ivl_expr_t expr, char op)
 {
+      const char *opcode;
+      const char *jmp_type;
+
+      switch (op) {
+	  case 'a':
+	    opcode = "and";
+	    jmp_type = "0";
+	    break;
+	  case 'o':
+	    opcode = "or";
+	    jmp_type = "1";
+	    break;
+	  default:
+	    assert(0);
+	    break;
+      }
+
+      unsigned label_out = local_count++;
+
       ivl_expr_t le = ivl_expr_oper1(expr);
       ivl_expr_t re = ivl_expr_oper2(expr);
 
-	/* Push the left expression. Reduce it to a single bit if
-	   necessary. */
-      draw_eval_vec4(le);
-      if (ivl_expr_width(le) > 1)
-	    fprintf(vvp_out, "    %%or/r;\n");
-
-	/* Now push the right expression. Again, reduce to a single
-	   bit if necessary. */
+      /* Evaluate the left expression as a conditon and skip the right expression
+       * if the left is false. */
+      int flag = draw_eval_condition(le);
+      fprintf(vvp_out, "    %%flag_get/vec4 %d;\n", flag);
+      fprintf(vvp_out, "    %%jmp/%s T_%u.%u, %d;\n", jmp_type, thread_count,
+	      label_out, flag);
+      clr_flag(flag);
+      /* Now push the right expression. Reduce to a single bit if necessary. */
       draw_eval_vec4(re);
       if (ivl_expr_width(re) > 1)
 	    fprintf(vvp_out, "    %%or/r;\n");
 
-      fprintf(vvp_out, "    %%and;\n");
+      fprintf(vvp_out, "    %%%s;\n", opcode);
 
+      fprintf(vvp_out, "T_%u.%u;\n", thread_count, label_out);
       if (ivl_expr_width(expr) > 1)
 	    fprintf(vvp_out, "    %%pad/u %u;\n", ivl_expr_width(expr));
 }
@@ -632,29 +667,6 @@ static void draw_binary_vec4_le(ivl_expr_t expr)
       }
 }
 
-static void draw_binary_vec4_lor(ivl_expr_t expr)
-{
-      ivl_expr_t le = ivl_expr_oper1(expr);
-      ivl_expr_t re = ivl_expr_oper2(expr);
-
-	/* Push the left expression. Reduce it to a single bit if
-	   necessary. */
-      draw_eval_vec4(le);
-      if (ivl_expr_width(le) > 1)
-	    fprintf(vvp_out, "    %%or/r;\n");
-
-	/* Now push the right expression. Again, reduce to a single
-	   bit if necessary. */
-      draw_eval_vec4(re);
-      if (ivl_expr_width(re) > 1)
-	    fprintf(vvp_out, "    %%or/r;\n");
-
-      fprintf(vvp_out, "    %%or;\n");
-
-      if (ivl_expr_width(expr) > 1)
-	    fprintf(vvp_out, "    %%pad/u %u;\n", ivl_expr_width(expr));
-}
-
 static void draw_binary_vec4_lrs(ivl_expr_t expr)
 {
       ivl_expr_t le = ivl_expr_oper1(expr);
@@ -695,7 +707,8 @@ static void draw_binary_vec4(ivl_expr_t expr)
 {
       switch (ivl_expr_opcode(expr)) {
 	  case 'a': /* Logical && */
-	    draw_binary_vec4_land(expr);
+	  case 'o': /* || (logical or) */
+	    draw_binary_vec4_logical(expr, ivl_expr_opcode(expr));
 	    break;
 
 	  case '+':
@@ -738,10 +751,6 @@ static void draw_binary_vec4(ivl_expr_t expr)
 	    draw_binary_vec4_lrs(expr);
 	    break;
 
-	  case 'o': /* || (logical or) */
-	    draw_binary_vec4_lor(expr);
-	    break;
-
 	  case 'q': /* -> (logical implication) */
 	    draw_binary_vec4_limpl(expr);
 	    break;
@@ -1398,10 +1407,21 @@ void draw_eval_vec4(ivl_expr_t expr)
 	    draw_property_vec4(expr);
 	    return;
 
+	  case IVL_EX_NULL:
+	    fprintf(stderr, "%s:%u: Error: 'null' used in an expression\n",
+	                    ivl_expr_file(expr), ivl_expr_lineno(expr));
+	    fprintf(vvp_out, "; Error 'null' used in an expression\n");
+	    vvp_errors += 1;
+	    return;
+
 	  default:
 	    break;
       }
 
-      fprintf(stderr, "XXXX Evaluate VEC4 expression (%d)\n", ivl_expr_type(expr));
-      fprintf(vvp_out, "; XXXX Evaluate VEC4 expression (%d)\n", ivl_expr_type(expr));
+      fprintf(stderr, "%s:%u: Sorry: cannot evaluate VEC4 expression (%d)\n",
+                      ivl_expr_file(expr), ivl_expr_lineno(expr),
+                      ivl_expr_type(expr));
+      fprintf(vvp_out, "; Sorry: cannot evaluate VEC4 expression (%d)\n",
+                       ivl_expr_type(expr));
+      vvp_errors += 1;
 }
