@@ -21,6 +21,7 @@
 # include  <string.h>
 # include  <assert.h>
 # include  <stdlib.h>
+# include  <limits.h>
 
 /*
  * These functions handle the blocking assignment. Use the %set
@@ -101,8 +102,10 @@ static void get_vec_from_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice
 	   it to select the word, and pay no further heed to the
 	   expression itself. */
       if (word_ix && number_is_immediate(word_ix, IMM_WID, 0)) {
-	    assert(! number_is_unknown(word_ix));
-	    use_word = get_number_immediate(word_ix);
+	    if (number_is_unknown(word_ix))
+		  use_word = ULONG_MAX; // The largest valid index is ULONG_MAX - 1
+	    else
+		  use_word = get_number_immediate(word_ix);
 	    word_ix = 0;
       }
 
@@ -147,9 +150,9 @@ static void get_vec_from_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice
 
 	    fprintf(vvp_out, "    %%load/vec4 v%p_%lu;\n", sig, use_word);
 	    draw_eval_vec4(part_off_ex);
-	    fprintf(vvp_out, "    %%flag_mov %u, 4;\n", slice->u_.part_select_dynamic.x_flag);
 	    fprintf(vvp_out, "    %%dup/vec4;\n");
 	    fprintf(vvp_out, "    %%ix/vec4 %d;\n", slice->u_.part_select_dynamic.word_idx_reg);
+	    fprintf(vvp_out, "    %%flag_mov %u, 4;\n", slice->u_.part_select_dynamic.x_flag);
 	    fprintf(vvp_out, "    %%part/u %u;\n", wid);
 
       } else if (ivl_signal_dimensions(sig) > 0 && word_ix == 0) {
@@ -161,10 +164,15 @@ static void get_vec_from_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice
 	    if (use_word < ivl_signal_array_count(sig)) {
 		  fprintf(vvp_out, "    %%ix/load 3, %lu, 0;\n",
 			  use_word);
+		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  fprintf(vvp_out, "    %%load/vec4a v%p, 3;\n", sig);
 	    } else {
-		  assert(wid <= 32);
-		  fprintf(vvp_out, "    %%pushi/vec4 4294967295, 4294967295, %u;\n", wid);
+		  if (wid <= 32) {
+			fprintf(vvp_out, "    %%pushi/vec4 4294967295, 4294967295, %u;\n", wid);
+		  } else {
+			fprintf(vvp_out, "    %%pushi/vec4 4294967295, 4294967295, 32;\n");
+			fprintf(vvp_out, "    %%pad/s %u;\n", wid);
+		  }
 	    }
 
       } else if (ivl_signal_dimensions(sig) > 0 && word_ix != 0) {
@@ -320,6 +328,7 @@ static void put_vec_to_lval_slice(ivl_lval_t lval, struct vec_slice_info*slice,
 		  clr_word(word_idx);
 	    } else {
 		  fprintf(vvp_out," ; Skip this slice write to v%p [%lu]\n", sig, slice->u_.memory_word_static.use_word);
+		  fprintf(vvp_out,"    %%pop/vec4 1;\n");
 	    }
 	    break;
 
@@ -639,8 +648,10 @@ static void get_real_from_lval(ivl_lval_t lval, struct real_lval_info*slice)
 	   it to select the word, and pay no further heed to the
 	   expression itself. */
       if (word_ix && number_is_immediate(word_ix, IMM_WID, 0)) {
-	    assert(! number_is_unknown(word_ix));
-	    use_word = get_number_immediate(word_ix);
+	    if (number_is_unknown(word_ix))
+		  use_word = ULONG_MAX; // The largest valid index is ULONG_MAX - 1
+	    else
+		  use_word = get_number_immediate(word_ix);
 	    word_ix = 0;
       }
 
@@ -664,7 +675,8 @@ static void get_real_from_lval(ivl_lval_t lval, struct real_lval_info*slice)
 	    if (use_word < ivl_signal_array_count(sig)) {
 		  fprintf(vvp_out, "    %%ix/load 3, %lu, 0;\n",
 			  use_word);
-		  fprintf(vvp_out, "    %%load/reala v%p, 3;\n", sig);
+		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
+		  fprintf(vvp_out, "    %%load/ar v%p, 3;\n", sig);
 	    } else {
 		  fprintf(vvp_out, "    %%pushi/real 0, 0;\n");
 	    }
@@ -679,7 +691,7 @@ static void get_real_from_lval(ivl_lval_t lval, struct real_lval_info*slice)
 
 	    draw_eval_expr_into_integer(word_ix, slice->u_.memory_word_dynamic.word_idx_reg);
 	    fprintf(vvp_out, "    %%flag_mov %u, 4;\n", slice->u_.memory_word_dynamic.x_flag);
-	    fprintf(vvp_out, "    %%load/reala v%p, %d;\n", sig, slice->u_.memory_word_dynamic.word_idx_reg);
+	    fprintf(vvp_out, "    %%load/ar v%p, %d;\n", sig, slice->u_.memory_word_dynamic.word_idx_reg);
 
       } else {
 	    assert(0);
@@ -719,6 +731,7 @@ static void put_real_to_lval(ivl_lval_t lval, struct real_lval_info*slice)
 		  clr_word(word_idx);
 	    } else {
 		  fprintf(vvp_out," ; Skip this slice write to v%p [%lu]\n", sig, slice->u_.memory_word_static.use_word);
+		  fprintf(vvp_out,"    %%pop/real 1;\n");
 	    }
 	    break;
 
@@ -756,37 +769,11 @@ static void store_real_to_lval(ivl_lval_t lval)
 	    return;
       }
 
-	// For now, only support 1-dimensional arrays.
-      assert(ivl_signal_dimensions(var) == 1);
-
       ivl_expr_t word_ex = ivl_lval_idx(lval);
       int word_ix = allocate_word();
 
-	/* If the word index is a constant, then we can write
-	   directly to the word and save the index calculation.
-	   Out-of-bounds and undefined indices are converted to
-	   a canonical index of 'bx during elaboration, and we
-	   don't try to optimise that case. */
-      if (word_ex && number_is_immediate(word_ex, IMM_WID, 0) &&
-	  !number_is_unknown(word_ex)) {
-	    unsigned long use_word = get_number_immediate(word_ex);
-	    assert(use_word < ivl_signal_array_count(var));
-	    fprintf(vvp_out, "    %%ix/load %d, %lu, 0;\n",
-		    word_ix, use_word);
-	    fprintf(vvp_out, "    %%store/reala v%p, %d;\n",
-		    var, word_ix);
-
-      } else {
-	    unsigned do_store = transient_id++;
-	    unsigned end_store = transient_id++;
-	    draw_eval_expr_into_integer(word_ex, word_ix);
-	    fprintf(vvp_out, "    %%jmp/0 t_%u, 4;\n", do_store);
-	    fprintf(vvp_out, "    %%pop/real 1;\n");
-	    fprintf(vvp_out, "    %%jmp t_%u;\n", end_store);
-	    fprintf(vvp_out, "t_%u ;\n", do_store);
-	    fprintf(vvp_out, "    %%store/reala v%p, %d;\n", var, word_ix);
-	    fprintf(vvp_out, "t_%u ;\n", end_store);
-      }
+      draw_eval_expr_into_integer(word_ex, word_ix);
+      fprintf(vvp_out, "    %%store/reala v%p, %d;\n", var, word_ix);
 
       clr_word(word_ix);
 }
@@ -910,21 +897,6 @@ static int show_stmt_assign_sig_string(ivl_statement_t net)
       return 0;
 }
 
-unsigned width_of_packed_type(ivl_type_t net)
-{
-      unsigned idx;
-      unsigned width = 1;
-      for (idx = 0 ; idx < ivl_type_packed_dimensions(net) ; idx += 1) {
-	    int lsb = ivl_type_packed_lsb(net,idx);
-	    int msb = ivl_type_packed_msb(net,idx);
-	    if (lsb <= msb)
-		  width *= msb - lsb + 1;
-	    else
-		  width *= lsb - msb + 1;
-      }
-      return width;
-}
-
 /*
  * This function handles the special case that we assign an array
  * pattern to a dynamic array. Handle this by assigning each
@@ -947,14 +919,14 @@ static int show_stmt_assign_darray_pattern(ivl_statement_t net)
 #if 0
       unsigned element_width = 1;
       if (ivl_type_base(element_type) == IVL_VT_BOOL)
-	    element_width = width_of_packed_type(element_type);
+	    element_width = ivl_type_packed_width(element_type);
       else if (ivl_type_base(element_type) == IVL_VT_LOGIC)
-	    element_width = width_of_packed_type(element_type);
+	    element_width = ivl_type_packed_width(element_type);
 #endif
 
 // FIXME: At the moment we reallocate the array space.
 //        This probably should be a resize to avoid values glitching
-	/* Allocate at least enough space for the array patter. */
+	/* Allocate at least enough space for the array pattern. */
       fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", size_reg, ivl_expr_parms(rval));
 	/* This can not have have a X/Z value so clear flag 4. */
       fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
@@ -968,18 +940,21 @@ static int show_stmt_assign_darray_pattern(ivl_statement_t net)
 		case IVL_VT_LOGIC:
 		  draw_eval_vec4(ivl_expr_parm(rval,idx));
 		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  fprintf(vvp_out, "    %%store/dar/vec4 v%p_0;\n", var);
 		  break;
 
 		case IVL_VT_REAL:
 		  draw_eval_real(ivl_expr_parm(rval,idx));
 		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  fprintf(vvp_out, "    %%store/dar/r v%p_0;\n", var);
 		  break;
 
 		case IVL_VT_STRING:
 		  draw_eval_string(ivl_expr_parm(rval,idx));
 		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  fprintf(vvp_out, "    %%store/dar/str v%p_0;\n", var);
 		  break;
 
@@ -1038,12 +1013,36 @@ static int show_stmt_assign_sig_darray(ivl_statement_t net)
 		 elements of the l-value. */
 	    errors += show_stmt_assign_darray_pattern(net);
 
-      } else {
-	      /* There is no l-value mux, so this must be an
-		 assignment to the array as a whole. Evaluate the
-		 "object", and store the evaluated result. */
+      } else if (ivl_expr_type(rval) == IVL_EX_NEW) {
+	    // There is no l-value mux, and the r-value expression is
+	    // a "new" expression. Handle this by simply storing the
+	    // new object to the lval.
 	    errors += draw_eval_object(rval);
-	    fprintf(vvp_out, "    %%store/obj v%p_0;\n", var);
+	    fprintf(vvp_out, "    %%store/obj v%p_0; %s:%u: %s = new ...\n",
+		    var, ivl_stmt_file(net), ivl_stmt_lineno(net),
+		    ivl_signal_basename(var));
+
+      } else if (ivl_expr_type(rval) == IVL_EX_SIGNAL) {
+
+	    // There is no l-value mux, and the r-value expression is
+	    // a "signal" expression. Store a duplicate into the lvalue
+	    // By using the %dup/obj. Remember to pop the rvalue that
+	    // is no longer needed.
+	    errors += draw_eval_object(rval);
+	    fprintf(vvp_out, "    %%dup/obj;\n");
+	    fprintf(vvp_out, "    %%store/obj v%p_0; %s:%u: %s = <signal>\n",
+		    var, ivl_stmt_file(net), ivl_stmt_lineno(net),
+		    ivl_signal_basename(var));
+	    fprintf(vvp_out, "    %%pop/obj 1, 0;\n");
+
+      } else {
+	    // There is no l-value mux, so this must be an
+	    // assignment to the array as a whole. Evaluate the
+	    // "object", and store the evaluated result.
+	    errors += draw_eval_object(rval);
+	    fprintf(vvp_out, "    %%store/obj v%p_0; %s:%u: %s = <expr type %d>\n",
+		    var, ivl_stmt_file(net), ivl_stmt_lineno(net),
+		    ivl_signal_basename(var), ivl_expr_type(rval));
       }
 
       return errors;
@@ -1078,19 +1077,22 @@ static int show_stmt_assign_queue_pattern(ivl_signal_t var, ivl_expr_t rval,
 		case IVL_VT_LOGIC:
 		  draw_eval_vec4(ivl_expr_parm(rval,idx));
 		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  fprintf(vvp_out, "    %%store/qdar/v v%p_0, %d, %u;\n", var, max_idx,
-		                   width_of_packed_type(element_type));
+		                   ivl_type_packed_width(element_type));
 		  break;
 
 		case IVL_VT_REAL:
 		  draw_eval_real(ivl_expr_parm(rval,idx));
 		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  fprintf(vvp_out, "    %%store/qdar/r v%p_0, %d;\n", var, max_idx);
 		  break;
 
 		case IVL_VT_STRING:
 		  draw_eval_string(ivl_expr_parm(rval,idx));
 		  fprintf(vvp_out, "    %%ix/load 3, %u, 0;\n", idx);
+		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  fprintf(vvp_out, "    %%store/qdar/str v%p_0, %d;\n", var, max_idx);
 		  break;
 
@@ -1106,8 +1108,8 @@ static int show_stmt_assign_queue_pattern(ivl_signal_t var, ivl_expr_t rval,
 	    int del_idx = allocate_word();
 	    assert(del_idx >= 0);
 	      /* Save the first queue element to delete. */
-	    fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", del_idx, max_elems);
-	    fprintf(vvp_out, "    %%delete/tail v%p_0, %u;\n", var, del_idx);
+	    fprintf(vvp_out, "    %%ix/load %d, %u, 0;\n", del_idx, max_elems);
+	    fprintf(vvp_out, "    %%delete/tail v%p_0, %d;\n", var, del_idx);
 	    clr_word(del_idx);
       }
 
@@ -1135,7 +1137,7 @@ static int show_stmt_assign_sig_queue(ivl_statement_t net)
       int idx = allocate_word();
       assert(idx >= 0);
         /* Save the queue maximum index value to an integer register. */
-      fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", idx, ivl_signal_array_count(var));
+      fprintf(vvp_out, "    %%ix/load %d, %u, 0;\n", idx, ivl_signal_array_count(var));
 
       if (ivl_expr_type(rval) == IVL_EX_NULL) {
 	    errors += draw_eval_object(rval);
@@ -1164,7 +1166,7 @@ static int show_stmt_assign_sig_queue(ivl_statement_t net)
 		 index register 3. */
 	    draw_eval_expr_into_integer(mux, 3);
 	    fprintf(vvp_out, "    %%store/qdar/v v%p_0, %d, %u;\n", var, idx,
-	                     width_of_packed_type(element_type));
+	                     ivl_type_packed_width(element_type));
 
       } else if (ivl_expr_type(rval) == IVL_EX_ARRAY_PATTERN) {
 	      /* There is no l-value mux, but the r-value is an array
@@ -1185,7 +1187,7 @@ static int show_stmt_assign_sig_queue(ivl_statement_t net)
 		  assert(ivl_type_base(element_type) == IVL_VT_BOOL ||
 		         ivl_type_base(element_type) == IVL_VT_LOGIC);
 		  fprintf(vvp_out, "    %%store/qobj/v v%p_0, %d, %u;\n",
-		                   var, idx, width_of_packed_type(element_type));
+		                   var, idx, ivl_type_packed_width(element_type));
 	    }
       }
       clr_word(idx);
