@@ -1,6 +1,6 @@
+#define COPYRIGHT \
+  "Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)"
 /*
- * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
- *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
  *    General Public License as published by the Free Software
@@ -28,7 +28,7 @@
 
 static const char*version_string =
 "Icarus Verilog VVP Code Generator " VERSION " (" VERSION_TAG ")\n\n"
-"Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)\n\n"
+COPYRIGHT "\n\n"
 "  This program is free software; you can redistribute it and/or modify\n"
 "  it under the terms of the GNU General Public License as published by\n"
 "  the Free Software Foundation; either version 2 of the License, or\n"
@@ -102,7 +102,7 @@ int allocate_flag(void)
       int idx;
       for (idx = 0 ; idx < FLAGS_COUNT ; idx += 1) {
 	    int word = idx / 32;
-	    uint32_t mask = 1 << (idx%32);
+	    uint32_t mask = 1U << (idx%32);
 	    if (allocate_flag_mask[word] & mask)
 		  continue;
 
