@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -179,6 +179,8 @@ static void assign_to_array_word(ivl_signal_t lsig, ivl_expr_t word_ix,
 	      /* If needed use the global error state. */
 	    if (part_off_ex) {
 		  fprintf(vvp_out, "    %%flag_mov 4, %d;\n", error_flag);
+		  fprintf(vvp_out, "    %%ix/mov 3, %d;\n", word_ix_reg);
+		  clr_word(word_ix_reg);
 	    }
 	    fprintf(vvp_out, "    %%assign/vec4/a/e v%p, %d;\n", lsig, part_off_reg);
 
@@ -203,8 +205,6 @@ static void assign_to_array_word(ivl_signal_t lsig, ivl_expr_t word_ix,
 	    clr_word(delay_index);
       }
 
-      if (nevents != 0) fprintf(vvp_out, "    %%evctl/c;\n");
-
       clr_flag(error_flag);
       if (part_off_reg)
 	    clr_word(part_off_reg);
@@ -279,7 +279,6 @@ static void assign_to_lvector(ivl_lval_t lval,
 		  draw_eval_expr_into_integer(part_off_ex, offset_index);
 		  fprintf(vvp_out, "    %s/vec4/off/e v%p_%lu, %d;\n",
 			  assign_op, sig, use_word, offset_index);
-		  fprintf(vvp_out, "    %%evctl/c;\n");
 
 		  clr_word(offset_index);
 
@@ -312,7 +311,6 @@ static void assign_to_lvector(ivl_lval_t lval,
 		  fprintf(vvp_out, "    %%flag_set/imm 4, 0;\n");
 		  fprintf(vvp_out, "    %s/vec4/off/e v%p_%lu, %d;\n",
 			  assign_op, sig, use_word, offset_index);
-		  fprintf(vvp_out, "    %%evctl/c;\n");
 		  clr_word(offset_index);
 
 	    } else {
@@ -346,8 +344,6 @@ static void assign_to_lvector(ivl_lval_t lval,
 	      /* Event control delay... */
 	    fprintf(vvp_out, "    %s/vec4/e v%p_%lu;\n",
 		    assign_op, sig, use_word);
-	    fprintf(vvp_out, "    %%evctl/c;\n");
-
       } else {
 	      /*
 	       * The %assign can only take a 32 bit delay. For a larger
@@ -509,6 +505,8 @@ static int show_stmt_assign_nb(ivl_statement_t net)
 			ev = ivl_stmt_events(net, idx);
 			fprintf(vvp_out, ", E_%p", ev);
 		  }
+
+		  fprintf(vvp_out, ";\n");
 		  snprintf(name, sizeof(name), "Eassign_%u", cascade_counter);
 		  cascade_counter += 1;
 	    }
@@ -593,6 +591,9 @@ static int show_stmt_assign_nb(ivl_statement_t net)
 	}
       }
 
+      if (nevents)
+	    fprintf(vvp_out, "    %%evctl/c;\n");
+
       return 0;
 }
 
@@ -1284,21 +1285,8 @@ static int show_stmt_delay(ivl_statement_t net, ivl_scope_t sscope)
       return rc;
 }
 
-/*
- * The delayx statement is slightly more complex in that it is
- * necessary to calculate the delay first. Load the calculated delay
- * into and index register and use the %delayx instruction to do the
- * actual delay.
- */
-static int show_stmt_delayx(ivl_statement_t net, ivl_scope_t sscope)
+static void draw_expr_into_idx(ivl_expr_t expr, int use_idx)
 {
-      int rc = 0;
-      ivl_expr_t expr = ivl_stmt_delay_expr(net);
-      ivl_statement_t stmt = ivl_stmt_sub_stmt(net);
-
-      show_stmt_file_line(net, "Delay statement.");
-
-      int use_idx = allocate_word();
       switch (ivl_expr_value(expr)) {
 
 	  case IVL_VT_BOOL:
@@ -1317,6 +1305,25 @@ static int show_stmt_delayx(ivl_statement_t net, ivl_scope_t sscope)
 	  default:
 	    assert(0);
       }
+}
+
+
+/*
+ * The delayx statement is slightly more complex in that it is
+ * necessary to calculate the delay first. Load the calculated delay
+ * into and index register and use the %delayx instruction to do the
+ * actual delay.
+ */
+static int show_stmt_delayx(ivl_statement_t net, ivl_scope_t sscope)
+{
+      int rc = 0;
+      ivl_expr_t expr = ivl_stmt_delay_expr(net);
+      ivl_statement_t stmt = ivl_stmt_sub_stmt(net);
+
+      show_stmt_file_line(net, "Delay statement.");
+
+      int use_idx = allocate_word();
+      draw_expr_into_idx(expr, use_idx);
 
       fprintf(vvp_out, "    %%delayx %d;\n", use_idx);
       clr_word(use_idx);
@@ -1334,8 +1341,13 @@ static int show_stmt_disable(ivl_statement_t net, ivl_scope_t sscope)
 
 	/* A normal disable statement. */
       if (target) {
-	    show_stmt_file_line(net, "Disable statement.");
-	    fprintf(vvp_out, "    %%disable S_%p;\n", target);
+	    if (ivl_stmt_flow_control(net)) {
+		  show_stmt_file_line(net, "Flow control disable statement.");
+		  fprintf(vvp_out, "    %%disable/flow S_%p;\n", target);
+	    } else {
+		  show_stmt_file_line(net, "Disable statement.");
+		  fprintf(vvp_out, "    %%disable S_%p;\n", target);
+	    }
 	/* A SystemVerilog disable fork statement. */
       } else {
 	    show_stmt_file_line(net, "Disable fork statement.");
@@ -1643,6 +1655,34 @@ static int show_stmt_trigger(ivl_statement_t net)
       return 0;
 }
 
+/*
+ * The non-blocking trigger statement is straight forward. All we have to
+ * do is write a single bit of fake data to the event object.
+ */
+static int show_stmt_nb_trigger(ivl_statement_t net)
+{
+      ivl_event_t ev = ivl_stmt_events(net, 0);
+      assert(ev);
+
+      show_stmt_file_line(net, "Non-blocking event trigger statement.");
+
+      ivl_expr_t expr = ivl_stmt_delay_expr(net);
+      int use_idx = allocate_word();
+      if (expr) {
+	    draw_expr_into_idx(expr, use_idx);
+      } else {
+	    fprintf(vvp_out, "    %%ix/load %d, 0, 0;\n", use_idx);
+      }
+
+      fprintf(vvp_out, "    %%event/nb E_%p, %d;\n", ev, use_idx);
+      clr_word(use_idx);
+	// FIXME: VVP needs to be updated to correctly support %event/nb
+      fprintf(stderr, "%s:%u: vvp.tgt sorry: ->> is not currently supported.\n",
+                      ivl_stmt_file(net), ivl_stmt_lineno(net));
+      vvp_errors += 1;
+      return 0;
+}
+
 static int show_stmt_utask(ivl_statement_t net)
 {
       ivl_scope_t task = ivl_stmt_call(net);
@@ -1784,7 +1824,7 @@ static int show_insert_method(ivl_statement_t net)
       int idx = allocate_word();
       assert(idx >= 0);
 	/* Save the queue maximum index value to an integer register. */
-      fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", idx, ivl_signal_array_count(var));
+      fprintf(vvp_out, "    %%ix/load %d, %u, 0;\n", idx, ivl_signal_array_count(var));
 
       ivl_type_t element_type = ivl_type_element(var_type);
 
@@ -1795,19 +1835,19 @@ static int show_insert_method(ivl_statement_t net)
       switch (ivl_type_base(element_type)) {
 	  case IVL_VT_REAL:
 	    draw_eval_real(parm2);
-	    fprintf(vvp_out, "    %%qinsert/real v%p_0, %u;\n",
+	    fprintf(vvp_out, "    %%qinsert/real v%p_0, %d;\n",
 	            var, idx);
 	    break;
 	  case IVL_VT_STRING:
 	    draw_eval_string(parm2);
-	    fprintf(vvp_out, "    %%qinsert/str v%p_0, %u;\n",
+	    fprintf(vvp_out, "    %%qinsert/str v%p_0, %d;\n",
 	            var, idx);
 	    break;
 	  default:
 	    draw_eval_vec4(parm2);
-	    fprintf(vvp_out, "    %%qinsert/v v%p_0, %u, %u;\n",
+	    fprintf(vvp_out, "    %%qinsert/v v%p_0, %d, %u;\n",
 	            var, idx,
-	            width_of_packed_type(element_type));
+	            ivl_type_packed_width(element_type));
 	    break;
       }
       return 0;
@@ -1837,7 +1877,7 @@ static int show_push_frontback_method(ivl_statement_t net, bool is_front)
       int idx = allocate_word();
       assert(idx >= 0);
 	/* Save the queue maximum index value to an integer register. */
-      fprintf(vvp_out, "    %%ix/load %u, %u, 0;\n", idx, ivl_signal_array_count(var));
+      fprintf(vvp_out, "    %%ix/load %d, %u, 0;\n", idx, ivl_signal_array_count(var));
 
       ivl_type_t element_type = ivl_type_element(var_type);
 
@@ -1845,19 +1885,19 @@ static int show_push_frontback_method(ivl_statement_t net, bool is_front)
       switch (ivl_type_base(element_type)) {
 	  case IVL_VT_REAL:
 	    draw_eval_real(parm1);
-	    fprintf(vvp_out, "    %%store/%s/r v%p_0, %u;\n",
+	    fprintf(vvp_out, "    %%store/%s/r v%p_0, %d;\n",
 	            type_code, var, idx);
 	    break;
 	  case IVL_VT_STRING:
 	    draw_eval_string(parm1);
-	    fprintf(vvp_out, "    %%store/%s/str v%p_0, %u;\n",
+	    fprintf(vvp_out, "    %%store/%s/str v%p_0, %d;\n",
 	            type_code, var, idx);
 	    break;
 	  default:
 	    draw_eval_vec4(parm1);
-	    fprintf(vvp_out, "    %%store/%s/v v%p_0, %u, %u;\n",
+	    fprintf(vvp_out, "    %%store/%s/v v%p_0, %d, %u;\n",
 	            type_code, var, idx,
-	            width_of_packed_type(element_type));
+	            ivl_type_packed_width(element_type));
 	    break;
       }
       clr_word(idx);
@@ -2383,6 +2423,10 @@ static int show_statement(ivl_statement_t net, ivl_scope_t sscope)
 	    rc += show_stmt_trigger(net);
 	    break;
 
+	  case IVL_ST_NB_TRIGGER:
+	    rc += show_stmt_nb_trigger(net);
+	    break;
+
 	  case IVL_ST_UTASK:
 	    rc += show_stmt_utask(net);
 	    break;
