@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2011-2013 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2012-2015 / Stephen Williams (steve@icarus.com),
+ * Copyright CERN 2016
  * @author Maciej Suminski (maciej.suminski@cern.ch)
  *
  *    This source code is free software; you can redistribute it
@@ -23,6 +24,7 @@
 # include  "subprogram.h"
 # include  "parse_types.h"
 # include  "scope.h"
+# include  "library.h"
 # include  <iostream>
 # include  <typeinfo>
 # include  <cstring>
@@ -54,29 +56,97 @@ bool Expression::symbolic_compare(const Expression*) const
       return false;
 }
 
-ExpAttribute::ExpAttribute(ExpName*bas, perm_string nam)
-: base_(bas), name_(nam)
+ExpAttribute::ExpAttribute(perm_string nam, list<Expression*>*args)
+: name_(nam), args_(args)
 {
 }
 
 ExpAttribute::~ExpAttribute()
 {
-	/* Different attributes can point to the same base so we cannot delete this here.
-	 * Look at the vhdl_range test with valgrind to see this issue. */
-//      delete base_;
+      if(args_) {
+	    for(list<Expression*>::iterator it = args_->begin();
+                    it != args_->end(); ++it) {
+		delete *it;
+            }
+      }
+
+      delete args_;
 }
 
-Expression*ExpAttribute::clone() const
-{
-      return new ExpAttribute(static_cast<ExpName*>(base_->clone()), name_);
+list<Expression*>*ExpAttribute::clone_args() const {
+      list<Expression*>*new_args = NULL;
+
+      if(args_) {
+	    for(list<Expression*>::iterator it = args_->begin();
+                    it != args_->end(); ++it) {
+		new_args->push_back((*it)->clone());
+            }
+      }
+
+      return new_args;
 }
 
-void ExpAttribute::visit(ExprVisitor& func)
+void ExpAttribute::visit_args(ExprVisitor& func)
 {
-      base_->visit(func);
+      func.down();
       func(this);
+
+      if(args_) {
+          for(list<Expression*>::iterator it = args_->begin();
+                    it != args_->end(); ++it) {
+              (*it)->visit(func);
+          }
+      }
+
+      func.up();
 }
 
+ExpObjAttribute::ExpObjAttribute(ExpName*base, perm_string name, list<Expression*>*args)
+: ExpAttribute(name, args), base_(base)
+{
+}
+
+ExpObjAttribute::~ExpObjAttribute()
+{
+    delete base_;
+}
+
+Expression*ExpObjAttribute::clone() const
+{
+      return new ExpObjAttribute(static_cast<ExpName*>(base_->clone()),
+                                 name_, clone_args());
+}
+
+void ExpObjAttribute::visit(ExprVisitor&func)
+{
+      func.down();
+      func(this);
+      visit_args(func);
+      base_->visit(func);
+      func.up();
+}
+
+ExpTypeAttribute::ExpTypeAttribute(const VType*base, perm_string name, list<Expression*>*args)
+: ExpAttribute(name, args), base_(base)
+{
+}
+
+Expression*ExpTypeAttribute::clone() const
+{
+      return new ExpTypeAttribute(base_, name_, clone_args());
+}
+
+void ExpTypeAttribute::visit(ExprVisitor&func)
+{
+      func.down();
+      func(this);
+      visit_args(func);
+      func.up();
+}
+
+const perm_string ExpAttribute::LEFT = perm_string::literal("left");
+const perm_string ExpAttribute::RIGHT = perm_string::literal("right");
+
 ExpBinary::ExpBinary(Expression*op1, Expression*op2)
 : operand1_(op1), operand2_(op2)
 {
@@ -88,21 +158,23 @@ ExpBinary::~ExpBinary()
       delete operand2_;
 }
 
-bool ExpBinary::eval_operand1(ScopeBase*scope, int64_t&val) const
+bool ExpBinary::eval_operand1(Entity*ent, ScopeBase*scope, int64_t&val) const
 {
-      return operand1_->evaluate(scope, val);
+      return operand1_->evaluate(ent, scope, val);
 }
 
-bool ExpBinary::eval_operand2(ScopeBase*scope, int64_t&val) const
+bool ExpBinary::eval_operand2(Entity*ent, ScopeBase*scope, int64_t&val) const
 {
-      return operand2_->evaluate(scope, val);
+      return operand2_->evaluate(ent, scope, val);
 }
 
-void ExpBinary::visit(ExprVisitor& func)
+void ExpBinary::visit(ExprVisitor&func)
 {
+      func.down();
+      func(this);
       operand1_->visit(func);
       operand2_->visit(func);
-      func(this);
+      func.up();
 }
 
 ExpUnary::ExpUnary(Expression*op1)
@@ -115,10 +187,12 @@ ExpUnary::~ExpUnary()
       delete operand1_;
 }
 
-void ExpUnary::visit(ExprVisitor& func)
+void ExpUnary::visit(ExprVisitor&func)
 {
-    operand1_->visit(func);
+    func.down();
     func(this);
+    operand1_->visit(func);
+    func.up();
 }
 
 ExpAggregate::ExpAggregate(std::list<element_t*>*el)
@@ -165,8 +239,11 @@ Expression* ExpAggregate::clone() const
       return new ExpAggregate(new_elements);
 }
 
-void ExpAggregate::visit(ExprVisitor& func)
+void ExpAggregate::visit(ExprVisitor&func)
 {
+    func.down();
+    func(this);
+
     for(std::vector<element_t*>::iterator it = elements_.begin();
             it != elements_.end(); ++it) {
         (*it)->extract_expression()->visit(func);
@@ -180,7 +257,7 @@ void ExpAggregate::visit(ExprVisitor& func)
         it->expr->visit(func);
     }
 
-    func(this);
+    func.up();
 }
 
 ExpAggregate::choice_t::choice_t(Expression*exp)
@@ -192,7 +269,7 @@ ExpAggregate::choice_t::choice_t()
 {
 }
 
-ExpAggregate::choice_t::choice_t(prange_t*rang)
+ExpAggregate::choice_t::choice_t(ExpRange*rang)
 : range_(rang)
 {
 }
@@ -203,7 +280,7 @@ ExpAggregate::choice_t::choice_t(const choice_t&other)
         expr_.reset(e->clone());
 
     if(other.range_.get())
-        range_.reset(new prange_t(*other.range_.get()));
+        range_.reset(static_cast<ExpRange*>(other.range_.get()->clone()));
 }
 
 ExpAggregate::choice_t::~choice_t()
@@ -221,7 +298,7 @@ Expression*ExpAggregate::choice_t::simple_expression(bool detach_flag)
       return res;
 }
 
-prange_t*ExpAggregate::choice_t::range_expressions(void)
+ExpRange*ExpAggregate::choice_t::range_expressions(void)
 {
       return range_.get();
 }
@@ -304,11 +381,13 @@ ExpConcat::~ExpConcat()
       delete operand2_;
 }
 
-void ExpConcat::visit(ExprVisitor& func)
+void ExpConcat::visit(ExprVisitor&func)
 {
-    operand1_->visit(func);
-    operand2_->visit(func);
-    func(this);
+      func.down();
+      func(this);
+      operand1_->visit(func);
+      operand2_->visit(func);
+      func.up();
 }
 
 ExpConditional::ExpConditional(Expression*co, list<Expression*>*tru,
@@ -342,14 +421,16 @@ Expression*ExpConditional::clone() const
       return new ExpConditional(NULL, NULL, new_options);
 }
 
-void ExpConditional::visit(ExprVisitor& func)
+void ExpConditional::visit(ExprVisitor&func)
 {
-      for(std::list<case_t*>::iterator it = options_.begin();
-              it != options_.end(); ++it) {
-          (*it)->visit(func);
-      }
-
+      func.down();
       func(this);
+
+      for(std::list<case_t*>::iterator it = options_.begin();
+              it != options_.end(); ++it)
+          (*it)->visit(func);
+
+      func.up();
 }
 
 ExpConditional::case_t::case_t(Expression*cond, std::list<Expression*>*tru)
@@ -411,15 +492,16 @@ Expression*ExpSelected::clone() const
       return new ExpSelected(selector_->clone(), new_options);
 }
 
-void ExpConditional::case_t::visit(ExprVisitor& func)
+void ExpConditional::case_t::visit(ExprVisitor&func)
 {
+      func.down();
       if(cond_)
-          func(cond_);
+          cond_->visit(func);
 
       for(std::list<Expression*>::iterator it = true_clause_.begin();
-              it != true_clause_.end(); ++it) {
-          func(*it);
-      }
+              it != true_clause_.end(); ++it)
+          (*it)->visit(func);
+      func.up();
 }
 
 ExpEdge::ExpEdge(ExpEdge::fun_t typ, Expression*op)
@@ -469,14 +551,18 @@ Expression*ExpFunc::clone() const {
     return f;
 }
 
-void ExpFunc::visit(ExprVisitor& func) {
+void ExpFunc::visit(ExprVisitor&func)
+{
+    func.down();
+    func(this);
+
     if(!argv_.empty()) {
         for(std::vector<Expression*>::iterator it = argv_.begin();
                 it != argv_.end(); ++it)
             (*it)->visit(func);
     }
 
-    func(this);
+    func.up();
 }
 
 const VType* ExpFunc::func_ret_type() const
@@ -484,6 +570,32 @@ const VType* ExpFunc::func_ret_type() const
     return def_ ? def_->peek_return_type() : NULL;
 }
 
+SubprogramHeader*ExpFunc::match_signature(Entity*ent, ScopeBase*scope) const
+{
+    SubprogramHeader*prog = NULL;
+    list<const VType*> arg_types;
+
+      // Create a list of argument types to find a matching subprogram
+    for(vector<Expression*>::const_iterator it = argv_.begin();
+            it != argv_.end(); ++it) {
+        arg_types.push_back((*it)->probe_type(ent, scope));
+    }
+
+    prog = scope->match_subprogram(name_, &arg_types);
+
+    if(!prog)
+        prog = library_match_subprogram(name_, &arg_types);
+
+    if(!prog) {
+        cerr << get_fileline() << ": sorry: could not find function ";
+        emit_subprogram_sig(cerr, name_, arg_types);
+        cerr << endl;
+        ivl_assert(*this, false);
+    }
+
+    return prog;
+}
+
 ExpInteger::ExpInteger(int64_t val)
 : value_(val)
 {
@@ -493,7 +605,7 @@ ExpInteger::~ExpInteger()
 {
 }
 
-bool ExpInteger::evaluate(ScopeBase*, int64_t&val) const
+bool ExpInteger::evaluate(Entity*, ScopeBase*, int64_t&val) const
 {
       val = value_;
       return true;
@@ -518,39 +630,54 @@ ExpLogical::~ExpLogical()
 }
 
 ExpName::ExpName(perm_string nn)
-: name_(nn), index_(0), lsb_(0)
+: name_(nn), indices_(NULL)
 {
 }
 
 ExpName::ExpName(perm_string nn, list<Expression*>*indices)
-: name_(nn), index_(0), lsb_(0)
-{
-	/* For now, assume a single index. */
-      ivl_assert(*this, indices->size() == 1);
-
-      index_ = indices->front();
-      indices->pop_front();
-}
-
-ExpName::ExpName(perm_string nn, Expression*msb, Expression*lsb)
-: name_(nn), index_(msb), lsb_(lsb)
+: name_(nn), indices_(indices)
 {
 }
 
-ExpName::ExpName(ExpName*prefix, perm_string nn)
-: prefix_(prefix), name_(nn), index_(0), lsb_(0)
-{
-}
-
-ExpName::ExpName(ExpName*prefix, perm_string nn, Expression*msb, Expression*lsb)
-: prefix_(prefix), name_(nn), index_(msb), lsb_(lsb)
+ExpName::ExpName(ExpName*prefix, perm_string nn, std::list<Expression*>*indices)
+: prefix_(prefix), name_(nn), indices_(indices)
 {
 }
 
 ExpName::~ExpName()
 {
-      delete index_;
-      delete lsb_;
+    if(indices_) {
+        for(list<Expression*>::iterator it = indices_->begin();
+                it != indices_->end(); ++it) {
+            delete *it;
+        }
+
+        delete indices_;
+    }
+}
+
+Expression*ExpName::clone() const {
+    list<Expression*>*new_indices = NULL;
+
+    if(indices_) {
+        new_indices = new list<Expression*>();
+
+        for(list<Expression*>::const_iterator it = indices_->begin();
+                it != indices_->end(); ++it) {
+            new_indices->push_back((*it)->clone());
+        }
+    }
+
+    return new ExpName(static_cast<ExpName*>(safe_clone(prefix_.get())),
+            name_, new_indices);
+}
+
+void ExpName::add_index(std::list<Expression*>*idx)
+{
+      if(!indices_)
+          indices_ = new list<Expression*>();
+
+      indices_->splice(indices_->end(), *idx);
 }
 
 bool ExpName::symbolic_compare(const Expression*that) const
@@ -562,42 +689,69 @@ bool ExpName::symbolic_compare(const Expression*that) const
       if (name_ != that_name->name_)
 	    return false;
 
-      if (that_name->index_ && !index_)
+      if (that_name->indices_ && !indices_)
 	    return false;
-      if (index_ && !that_name->index_)
+      if (indices_ && !that_name->indices_)
 	    return false;
 
-      if (index_) {
-	    assert(that_name->index_);
-	    return index_->symbolic_compare(that_name->index_);
+      if (indices_) {
+	    assert(that_name->indices_);
+
+            if(indices_->size() != that_name->indices_->size())
+                return false;
+
+            list<Expression*>::const_iterator it, jt;
+            it = indices_->begin();
+            jt = that_name->indices_->begin();
+
+            for(unsigned int i = 0; i < indices_->size(); ++i) {
+                if(!(*it)->symbolic_compare(*jt))
+                    return false;
+
+                ++it;
+                ++jt;
+            }
       }
 
       return true;
 }
 
-void ExpName::set_range(Expression*msb, Expression*lsb)
+Expression*ExpName::index(unsigned int number) const
 {
-      assert(index_==0);
-      index_ = msb;
-      assert(lsb_==0);
-      lsb_ = lsb;
+    if(!indices_)
+        return NULL;
+
+    if(number >= indices_->size())
+        return NULL;
+
+    if(number == 0)
+        return indices_->front();
+
+    list<Expression*>::const_iterator it = indices_->begin();
+    advance(it, number);
+
+    return *it;
 }
 
-void ExpName::visit(ExprVisitor& func)
+void ExpName::visit(ExprVisitor&func)
 {
+      func.down();
+      func(this);
+
       if(prefix_.get())
           prefix_.get()->visit(func);
 
-      if(index_)
-          index_->visit(func);
+      if(indices_) {
+          for(list<Expression*>::const_iterator it = indices_->begin();
+                  it != indices_->end(); ++it) {
+              (*it)->visit(func);
+          }
+      }
 
-      if(lsb_)
-          lsb_->visit(func);
-
-      func(this);
+      func.up();
 }
 
-int ExpName::index_t::emit(ostream&out, Entity*ent, ScopeBase*scope)
+int ExpName::index_t::emit(ostream&out, Entity*ent, ScopeBase*scope) const
 {
       int errors = 0;
 
@@ -628,16 +782,45 @@ ExpRelation::~ExpRelation()
 {
 }
 
+ExpScopedName::ExpScopedName(perm_string scope, ExpName*exp)
+: scope_name_(scope), scope_(NULL), name_(exp)
+{
+}
+
+ExpScopedName::~ExpScopedName()
+{
+    delete name_;
+}
+
+void ExpScopedName::visit(ExprVisitor&func)
+{
+    func.down();
+    func(this);
+    name_->visit(func);
+    func.up();
+}
+
+ScopeBase*ExpScopedName::get_scope(const ScopeBase*scope)
+{
+    if(!scope_)
+        scope_ = scope->find_scope(scope_name_);
+
+    return scope_;
+}
+
+ScopeBase*ExpScopedName::get_scope(const ScopeBase*scope) const
+{
+    return scope_ ? scope_ : scope->find_scope(scope_name_);
+}
+
 ExpShift::ExpShift(ExpShift::shift_t op, Expression*op1, Expression*op2)
 : ExpBinary(op1, op2), shift_(op)
 {
 }
 
 ExpString::ExpString(const char* value)
-: value_(strlen(value))
+: value_(value)
 {
-      for(size_t idx = 0; idx < value_.size(); idx += 1)
-	    value_[idx] = value[idx];
 }
 
 ExpString::~ExpString()
@@ -662,6 +845,15 @@ ExpUNot::~ExpUNot()
 {
 }
 
+ExpUMinus::ExpUMinus(Expression*op1)
+: ExpUnary(op1)
+{
+}
+
+ExpUMinus::~ExpUMinus()
+{
+}
+
 ExpCast::ExpCast(Expression*base, const VType*type) :
     base_(base), type_(type)
 {
@@ -671,10 +863,12 @@ ExpCast::~ExpCast()
 {
 }
 
-void ExpCast::visit(ExprVisitor& func)
+void ExpCast::visit(ExprVisitor&func)
 {
-    base_->visit(func);
+    func.down();
     func(this);
+    base_->visit(func);
+    func.up();
 }
 
 ExpNew::ExpNew(Expression*size) :
@@ -687,10 +881,12 @@ ExpNew::~ExpNew()
     delete size_;
 }
 
-void ExpNew::visit(ExprVisitor& func)
+void ExpNew::visit(ExprVisitor&func)
 {
-    size_->visit(func);
+    func.down();
     func(this);
+    size_->visit(func);
+    func.up();
 }
 
 ExpTime::ExpTime(uint64_t amount, timeunit_t unit)
@@ -714,3 +910,95 @@ double ExpTime::to_fs() const
 
     return val;
 }
+
+ExpRange::ExpRange(Expression*left_idx, Expression*right_idx, range_dir_t dir)
+: left_(left_idx), right_(right_idx), direction_(dir), range_expr_(false),
+    range_base_(NULL)
+{
+}
+
+ExpRange::ExpRange(ExpName*base, bool reverse_range)
+: left_(NULL), right_(NULL), direction_(AUTO), range_expr_(true),
+    range_base_(base), range_reverse_(reverse_range)
+{
+}
+
+ExpRange::~ExpRange()
+{
+    delete left_;
+    delete right_;
+    delete range_base_;
+}
+
+Expression*ExpRange::clone() const
+{
+    if(range_expr_)
+        return new ExpRange(static_cast<ExpName*>(range_base_->clone()), range_reverse_);
+    else
+        return new ExpRange(left_->clone(), right_->clone(), direction_);
+}
+
+Expression* ExpRange::msb()
+{
+    ivl_assert(*this, direction() != AUTO);
+
+    switch(direction()) {
+        case DOWNTO: return left_;
+        case TO: return right_;
+        default: return NULL;
+    }
+
+    return NULL;
+}
+
+Expression* ExpRange::lsb()
+{
+    ivl_assert(*this, direction() != AUTO);
+
+    switch(direction()) {
+        case DOWNTO: return right_;
+        case TO: return left_;
+        default: return NULL;
+    }
+
+    return NULL;
+}
+
+Expression*ExpRange::left()
+{
+    if(range_expr_ && !left_)
+        // TODO check if it is an object or type
+        left_ = new ExpObjAttribute(static_cast<ExpName*>(range_base_->clone()),
+                                    ExpAttribute::LEFT, NULL);
+
+    return left_;
+}
+
+Expression*ExpRange::right()
+{
+    if(range_expr_ && !right_)
+        // TODO check if it is an object or type
+        right_ = new ExpObjAttribute(static_cast<ExpName*>(range_base_->clone()),
+                                    ExpAttribute::RIGHT, NULL);
+    return right_;
+}
+
+ExpDelay::ExpDelay(Expression*expr, Expression*delay)
+: expr_(expr), delay_(delay)
+{
+}
+
+ExpDelay::~ExpDelay()
+{
+    delete expr_;
+    delete delay_;
+}
+
+void ExpDelay::visit(ExprVisitor&func)
+{
+    func.down();
+    func(this);
+    expr_->visit(func);
+    delay_->visit(func);
+    func.up();
+}
