@@ -1,7 +1,7 @@
 #ifndef IVL_expression_H
 #define IVL_expression_H
 /*
- * Copyright (c) 2011-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2011-2021 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2015 / Stephen Williams (steve@icarus.com),
  * Copyright CERN 2016
  * @author Maciej Suminski (maciej.suminski@cern.ch)
@@ -118,7 +118,7 @@ class Expression : public LineInfo {
 	// The emit virtual method is called by architecture emit to
 	// output the generated code for the expression. The derived
 	// class fills in the details of what exactly happened.
-      virtual int emit(ostream&out, Entity*ent, ScopeBase*scope) const =0;
+      virtual int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const =0;
 
 	// The emit_package virtual message is similar, but is called
 	// in a package context and to emit SV packages.
@@ -143,8 +143,8 @@ class Expression : public LineInfo {
       virtual bool is_primary(void) const;
 
 	// Debug dump of the expression.
-      virtual void dump(ostream&out, int indent = 0) const =0;
-      virtual ostream& dump_inline(ostream&out) const;
+      virtual void dump(std::ostream&out, int indent = 0) const =0;
+      virtual std::ostream& dump_inline(std::ostream&out) const;
 
 	// Recursively visits a tree of expressions (useful for complex expressions).
       virtual void visit(ExprVisitor& func) { func.down(); func(this); func.up(); }
@@ -175,7 +175,7 @@ static inline void FILE_NAME(Expression*tgt, const LineInfo*src)
       tgt->set_line(*src);
 }
 
-static inline ostream& operator <<(ostream&out, const Expression&exp)
+static inline std::ostream& operator <<(std::ostream&out, const Expression&exp)
 {
       return exp.dump_inline(out);
 }
@@ -197,8 +197,8 @@ class ExpUnary : public Expression {
       inline void write_to_stream_operand1(std::ostream&fd) const
       { operand1_->write_to_stream(fd); }
 
-      int emit_operand1(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump_operand1(ostream&out, int indent = 0) const;
+      int emit_operand1(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump_operand1(std::ostream&out, int indent = 0) const;
 
     private:
       Expression*operand1_;
@@ -223,8 +223,8 @@ class ExpBinary : public Expression {
     protected:
 
       int elaborate_exprs(Entity*, ScopeBase*, const VType*);
-      int emit_operand1(ostream&out, Entity*ent, ScopeBase*scope) const;
-      int emit_operand2(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit_operand1(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit_operand2(std::ostream&out, Entity*ent, ScopeBase*scope) const;
 
       bool eval_operand1(Entity*ent, ScopeBase*scope, int64_t&val) const;
       bool eval_operand2(Entity*ent, ScopeBase*scope, int64_t&val) const;
@@ -234,7 +234,7 @@ class ExpBinary : public Expression {
       inline void write_to_stream_operand2(std::ostream&out) const
           { operand2_->write_to_stream(out); }
 
-      void dump_operands(ostream&out, int indent = 0) const;
+      void dump_operands(std::ostream&out, int indent = 0) const;
 
     private:
       virtual const VType*resolve_operand_types_(const VType*t1, const VType*t2) const;
@@ -274,7 +274,7 @@ class ExpAggregate : public Expression {
 	    ExpRange*range_expressions(void);
 
 	    void write_to_stream(std::ostream&fd);
-	    void dump(ostream&out, int indent) const;
+	    void dump(std::ostream&out, int indent) const;
 
 	  private:
 	    std::unique_ptr<Expression>expr_;
@@ -291,6 +291,10 @@ class ExpAggregate : public Expression {
 	        expr = safe_clone(other.expr);
 	    }
 
+# if __cplusplus >= 201103L
+	    choice_element& operator = (const choice_element&that) = default;
+# endif
+
 	    choice_t*choice;
 	    Expression*expr;
 	    bool alias_flag;
@@ -311,7 +315,7 @@ class ExpAggregate : public Expression {
 	    inline Expression* extract_expression() { return val_; }
 	    void write_to_stream(std::ostream&fd) const;
 
-	    void dump(ostream&out, int indent) const;
+	    void dump(std::ostream&out, int indent) const;
 
 	  private:
 	    std::vector<choice_t*>fields_;
@@ -329,15 +333,15 @@ class ExpAggregate : public Expression {
       const VType*fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*atype) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
       void visit(ExprVisitor& func);
 
     private:
       int elaborate_expr_array_(Entity*ent, ScopeBase*scope, const VTypeArray*ltype);
       int elaborate_expr_record_(Entity*ent, ScopeBase*scope, const VTypeRecord*ltype);
-      int emit_array_(ostream&out, Entity*ent, ScopeBase*scope, const VTypeArray*ltype) const;
-      int emit_record_(ostream&out, Entity*ent, ScopeBase*scope, const VTypeRecord*ltype) const;
+      int emit_array_(std::ostream&out, Entity*ent, ScopeBase*scope, const VTypeArray*ltype) const;
+      int emit_record_(std::ostream&out, Entity*ent, ScopeBase*scope, const VTypeRecord*ltype) const;
 
     private:
 	// This is the elements as directly parsed.
@@ -363,9 +367,9 @@ class ExpArithmetic : public ExpBinary {
 
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       virtual bool evaluate(Entity*ent, ScopeBase*scope, int64_t&val) const;
-      void dump(ostream&out, int indent = 0) const;
+      void dump(std::ostream&out, int indent = 0) const;
 
     private:
       const VType* resolve_operand_types_(const VType*t1, const VType*t2) const;
@@ -407,13 +411,13 @@ class ExpObjAttribute : public ExpAttribute {
 
       inline const ExpName* peek_base() const { return base_; }
 
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       const VType*probe_type(Entity*ent, ScopeBase*scope) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
 	// Some attributes can be evaluated at compile time
       bool evaluate(Entity*ent, ScopeBase*scope, int64_t&val) const;
-      void dump(ostream&out, int indent = 0) const;
+      void dump(std::ostream&out, int indent = 0) const;
       void visit(ExprVisitor& func);
 
     private:
@@ -429,14 +433,14 @@ class ExpTypeAttribute : public ExpAttribute {
 
       inline const VType* peek_base() const { return base_; }
 
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       const VType*probe_type(Entity*ent, ScopeBase*scope) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
 	// Some attributes can be evaluated at compile time
       bool evaluate(ScopeBase*scope, int64_t&val) const;
       bool evaluate(Entity*ent, ScopeBase*scope, int64_t&val) const;
-      void dump(ostream&out, int indent = 0) const;
+      void dump(std::ostream&out, int indent = 0) const;
       void visit(ExprVisitor& func);
 
     private:
@@ -455,8 +459,8 @@ class ExpBitstring : public Expression {
       const VType*fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*atype) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
 
     private:
       std::vector<char>value_;
@@ -475,14 +479,14 @@ class ExpCharacter : public Expression {
       const VType*fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*atype) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       bool is_primary(void) const;
-      void dump(ostream&out, int indent = 0) const;
+      void dump(std::ostream&out, int indent = 0) const;
 
       char value() const { return value_; }
 
     private:
-      int emit_primitive_bit_(ostream&out, Entity*ent, ScopeBase*scope,
+      int emit_primitive_bit_(std::ostream&out, Entity*ent, ScopeBase*scope,
 			      const VTypePrimitive*etype) const;
 
     private:
@@ -503,9 +507,9 @@ class ExpConcat : public Expression {
       const VType*fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*atype) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       bool is_primary(void) const;
-      void dump(ostream&out, int indent = 0) const;
+      void dump(std::ostream&out, int indent = 0) const;
       void visit(ExprVisitor& func);
 
     private:
@@ -535,9 +539,9 @@ class ExpConditional : public Expression {
 	    inline const std::list<Expression*>& true_clause() const { return true_clause_; }
 
 	    int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*lt);
-	    int emit_option(ostream&out, Entity*ent, ScopeBase*scope) const;
-	    int emit_default(ostream&out, Entity*ent, ScopeBase*scope) const;
-	    void dump(ostream&out, int indent = 0) const;
+	    int emit_option(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+	    int emit_default(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+	    void dump(std::ostream&out, int indent = 0) const;
             std::list<Expression*>& extract_true_clause() { return true_clause_; }
             void visit(ExprVisitor& func);
 
@@ -556,8 +560,8 @@ class ExpConditional : public Expression {
       const VType*probe_type(Entity*ent, ScopeBase*scope) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
       void visit(ExprVisitor& func);
 
     protected:
@@ -596,8 +600,8 @@ class ExpEdge : public ExpUnary {
       inline fun_t edge_fun() const { return fun_; }
 
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
 
     private:
       fun_t fun_;
@@ -622,8 +626,8 @@ class ExpFunc : public Expression {
       const VType*probe_type(Entity*ent, ScopeBase*scope) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
       void visit(ExprVisitor& func); // NOTE: does not handle expressions in subprogram body
 
       // Returns a subprogram header that matches the function call
@@ -647,12 +651,12 @@ class ExpInteger : public Expression {
       const VType*probe_type(Entity*ent, ScopeBase*scope) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       int emit_package(std::ostream&out) const;
       bool is_primary(void) const { return true; }
       bool evaluate(Entity*ent, ScopeBase*scope, int64_t&val) const;
-      void dump(ostream&out, int indent = 0) const;
-      virtual ostream& dump_inline(ostream&out) const;
+      void dump(std::ostream&out, int indent = 0) const;
+      virtual std::ostream& dump_inline(std::ostream&out) const;
 
     private:
       int64_t value_;
@@ -670,11 +674,11 @@ class ExpReal : public Expression {
       const VType*probe_type(Entity*ent, ScopeBase*scope) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       int emit_package(std::ostream&out) const;
       bool is_primary(void) const;
-      void dump(ostream&out, int indent = 0) const;
-      virtual ostream& dump_inline(ostream&out) const;
+      void dump(std::ostream&out, int indent = 0) const;
+      virtual std::ostream& dump_inline(std::ostream&out) const;
 
     private:
       double value_;
@@ -697,8 +701,8 @@ class ExpLogical : public ExpBinary {
 
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
 
     private:
       fun_t fun_;
@@ -725,12 +729,12 @@ class ExpName : public Expression {
       const VType* fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*host) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit_indices(ostream&out, Entity*ent, ScopeBase*scope) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit_indices(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       bool is_primary(void) const;
       bool evaluate(Entity*ent, ScopeBase*scope, int64_t&val) const;
       bool symbolic_compare(const Expression*that) const;
-      void dump(ostream&out, int indent = 0) const;
+      void dump(std::ostream&out, int indent = 0) const;
       inline const char* name() const { return name_; }
       inline const perm_string& peek_name() const { return name_; }
       void add_index(std::list<Expression*>*idx);
@@ -747,7 +751,7 @@ class ExpName : public Expression {
                 delete offset_;
           }
 
-          int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+          int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
 
       private:
           Expression*idx_;
@@ -761,23 +765,23 @@ class ExpName : public Expression {
       const VType* probe_prefix_type_(Entity*ent, ScopeBase*scope) const;
       const VType* probe_prefixed_type_(Entity*ent, ScopeBase*scope) const;
 
-      int emit_as_prefix_(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit_as_prefix_(std::ostream&out, Entity*ent, ScopeBase*scope) const;
 
 	// There are some workarounds required for constant arrays/records, as
 	// they are currently emitted as flat localparams (without any type
 	// information). The following workarounds adjust the access indices
 	// to select appropriate parts of the localparam.
-      bool try_workarounds_(ostream&out, Entity*ent, ScopeBase*scope,
-                            list<index_t*>&indices, int&data_size) const;
+      bool try_workarounds_(std::ostream&out, Entity*ent, ScopeBase*scope,
+                            std::list<index_t*>&indices, int&data_size) const;
 
       bool check_const_array_workaround_(const VTypeArray*arr, ScopeBase*scope,
-                                         list<index_t*>&indices, int&data_size) const;
+                                         std::list<index_t*>&indices, int&data_size) const;
 
       bool check_const_record_workaround_(const VTypeRecord*rec, ScopeBase*scope,
-                                          list<index_t*>&indices, int&data_size) const;
+                                          std::list<index_t*>&indices, int&data_size) const;
 
-      int emit_workaround_(ostream&out, Entity*ent, ScopeBase*scope,
-                           const list<index_t*>&indices, int field_size) const;
+      int emit_workaround_(std::ostream&out, Entity*ent, ScopeBase*scope,
+                           const std::list<index_t*>&indices, int field_size) const;
 
     private:
       Expression*index(unsigned int number) const;
@@ -794,7 +798,7 @@ class ExpNameALL : public ExpName {
 
     public:
       const VType* probe_type(Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent =0) const;
+      void dump(std::ostream&out, int indent =0) const;
 };
 
 class ExpRelation : public ExpBinary {
@@ -815,8 +819,8 @@ class ExpRelation : public ExpBinary {
       const VType* probe_type(Entity*ent, ScopeBase*scope) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
 
     private:
       fun_t fun_;
@@ -852,7 +856,7 @@ class ExpScopedName : public Expression {
         void write_to_stream(std::ostream&fd) const
         { name_->write_to_stream(fd); }
 
-        int emit(ostream&out, Entity*ent, ScopeBase*scope) const {
+        int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const {
             out << scope_name_ << ".";
             return name_->emit(out, ent, scope);
         }
@@ -866,7 +870,7 @@ class ExpScopedName : public Expression {
         bool symbolic_compare(const Expression*that) const
         { return name_->symbolic_compare(that); }
 
-        void dump(ostream&out, int indent = 0) const;
+        void dump(std::ostream&out, int indent = 0) const;
 
         void visit(ExprVisitor&func);
 
@@ -893,9 +897,9 @@ class ExpShift : public ExpBinary {
 
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       bool evaluate(Entity*ent, ScopeBase*scope, int64_t&val) const;
-      void dump(ostream&out, int indent = 0) const;
+      void dump(std::ostream&out, int indent = 0) const;
 
     private:
       shift_t shift_;
@@ -913,9 +917,9 @@ class ExpString : public Expression {
       const VType*fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*atype) const;
       int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
       bool is_primary(void) const;
-      void dump(ostream&out, int indent = 0) const;
+      void dump(std::ostream&out, int indent = 0) const;
       const std::string& get_value() const { return value_; }
 
 	// Converts quotation marks (") to its escaped
@@ -923,7 +927,7 @@ class ExpString : public Expression {
       static std::string escape_quot(const std::string& str);
 
     private:
-      int emit_as_array_(ostream&out, Entity*ent, ScopeBase*scope, const VTypeArray*arr) const;
+      int emit_as_array_(std::ostream&out, Entity*ent, ScopeBase*scope, const VTypeArray*arr) const;
 
     private:
       std::string value_;
@@ -938,8 +942,8 @@ class ExpUAbs : public ExpUnary {
       Expression*clone() const { return new ExpUAbs(peek_operand()->clone()); }
 
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
 };
 
 class ExpUNot : public ExpUnary {
@@ -951,8 +955,8 @@ class ExpUNot : public ExpUnary {
       Expression*clone() const { return new ExpUNot(peek_operand()->clone()); }
 
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
 };
 
 class ExpUMinus : public ExpUnary {
@@ -964,8 +968,8 @@ class ExpUMinus : public ExpUnary {
       Expression*clone() const { return new ExpUMinus(peek_operand()->clone()); }
 
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
 };
 
 /*
@@ -983,8 +987,8 @@ class ExpCast : public Expression {
             return base_->elaborate_expr(ent, scope, type_);
       }
       void write_to_stream(std::ostream&fd) const;
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
       void visit(ExprVisitor& func);
 
     private:
@@ -1006,8 +1010,8 @@ class ExpNew : public Expression {
 
       // There is no 'new' in VHDL - do not emit anything
       void write_to_stream(std::ostream&) const {};
-      int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-      void dump(ostream&out, int indent = 0) const;
+      int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+      void dump(std::ostream&out, int indent = 0) const;
       void visit(ExprVisitor& func);
 
     private:
@@ -1024,9 +1028,9 @@ class ExpTime : public Expression {
 
         int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
         void write_to_stream(std::ostream&) const;
-        int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
+        int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
         //bool evaluate(Entity*ent, ScopeBase*scope, int64_t&val) const;
-        void dump(ostream&out, int indent = 0) const;
+        void dump(std::ostream&out, int indent = 0) const;
 
     private:
         // Returns the time value expressed in femtoseconds
@@ -1059,8 +1063,8 @@ class ExpRange : public Expression {
 
         int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
         void write_to_stream(std::ostream&) const;
-        int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-        void dump(ostream&out, int indent = 0) const;
+        int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+        void dump(std::ostream&out, int indent = 0) const;
     private:
         // Regular range related fields
         Expression*left_, *right_;
@@ -1085,8 +1089,8 @@ public:
 
     int elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype);
     void write_to_stream(std::ostream&) const;
-    int emit(ostream&out, Entity*ent, ScopeBase*scope) const;
-    void dump(ostream&out, int indent = 0) const;
+    int emit(std::ostream&out, Entity*ent, ScopeBase*scope) const;
+    void dump(std::ostream&out, int indent = 0) const;
     void visit(ExprVisitor& func);
 
     const Expression*peek_expr() const { return expr_; }
