@@ -1,6 +1,8 @@
 /*
  * Copyright (c) 2011-2013 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2012-2013 / Stephen Williams (steve@icarus.com)
+ * Copyright CERN 2016
+ * @author Maciej Suminski (maciej.suminski@cern.ch)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -24,7 +26,6 @@
 # include  "entity.h"
 # include  "vsignal.h"
 # include  "subprogram.h"
-# include  "library.h"
 # include  "std_types.h"
 # include  <iostream>
 # include  <typeinfo>
@@ -57,7 +58,8 @@ const VType* Expression::fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*
       return res;
 }
 
-const VType*ExpName::elaborate_adjust_type_with_range_(Entity*, ScopeBase*scope, const VType*type)
+const VType*ExpName::elaborate_adjust_type_with_range_(Entity*ent, ScopeBase*scope,
+                                                       const VType*type)
 {
 	// Unfold typedefs
       while (const VTypeDef*tdef = dynamic_cast<const VTypeDef*>(type)) {
@@ -65,23 +67,24 @@ const VType*ExpName::elaborate_adjust_type_with_range_(Entity*, ScopeBase*scope,
       }
 
       if (const VTypeArray*array = dynamic_cast<const VTypeArray*>(type)) {
-	    if (index_ && !lsb_) {
-		    // If the name is an array or a vector, then an
-		    // indexed name has the type of the element.
-		  type = array->element_type();
+	    Expression*idx = index(0);
 
-	    } else if (index_ && lsb_) {
+	    if (ExpRange*range = dynamic_cast<ExpRange*>(idx)) {
 		    // If the name is an array, then a part select is
 		    // also an array, but with different bounds.
 		  int64_t use_msb, use_lsb;
-		  bool flag;
+		  bool flag = true;
 
-		  flag = index_->evaluate(scope, use_msb);
-		  ivl_assert(*this, flag);
-		  flag = lsb_->evaluate(scope, use_lsb);
-		  ivl_assert(*this, flag);
+		  flag &= range->msb()->evaluate(ent, scope, use_msb);
+		  flag &= range->lsb()->evaluate(ent, scope, use_lsb);
 
-		  type = new VTypeArray(array->element_type(), use_msb, use_lsb);
+                  if(flag)
+                    type = new VTypeArray(array->element_type(), use_msb, use_lsb);
+	    }
+	    else if(idx) {
+		    // If the name is an array or a vector, then an
+		    // indexed name has the type of the element.
+		  type = array->element_type();
 	    }
       }
 
@@ -96,10 +99,14 @@ int ExpName::elaborate_lval_(Entity*ent, ScopeBase*scope, bool is_sequ, ExpName*
 	    debug_log_file << get_fileline() << ": ExpName::elaborate_lval_: "
 			   << "name_=" << name_
 			   << ", suffix->name()=" << suffix->name();
-	    if (index_)
-		  debug_log_file << ", index_=" << *index_;
-	    if (lsb_)
-		  debug_log_file << ", lsb_=" << *lsb_;
+	    if (indices_) {
+		for(list<Expression*>::const_iterator it = indices_->begin();
+		        it != indices_->end(); ++it) {
+		    debug_log_file << "[";
+		    debug_log_file << **it;
+		    debug_log_file << "]";
+		}
+	    }
 	    debug_log_file << endl;
       }
 
@@ -216,36 +223,44 @@ int ExpName::elaborate_lval(Entity*ent, ScopeBase*scope, bool is_sequ)
 
       const VType*found_type = 0;
 
-      if (const InterfacePort*cur = ent->find_port(name_)) {
-	    if (cur->mode != PORT_OUT && cur->mode != PORT_INOUT) {
-		  cerr << get_fileline() << ": error: Assignment to "
-			"input port " << name_ << "." << endl;
-		  return errors += 1;
-	    }
+      if (ent) {
+          if (const InterfacePort*cur = ent->find_port(name_)) {
+                  if (cur->mode != PORT_OUT && cur->mode != PORT_INOUT) {
+                      cerr << get_fileline() << ": error: Assignment to "
+                              "input port " << name_ << "." << endl;
+                      return errors += 1;
+                  }
 
-	    if (is_sequ)
-		  ent->set_declaration_l_value(name_, is_sequ);
+                  if (is_sequ)
+                      ent->set_declaration_l_value(name_, is_sequ);
 
-	    found_type = cur->type;
+                  found_type = cur->type;
 
-      } else if (ent->find_generic(name_)) {
+          } else if (ent->find_generic(name_)) {
 
-	    cerr << get_fileline() << ": error: Assignment to generic "
-		 << name_ << " from entity "
-		 << ent->get_name() << "." << endl;
-	    return 1;
+                  cerr << get_fileline() << ": error: Assignment to generic "
+                      << name_ << " from entity "
+                      << ent->get_name() << "." << endl;
+                  return 1;
+          }
+      }
 
-      } else if (Signal*sig = scope->find_signal(name_)) {
-	      // Tell the target signal that this may be a sequential l-value.
-	    if (is_sequ) sig->count_ref_sequ();
+      if (!found_type && scope) {
+        if (Signal*sig = scope->find_signal(name_)) {
+            // Tell the target signal that this may be a sequential l-value.
+            if (is_sequ) sig->count_ref_sequ();
 
-	    found_type = sig->peek_type();
+            found_type = sig->peek_type();
 
-      } else if (Variable*var = scope->find_variable(name_)) {
-	      // Tell the target signal that this may be a sequential l-value.
-	    if (is_sequ) var->count_ref_sequ();
+        } else if (Variable*var = scope->find_variable(name_)) {
+                // Tell the target signal that this may be a sequential l-value.
+                if (is_sequ) var->count_ref_sequ();
 
-	    found_type = var->peek_type();
+                found_type = var->peek_type();
+
+        } else if (const InterfacePort*port = scope->find_param(name_)) {
+                found_type = port->type;
+        }
       }
 
       if (found_type == 0) {
@@ -316,14 +331,10 @@ int ExpName::elaborate_rval(Entity*ent, ScopeBase*scope, const InterfacePort*lva
       return errors;
 }
 
-int ExpNameALL::elaborate_lval(Entity*ent, ScopeBase*scope, bool is_sequ)
-{
-      return Expression::elaborate_lval(ent, scope, is_sequ);
-}
-
 int Expression::elaborate_expr(Entity*, ScopeBase*, const VType*)
 {
-      cerr << get_fileline() << ": internal error: I don't know how to elaborate expression type=" << typeid(*this).name() << endl;
+      cerr << get_fileline() << ": internal error: I don't know how to "
+           << "elaborate expression type=" << typeid(*this).name() << endl;
       return 1;
 }
 
@@ -337,8 +348,10 @@ const VType* ExpBinary::probe_type(Entity*ent, ScopeBase*scope) const
       if (t2 == 0)
 	    return t1;
 
-      if (t1 == t2)
+      if (t1->type_match(t2))
 	    return t1;
+      if (t2->type_match(t1))
+	    return t2;
 
       if (const VType*tb = resolve_operand_types_(t1, t2))
 	    return tb;
@@ -376,6 +389,11 @@ const VType*ExpUnary::fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*aty
       return operand1_->fit_type(ent, scope, atype);
 }
 
+const VType*ExpUnary::probe_type(Entity*ent, ScopeBase*scope) const
+{
+      return operand1_->probe_type(ent, scope);
+}
+
 int ExpUnary::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype)
 {
       ivl_assert(*this, ltype != 0);
@@ -383,11 +401,6 @@ int ExpUnary::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype)
       return operand1_->elaborate_expr(ent, scope, ltype);
 }
 
-const VType*ExpAggregate::probe_type(Entity*ent, ScopeBase*scope) const
-{
-      return Expression::probe_type(ent, scope);
-}
-
 const VType*ExpAggregate::fit_type(Entity*, ScopeBase*, const VTypeArray*host) const
 {
       ivl_assert(*this, elements_.size() == 1);
@@ -398,13 +411,13 @@ const VType*ExpAggregate::fit_type(Entity*, ScopeBase*, const VTypeArray*host) c
       elements_[0]->map_choices(&ce[0]);
 
       ivl_assert(*this, ce.size() == 1);
-      prange_t*prange = ce[0].choice->range_expressions();
+      ExpRange*prange = ce[0].choice->range_expressions();
       ivl_assert(*this, prange);
 
       Expression*use_msb = prange->msb();
       Expression*use_lsb = prange->lsb();
 
-      ivl_assert(*this, host->dimensions() == 1);
+      ivl_assert(*this, host->dimensions().size() == 1);
       vector<VTypeArray::range_t> range (1);
 
       range[0] = VTypeArray::range_t(use_msb, use_lsb);
@@ -565,6 +578,7 @@ int ExpArithmetic::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype
 
 const VType* ExpArithmetic::resolve_operand_types_(const VType*t1, const VType*t2) const
 {
+    // Ranges
       while (const VTypeRange*tmp = dynamic_cast<const VTypeRange*> (t1))
 	    t1 = tmp->base_type();
       while (const VTypeRange*tmp = dynamic_cast<const VTypeRange*> (t2))
@@ -572,31 +586,107 @@ const VType* ExpArithmetic::resolve_operand_types_(const VType*t1, const VType*t
 
       if (t1->type_match(t2))
 	    return t1;
-      if (t2->type_match(t2))
-	    return t2;
 
-      return 0;
+    // Signed & unsigned (resized to the widest argument)
+    const VTypeArray*t1_arr = dynamic_cast<const VTypeArray*>(t1);
+    const VTypeArray*t2_arr = dynamic_cast<const VTypeArray*>(t2);
+
+    if(t1_arr && t2_arr) {
+        const VTypeArray*t1_parent = t1_arr->get_parent_type();
+        const VTypeArray*t2_parent = t2_arr->get_parent_type();
+
+        if(t1_parent == t2_parent
+                && (t1_parent == &primitive_SIGNED || t1_parent == &primitive_UNSIGNED)) {
+            int t1_size = t1_arr->get_width(NULL);
+            int t2_size = t2_arr->get_width(NULL);
+
+            // Easy, the same sizes, so we do not need to resize
+            if(t1_size == t2_size && t1_size > 0)
+                return t1;  // == t2
+
+            VTypeArray*resolved = new VTypeArray(t1_parent->element_type(),
+                    std::max(t1_size, t2_size) - 1, 0, t1_parent->signed_vector());
+            resolved->set_parent_type(t1_parent);
+
+            return resolved;
+        }
+
+    } else if(t1_arr) {
+        if(const VTypePrimitive*prim = dynamic_cast<const VTypePrimitive*>(t2)) {
+            const VTypeArray*t1_parent = t1_arr->get_parent_type();
+            VTypePrimitive::type_t t2_type = prim->type();
+
+            if((t2_type == VTypePrimitive::NATURAL || t2_type == VTypePrimitive::INTEGER)
+                    && t1_parent == &primitive_SIGNED)
+                return t1;
+
+            if((t2_type == VTypePrimitive::NATURAL) && t1_parent == &primitive_UNSIGNED)
+                return t1;
+        }
+
+    } else if(t2_arr) {
+        if(const VTypePrimitive*prim = dynamic_cast<const VTypePrimitive*>(t1)) {
+            const VTypeArray*t2_parent = t2_arr->get_parent_type();
+            VTypePrimitive::type_t t1_type = prim->type();
+
+            if((t1_type == VTypePrimitive::NATURAL || t1_type == VTypePrimitive::INTEGER)
+                    && t2_parent == &primitive_SIGNED)
+                return t2;
+
+            if((t1_type == VTypePrimitive::NATURAL) && t2_parent == &primitive_UNSIGNED)
+                return t2;
+        }
+    }
+
+    return 0;
 }
 
-const VType* ExpAttribute::probe_type(Entity*ent, ScopeBase*scope) const
+int ExpAttribute::elaborate_args(Entity*ent, ScopeBase*scope, const VType*ltype)
 {
-      base_->probe_type(ent, scope);
+      int errors = 0;
 
-      if (name_ == "length" || name_ == "left" || name_ == "right") {
-	    return &primitive_NATURAL;
+      if(args_) {
+	    for(list<Expression*>::iterator it = args_->begin();
+                    it != args_->end(); ++it) {
+		errors += (*it)->elaborate_expr(ent, scope, ltype);
+            }
       }
 
-      return 0;
+      return errors;
 }
 
-int ExpAttribute::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*)
+int ExpObjAttribute::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*)
 {
       int errors = 0;
       const VType*sub_type = base_->probe_type(ent, scope);
+
+      errors += elaborate_args(ent, scope, sub_type);
       errors += base_->elaborate_expr(ent, scope, sub_type);
+
       return errors;
 }
 
+const VType* ExpObjAttribute::probe_type(Entity*, ScopeBase*) const
+{
+      if (name_ == "length" || name_ == "left" || name_ == "right")
+	    return &primitive_NATURAL;
+
+      return NULL;
+}
+
+int ExpTypeAttribute::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype)
+{
+      return elaborate_args(ent, scope, ltype);
+}
+
+const VType* ExpTypeAttribute::probe_type(Entity*, ScopeBase*) const
+{
+      if(name_ == "image")
+	    return &primitive_STRING;
+
+      return NULL;
+}
+
 const VType*ExpBitstring::fit_type(Entity*, ScopeBase*, const VTypeArray*atype) const
 {
 	// Really should check that this string can work with the
@@ -638,7 +728,7 @@ const VType*ExpConcat::fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*at
 
 	    if(const VTypeArray*arr = dynamic_cast<const VTypeArray*>(types[i])) {
 		types[i] = arr->element_type();
-		ivl_assert(*this, arr->dimensions() == 1);
+		ivl_assert(*this, arr->dimensions().size() == 1);
 		const VTypeArray::range_t&dim = arr->dimension(0);
 		sizes[i] = new ExpArithmetic(ExpArithmetic::MINUS, dim.msb(), dim.lsb());
 	    } else {
@@ -653,8 +743,8 @@ const VType*ExpConcat::fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*at
                             new ExpArithmetic(ExpArithmetic::PLUS, sizes[0], sizes[1]),
                             new ExpInteger(1));
 
-      std::list<prange_t*> ranges;
-      ranges.push_front(new prange_t(size, new ExpInteger(0), true));
+      std::list<ExpRange*> ranges;
+      ranges.push_front(new ExpRange(size, new ExpInteger(0), ExpRange::DOWNTO));
       const VType*array = new VTypeArray(types[1], &ranges);
 
       return array;
@@ -693,7 +783,7 @@ int ExpConcat::elaborate_expr_array_(Entity*ent, ScopeBase*scope, const VTypeArr
       int errors = 0;
 
 	// For now, only support single-dimension arrays here.
-      ivl_assert(*this, atype->dimensions() == 1);
+      ivl_assert(*this, atype->dimensions().size() == 1);
 
       const VType*type1 = operand1_->fit_type(ent, scope, atype);
       ivl_assert(*this, type1);
@@ -749,60 +839,37 @@ int ExpConditional::case_t::elaborate_expr(Entity*ent, ScopeBase*scope, const VT
       return errors;
 }
 
-const VType*ExpFunc::probe_type(Entity*, ScopeBase*scope) const
+const VType*ExpFunc::probe_type(Entity*ent, ScopeBase*scope) const
 {
-      SubprogramHeader*prog = def_;
+      if(!def_)
+          def_ = match_signature(ent, scope);
 
-      if(!prog) {
-          prog = scope->find_subprogram(name_);
-      }
-
-      if(!prog)
-          prog = library_find_subprogram(name_);
-
-      if(!prog) {
-          cerr << get_fileline() << ": sorry: VHDL function " << name_ << " not yet implemented" << endl;
-          ivl_assert(*this, false);
-      }
-
-      return prog->peek_return_type();
+      return def_ ? def_->exact_return_type(argv_, ent, scope) : NULL;
 }
 
 int ExpFunc::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*)
 {
       int errors = 0;
 
-      ivl_assert(*this, scope);
-      SubprogramHeader*prog = scope->find_subprogram(name_);
+      if(def_)
+          return 0;
 
-      if(!prog)
-            prog = library_find_subprogram(name_);
+      def_ = match_signature(ent, scope);
 
-      ivl_assert(*this, def_==0);
-      def_ = prog;
+      if(!def_)
+          return 1;
 
 	// Elaborate arguments
-      for (size_t idx = 0 ; idx < argv_.size() ; idx += 1) {
-	    const VType*tmp = argv_[idx]->probe_type(ent, scope);
-	    const VType*param_type = prog ? prog->peek_param_type(idx) : NULL;
-
-	    if(!tmp && param_type)
-	        tmp = param_type;
-
-	    errors += argv_[idx]->elaborate_expr(ent, scope, tmp);
+      for (size_t idx = 0; idx < argv_.size(); ++idx) {
+	    errors += def_->elaborate_argument(argv_[idx], idx, ent, scope);
       }
 
 	// SystemVerilog functions work only with defined size data types, therefore
 	// if header does not specify argument or return type size, create a function
 	// instance that work with this particular size.
       if(def_ && !def_->is_std() && def_->unbounded()) {
-            def_ = prog->make_instance(argv_, scope);
-            name_ = def_->name();
-      }
-
-      if(!def_) {
-            cerr << get_fileline() << ": error: could not find function " << name_ << endl;
-            ++errors;
+            def_ = def_->make_instance(argv_, scope);
+            name_ = def_->name();   // TODO necessary?
       }
 
       return errors;
@@ -810,12 +877,15 @@ int ExpFunc::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*)
 
 const VType* ExpFunc::fit_type(Entity*ent, ScopeBase*scope, const VTypeArray*) const
 {
-      return probe_type(ent, scope);
+    return probe_type(ent, scope);
 }
 
 const VType* ExpInteger::probe_type(Entity*, ScopeBase*) const
 {
-      return &primitive_INTEGER;
+      if(value_ >= 0)
+          return &primitive_NATURAL;
+      else
+          return &primitive_INTEGER;
 }
 
 int ExpInteger::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype)
@@ -880,40 +950,49 @@ const VType* ExpName::probe_prefix_type_(Entity*ent, ScopeBase*scope) const
  */
 const VType* ExpName::probe_prefixed_type_(Entity*ent, ScopeBase*scope) const
 {
-	// First, get the type of the prefix.
+      // First, get the type of the prefix.
       const VType*prefix_type = prefix_->probe_prefix_type_(ent, scope);
       if (prefix_type == 0) {
-	    return 0;
+          return 0;
       }
 
       while (const VTypeDef*def = dynamic_cast<const VTypeDef*> (prefix_type)) {
-	    prefix_type = def->peek_definition();
+          prefix_type = def->peek_definition();
       }
 
-	// If the prefix type is a record, then the current name is
-	// the name of a member.
-      if (const VTypeRecord*pref_record = dynamic_cast<const VTypeRecord*> (prefix_type)) {
-	    const VTypeRecord::element_t*element = pref_record->element_by_name(name_);
-	    ivl_assert(*this, element);
+      const VType*element_type = prefix_type;
+      bool type_changed = true;
 
-	    const VType*element_type = element->peek_type();
-	    ivl_assert(*this, element_type);
+      // Keep unwinding the type until we find the basic element type
+      while (type_changed) {
+          type_changed = false;
 
-	    return element_type;
+          // If the prefix type is a record, then the current name is
+          // the name of a member.
+          if (const VTypeRecord*pref_record = dynamic_cast<const VTypeRecord*>(element_type)) {
+              const VTypeRecord::element_t*element = pref_record->element_by_name(name_);
+              ivl_assert(*this, element);
+
+              element_type = element->peek_type();
+              ivl_assert(*this, element_type);
+              type_changed = true;
+          }
+
+          if (const VTypeArray*pref_array = dynamic_cast<const VTypeArray*>(element_type)) {
+              element_type = pref_array->basic_type(false);
+              ivl_assert(*this, element_type);
+              type_changed = true;
+          }
       }
 
-      if (const VTypeArray*pref_array = dynamic_cast<const VTypeArray*> (prefix_type)) {
-            const VType*element_type = pref_array->element_type();
-            ivl_assert(*this, element_type);
-
-            return element_type;
+      if(!element_type) {
+          cerr << get_fileline() << ": sorry: I don't know how to probe "
+              << "prefix type " << typeid(*prefix_type).name()
+              << " of " << name_ << "." << endl;
+          return NULL;
       }
 
-      cerr << get_fileline() << ": sorry: I don't know how to probe "
-	   << "prefix type " << typeid(*prefix_type).name()
-	   << " of " << name_ << "." << endl;
-
-      return 0;
+      return element_type;
 }
 
 const VType* ExpName::probe_type(Entity*ent, ScopeBase*scope) const
@@ -923,31 +1002,31 @@ const VType* ExpName::probe_type(Entity*ent, ScopeBase*scope) const
 
       if(ent) {
         if (const InterfacePort*cur = ent->find_port(name_)) {
-                ivl_assert(*this, cur->type);
-                return cur->type;
+            ivl_assert(*this, cur->type);
+            return cur->type;
         }
 
         if (const InterfacePort*cur = ent->find_generic(name_)) {
-                ivl_assert(*this, cur->type);
-                return cur->type;
+            ivl_assert(*this, cur->type);
+            return cur->type;
         }
       }
 
       if(scope) {
         if (Signal*sig = scope->find_signal(name_))
-                return sig->peek_type();
+            return sig->peek_type();
 
         if (Variable*var = scope->find_variable(name_))
-                return var->peek_type();
+            return var->peek_type();
 
         const VType*type = 0;
         Expression*cval = 0;
         if (scope->find_constant(name_, type, cval))
-                return type;
+            return type;
 
         Architecture*arc = dynamic_cast<Architecture*>(scope);
         if (arc && (type = arc->probe_genvar_type(name_))) {
-                return type;
+            return type;
         }
 
         if (const InterfacePort*port = scope->find_param(name_)) {
@@ -959,8 +1038,14 @@ const VType* ExpName::probe_type(Entity*ent, ScopeBase*scope) const
         }
       }
 
-      cerr << get_fileline() << ": error: Signal/variable " << name_
-	   << " not found in this context." << endl;
+      if(ent || scope) {
+          // Do not display error messages if there was no entity or scope
+          // specified. There are functions that are called without any specific
+          // context and they still may want to probe the expression type.
+        cerr << get_fileline() << ": error: Signal/variable " << name_
+             << " not found in this context." << endl;
+      }
+
       return 0;
 }
 
@@ -979,11 +1064,12 @@ int ExpName::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype)
       if(prefix_.get())
 	    prefix_.get()->elaborate_expr(ent, scope, NULL);
 
-      if(index_)
-	    index_->elaborate_expr(ent, scope, &primitive_INTEGER);
-
-      if(lsb_)
-	    lsb_->elaborate_expr(ent, scope, &primitive_INTEGER);
+      if (indices_) {
+          for(list<Expression*>::const_iterator it = indices_->begin();
+                  it != indices_->end(); ++it) {
+              (*it)->elaborate_expr(ent, scope, &primitive_INTEGER);
+          }
+      }
 
       return 0;
 }
@@ -1058,3 +1144,26 @@ int ExpTime::elaborate_expr(Entity*, ScopeBase*, const VType*)
       set_type(&primitive_INTEGER);
       return 0;
 }
+
+int ExpRange::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*)
+{
+    int errors = 0;
+
+    if(left_)
+        errors += left_->elaborate_expr(ent, scope, &primitive_INTEGER);
+
+    if(right_)
+        errors += right_->elaborate_expr(ent, scope, &primitive_INTEGER);
+
+    return errors;
+}
+
+int ExpDelay::elaborate_expr(Entity*ent, ScopeBase*scope, const VType*ltype)
+{
+    int errors = 0;
+
+    errors += expr_->elaborate_expr(ent, scope, ltype);
+    errors += delay_->elaborate_expr(ent, scope, ltype);
+
+    return errors;
+}
