@@ -1,7 +1,7 @@
 #ifndef IVL_library_H
 #define IVL_library_H
 /*
- * Copyright (c) 2011-2014 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2011-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -30,7 +30,6 @@ void library_add_directory(const char*directory);
 int elaborate_libraries(void);
 int emit_packages(void);
 
-SubprogramHeader*library_match_subprogram(perm_string name, const list<const VType*>*params);
+SubprogramHeader*library_match_subprogram(perm_string name, const std::list<const VType*>*params);
 
 #endif /* IVL_library_H */
-
