@@ -1,6 +1,6 @@
 
 const char COPYRIGHT[] =
-      "Copyright (c) 2011-2015 Stephen Williams (steve@icarus.com)\n"
+      "Copyright (c) 2011-2021 Stephen Williams (steve@icarus.com)\n"
       "Copyright CERN 2012 / Stephen Williams (steve@icarus.com)";
 /*
  *    This source code is free software; you can redistribute it
@@ -22,6 +22,8 @@ const char COPYRIGHT[] =
 # include  "version_base.h"
 # include  "version_tag.h"
 
+using namespace std;
+
 /*
  * Usage:  vhdlpp [flags] sourcefile...
  * Flags:
@@ -168,7 +170,7 @@ int main(int argc, char*argv[])
 	    debug_log_file.open(debug_log_path);
       }
 
-      if ( (rc = mkdir(work_path, 0777)) < 0 ) {
+      if ( (mkdir(work_path, 0777)) < 0 ) {
 	    if (errno != EEXIST) {
 		  fprintf(stderr, "Icarus Verilog VHDL unable to create work directory %s, errno=%d\n", work_path, errno);
 		  return -1;
@@ -176,7 +178,7 @@ int main(int argc, char*argv[])
 	    struct stat stat_buf;
 	    rc = stat(work_path, &stat_buf);
 
-	    if (!S_ISDIR(stat_buf.st_mode)) {
+	    if (rc || !S_ISDIR(stat_buf.st_mode)) {
 		  fprintf(stderr, "Icarus Verilog VHDL work path `%s' is not a directory.\n", work_path);
 		  return -1;
 	    }
