@@ -1,7 +1,7 @@
 #ifndef IVL_parse_types_H
 #define IVL_parse_types_H
 /*
- * Copyright (c) 2011-2015 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2011-2021 Stephen Williams (steve@icarus.com)
  * Copyright CERN 2013 / Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
@@ -31,7 +31,7 @@ class named_expr_t {
 
       perm_string name() const { return name_; }
       Expression* expr() const { return expr_; }
-      void dump(ostream&out, int indent) const;
+      void dump(std::ostream&out, int indent) const;
     private:
       perm_string name_;
       Expression* expr_;
@@ -57,7 +57,7 @@ class entity_aspect_t {
 
 class instant_list_t {
     public:
-      typedef enum { ALL = 0, OTHERS, NONE } application_domain_t;
+      typedef enum { ALL = 0, OTHERS, NO_DOMAIN } application_domain_t;
 
       instant_list_t(application_domain_t d, std::list<perm_string>* l) : domain_(d), labels_(l) {}
       ~instant_list_t() { delete labels_; }
@@ -78,7 +78,7 @@ struct adding_term {
 // (VHDL-2008 6.4.2.5)
 class file_open_info_t {
     public:
-      file_open_info_t(ExpString*filename__, ExpName*kind__ = NULL)
+      explicit file_open_info_t(ExpString*filename__, ExpName*kind__ = NULL)
         : kind_(kind__), filename_(filename__) {
           // By default files are opened in read-only mode
           if(!kind_) kind_ = new ExpName(perm_string::literal("read_mode"));
