@@ -1,7 +1,7 @@
 #ifndef IVL_std_funcs_H
 #define IVL_std_funcs_H
 /*
- * Copyright CERN 2015
+ * Copyright CERN 2016
  * @author Maciej Suminski (maciej.suminski@cern.ch)
  *
  *    This source code is free software; you can redistribute it
@@ -28,7 +28,10 @@ void preload_std_funcs();
 // Destroys subprogram headers for standard VHDL library functions.
 void delete_std_funcs();
 
+// Adds a subprogram to the standard library subprogram set
+void register_std_subprogram(SubprogramHeader*header);
+
 // Returns subprogram header for a requested function or NULL if it does not exist.
-SubprogramHeader*find_std_subprogram(perm_string name);
+SubHeaderList find_std_subprogram(perm_string name);
 
 #endif /* IVL_std_funcs_H */
