@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -33,30 +33,22 @@
 # include  "ivl_alloc.h"
 
 static char *dump_path = NULL;
+static int dump_no_date = 0;
 static struct fstContext *dump_file = NULL;
 
 static struct t_vpi_time zero_delay = { vpiSimTime, 0, 0, 0.0 };
 
-struct vcd_info {
-      vpiHandle item;
-      vpiHandle cb;
-      struct t_vpi_time time;
-      struct vcd_info *next;
-      struct vcd_info *dmp_next;
-      fstHandle handle;
-      int scheduled;
-};
-
-
+DECLARE_VCD_INFO(vcd_info, fstHandle);
 static struct vcd_info *vcd_list = NULL;
+static struct vcd_info *vcd_const_list = NULL;
 static struct vcd_info *vcd_dmp_list = NULL;
+
 static PLI_UINT64 vcd_cur_time = 0;
 static int dump_is_off = 0;
 static long dump_limit = 0;
 static int dump_is_full = 0;
 static int finish_status = 0;
 
-
 static enum lxm_optimum_mode_e {
       LXM_NONE  = 0,
       LXM_SPACE = 1,
@@ -81,11 +73,15 @@ static void show_this_item(struct vcd_info*info)
       if (type == vpiRealVar) {
 	    value.format = vpiRealVal;
 	    vpi_get_value(info->item, &value);
-	    fstWriterEmitValueChange(dump_file, info->handle, &value.value.real);
+	    fstWriterEmitValueChange(dump_file, info->ident, &value.value.real);
+      } else if (type == vpiParameter && vpi_get(vpiConstType, info->item) == vpiRealConst) {
+	    value.format = vpiRealVal;
+	    vpi_get_value(info->item, &value);
+	    fstWriterEmitValueChange(dump_file, info->ident, &value.value.real);
       } else {
 	    value.format = vpiBinStrVal;
 	    vpi_get_value(info->item, &value);
-	    fstWriterEmitValueChange(dump_file, info->handle, (type != vpiNamedEvent) ? value.value.str : "1");
+	    fstWriterEmitValueChange(dump_file, info->ident, (type != vpiNamedEvent) ? value.value.str : "1");
       }
 }
 
@@ -97,14 +93,14 @@ static void show_this_item_x(struct vcd_info*info)
       if (type == vpiRealVar) {
 	      /* Some tools dump nothing here...? */
             double mynan = strtod("NaN", NULL);
-	    fstWriterEmitValueChange(dump_file, info->handle, &mynan);
+	    fstWriterEmitValueChange(dump_file, info->ident, &mynan);
       } else if (type == vpiNamedEvent) {
 	    /* Do nothing for named events. */
       } else {
 	    int siz = vpi_get(vpiSize, info->item);
 	    char *xmem = malloc(siz);
 	    memset(xmem, 'x', siz);
-	    fstWriterEmitValueChange(dump_file, info->handle, xmem);
+	    fstWriterEmitValueChange(dump_file, info->ident, xmem);
 	    free(xmem);
       }
 }
@@ -125,26 +121,6 @@ __inline__ static int dump_header_pending(void)
       return dumpvars_status != 2;
 }
 
-/*
- * This function writes out all the traced variables, whether they
- * changed or not.
- */
-static void vcd_checkpoint(void)
-{
-      struct vcd_info*cur;
-
-      for (cur = vcd_list ;  cur ;  cur = cur->next)
-	    show_this_item(cur);
-}
-
-static void vcd_checkpoint_x(void)
-{
-      struct vcd_info*cur;
-
-      for (cur = vcd_list ;  cur ;  cur = cur->next)
-	    show_this_item_x(cur);
-}
-
 static PLI_INT32 variable_cb_2(p_cb_data cause)
 {
       struct vcd_info* info = vcd_dmp_list;
@@ -211,7 +187,8 @@ static PLI_INT32 dumpvars_cb(p_cb_data cause)
       if (!dump_is_off) {
 	    fstWriterEmitTimeChange(dump_file, dumpvars_time);
 	    /* nothing to do for  $dumpvars... */
-	    vcd_checkpoint();
+	    ITERATE_VCD_INFO(vcd_const_list, vcd_info, next, show_this_item);
+	    ITERATE_VCD_INFO(vcd_list, vcd_info, next, show_this_item);
 	    /* ...nothing to do for $end */
       }
 
@@ -239,6 +216,11 @@ static PLI_INT32 finish_cb(p_cb_data cause)
 	    free(cur);
       }
       vcd_list = 0;
+      for (cur = vcd_const_list ; cur ; cur = next) {
+	    next = cur->next;
+	    free(cur);
+      }
+      vcd_const_list = 0;
       vcd_names_delete(&fst_tab);
       vcd_names_delete(&fst_var);
       nexus_ident_delete();
@@ -302,7 +284,7 @@ static PLI_INT32 sys_dumpoff_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
 
       fstWriterEmitDumpActive(dump_file, 0); /* $dumpoff */
-      vcd_checkpoint_x();
+      ITERATE_VCD_INFO(vcd_list, vcd_info, next, show_this_item_x);
 
       return 0;
 }
@@ -331,7 +313,7 @@ static PLI_INT32 sys_dumpon_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
 
       fstWriterEmitDumpActive(dump_file, 1); /* $dumpon */
-      vcd_checkpoint();
+      ITERATE_VCD_INFO(vcd_list, vcd_info, next, show_this_item);
 
       return 0;
 }
@@ -357,7 +339,7 @@ static PLI_INT32 sys_dumpall_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
 
       /* nothing to do for $dumpall... */
-      vcd_checkpoint();
+      ITERATE_VCD_INFO(vcd_list, vcd_info, next, show_this_item);
 
       return 0;
 }
@@ -372,6 +354,7 @@ static void open_dumpfile(vpiHandle callh)
 	    vpi_printf("FST Error: %s:%d: ", vpi_get_str(vpiFile, callh),
 	               (int)vpi_get(vpiLineNo, callh));
 	    vpi_printf("Unable to open %s for output.\n", dump_path);
+	    vpip_set_return_value(1);
 	    vpi_control(vpiFinish, 1);
 	    free(dump_path);
 	    dump_path = 0;
@@ -398,7 +381,10 @@ static void open_dumpfile(vpiHandle callh)
 		  prec -= 1;
 	    }
 
-	    fstWriterSetDate(dump_file, asctime(localtime(&walltime)));
+		if (!dump_no_date)
+			fstWriterSetDate(dump_file, asctime(localtime(&walltime)));
+		else
+			fstWriterSetDate(dump_file, "");
 	    fstWriterSetVersion(dump_file, "Icarus Verilog");
 	    sprintf(scale_buf, "\t%u%s\n", scale, units_names[udx]);
 	    fstWriterSetTimescaleFromString(dump_file, scale_buf);
@@ -501,7 +487,13 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	  case vpiNamedEvent: type = FST_VT_VCD_EVENT; break;
 	  case vpiIntVar:
 	  case vpiIntegerVar: type = FST_VT_VCD_INTEGER; break;
-	  case vpiParameter:  type = FST_VT_VCD_PARAMETER; break;
+	    /* FST (gtkwave) assumes PARAMETER are bit, so handle other cases */
+	  case vpiParameter:
+	    switch (vpi_get(vpiConstType, item)) {
+		case vpiRealConst: type = FST_VT_VCD_REAL; break;
+		default: type = FST_VT_VCD_PARAMETER; break;
+	    }
+	    break;
 	    /* Icarus converts realtime to real. */
 	  case vpiRealVar:    type = FST_VT_VCD_REAL; break;
 	  case vpiMemoryWord:
@@ -533,11 +525,12 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	  case vpiFunction:   stype = FST_ST_VCD_FUNCTION; break;
 	  case vpiGenScope:   stype = FST_ST_VCD_GENERATE; break;
 	  case vpiModule:     stype = FST_ST_VCD_MODULE; break;
+	  case vpiPackage:    stype = FST_ST_VCD_PACKAGE; break;
 	  case vpiTask:       stype = FST_ST_VCD_TASK; break;
 
 	  default:
 	    vpi_printf("FST warning: $dumpvars: Unsupported argument "
-	               "type (%s)\n", vpi_get_str(vpiType, item));
+	               "type (%s).\n", vpi_get_str(vpiType, item));
 	    return;
       }
 
@@ -568,10 +561,6 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
 	/* Generate the $var or $scope commands. */
       switch (item_type) {
-	  case vpiParameter:
-	    vpi_printf("FST sorry: $dumpvars: can not dump parameters.\n");
-	    break;
-
 	  case vpiNamedEvent:
 	  case vpiIntegerVar:
 	  case vpiBitVar:
@@ -642,7 +631,7 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
 		  info->time.type = vpiSimTime;
 		  info->item  = item;
-		  info->handle = new_ident;
+		  info->ident = new_ident;
 		  info->scheduled = 0;
 
 		  cb.time      = &info->time;
@@ -661,7 +650,39 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
 	    break;
 
+	  case vpiParameter:
+
+	    /* If we are skipping all pamaeters then just return. */
+            if (skip) return;
+
+	      /* Skip this signal if it has already been included.
+	       * This can only happen for implicitly given signals. */
+	    if (vcd_names_search(&fst_var, fullname)) return;
+
+	    name = vpi_get_str(vpiName, item);
+	    if (is_escaped_id(name)) {
+		  escname = malloc(strlen(name) + 2);
+		  sprintf(escname, "\\%s", name);
+	    } else escname = strdup(name);
+
+	    size = vpi_get(vpiSize, item);
+	    dir = FST_VD_IMPLICIT;
+	    new_ident = fstWriterCreateVar(dump_file, type,
+		                                 dir, size, escname, 0);
+	    free(escname);
+
+	    info = malloc(sizeof(*info));
+	    info->item = item;
+	    info->ident = new_ident;
+	    info->scheduled = 0;
+	    info->dmp_next = 0;
+	    info->next = vcd_const_list;
+	    info->cb = NULL;
+	    vcd_const_list = info;
+	    break;
+
 	  case vpiModule:
+	  case vpiPackage:
 	  case vpiGenScope:
 	  case vpiFunction:
 	  case vpiTask:
@@ -676,7 +697,7 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 			/* Value */
 			vpiNamedEvent,
 			vpiNet,
-			/* vpiParameter, */
+			vpiParameter,
 			vpiReg,
 			vpiVariables,
 			/* Scope */
@@ -822,10 +843,10 @@ static PLI_INT32 sys_dumpvars_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
       if (!depth) depth = 10000;
 
-        /* This dumps all the modules in the design if none are given. */
+        /* This dumps all the instances in the design if none are given. */
       if (!argv || !(item = vpi_scan(argv))) {
-	    argv = vpi_iterate(vpiModule, 0x0);
-	    assert(argv);  /* There must be at least one top level module. */
+	    argv = vpi_iterate(vpiInstance, 0x0);
+	    assert(argv);  /* There must be at least one top level instance. */
 	    item = vpi_scan(argv);
       }
 
@@ -915,7 +936,9 @@ void sys_fst_register(void)
 		  lxm_optimum_mode = LXM_BOTH;
 	    } else if (strcmp(vlog_info.argv[idx],"-fst-speed-space") == 0) {
 		  lxm_optimum_mode = LXM_BOTH;
-	    }
+		} else if (strcmp(vlog_info.argv[idx],"-no-date") == 0) {
+		  dump_no_date = 1;
+		}
       }
 
       /* All the compiletf routines are located in vcd_priv.c. */
