@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2016 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -33,22 +33,25 @@
 
 static char *dump_path = NULL;
 static FILE *dump_file = NULL;
+static int   dump_no_date = 0;
 
 static struct t_vpi_time zero_delay = { vpiSimTime, 0, 0, 0.0 };
 
-struct vcd_info {
-      vpiHandle item;
-      vpiHandle cb;
-      struct t_vpi_time time;
-      const char *ident;
-      struct vcd_info *next;
-      struct vcd_info *dmp_next;
-      int scheduled;
-};
-
-
+/*
+ * The vcd_list is the list of all the objects that are tracked for
+ * dumping. The vcd_checkpoint goes through the list to dump the current
+ * values for everything. When the item has a value change, it is added to the
+ * vcd_dmp_list for dumping in the current time step.
+ *
+ * The vcd_const_list is a list of all of the parameters that are being
+ * dumped. This list is scanned less often, since parameters do not change
+ * values.
+ */
+DECLARE_VCD_INFO(vcd_info, const char*);
+static struct vcd_info *vcd_const_list = NULL;
 static struct vcd_info *vcd_list = NULL;
 static struct vcd_info *vcd_dmp_list = NULL;
+
 static PLI_UINT64 vcd_cur_time = 0;
 static int dump_is_off = 0;
 static long dump_limit = 0;
@@ -112,6 +115,11 @@ static void show_this_item(struct vcd_info*info)
 	    fprintf(dump_file, "r%.16g %s\n", value.value.real, info->ident);
       } else if (type == vpiNamedEvent) {
 	    fprintf(dump_file, "1%s\n", info->ident);
+      } else if (type == vpiParameter && vpi_get(vpiConstType, info->item) == vpiRealConst) {
+
+	    value.format = vpiRealVal;
+	    vpi_get_value(info->item, &value);
+	    fprintf(dump_file, "r%.16g %s\n", value.value.real, info->ident);
       } else if (vpi_get(vpiSize, info->item) == 1) {
 	    value.format = vpiBinStrVal;
 	    vpi_get_value(info->item, &value);
@@ -157,25 +165,6 @@ __inline__ static int dump_header_pending(void)
       return dumpvars_status != 2;
 }
 
-/*
- * This function writes out all the traced variables, whether they
- * changed or not.
- */
-static void vcd_checkpoint(void)
-{
-      struct vcd_info*cur;
-
-      for (cur = vcd_list ;  cur ;  cur = cur->next)
-	    show_this_item(cur);
-}
-
-static void vcd_checkpoint_x(void)
-{
-      struct vcd_info*cur;
-
-      for (cur = vcd_list ;  cur ;  cur = cur->next)
-	    show_this_item_x(cur);
-}
 
 static PLI_INT32 variable_cb_2(p_cb_data cause)
 {
@@ -231,6 +220,11 @@ static PLI_INT32 variable_cb_1(p_cb_data cause)
       return 0;
 }
 
+/*
+ * This is called at the end of the timestep where the $dumpvars task is
+ * called. This allows for values to settle for the timestep, so that the
+ * checkpoint gets the current values.
+ */
 static PLI_INT32 dumpvars_cb(p_cb_data cause)
 {
       if (dumpvars_status != 1) return 0;
@@ -243,9 +237,15 @@ static PLI_INT32 dumpvars_cb(p_cb_data cause)
       fprintf(dump_file, "$enddefinitions $end\n");
 
       if (!dump_is_off) {
+	    fprintf(dump_file, "$comment Show the parameter values. $end\n");
+	    fprintf(dump_file, "$dumpall\n");
+	    ITERATE_VCD_INFO(vcd_const_list, vcd_info, next, show_this_item);
+	    fprintf(dump_file, "$end\n");
+
 	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", dumpvars_time);
+
 	    fprintf(dump_file, "$dumpvars\n");
-	    vcd_checkpoint();
+	    ITERATE_VCD_INFO(vcd_list, vcd_info, next, show_this_item);
 	    fprintf(dump_file, "$end\n");
       }
 
@@ -274,6 +274,12 @@ static PLI_INT32 finish_cb(p_cb_data cause)
 	    free(cur);
       }
       vcd_list = 0;
+      for (cur = vcd_const_list ; cur ; cur = next) {
+	    next = cur->next;
+	    free((char *)cur->ident);
+	    free(cur);
+      }
+      vcd_const_list = 0;
       vcd_names_delete(&vcd_tab);
       vcd_names_delete(&vcd_var);
       nexus_ident_delete();
@@ -337,7 +343,7 @@ static PLI_INT32 sys_dumpoff_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
 
       fprintf(dump_file, "$dumpoff\n");
-      vcd_checkpoint_x();
+      ITERATE_VCD_INFO(vcd_list, vcd_info, next, show_this_item_x);
       fprintf(dump_file, "$end\n");
 
       return 0;
@@ -367,7 +373,7 @@ static PLI_INT32 sys_dumpon_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
 
       fprintf(dump_file, "$dumpon\n");
-      vcd_checkpoint();
+      ITERATE_VCD_INFO(vcd_list, vcd_info, next, show_this_item);
       fprintf(dump_file, "$end\n");
 
       return 0;
@@ -394,7 +400,7 @@ static PLI_INT32 sys_dumpall_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
 
       fprintf(dump_file, "$dumpall\n");
-      vcd_checkpoint();
+      ITERATE_VCD_INFO(vcd_list, vcd_info, next, show_this_item);
       fprintf(dump_file, "$end\n");
 
       return 0;
@@ -410,6 +416,7 @@ static void open_dumpfile(vpiHandle callh)
 	    vpi_printf("VCD Error: %s:%d: ", vpi_get_str(vpiFile, callh),
 	               (int)vpi_get(vpiLineNo, callh));
 	    vpi_printf("Unable to open %s for output.\n", dump_path);
+	    vpip_set_return_value(1);
 	    vpi_control(vpiFinish, 1);
 	    free(dump_path);
 	    dump_path = 0;
@@ -435,9 +442,11 @@ static void open_dumpfile(vpiHandle callh)
 		  prec -= 1;
 	    }
 
-	    fprintf(dump_file, "$date\n");
-	    fprintf(dump_file, "\t%s",asctime(localtime(&walltime)));
-	    fprintf(dump_file, "$end\n");
+		if (!dump_no_date) {
+			fprintf(dump_file, "$date\n");
+			fprintf(dump_file, "\t%s",asctime(localtime(&walltime)));
+			fprintf(dump_file, "$end\n");
+		}
 	    fprintf(dump_file, "$version\n");
 	    fprintf(dump_file, "\tIcarus Verilog\n");
 	    fprintf(dump_file, "$end\n");
@@ -529,7 +538,13 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	  case vpiNamedEvent: type = "event"; break;
 	  case vpiIntVar:
 	  case vpiIntegerVar: type = "integer"; break;
-	  case vpiParameter:  type = "parameter"; break;
+	    /* VCD doesn't support real parameters, so lie. */
+	  case vpiParameter:
+	    switch (vpi_get(vpiConstType, item)) {
+		case vpiRealConst: type = "real"; break;
+		default: type = "parameter"; break;
+	    }
+	    break;
 	    /* Icarus converts realtime to real. */
 	  case vpiRealVar:    type = "real"; break;
 	  case vpiMemoryWord:
@@ -561,11 +576,12 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	  case vpiNamedFork:  type = "fork"; break;
 	  case vpiFunction:   type = "function"; break;
 	  case vpiModule:     type = "module"; break;
+	  case vpiPackage:    type = "package"; break;
 	  case vpiTask:       type = "task"; break;
 
 	  default:
 	    vpi_printf("VCD warning: $dumpvars: Unsupported argument "
-	               "type (%s)\n", vpi_get_str(vpiType, item));
+	               "type (%s).\n", vpi_get_str(vpiType, item));
 	    return;
       }
 
@@ -596,10 +612,6 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 
 	/* Generate the $var or $scope commands. */
       switch (item_type) {
-	  case vpiParameter:
-	    vpi_printf("VCD sorry: $dumpvars: can not dump parameters.\n");
-	    break;
-
 	  case vpiNamedEvent:
 	  case vpiIntegerVar:
 	  case vpiBitVar:
@@ -613,7 +625,6 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	  case vpiTimeVar:
 	  case vpiNet:
 
-
 	      /* If we are skipping all signal or this is in an automatic
 	       * scope then just return. */
             if (skip || vpi_get(vpiAutomatic, item)) return;
@@ -679,6 +690,35 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	    fprintf(dump_file, " $end\n");
 	    break;
 
+	  case vpiParameter:
+
+	    /* If we are skipping all pamaeters then just return. */
+            if (skip) return;
+
+	    size = vpi_get(vpiSize, item);
+
+	    /* Declare the parameter in the VCD file. */
+	    name = vpi_get_str(vpiName, item);
+	    prefix = is_escaped_id(name) ? "\\" : "";
+
+	    ident = strdup(vcdid);
+	    gen_new_vcd_id();
+
+	    /* Make an info item to go in the vcd_const_list. */
+	    info = malloc(sizeof(*info));
+	    info->item = item;
+	    info->ident = ident;
+	    info->scheduled = 0;
+	    info->dmp_next = 0;
+	    info->next = vcd_const_list;
+	    vcd_const_list = info;
+	    info->cb = NULL;
+
+	    /* Generate the $var record. Now the parameter is declared. */
+	    fprintf(dump_file, "$var %s %u %s %s%s $end\n",
+		    type, size, ident, prefix, name);
+	    break;
+
 	  case vpiModule:
 	  case vpiGenScope:
 	  case vpiFunction:
@@ -692,7 +732,7 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 			/* Value */
 			vpiNamedEvent,
 			vpiNet,
-			/* vpiParameter, */
+			vpiParameter,
 			vpiReg,
 			vpiVariables,
 			/* Scope */
@@ -731,6 +771,11 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 		  fprintf(dump_file, "$upscope $end\n");
 	    }
 	    break;
+
+	  case vpiPackage:
+	    vpi_printf("VCD warning: $dumpvars: Package (%s) is not dumpable "
+	               "with VCD.\n", vpi_get_str(vpiFullName, item));
+	    break;
       }
 }
 
@@ -798,10 +843,10 @@ static PLI_INT32 sys_dumpvars_calltf(ICARUS_VPI_CONST PLI_BYTE8*name)
       }
       if (!depth) depth = 10000;
 
-        /* This dumps all the modules in the design if none are given. */
+        /* This dumps all the instances in the design if none are given. */
       if (!argv || !(item = vpi_scan(argv))) {
-	    argv = vpi_iterate(vpiModule, 0x0);
-	    assert(argv);  /* There must be at least one top level module. */
+	    argv = vpi_iterate(vpiInstance, 0x0);
+	    assert(argv);  /* There must be at least one top level instance. */
 	    item = vpi_scan(argv);
       }
 
@@ -873,6 +918,17 @@ void sys_vcd_register(void)
 {
       s_vpi_systf_data tf_data;
       vpiHandle res;
+      int idx;
+      struct t_vpi_vlog_info vlog_info;
+
+      /* Scan the extended arguments */
+      vpi_get_vlog_info(&vlog_info);
+
+      for (idx = 0 ;  idx < vlog_info.argc ;  idx += 1) {
+        if (strcmp(vlog_info.argv[idx],"-no-date") == 0) {
+          dump_no_date = 1;
+        }
+      }
 
       /* All the compiletf routines are located in vcd_priv.c. */
 
