@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2017 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -25,6 +25,8 @@
 # include  <cstdlib>
 # include  <cmath>
 
+using namespace std;
+
 vvp_arith_::vvp_arith_(unsigned wid)
 : wid_(wid), op_a_(wid), op_b_(wid), x_val_(wid)
 {
@@ -35,7 +37,7 @@ vvp_arith_::vvp_arith_(unsigned wid)
       }
 }
 
-void vvp_arith_::dispatch_operand_(vvp_net_ptr_t ptr, vvp_vector4_t bit)
+void vvp_arith_::dispatch_operand_(vvp_net_ptr_t ptr, const vvp_vector4_t&bit)
 {
       unsigned port = ptr.port();
       switch (port) {
@@ -52,10 +54,9 @@ void vvp_arith_::dispatch_operand_(vvp_net_ptr_t ptr, vvp_vector4_t bit)
 }
 
 void vvp_arith_::recv_vec4_pv(vvp_net_ptr_t ptr, const vvp_vector4_t&bit,
-			      unsigned base, unsigned wid, unsigned vwid,
-                              vvp_context_t ctx)
+			      unsigned base, unsigned vwid, vvp_context_t ctx)
 {
-      recv_vec4_pv_(ptr, bit, base, wid, vwid, ctx);
+      recv_vec4_pv_(ptr, bit, base, vwid, ctx);
 }
 
 vvp_arith_abs::vvp_arith_abs()
@@ -89,10 +90,9 @@ void vvp_arith_abs::recv_vec4(vvp_net_ptr_t ptr, const vvp_vector4_t&bit,
 }
 
 void vvp_arith_abs::recv_vec4_pv(vvp_net_ptr_t ptr, const vvp_vector4_t&bit,
-				 unsigned base, unsigned wid, unsigned vwid,
-				 vvp_context_t ctx)
+				 unsigned base, unsigned vwid, vvp_context_t ctx)
 {
-      recv_vec4_pv_(ptr, bit, base, wid, vwid, ctx);
+      recv_vec4_pv_(ptr, bit, base, vwid, ctx);
 }
 
 void vvp_arith_abs::recv_real(vvp_net_ptr_t ptr, double bit,
@@ -135,10 +135,10 @@ void vvp_arith_cast_real::recv_vec4(vvp_net_ptr_t ptr, const vvp_vector4_t&bit,
 }
 
 void vvp_arith_cast_real::recv_vec4_pv(vvp_net_ptr_t ptr, const vvp_vector4_t&bit,
-				       unsigned base, unsigned wid, unsigned vwid,
+				       unsigned base, unsigned vwid,
 				       vvp_context_t ctx)
 {
-      recv_vec4_pv_(ptr, bit, base, wid, vwid, ctx);
+      recv_vec4_pv_(ptr, bit, base, vwid, ctx);
 }
 
 vvp_arith_cast_vec2::vvp_arith_cast_vec2(unsigned wid)
@@ -159,15 +159,14 @@ void vvp_arith_cast_vec2::recv_real(vvp_net_ptr_t ptr, double bit,
 void vvp_arith_cast_vec2::recv_vec4(vvp_net_ptr_t ptr, const vvp_vector4_t&bit,
                                     vvp_context_t)
 {
-      vvp_vector2_t tmp = bit;
+      vvp_vector2_t tmp = vvp_vector2_t(bit);
       ptr.ptr()->send_vec4(vector2_to_vector4(tmp,wid_), 0);
 }
 
 void vvp_arith_cast_vec2::recv_vec4_pv(vvp_net_ptr_t ptr, const vvp_vector4_t&bit,
-				       unsigned base, unsigned wid, unsigned vwid,
-				       vvp_context_t ctx)
+				       unsigned base, unsigned vwid, vvp_context_t ctx)
 {
-      recv_vec4_pv_(ptr, bit, base, wid, vwid, ctx);
+      recv_vec4_pv_(ptr, bit, base, vwid, ctx);
 }
 
 // Division
