@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2013 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2007-2022 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -36,6 +36,8 @@
 # include  <cassert>
 # include  "ivl_alloc.h"
 
+using namespace std;
+
 unsigned long count_net_arrays = 0;
 unsigned long count_net_array_words = 0;
 unsigned long count_var_arrays = 0;
@@ -210,18 +212,11 @@ void __vpiArray::get_word_value(struct __vpiArrayWord*word, p_vpi_value vp)
             case vpiHexStrVal:
             case vpiScalarVal:
             case vpiIntVal:
-            {
-                vvp_vector4_t v;
-                vals->get_word(index, v);
-                vpip_vec4_get_value(v, vals_width, signed_flag, vp);
-            }
-            break;
-
             case vpiVectorVal:
             {
                 vvp_vector4_t v;
                 vals->get_word(index, v);
-                vpip_vec2_get_value(v, vals_width, signed_flag, vp);
+                vpip_vec4_get_value(v, vals_width, signed_flag, vp);
             }
             break;
 
@@ -607,10 +602,21 @@ void __vpiArray::set_word(unsigned address, unsigned part_off, const vvp_vector4
 
       if (vals) {
 	    assert(nets == 0);
-	      // FIXME: For now, assume no part select of word?
-	    assert(part_off==0);
-	    assert(val.size() == vals_width);
-	    vals->set_word(address, val);
+	    if (part_off != 0 || val.size() != vals_width) {
+		  vvp_vector4_t tmp;
+		  vals->get_word(address, tmp);
+		  if ((part_off + val.size()) > tmp.size()) {
+			cerr << "part_off=" << part_off
+			     << " val.size()=" << val.size()
+			     << " vals[address].size()=" << tmp.size()
+			     << " vals_width=" << vals_width << endl;
+			assert(0);
+		  }
+		  tmp.set_vec(part_off, val);
+		  vals->set_word(address, tmp);
+	    } else {
+		  vals->set_word(address, val);
+	    }
 	    word_change(address);
 	    return;
       }
@@ -622,7 +628,7 @@ void __vpiArray::set_word(unsigned address, unsigned part_off, const vvp_vector4
       struct __vpiSignal*vsig = dynamic_cast<__vpiSignal*>(word);
       assert(vsig);
 
-      vsig->node->send_vec4_pv(val, part_off, val.size(), vpip_size(vsig), 0);
+      vsig->node->send_vec4_pv(val, part_off, vpip_size(vsig), 0);
       word_change(address);
 }
 
@@ -724,6 +730,9 @@ double __vpiArray::get_word_r(unsigned address)
 	    return val;
       }
 
+      if (address >= get_size())
+	    return 0.0;
+
       assert(nets);
       vpiHandle word = nets[address];
       struct __vpiRealVar*vsig = dynamic_cast<__vpiRealVar*>(word);
@@ -1116,8 +1125,8 @@ void vvp_fun_arrayport_sa::check_word_change(unsigned long addr)
 class vvp_fun_arrayport_aa  : public vvp_fun_arrayport, public automatic_hooks_s {
 
     public:
-      explicit vvp_fun_arrayport_aa(vvp_array_t mem, vvp_net_t*net);
-      explicit vvp_fun_arrayport_aa(vvp_array_t mem, vvp_net_t*net, long addr);
+      explicit vvp_fun_arrayport_aa(__vpiScope*context_scope, vvp_array_t mem, vvp_net_t*net);
+      explicit vvp_fun_arrayport_aa(__vpiScope*context_scope, vvp_array_t mem, vvp_net_t*net, long addr);
       ~vvp_fun_arrayport_aa();
 
       void alloc_instance(vvp_context_t context);
@@ -1138,18 +1147,16 @@ class vvp_fun_arrayport_aa  : public vvp_fun_arrayport, public automatic_hooks_s
       unsigned context_idx_;
 };
 
-vvp_fun_arrayport_aa::vvp_fun_arrayport_aa(vvp_array_t mem, vvp_net_t*net)
-: vvp_fun_arrayport(mem, net)
+vvp_fun_arrayport_aa::vvp_fun_arrayport_aa(__vpiScope*context_scope, vvp_array_t mem, vvp_net_t*net)
+: vvp_fun_arrayport(mem, net), context_scope_(context_scope)
 {
-      context_scope_ = vpip_peek_context_scope();
-      context_idx_ = vpip_add_item_to_context(this, context_scope_);
+      context_idx_ = vpip_add_item_to_context(this, context_scope);
 }
 
-vvp_fun_arrayport_aa::vvp_fun_arrayport_aa(vvp_array_t mem, vvp_net_t*net, long addr)
-: vvp_fun_arrayport(mem, net, addr)
+vvp_fun_arrayport_aa::vvp_fun_arrayport_aa(__vpiScope*context_scope, vvp_array_t mem, vvp_net_t*net, long addr)
+: vvp_fun_arrayport(mem, net, addr), context_scope_(context_scope)
 {
-      context_scope_ = vpip_peek_context_scope();
-      context_idx_ = vpip_add_item_to_context(this, context_scope_);
+      context_idx_ = vpip_add_item_to_context(this, context_scope);
 }
 
 vvp_fun_arrayport_aa::~vvp_fun_arrayport_aa()
@@ -1364,6 +1371,7 @@ class array_port_resolv_list_t : public resolv_list_s {
       explicit array_port_resolv_list_t(char* lab, bool use_addr__,
                                         long addr__);
 
+      __vpiScope*context_scope;
       vvp_net_t*ptr;
       bool use_addr;
       long addr;
@@ -1376,6 +1384,10 @@ array_port_resolv_list_t::array_port_resolv_list_t(char *lab, bool use_addr__,
                                                    long addr__)
 : resolv_list_s(lab), use_addr(use_addr__), addr(addr__)
 {
+      if (vpip_peek_current_scope()->is_automatic())
+            context_scope = vpip_peek_context_scope();
+      else
+            context_scope = 0;
       ptr = new vvp_net_t;
 }
 
@@ -1389,13 +1401,13 @@ bool array_port_resolv_list_t::resolve(bool mes)
 
       vvp_fun_arrayport*fun;
       if (use_addr)
-            if (vpip_peek_current_scope()->is_automatic())
-                  fun = new vvp_fun_arrayport_aa(mem, ptr, addr);
+            if (context_scope)
+                  fun = new vvp_fun_arrayport_aa(context_scope, mem, ptr, addr);
             else
                   fun = new vvp_fun_arrayport_sa(mem, ptr, addr);
       else
-            if (vpip_peek_current_scope()->is_automatic())
-                  fun = new vvp_fun_arrayport_aa(mem, ptr);
+            if (context_scope)
+                  fun = new vvp_fun_arrayport_aa(context_scope, mem, ptr);
             else
                   fun = new vvp_fun_arrayport_sa(mem, ptr);
       ptr->fun = fun;
@@ -1460,7 +1472,7 @@ value_callback*vpip_array_word_change(p_cb_data data)
       struct __vpiArray*parent = 0;
       array_word_value_callback*cbh = 0;
       if (struct __vpiArrayWord*word = array_var_word_from_handle(data->obj)) {
-	    parent = (__vpiArray*) word->get_parent();
+	    parent = static_cast<__vpiArray*>(word->get_parent());
 	    unsigned addr = word->get_index();
 	    cbh = new array_word_value_callback(data);
 	    cbh->word_addr = addr;
