@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2013 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2012-2019 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -36,7 +36,7 @@ using namespace std;
  */
 class class_property_t {
     public:
-      explicit inline class_property_t() { }
+      inline class_property_t() { }
       virtual ~class_property_t() =0;
 	// How much space does an instance of this property require?
       virtual size_t instance_size() const =0;
@@ -143,7 +143,7 @@ template <class T> class property_atom : public class_property_t {
 
 class property_bit : public class_property_t {
     public:
-      inline property_bit(size_t wid): wid_(wid) { }
+      explicit inline property_bit(size_t wid): wid_(wid) { }
       ~property_bit() { }
 
       size_t instance_size() const { return sizeof(vvp_vector2_t); }
@@ -168,7 +168,7 @@ class property_bit : public class_property_t {
 
 class property_logic : public class_property_t {
     public:
-      inline property_logic(size_t wid): wid_(wid) { }
+      explicit inline property_logic(size_t wid): wid_(wid) { }
       ~property_logic() { }
 
       size_t instance_size() const { return sizeof(vvp_vector4_t); }
@@ -441,6 +441,9 @@ void class_type::set_property(size_t idx, const string&name, const string&type,
       } else if (type[0] == 'L') {
 	    size_t wid = strtoul(type.c_str()+1,0,0);
 	    properties_[idx].type = new property_logic(wid);
+      } else if (type[0] == 's' && type[1] == 'L') {
+	    size_t wid = strtoul(type.c_str()+2,0,0);
+	    properties_[idx].type = new property_logic(wid);
       } else {
 	    properties_[idx].type = 0;
       }
@@ -595,7 +598,7 @@ void compile_class_property(unsigned idx, char*nam, char*typ, uint64_t array_siz
 
 void compile_class_done(void)
 {
-      struct __vpiScope*scope = vpip_peek_current_scope();
+      __vpiScope*scope = vpip_peek_current_scope();
       assert(scope);
       assert(compile_class);
       compile_class->finish_setup();
