@@ -1,7 +1,7 @@
 #ifndef IVL_codes_H
 #define IVL_codes_H
 /*
- * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -41,8 +41,6 @@ extern bool of_ANDR(vthread_t thr, vvp_code_t code);
 extern bool of_ASSIGN_AR(vthread_t thr, vvp_code_t code);
 extern bool of_ASSIGN_ARD(vthread_t thr, vvp_code_t code);
 extern bool of_ASSIGN_ARE(vthread_t thr, vvp_code_t code);
-extern bool of_ASSIGN_D(vthread_t thr, vvp_code_t code);
-extern bool of_ASSIGN_MV(vthread_t thr, vvp_code_t code);
 extern bool of_ASSIGN_VEC4(vthread_t thr, vvp_code_t code);
 extern bool of_ASSIGN_VEC4D(vthread_t thr, vvp_code_t code);
 extern bool of_ASSIGN_VEC4E(vthread_t thr, vvp_code_t code);
@@ -81,8 +79,6 @@ extern bool of_CMPIU(vthread_t thr, vvp_code_t code);
 extern bool of_CMPWE(vthread_t thr, vvp_code_t code);
 extern bool of_CMPWNE(vthread_t thr, vvp_code_t code);
 extern bool of_CMPWR(vthread_t thr, vvp_code_t code);
-extern bool of_CMPWS(vthread_t thr, vvp_code_t code);
-extern bool of_CMPWU(vthread_t thr, vvp_code_t code);
 extern bool of_CMPX(vthread_t thr, vvp_code_t code);
 extern bool of_CMPZ(vthread_t thr, vvp_code_t code);
 extern bool of_CONCAT_STR(vthread_t thr, vvp_code_t code);
@@ -103,14 +99,17 @@ extern bool of_DELETE_ELEM(vthread_t thr, vvp_code_t code);
 extern bool of_DELETE_OBJ(vthread_t thr, vvp_code_t code);
 extern bool of_DELETE_TAIL(vthread_t thr, vvp_code_t code);
 extern bool of_DISABLE(vthread_t thr, vvp_code_t code);
+extern bool of_DISABLE_FLOW(vthread_t thr, vvp_code_t code);
 extern bool of_DISABLE_FORK(vthread_t thr, vvp_code_t code);
 extern bool of_DIV(vthread_t thr, vvp_code_t code);
 extern bool of_DIV_S(vthread_t thr, vvp_code_t code);
 extern bool of_DIV_WR(vthread_t thr, vvp_code_t code);
+extern bool of_DUP_OBJ(vthread_t thr, vvp_code_t code);
 extern bool of_DUP_REAL(vthread_t thr, vvp_code_t code);
 extern bool of_DUP_VEC4(vthread_t thr, vvp_code_t code);
 extern bool of_END(vthread_t thr, vvp_code_t code);
 extern bool of_EVENT(vthread_t thr, vvp_code_t code);
+extern bool of_EVENT_NB(vthread_t thr, vvp_code_t code);
 extern bool of_EVCTL(vthread_t thr, vvp_code_t code);
 extern bool of_EVCTLC(vthread_t thr, vvp_code_t code);
 extern bool of_EVCTLI(vthread_t thr, vvp_code_t code);
@@ -162,7 +161,6 @@ extern bool of_MIN_WR(vthread_t thr, vvp_code_t code);
 extern bool of_MOD(vthread_t thr, vvp_code_t code);
 extern bool of_MOD_S(vthread_t thr, vvp_code_t code);
 extern bool of_MOD_WR(vthread_t thr, vvp_code_t code);
-extern bool of_MOV_WU(vthread_t thr, vvp_code_t code);
 extern bool of_MUL(vthread_t thr, vvp_code_t code);
 extern bool of_MULI(vthread_t thr, vvp_code_t code);
 extern bool of_MUL_WR(vthread_t thr, vvp_code_t code);
@@ -221,7 +219,6 @@ extern bool of_SCOPY(vthread_t thr, vvp_code_t code);
 extern bool of_SET_DAR_OBJ_REAL(vthread_t thr, vvp_code_t code);
 extern bool of_SET_DAR_OBJ_STR(vthread_t thr, vvp_code_t code);
 extern bool of_SET_DAR_OBJ_VEC4(vthread_t thr, vvp_code_t code);
-extern bool of_SET_X0_X(vthread_t thr, vvp_code_t code);
 extern bool of_SHIFTL(vthread_t thr, vvp_code_t code);
 extern bool of_SHIFTR(vthread_t thr, vvp_code_t code);
 extern bool of_SHIFTR_S(vthread_t thr, vvp_code_t code);
