@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2016 Stephen Williams <steve@icarus.com>
+ * Copyright (c) 2005-2021 Stephen Williams <steve@icarus.com>
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -31,6 +31,8 @@
 #include <cmath>
 #include "ivl_alloc.h"
 
+using namespace std;
+
 void vvp_delay_t::calculate_min_delay_()
 {
       min_delay_ = rise_;
@@ -166,13 +168,13 @@ vvp_fun_delay::vvp_fun_delay(vvp_net_t*n, unsigned width, const vvp_delay_t&d)
 	// to simulation time units.
       __vpiScope*scope = vpip_peek_current_scope();
 
-      int pow = scope->time_units - scope->time_precision;
+      int powr = scope->time_units - scope->time_precision;
       round_ = 1;
-      for (int lp = 0; lp < pow; lp += 1) round_ *= 10;
+      for (int lp = 0; lp < powr; lp += 1) round_ *= 10;
 
-      pow = scope->time_precision - vpip_get_time_precision();
+      powr = scope->time_precision - vpip_get_time_precision();
       scale_ = 1;
-      for (int lp = 0; lp < pow; lp += 1) scale_ *= 10;
+      for (int lp = 0; lp < powr; lp += 1) scale_ *= 10;
 }
 
 vvp_fun_delay::~vvp_fun_delay()
@@ -333,10 +335,9 @@ void vvp_fun_delay::recv_vec4(vvp_net_ptr_t port, const vvp_vector4_t&bit,
 }
 
 void vvp_fun_delay::recv_vec4_pv(vvp_net_ptr_t ptr, const vvp_vector4_t&bit,
-			         unsigned base, unsigned wid, unsigned vwid,
-                                 vvp_context_t ctx)
+			         unsigned base, unsigned vwid, vvp_context_t ctx)
 {
-      recv_vec4_pv_(ptr, bit, base, wid, vwid, ctx);
+      recv_vec4_pv_(ptr, bit, base, vwid, ctx);
 }
 
 /* See the recv_vec4 comment above. */
@@ -405,9 +406,9 @@ void vvp_fun_delay::recv_vec8(vvp_net_ptr_t port, const vvp_vector8_t&bit)
 }
 
 void vvp_fun_delay::recv_vec8_pv(vvp_net_ptr_t ptr, const vvp_vector8_t&bit,
-			         unsigned base, unsigned wid, unsigned vwid)
+			         unsigned base, unsigned vwid)
 {
-      recv_vec8_pv_(ptr, bit, base, wid, vwid);
+      recv_vec8_pv_(ptr, bit, base, vwid);
 }
 
 void vvp_fun_delay::recv_real(vvp_net_ptr_t port, double bit,
