@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2004-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -107,7 +107,7 @@ evctl_vector::evctl_vector(vvp_net_ptr_t ptr, const vvp_vector4_t&value,
 void evctl_vector::run_run()
 {
       if (wid_ != 0) {
-	    vvp_send_vec4_pv(ptr_, value_, off_, value_.size(), wid_, 0);
+	    vvp_send_vec4_pv(ptr_, value_, off_, wid_, 0);
       } else {
 	    vvp_send_vec4(ptr_, value_, 0);
       }
@@ -199,6 +199,19 @@ const vvp_fun_edge::edge_t vvp_edge_negedge
       | VVP_EDGE(BIT4_Z,BIT4_0)
       ;
 
+const vvp_fun_edge::edge_t vvp_edge_edge
+      = VVP_EDGE(BIT4_0,BIT4_1)
+      | VVP_EDGE(BIT4_1,BIT4_0)
+      | VVP_EDGE(BIT4_0,BIT4_X)
+      | VVP_EDGE(BIT4_X,BIT4_0)
+      | VVP_EDGE(BIT4_0,BIT4_Z)
+      | VVP_EDGE(BIT4_Z,BIT4_0)
+      | VVP_EDGE(BIT4_X,BIT4_1)
+      | VVP_EDGE(BIT4_1,BIT4_X)
+      | VVP_EDGE(BIT4_Z,BIT4_1)
+      | VVP_EDGE(BIT4_1,BIT4_Z)
+      ;
+
 const vvp_fun_edge::edge_t vvp_edge_none    = 0;
 
 struct vvp_fun_edge_state_s : public waitable_state_s {
@@ -265,13 +278,12 @@ void vvp_fun_edge_sa::recv_vec4(vvp_net_ptr_t port, const vvp_vector4_t&bit,
 }
 
 void vvp_fun_edge_sa::recv_vec4_pv(vvp_net_ptr_t port, const vvp_vector4_t&bit,
-				   unsigned base, unsigned wid, unsigned vwid,
-				   vvp_context_t)
+				   unsigned base, unsigned vwid, vvp_context_t)
 {
       assert(base == 0);
       if (recv_vec4_(bit, bits_[port.port()], threads_)) {
 	    vvp_net_t*net = port.ptr();
-	    net->send_vec4_pv(bit, base, wid, vwid, 0);
+	    net->send_vec4_pv(bit, base, vwid, 0);
       }
 }
 
@@ -369,7 +381,7 @@ class anyedge_vec4_value : public anyedge_value {
       bool recv_vec4(const vvp_vector4_t&bit);
 
       bool recv_vec4_pv(const vvp_vector4_t&bit, unsigned base,
-			unsigned wid, unsigned vwid);
+			unsigned vwid);
 
     private:
       vvp_vector4_t old_bits;
@@ -516,13 +528,12 @@ bool anyedge_vec4_value::recv_vec4(const vvp_vector4_t&bit)
 }
 
 bool anyedge_vec4_value::recv_vec4_pv(const vvp_vector4_t&bit, unsigned base,
-				      unsigned wid, unsigned vwid)
+				      unsigned vwid)
 {
       vvp_vector4_t tmp = old_bits;
       if (tmp.size() == 0)
 	    tmp = vvp_vector4_t(vwid, BIT4_Z);
-      assert(wid == bit.size());
-      assert(base+wid <= vwid);
+      assert(base + bit.size()<= vwid);
       assert(tmp.size() == vwid);
       tmp.set_vec(base, bit);
 
@@ -591,12 +602,11 @@ void vvp_fun_anyedge_sa::recv_vec4(vvp_net_ptr_t port, const vvp_vector4_t&bit,
 }
 
 void vvp_fun_anyedge_sa::recv_vec4_pv(vvp_net_ptr_t port, const vvp_vector4_t&bit,
-				      unsigned base, unsigned wid, unsigned vwid,
-				      vvp_context_t)
+				      unsigned base, unsigned vwid, vvp_context_t)
 {
       anyedge_vec4_value*value = get_vec4_value(last_value_[port.port()]);
       assert(value);
-      if (value->recv_vec4_pv(bit, base, wid, vwid)) {
+      if (value->recv_vec4_pv(bit, base, vwid)) {
 	    run_waiting_threads_(threads_);
 	    vvp_net_t*net = port.ptr();
 	    net->send_vec4(bit, 0);
@@ -627,6 +637,18 @@ void vvp_fun_anyedge_sa::recv_string(vvp_net_ptr_t port, const std::string&bit,
       }
 }
 
+/*
+ * An anyedge receiving an object should do nothing with it, but should
+ * trigger waiting threads.
+ */
+void vvp_fun_anyedge_sa::recv_object(vvp_net_ptr_t port, vvp_object_t,
+				     vvp_context_t)
+{
+      run_waiting_threads_(threads_);
+      vvp_net_t*net = port.ptr();
+      net->send_vec4(vvp_vector4_t(), 0);
+}
+
 vvp_fun_anyedge_aa::vvp_fun_anyedge_aa()
 {
       context_scope_ = vpip_peek_context_scope();
@@ -755,7 +777,8 @@ void vvp_fun_anyedge_aa::recv_string(vvp_net_ptr_t port, const std::string&bit,
       }
 }
 
-vvp_fun_event_or::vvp_fun_event_or()
+vvp_fun_event_or::vvp_fun_event_or(vvp_net_t*base_net)
+: base_net_(base_net)
 {
 }
 
@@ -763,8 +786,8 @@ vvp_fun_event_or::~vvp_fun_event_or()
 {
 }
 
-vvp_fun_event_or_sa::vvp_fun_event_or_sa()
-: threads_(0)
+vvp_fun_event_or_sa::vvp_fun_event_or_sa(vvp_net_t*base_net)
+: vvp_fun_event_or(base_net), threads_(0)
 {
 }
 
@@ -780,15 +803,15 @@ vthread_t vvp_fun_event_or_sa::add_waiting_thread(vthread_t thread)
       return tmp;
 }
 
-void vvp_fun_event_or_sa::recv_vec4(vvp_net_ptr_t port, const vvp_vector4_t&bit,
+void vvp_fun_event_or_sa::recv_vec4(vvp_net_ptr_t, const vvp_vector4_t&bit,
                                     vvp_context_t)
 {
       run_waiting_threads_(threads_);
-      vvp_net_t*net = port.ptr();
-      net->send_vec4(bit, 0);
+      base_net_->send_vec4(bit, 0);
 }
 
-vvp_fun_event_or_aa::vvp_fun_event_or_aa()
+vvp_fun_event_or_aa::vvp_fun_event_or_aa(vvp_net_t*base_net)
+: vvp_fun_event_or(base_net)
 {
       context_scope_ = vpip_peek_context_scope();
       context_idx_ = vpip_add_item_to_context(this, context_scope_);
@@ -839,8 +862,7 @@ void vvp_fun_event_or_aa::recv_vec4(vvp_net_ptr_t port, const vvp_vector4_t&bit,
                   (vvp_get_context_item(context, context_idx_));
 
             run_waiting_threads_(state->threads);
-            vvp_net_t*net = port.ptr();
-            net->send_vec4(bit, context);
+            base_net_->send_vec4(bit, context);
       } else {
             context = context_scope_->live_contexts;
             while (context) {
@@ -963,7 +985,7 @@ void compile_event(char*label, char*type, unsigned argc, struct symb_s*argv)
 	    return;
       }
 
-      if (strcmp(type,"edge") == 0) {
+      if (strcmp(type,"anyedge") == 0) {
 
 	    free(type);
 
@@ -975,20 +997,22 @@ void compile_event(char*label, char*type, unsigned argc, struct symb_s*argv)
 
       } else {
 
-	    vvp_fun_edge::edge_t edge = vvp_edge_none;
+	    vvp_fun_edge::edge_t edge_type = vvp_edge_none;
 
 	    if (strcmp(type,"posedge") == 0)
-		  edge = vvp_edge_posedge;
+		  edge_type = vvp_edge_posedge;
 	    else if (strcmp(type,"negedge") == 0)
-		  edge = vvp_edge_negedge;
+		  edge_type = vvp_edge_negedge;
+	    else if (strcmp(type,"edge") == 0)
+		  edge_type = vvp_edge_edge;
 
 	    assert(argc <= 4);
 	    free(type);
 
             if (vpip_peek_current_scope()->is_automatic()) {
-                  fun = new vvp_fun_edge_aa(edge);
+                  fun = new vvp_fun_edge_aa(edge_type);
             } else {
-                  fun = new vvp_fun_edge_sa(edge);
+                  fun = new vvp_fun_edge_sa(edge_type);
             }
 
       }
@@ -1005,20 +1029,25 @@ void compile_event(char*label, char*type, unsigned argc, struct symb_s*argv)
 
 static void compile_event_or(char*label, unsigned argc, struct symb_s*argv)
 {
-      vvp_net_t* ptr = new vvp_net_t;
+      vvp_net_t*base_net = new vvp_net_t;
       if (vpip_peek_current_scope()->is_automatic()) {
-            ptr->fun = new vvp_fun_event_or_aa;
+            base_net->fun = new vvp_fun_event_or_aa(base_net);
       } else {
-            ptr->fun = new vvp_fun_event_or_sa;
+            base_net->fun = new vvp_fun_event_or_sa(base_net);
       }
-      define_functor_symbol(label, ptr);
+      define_functor_symbol(label, base_net);
       free(label);
 
-	/* This is a very special case. Point all the source inputs to
-	   the same input. It doesn't matter that the streams get
-	   tangled because data values are irrelevant. */
+	/* This is a simplified version of a wide functor. We don't
+	   care about the data values or what port they arrived on,
+	   so we can use a single shared functor. */
+      vvp_net_t*curr_net = base_net;
       for (unsigned idx = 0 ;  idx < argc ;  idx += 1) {
-	    input_connect(ptr, 0, argv[idx].text);
+	    if (idx > 0 && (idx % 4) == 0) {
+		  curr_net = new vvp_net_t;
+		  curr_net->fun = base_net->fun;
+	    }
+	    input_connect(curr_net, idx % 4, argv[idx].text);
       }
       free(argv);
 }
