@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2020 Cary R. (cygcary@yahoo.com)
+ * Copyright (C) 2011-2021 Cary R. (cygcary@yahoo.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,12 +19,18 @@
 # include "compile.h"
 # include "vpi_priv.h"
 
-struct __vpiFileLine : public __vpiHandle {
-      __vpiFileLine();
+class __vpiFileLine : public __vpiHandle {
+    public:
+      explicit __vpiFileLine(const char*desc_, long file_idx_, long lineno_);
       int get_type_code(void) const;
       int vpi_get(int code);
       char* vpi_get_str(int code);
 
+      const char* get_description() const { return description; };
+      unsigned get_file_idx() const { return file_idx; };
+      unsigned get_lineno() const { return lineno; };
+
+    private:
       const char *description;
       unsigned file_idx;
       unsigned lineno;
@@ -35,12 +41,12 @@ bool code_is_instrumented = false;
 
 static int file_line_get(int type, vpiHandle ref)
 {
-      struct __vpiFileLine*rfp = dynamic_cast<__vpiFileLine*>(ref);
+      __vpiFileLine*rfp = dynamic_cast<__vpiFileLine*>(ref);
       assert(rfp);
 
       switch (type) {
 	case vpiLineNo:
-	    return rfp->lineno;
+	    return rfp->get_lineno();
 	default:
 	    return vpiUndefined;
       }
@@ -48,23 +54,30 @@ static int file_line_get(int type, vpiHandle ref)
 
 static char *file_line_get_str(int type, vpiHandle ref)
 {
-      struct __vpiFileLine*rfp = dynamic_cast<__vpiFileLine*>(ref);
+      __vpiFileLine*rfp = dynamic_cast<__vpiFileLine*>(ref);
       assert(rfp);
 
       switch (type) {
 	case vpiFile:
-	    assert(rfp->file_idx < file_names.size());
-	    return simple_set_rbuf_str(file_names[rfp->file_idx]);
+	    assert(rfp->get_file_idx() < file_names.size());
+	    return simple_set_rbuf_str(file_names[rfp->get_file_idx()]);
 	case _vpiDescription:
-	    if (rfp->description) return simple_set_rbuf_str(rfp->description);
-	    else return simple_set_rbuf_str("Procedural tracing.");
+	    if (rfp->get_description()) {
+		  return simple_set_rbuf_str(rfp->get_description());
+	    } else return simple_set_rbuf_str("Procedural tracing.");
 	default:
 	    return 0;
       }
 }
 
-inline __vpiFileLine::__vpiFileLine()
-{ }
+inline __vpiFileLine::__vpiFileLine(const char*desc_, long file_idx_,
+                                    long lineno_)
+{
+      if (desc_) description = vpip_name_string(desc_);
+      else description = 0;
+      file_idx = (unsigned) file_idx_;
+      lineno = (unsigned) lineno_;
+}
 
 int __vpiFileLine::get_type_code(void) const
 { return _vpiFileLine; }
@@ -78,15 +91,10 @@ char* __vpiFileLine::vpi_get_str(int code)
 
 vpiHandle vpip_build_file_line(char*description, long file_idx, long lineno)
 {
-      struct __vpiFileLine*obj = new struct __vpiFileLine;
+      __vpiFileLine*obj = new __vpiFileLine(description, file_idx, lineno);
 
 	/* You can turn on the diagnostic output if we find a %file_line. */
       code_is_instrumented = true;
 
-      if (description) obj->description = vpip_name_string(description);
-      else obj->description = 0;
-      obj->file_idx = (unsigned) file_idx;
-      obj->lineno = (unsigned) lineno;
-
       return obj;
 }
