@@ -1,6 +1,6 @@
+const char COPYRIGHT[] =
+  "Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)";
 /*
- * Copyright (c) 2001-2020 Stephen Williams (steve@icarus.com)
- *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
  *    General Public License as published by the Free Software
@@ -31,6 +31,7 @@
 # include  <cstdlib>
 # include  <cstring>
 # include  <unistd.h>
+# include  <cassert>
 #ifdef CHECK_WITH_VALGRIND
 # include  <pthread.h>
 #endif
@@ -48,6 +49,8 @@
 # include  <windows.h>
 #endif
 
+using namespace std;
+
 ofstream debug_file;
 
 #if defined(__MINGW32__) && !defined(HAVE_GETOPT_H)
@@ -155,6 +158,7 @@ void verify_version(char*ivl_ver, char*commit)
       if (rc == 2) {
 	    file_extra[0] = 0;
 	    rc = sscanf(ivl_ver, "%d.%d %127s", &file_major, &file_minor, file_extra);
+	    assert((rc == 2) || (rc == 3));
 	    file_minor2 = 0;
       }
       delete[] ivl_ver;
@@ -339,7 +343,7 @@ int main(int argc, char*argv[])
       if (version_flag) {
 	    fprintf(stderr, "Icarus Verilog runtime version " VERSION " ("
 	                    VERSION_TAG ")\n\n");
-	    fprintf(stderr, "Copyright 1998-2020 Stephen Williams\n\n");
+	    fprintf(stderr, "%s\n\n", COPYRIGHT);
 	    fprintf(stderr,
 "  This program is free software; you can redistribute it and/or modify\n"
 "  it under the terms of the GNU General Public License as published by\n"
