@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -17,9 +17,11 @@
  *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+# include  "config.h"
 # include  "schedule.h"
 # include  "vthread.h"
 # include  "vpi_priv.h"
+# include  "vvp_net_sig.h"
 # include  "slab.h"
 # include  "compile.h"
 # include  <new>
@@ -27,8 +29,13 @@
 # include  <csignal>
 # include  <cstdlib>
 # include  <cassert>
-
 # include  <iostream>
+#ifdef CHECK_WITH_VALGRIND
+# include  "vvp_cleanup.h"
+# include  "ivl_alloc.h"
+#endif
+
+using namespace std;
 
 unsigned long count_assign_events = 0;
 unsigned long count_gen_events = 0;
@@ -69,8 +76,10 @@ void event_s::single_step_display(void)
 struct event_time_s {
       event_time_s() {
 	    count_time_events += 1;
+	    delay = 0;
 	    start = 0;
 	    active = 0;
+	    inactive = 0;
 	    nbassign = 0;
 	    rwsync = 0;
 	    rosync = 0;
@@ -81,6 +90,7 @@ struct event_time_s {
 
       struct event_s*start;
       struct event_s*active;
+      struct event_s*inactive;
       struct event_s*nbassign;
       struct event_s*rwsync;
       struct event_s*rosync;
@@ -121,7 +131,7 @@ void vthread_event_s::run_run(void)
 
 void vthread_event_s::single_step_display(void)
 {
-      struct __vpiScope*scope = vthread_scope(thr);
+      __vpiScope*scope = vthread_scope(thr);
       cerr << "vthread_event: Resume thread"
 	   << " scope=" << scope->vpi_get_str(vpiFullName)
 	   << endl;
@@ -154,14 +164,14 @@ void del_thr_event_s::run_run(void)
 
 void del_thr_event_s::single_step_display(void)
 {
-      struct __vpiScope*scope = vthread_scope(thr);
+      __vpiScope*scope = vthread_scope(thr);
       cerr << "del_thr_event: Reap completed thread"
 	   << " scope=" << scope->vpi_get_str(vpiFullName) << endl;
 }
 
 struct assign_vector4_event_s  : public event_s {
 	/* The default constructor. */
-      assign_vector4_event_s(const vvp_vector4_t&that) : val(that) {
+      explicit assign_vector4_event_s(const vvp_vector4_t&that) : val(that) {
 	    base = 0;
 	    vwid = 0;
       }
@@ -185,7 +195,7 @@ void assign_vector4_event_s::run_run(void)
 {
       count_assign_events += 1;
       if (vwid > 0)
-	    vvp_send_vec4_pv(ptr, val, base, val.size(), vwid, 0);
+	    vvp_send_vec4_pv(ptr, val, base, vwid, 0);
       else
 	    vvp_send_vec4(ptr, val, 0);
 }
@@ -319,13 +329,85 @@ void assign_array_word_s::operator delete(void*ptr)
 
 unsigned long count_assign_aword_pool(void) { return array_w_heap.pool; }
 
+struct force_vector4_event_s  : public event_s {
+	/* The default constructor. */
+      explicit force_vector4_event_s(const vvp_vector4_t&that): val(that) {
+	    net = NULL;
+	    base = 0;
+	    vwid = 0;
+      }
+	/* Where to do the force. */
+      vvp_net_t*net;
+	/* Value to force. */
+      vvp_vector4_t val;
+	/* Offset of the part into the destination. */
+      unsigned base;
+	/* Width of the destination vector. */
+      unsigned vwid;
+
+      void run_run(void);
+      void single_step_display(void);
+
+      static void* operator new(size_t);
+      static void operator delete(void*);
+};
+
+void force_vector4_event_s::run_run(void)
+{
+      count_assign_events += 1;
+
+      unsigned wid = val.size();
+      if ((base + wid) > vwid)
+	    wid = vwid - base;
+
+	// Make a mask of which bits are to be forced, 0 for unforced
+	// bits and 1 for forced bits.
+      vvp_vector2_t mask (vvp_vector2_t::FILL0, vwid);
+      for (unsigned idx = 0 ; idx < wid ; idx += 1)
+	    mask.set_bit(base+idx, 1);
+
+      vvp_vector4_t tmp (vwid, BIT4_Z);
+
+	// vvp_net_t::force_vec4 propagates all the bits of the
+	// forced vector value, regardless of the mask. This
+	// ensures the unforced bits retain their current value.
+      vvp_signal_value*sig = dynamic_cast<vvp_signal_value*>(net->fil);
+      assert(sig);
+      sig->vec4_value(tmp);
+
+      tmp.set_vec(base, val);
+      net->force_vec4(tmp, mask);
+}
+
+void force_vector4_event_s::single_step_display(void)
+{
+      cerr << "force_vector4_event: Force val=" << val
+	   << ", vwid=" << vwid << ", base=" << base << endl;
+}
+
+static const size_t FORCE4_CHUNK_COUNT = 8192 / sizeof(struct force_vector4_event_s);
+static slab_t<sizeof(force_vector4_event_s),FORCE4_CHUNK_COUNT> force4_heap;
+
+inline void* force_vector4_event_s::operator new(size_t size)
+{
+      assert(size == sizeof(force_vector4_event_s));
+      return force4_heap.alloc_slab();
+}
+
+void force_vector4_event_s::operator delete(void*dptr)
+{
+      force4_heap.free_slab(dptr);
+}
+
+unsigned long count_force4_pool(void) { return force4_heap.pool; }
+
 /*
  * This class supports the propagation of vec4 outputs from a
  * vvp_net_t object.
  */
 struct propagate_vector4_event_s : public event_s {
 	/* The default constructor. */
-      propagate_vector4_event_s(const vvp_vector4_t&that) : val(that) {
+      explicit propagate_vector4_event_s(const vvp_vector4_t&that) : val(that) {
 	    net = NULL;
       }
 	/* A constructor that makes the val directly. */
@@ -597,14 +679,13 @@ static void schedule_final_event(struct event_s*cur)
  * itself, and the structure is placed in the right place in the
  * queue.
  */
-typedef enum event_queue_e { SEQ_START, SEQ_ACTIVE, SEQ_NBASSIGN,
+typedef enum event_queue_e { SEQ_START, SEQ_ACTIVE, SEQ_INACTIVE, SEQ_NBASSIGN,
 			     SEQ_RWSYNC, SEQ_ROSYNC, DEL_THREAD } event_queue_t;
 
 static void schedule_event_(struct event_s*cur, vvp_time64_t delay,
 			    event_queue_t select_queue)
 {
       cur->next = cur;
-
       struct event_time_s*ctim = sched_list;
 
       if (sched_list == 0) {
@@ -672,6 +753,11 @@ static void schedule_event_(struct event_s*cur, vvp_time64_t delay,
 	    q = &ctim->active;
 	    break;
 
+	  case SEQ_INACTIVE:
+	    assert(delay == 0);
+	    q = &ctim->inactive;
+	    break;
+
 	  case SEQ_NBASSIGN:
 	    q = &ctim->nbassign;
 	    break;
@@ -737,6 +823,23 @@ void schedule_vthread(vthread_t thr, vvp_time64_t delay, bool push_flag)
       }
 }
 
+void schedule_t0_trigger(vvp_net_ptr_t ptr)
+{
+      vvp_vector4_t bit (1, BIT4_X);
+      struct assign_vector4_event_s*cur = new struct assign_vector4_event_s(bit);
+      cur->ptr = ptr;
+      schedule_event_(cur, 0, SEQ_INACTIVE);
+}
+
+void schedule_inactive(vthread_t thr)
+{
+      struct vthread_event_s*cur = new vthread_event_s;
+
+      cur->thr = thr;
+      vthread_mark_scheduled(thr);
+      schedule_event_(cur, 0, SEQ_INACTIVE);
+}
+
 void schedule_init_vthread(vthread_t thr)
 {
       struct vthread_event_s*cur = new vthread_event_s;
@@ -769,6 +872,18 @@ void schedule_assign_vector(vvp_net_ptr_t ptr,
       schedule_event_(cur, delay, SEQ_NBASSIGN);
 }
 
+void schedule_force_vector(vvp_net_t*net,
+			    unsigned base, unsigned vwid,
+			    const vvp_vector4_t&bit,
+			    vvp_time64_t delay)
+{
+      struct force_vector4_event_s*cur = new struct force_vector4_event_s(bit);
+      cur->net = net;
+      cur->base = base;
+      cur->vwid = vwid;
+      schedule_event_(cur, delay, SEQ_NBASSIGN);
+}
+
 void schedule_propagate_vector(vvp_net_t*net,
 			       vvp_time64_t delay,
 			       const vvp_vector4_t&src)
@@ -779,10 +894,23 @@ void schedule_propagate_vector(vvp_net_t*net,
       schedule_event_(cur, delay, SEQ_NBASSIGN);
 }
 
+// FIXME: This needs to create a non-blocking event, but only one per time slot.
+//        Is schedule_event_ or execution actually filtering since the net is
+//        already X because this is not triggering?
+void schedule_propagate_event(vvp_net_t*net,
+                              vvp_time64_t delay)
+{
+      vvp_vector4_t tmp (1, BIT4_X);
+      struct propagate_vector4_event_s*cur
+	    = new struct propagate_vector4_event_s(tmp);
+      cur->net = net;
+      schedule_event_(cur, delay, SEQ_NBASSIGN);
+}
+
 void schedule_assign_array_word(vvp_array_t mem,
 				unsigned word_addr,
 				unsigned off,
-				vvp_vector4_t val,
+				const vvp_vector4_t&val,
 				vvp_time64_t delay)
 {
       struct assign_array_word_s*cur = new struct assign_array_word_s;
@@ -814,7 +942,7 @@ void schedule_set_vector(vvp_net_ptr_t ptr, const vvp_vector4_t&bit)
       schedule_event_(cur, 0, SEQ_ACTIVE);
 }
 
-void schedule_set_vector(vvp_net_ptr_t ptr, vvp_vector8_t bit)
+void schedule_set_vector(vvp_net_ptr_t ptr, const vvp_vector8_t&bit)
 {
       struct assign_vector8_event_s*cur = new struct assign_vector8_event_s;
       cur->ptr = ptr;
@@ -830,7 +958,7 @@ void schedule_set_vector(vvp_net_ptr_t ptr, double bit)
       schedule_event_(cur, 0, SEQ_ACTIVE);
 }
 
-void schedule_init_vector(vvp_net_ptr_t ptr, vvp_vector4_t bit)
+void schedule_init_vector(vvp_net_ptr_t ptr, const vvp_vector4_t&bit)
 {
       struct assign_vector4_event_s*cur = new struct assign_vector4_event_s(bit);
       cur->ptr = ptr;
@@ -839,7 +967,7 @@ void schedule_init_vector(vvp_net_ptr_t ptr, vvp_vector4_t bit)
       schedule_init_event(cur);
 }
 
-void schedule_init_vector(vvp_net_ptr_t ptr, vvp_vector8_t bit)
+void schedule_init_vector(vvp_net_ptr_t ptr, const vvp_vector8_t&bit)
 {
       struct assign_vector8_event_s*cur = new struct assign_vector8_event_s;
       cur->ptr = ptr;
@@ -917,6 +1045,20 @@ void schedule_at_start_of_simtime(vvp_gen_event_t obj, vvp_time64_t delay)
       schedule_event_(cur, delay, SEQ_START);
 }
 
+/*
+ * In the vvp runtime of Icarus Verilog, the SEQ_RWSYNC time step is
+ * after all of the non-blocking assignments, so is effectively the
+ * same as the ReadWriteSync time.
+ */
+void schedule_at_end_of_simtime(vvp_gen_event_t obj, vvp_time64_t delay)
+{
+      struct generic_event_s*cur = new generic_event_s;
+
+      cur->obj = obj;
+      cur->delete_obj_when_done = false;
+      schedule_event_(cur, delay, SEQ_RWSYNC);
+}
+
 static vvp_time64_t schedule_time;
 vvp_time64_t schedule_simtime(void)
 { return schedule_time; }
@@ -968,7 +1110,7 @@ static void run_rosync(struct event_time_s*ctim)
 	    delete cur;
       }
 
-      if (ctim->active || ctim->nbassign || ctim->rwsync) {
+      if (ctim->active || ctim->inactive || ctim->nbassign || ctim->rwsync) {
 	    cerr << "SCHEDULER ERROR: read-only sync events "
 		 << "created RW events!" << endl;
       }
@@ -1069,21 +1211,26 @@ void schedule_simulate(void)
 		 queues. If there are not events at all, then release
 		 the event_time object. */
 	    if (ctim->active == 0) {
-		  ctim->active = ctim->nbassign;
-		  ctim->nbassign = 0;
+		  ctim->active = ctim->inactive;
+		  ctim->inactive = 0;
 
 		  if (ctim->active == 0) {
-			ctim->active = ctim->rwsync;
-			ctim->rwsync = 0;
+			ctim->active = ctim->nbassign;
+			ctim->nbassign = 0;
 
-			  /* If out of rw events, then run the rosync
-			     events and delete this time step. This also
-			     deletes threads as needed. */
 			if (ctim->active == 0) {
-			      run_rosync(ctim);
-			      sched_list = ctim->next;
-			      delete ctim;
-			      continue;
+			      ctim->active = ctim->rwsync;
+			      ctim->rwsync = 0;
+
+				/* If out of rw events, then run the rosync
+				   events and delete this time step. This also
+				   deletes threads as needed. */
+			      if (ctim->active == 0) {
+				    run_rosync(ctim);
+				    sched_list = ctim->next;
+				    delete ctim;
+				    continue;
+			      }
 			}
 		  }
 	    }
@@ -1130,4 +1277,21 @@ void schedule_simulate(void)
 
       // Execute post-simulation callbacks
       vpiPostsim();
+#ifdef CHECK_WITH_VALGRIND
+      schedule_delete();
+#endif
 }
+
+#ifdef CHECK_WITH_VALGRIND
+void schedule_delete(void)
+{
+      vthread_event_heap.delete_pool();
+      assign4_heap.delete_pool();
+      assign8_heap.delete_pool();
+      assignr_heap.delete_pool();
+      array_w_heap.delete_pool();
+      array_r_w_heap.delete_pool();
+      generic_event_heap.delete_pool();
+      event_time_heap.delete_pool();
+}
+#endif
