@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -37,6 +37,9 @@
 # include  <cstdio>
 # include  <cassert>
 # include  <cstdlib>
+
+using namespace std;
+
 /*
  * Callback handles are created when the VPI function registers a
  * callback. The handle is stored by the run time, and it triggered
@@ -320,9 +323,7 @@ void sync_cb::run_run()
       if (cur->cb_data.cb_rtn != 0) {
 	    assert(vpi_mode_flag == VPI_MODE_NONE);
 	    vpi_mode_flag = sync_flag? VPI_MODE_ROSYNC : VPI_MODE_RWSYNC;
-	    vpip_cur_task = dynamic_cast<__vpiSysTaskCall*>(cur->cb_data.obj);
 	    (cur->cb_data.cb_rtn)(&cur->cb_data);
-	    vpip_cur_task = 0;
 	    vpi_mode_flag = VPI_MODE_NONE;
       }
 
@@ -514,7 +515,9 @@ void vpiEndOfCompile(void) {
       while (EndOfCompile) {
 	    cur = EndOfCompile;
 	    EndOfCompile = dynamic_cast<simulator_callback*>(cur->next);
-	    (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    if (cur->cb_data.cb_rtn != 0) {
+	        (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    }
 	    delete cur;
       }
 
@@ -534,7 +537,9 @@ void vpiStartOfSim(void) {
       while (StartOfSimulation) {
 	    cur = StartOfSimulation;
 	    StartOfSimulation = dynamic_cast<simulator_callback*>(cur->next);
-	    (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    if (cur->cb_data.cb_rtn != 0) {
+	        (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    }
 	    delete cur;
       }
 
@@ -553,10 +558,12 @@ void vpiPostsim(void) {
       while (EndOfSimulation) {
 	    cur = EndOfSimulation;
 	    EndOfSimulation = dynamic_cast<simulator_callback*>(cur->next);
-	      /* Only set the time if it is not NULL. */
-	    if (cur->cb_data.time)
-	          vpip_time_to_timestruct(cur->cb_data.time, schedule_simtime());
-	    (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    if (cur->cb_data.cb_rtn != 0) {
+	        /* Only set the time if it is not NULL. */
+	        if (cur->cb_data.time)
+	            vpip_time_to_timestruct(cur->cb_data.time, schedule_simtime());
+	        (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    }
 	    delete cur;
       }
 
@@ -577,7 +584,9 @@ void vpiNextSimTime(void)
       while (NextSimTime) {
 	    cur = NextSimTime;
 	    NextSimTime = dynamic_cast<simulator_callback*>(cur->next);
-	    (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    if (cur->cb_data.cb_rtn != 0) {
+	        (cur->cb_data.cb_rtn)(&cur->cb_data);
+	    }
 	    delete cur;
       }
 
@@ -685,7 +694,8 @@ void callback_execute(struct __vpiCallback*cur)
 	  case vpiScaledRealTime: {
 	    cur->cb_data.time->real =
 	         vpip_time_to_scaled_real(schedule_simtime(),
-	             (__vpiScope *) vpi_handle(vpiScope, cur->cb_data.obj));
+	             static_cast<__vpiScope *>(vpi_handle(vpiScope,
+	                                                  cur->cb_data.obj)));
 	    break;
 	  }
 	  case vpiSuppressTime:
@@ -850,7 +860,8 @@ static double vlg_round(double rval)
 
 static void real_signal_value(struct t_vpi_value*vp, double rval)
 {
-      char*rbuf = (char *) need_result_buf(64 + 1, RBUF_VAL);
+      static const size_t RBUF_SIZE = 64 + 1;
+      char*rbuf = (char *) need_result_buf(RBUF_SIZE, RBUF_VAL);
 
       switch (vp->format) {
 	  case vpiObjTypeVal:
@@ -871,15 +882,15 @@ static void real_signal_value(struct t_vpi_value*vp, double rval)
 	    break;
 
 	  case vpiDecStrVal:
-	    if (isnan(rval))
-		  sprintf(rbuf, "%s", "nan");
+	    if (std::isnan(rval))
+		  snprintf(rbuf, RBUF_SIZE, "%s", "nan");
 	    else
-		  sprintf(rbuf, "%0.0f", vlg_round(rval));
+		  snprintf(rbuf, RBUF_SIZE, "%0.0f", vlg_round(rval));
 	    vp->value.str = rbuf;
 	    break;
 
 	  case vpiHexStrVal:
-	    sprintf(rbuf, "%" PRIx64, (uint64_t)vlg_round(rval));
+	    snprintf(rbuf, RBUF_SIZE, "%" PRIx64, (uint64_t)vlg_round(rval));
 	    vp->value.str = rbuf;
 	    break;
 
