@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -137,21 +137,28 @@ void __vpiStringConst::vpi_get_value(p_vpi_value vp)
 	    break;
 
           case vpiDecStrVal:
-	      if (size > 4){
+	    // Take the (up to 4) characters of the text, convert the characters
+	    // to a numerical value, and convert that value to a decimal string.
+	    // For example, the string "A" is ASCII 65, so the resulting string
+	    // will be "65". and the string "AB" is 65*256 + 66 == 16706, so
+	    // the resulting string is "16706". The "size" is the number of
+	    // characters of input text to put to work.
+	    if (size > 4){
 		  // We only support standard integers. Ignore other bytes...
 		  size = 4;
 		  fprintf(stderr, "Warning (vpi_const.cc): %%d on constant strings only looks "
 			  "at first 4 bytes!\n");
-	      }
-	      rbuf = (char *) need_result_buf(size + 1, RBUF_VAL);
-	      uint_value = 0;
-	      for(unsigned i=0; i<size; i += 1){
+	    }
+	    static const size_t RBUF_USE_SIZE = 12;
+	    rbuf = (char *) need_result_buf(RBUF_USE_SIZE, RBUF_VAL);
+	    uint_value = 0;
+	    for(unsigned i=0; i<size; i += 1){
 		  uint_value <<=8;
 		  uint_value += (unsigned char)(value_[i]);
-	      }
-	      sprintf(rbuf, "%u", uint_value);
-	      vp->value.str = rbuf;
-	      break;
+	    }
+	    snprintf(rbuf, RBUF_USE_SIZE, "%u", uint_value);
+	    vp->value.str = rbuf;
+	    break;
 
           case vpiBinStrVal:
 	      rbuf = (char *) need_result_buf(8 * size + 1, RBUF_VAL);
@@ -333,6 +340,7 @@ vpiHandle vpip_make_string_param(char*name, char*text,
 
 
 inline __vpiBinaryConst::__vpiBinaryConst()
+: signed_flag(0), sized_flag(0)
 { }
 
 int __vpiBinaryConst::get_type_code(void) const
@@ -395,7 +403,6 @@ void __vpiBinaryConst::vpi_get_value(p_vpi_value val)
       }
 }
 
-
 /*
  * Make a VPI constant from a vector string. The string is normally a
  * ASCII string, with each letter a 4-value bit. The first character
@@ -405,9 +412,6 @@ vpiHandle vpip_make_binary_const(unsigned wid, const char*bits)
 {
       struct __vpiBinaryConst*obj = new __vpiBinaryConst;
 
-      obj->signed_flag = 0;
-      obj->sized_flag = 0;
-
       const char*bp = bits;
       if (*bp == 's') {
 	    bp += 1;
@@ -577,7 +581,8 @@ int __vpiDecConst::vpi_get(int code)
 
 void __vpiDecConst::vpi_get_value(p_vpi_value vp)
 {
-      char*rbuf = (char *) need_result_buf(64 + 1, RBUF_VAL);
+      static const size_t RBUF_USE_SIZE = 64 + 1;
+      char*rbuf = (char *) need_result_buf(RBUF_USE_SIZE, RBUF_VAL);
       char*cp = rbuf;
 
       switch (vp->format) {
@@ -589,8 +594,7 @@ void __vpiDecConst::vpi_get_value(p_vpi_value vp)
 	  }
 
           case vpiDecStrVal:
-	      sprintf(rbuf, "%d", value);
-
+	      snprintf(rbuf, RBUF_USE_SIZE, "%d", value);
 	      vp->value.str = rbuf;
 	      break;
 
@@ -604,14 +608,12 @@ void __vpiDecConst::vpi_get_value(p_vpi_value vp)
 	      break;
 
           case vpiHexStrVal:
-	      sprintf(rbuf, "%08x", value);
-
+	      snprintf(rbuf, RBUF_USE_SIZE, "%08x", value);
 	      vp->value.str = rbuf;
 	      break;
 
           case vpiOctStrVal:
-	      sprintf(rbuf, "%011x", value);
-
+	      snprintf(rbuf, RBUF_USE_SIZE, "%011x", value);
 	      vp->value.str = rbuf;
 	      break;
 
@@ -757,6 +759,101 @@ vpiHandle vpip_make_real_param(char*name, double value,
       return obj;
 }
 
+/*
+ * Make a VPI null constant
+ */
+inline __vpiNullConst::__vpiNullConst()
+{ }
+
+int __vpiNullConst::get_type_code(void) const
+{ return vpiConstant; }
+
+int __vpiNullConst::vpi_get(int code)
+{
+      switch (code) {
+	  case vpiLineNo:
+	    return 0;  // Not implemented for now!
+
+	  case vpiSize:
+	    return 32;
+
+	  case vpiConstType:
+	    return vpiNullConst;
+
+	  case vpiSigned:
+	    return 0;
+
+          case vpiAutomatic:
+	    return 0;
+
+#if defined(CHECK_WITH_VALGRIND) || defined(BR916_STOPGAP_FIX)
+          case _vpiFromThr:
+	      return _vpiNoThr;
+#endif
+
+	  default:
+	    fprintf(stderr, "vvp error: get %d not supported "
+		    "by vpiNullConst\n", code);
+	    assert(0);
+	    return 0;
+      }
+}
+
+void __vpiNullConst::vpi_get_value(p_vpi_value val)
+{
+      static const size_t RBUF_USE_SIZE = 64 + 1;
+      char*rbuf = (char *) need_result_buf(RBUF_USE_SIZE, RBUF_VAL);
+
+      switch (val->format) {
+
+	  case vpiObjTypeVal:
+	    val->format = vpiStringVal;
+	    // fall through
+	  case vpiBinStrVal:
+	  case vpiDecStrVal:
+	  case vpiOctStrVal:
+	  case vpiHexStrVal:
+	  case vpiStringVal:
+	    snprintf(rbuf, RBUF_USE_SIZE, "null");
+	    val->value.str = rbuf;
+	    break;
+
+          case vpiScalarVal:
+	    val->value.scalar = vpi0;
+	    break;
+
+	  case vpiIntVal:
+	    val->value.integer = 0;
+	    break;
+
+	  case vpiVectorVal:
+	    val->value.vector = (p_vpi_vecval)
+	                        need_result_buf(2*sizeof(s_vpi_vecval),
+	                        RBUF_VAL);
+	    for (unsigned idx = 0; idx < 2; idx += 1) {
+		  val->value.vector[idx].aval = 0;
+		  val->value.vector[idx].bval = 0;
+	    }
+	    break;
+
+	  case vpiRealVal:
+	    val->value.real = 0.0;
+	    break;
+
+	  default:
+	    fprintf(stderr, "vvp error: format %d not supported "
+		    "by vpiNullConst\n", (int)val->format);
+	    val->format = vpiSuppressVal;
+	    break;
+      }
+}
+
+vpiHandle vpip_make_null_const()
+{
+      __vpiNullConst*obj = new __vpiNullConst;
+      return obj;
+}
+
 #ifdef CHECK_WITH_VALGRIND
 void constant_delete(vpiHandle item)
 {
@@ -774,6 +871,9 @@ void constant_delete(vpiHandle item)
 	  case vpiRealConst:
 	    delete dynamic_cast<__vpiRealConst*>(item);
 	    break;
+	  case vpiNullConst:
+	    delete dynamic_cast<__vpiNullConst*>(item);
+	    break;
 	  default:
 	    assert(0);
       }
