@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2019 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2022 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -64,12 +64,18 @@ void vpip_add_env_and_default_module_paths()
       if (disable_default_paths)
 	    return;
 
+#ifdef __MINGW32__
+      const char path_sep = ';';
+#else
+      const char path_sep = ':';
+#endif
+
       if (char *var = ::getenv("IVERILOG_VPI_MODULE_PATH")) {
             char *ptr = var;
             char *end = var+strlen(var);
             int len = 0;
             while (ptr <= end) {
-                  if (*ptr == 0 || *ptr == ':' || *ptr == ';') {
+                  if (*ptr == 0 || *ptr == path_sep) {
                         if (len > 0) {
                               vpip_add_module_path(strndup(var, len));
                         }
@@ -151,7 +157,11 @@ void vpip_load_module(const char*name)
 
       ivl_dll_t dll = 0;
       buf[0] = 0;                     /* terminate the string */
+#ifdef __MINGW32__
+      if (strchr(name, '\\') || strchr(name, '/')) {
+#else
       if (strchr(name, sep)) {
+#endif
 	      /* If the name has at least one directory character in
 		 it, then assume it is a complete name, maybe including any
 		 possible .vpi suffix. */
@@ -173,7 +183,7 @@ void vpip_load_module(const char*name)
 
 		  if (rc != 0) {
 			fprintf(stderr, "%s: Unable to find module file `%s' "
-				"or `%s.vpi'.\n", name,name,buf);
+				"or `%s.vpi'.\n", name, name, name);
 			return;
 		  }
 	    } else {
@@ -221,16 +231,13 @@ void vpip_load_module(const char*name)
 
 #if defined(__MINGW32__) || defined (__CYGWIN__)
       void*function = ivl_dlsym(dll, "vpip_set_callback");
-      if (function == 0) {
-	    fprintf(stderr, "%s: no vpip_set_callback()\n", name);
-	    ivl_dlclose(dll);
-	    return;
-      }
-      vpip_set_callback_t set_callback = (vpip_set_callback_t)function;
-      if (!set_callback(&vpi_routines, vpip_routines_version)) {
-	    fprintf(stderr, "Failed to link VPI module %s. Try rebuilding it with iverilog-vpi.\n", name);
-	    ivl_dlclose(dll);
-	    return;
+      if (function) {
+            vpip_set_callback_t set_callback = (vpip_set_callback_t)function;
+            if (!set_callback(&vpi_routines, vpip_routines_version)) {
+	          fprintf(stderr, "Failed to link VPI module %s. Try rebuilding it with iverilog-vpi.\n", name);
+	          ivl_dlclose(dll);
+	          return;
+            }
       }
 #endif
 
