@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2020 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2008-2021 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -39,6 +39,24 @@ FILE*vpi_trace = 0;
 static s_vpi_vlog_info  vpi_vlog_info;
 static s_vpi_error_info vpip_last_error = { 0, 0, 0, 0, 0, 0, 0 };
 
+const char* direction_as_string(int dir)
+{
+      switch (dir) {
+	  case vpiInput:
+	    return "input";
+	  case vpiOutput:
+	    return "output";
+	  case vpiInout:
+	    return "inout";
+	  case vpiMixedIO:
+	    return "mixed io";
+	  case vpiNoDirection:
+	    return "no direction";
+	  default:
+	    return "INVALID-DIRECTION";
+      }
+}
+
 __vpiHandle::~__vpiHandle()
 { }
 
@@ -111,7 +129,8 @@ unsigned vpip_size(__vpiSignal *sig)
 __vpiScope* vpip_scope(__vpiSignal*sig)
 {
       if (sig->is_netarray)
-	    return  (__vpiScope*) vpi_handle(vpiScope, sig->within.parent);
+	    return static_cast<__vpiScope*>(vpi_handle(vpiScope,
+	                                               sig->within.parent));
       else
 	    return sig->within.scope;
 }
@@ -119,7 +138,8 @@ __vpiScope* vpip_scope(__vpiSignal*sig)
 __vpiScope* vpip_scope(__vpiRealVar*sig)
 {
       if (sig->is_netarray)
-	    return  (__vpiScope*) vpi_handle(vpiScope, sig->within.parent);
+	    return static_cast<__vpiScope*>(vpi_handle(vpiScope,
+	                                               sig->within.parent));
       else
 	    return sig->within.scope;
 }
@@ -300,12 +320,12 @@ static const char* vpi_property_str(PLI_INT32 code)
           case vpiSize:
 	    return "vpiSize";
 	  default:
-	    sprintf(buf, "%d", (int)code);
+	    snprintf(buf, sizeof(buf), "%d", (int)code);
       }
       return buf;
 }
 
-static const char* vpi_type_values(PLI_INT32 code)
+const char* vpi_type_as_string(PLI_INT32 code)
 {
       static char buf[32];
       switch (code) {
@@ -382,7 +402,7 @@ static const char* vpi_type_values(PLI_INT32 code)
 	  case vpiUserSystf:
 	    return "vpiUserSystf";
 	  default:
-	    sprintf(buf, "%d", (int)code);
+	    snprintf(buf, sizeof(buf), "%d", (int)code);
       }
       return buf;
 }
@@ -398,7 +418,7 @@ PLI_INT32 vpi_get(int property, vpiHandle ref)
       if (property == vpiType) {
 	    if (vpi_trace) {
 		  fprintf(vpi_trace, "vpi_get(vpiType, %p) --> %s\n",
-			  ref, vpi_type_values(ref->get_type_code()));
+			  ref, vpi_type_as_string(ref->get_type_code()));
 	    }
 
 	    if (ref->get_type_code() == vpiMemory && is_net_array(ref))
@@ -442,7 +462,7 @@ char* vpi_get_str(PLI_INT32 property, vpiHandle ref)
       if (property == vpiType) {
 	    if (vpi_trace) {
 		  fprintf(vpi_trace, "vpi_get(vpiType, %p) --> %s\n",
-			  ref, vpi_type_values(ref->get_type_code()));
+			  ref, vpi_type_as_string(ref->get_type_code()));
 	    }
 
             PLI_INT32 type;
@@ -450,7 +470,7 @@ char* vpi_get_str(PLI_INT32 property, vpiHandle ref)
 		  type = vpiNetArray;
 	    else
 		  type = ref->get_type_code();
-	    return (char *)vpi_type_values(type);
+	    return (char *)vpi_type_as_string(type);
       }
 
       char*res = ref->vpi_get_str(property);
@@ -495,7 +515,7 @@ int vpip_time_units_from_handle(vpiHandle obj)
 
 	  default:
 	    fprintf(stderr, "ERROR: vpip_time_units_from_handle called with "
-		    "object handle type=%u\n", obj->get_type_code());
+		    "object handle type=%d\n", obj->get_type_code());
 	    assert(0);
 	    return 0;
       }
@@ -527,7 +547,7 @@ int vpip_time_precision_from_handle(vpiHandle obj)
 
 	  default:
 	    fprintf(stderr, "ERROR: vpip_time_precision_from_handle called "
-		    "with object handle type=%u\n", obj->get_type_code());
+		    "with object handle type=%d\n", obj->get_type_code());
 	    assert(0);
 	    return 0;
       }
@@ -1238,8 +1258,16 @@ static vpiHandle vpip_make_udp_iterator()
 static vpiHandle vpi_iterate_global(int type)
 {
       switch (type) {
+	  case vpiInstance:
+	    // fallthrough
 	  case vpiModule:
-	    return vpip_make_root_iterator();
+	    // fallthrough
+	  case vpiProgram:
+	    // fallthrough
+	  case vpiInterface:
+	    // fallthrough
+	  case vpiPackage:
+	    return vpip_make_root_iterator(type);
 
 	  case vpiUdpDefn:
 	    return vpip_make_udp_iterator();
