@@ -1,7 +1,7 @@
 #ifndef IVL_vpi_priv_H
 #define IVL_vpi_priv_H
 /*
- * Copyright (c) 2001-2018 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2021 Stephen Williams (steve@icarus.com)
  * Copyright (c) 2016 CERN Michele Castellana (michele.castellana@cern.ch)
  *
  *    This source code is free software; you can redistribute it
@@ -64,6 +64,18 @@ extern bool code_is_instrumented;
 extern vpiHandle vpip_build_file_line(char*description,
                                       long file_idx, long lineno);
 
+/*
+ * vpi_type_values returns the text form of the vpi type. If the type name
+ * is unknown, then return the number stringified. Note that the return
+ * value may return to a static buffer.
+ */
+extern const char* vpi_type_as_string(PLI_INT32 code);
+
+/*
+ * Return the port direction as a string.
+ */
+extern const char* direction_as_string(int dir);
+
 /*
  * Private VPI properties that are only used in the cleanup code.
  */
@@ -160,6 +172,9 @@ class __vpiDecConst : public __vpiHandle {
     public:
       explicit __vpiDecConst(int val =0);
       __vpiDecConst(const __vpiDecConst&that);
+# if __cplusplus >= 201103L
+      __vpiDecConst& operator = (const __vpiDecConst&that) = default;
+# endif
       int get_type_code(void) const;
       int vpi_get(int code);
       void vpi_get_value(p_vpi_value val);
@@ -326,7 +341,7 @@ extern void vpip_attach_to_current_scope(vpiHandle obj);
 extern __vpiScope* vpip_peek_context_scope(void);
 extern unsigned vpip_add_item_to_context(automatic_hooks_s*item,
                                          __vpiScope*scope);
-extern vpiHandle vpip_make_root_iterator(void);
+extern vpiHandle vpip_make_root_iterator(int type_code);
 extern void vpip_make_root_iterator(class __vpiHandle**&table,
 				    unsigned&ntable);
 
@@ -888,6 +903,16 @@ vpiHandle vpip_make_real_const(double value);
 vpiHandle vpip_make_real_param(char*name, double value, bool local_flag,
                                long file_idx, long lineno);
 
+class __vpiNullConst : public __vpiHandle {
+    public:
+      explicit __vpiNullConst();
+      int get_type_code(void) const;
+      int vpi_get(int code);
+      void vpi_get_value(p_vpi_value val);
+};
+
+vpiHandle vpip_make_null_const();
+
 /*
  *  This one looks like a constant, but really is a vector in the current
  *  thread.
